Module:       llvm-tablegen
Author:       Peter S. Housel
Copyright:    Original Code is Copyright 2009-2011 Gwydion Dylan Maintainers
              All rights reserved.
License:      See License.txt in this distribution for details.
Warranty:     Distributed WITHOUT WARRANTY OF ANY KIND

define class <tablegen-class-reference> (<object>)
  constant slot class-reference-class :: type-union(<tablegen-class>, <tablegen-multi-class>),
    required-init-keyword: class:;
  constant slot class-reference-template-arguments :: <sequence>,
    init-value: #(), init-keyword: template-arguments:;
end class;

define variable *tablegen-overrides* = make(<string-table>);

define function add-initializer
    (sequence :: <stretchy-sequence>,
     environment :: <string-table>,
     initializer :: <tablegen-initializer>)
 => (value :: <tablegen-value>);
  let value
    = element(*tablegen-overrides*, initializer.initializer-name, default: #f)
    | resolve-value(environment, initializer.initializer-value);
  add!(sequence, make(<tablegen-initializer>,
                      name: initializer.initializer-name,
                      type: initializer.initializer-type,
                      value: value));
  value
end function;

define function instantiate-class-reference
    (sequence :: <stretchy-sequence>,
     environment :: <string-table>,
     reference :: <tablegen-class-reference>,
     resolve? :: <boolean>)
 => ();
  let class = reference.class-reference-class;
  let template-arguments
    = reference.class-reference-template-arguments;
  for (parameter in class.record-template-parameters, index from 0)
    let parameter-name
      = format-to-string("%s:%s",
                         class.record-name,
                         parameter.initializer-name);
    let parameter-value
      = element(template-arguments, index, default: #f);
    if (parameter-value)
      environment[parameter-name] := parameter-value;
    else
      let value = resolve-value(environment, parameter.initializer-value);
      add!(sequence,
           make(<tablegen-initializer>,
                name: parameter-name,
                type: parameter.initializer-type,
                value: value));
      if (resolve?)
        environment[parameter-name] := value;
      end if;
    end if;
  end for;

  for (initializer in class.record-initializers)
    let value = add-initializer(sequence, environment, initializer);
    if (resolve?)
      environment[initializer.initializer-name] := value;
    end if;
  end for;
end function;

define variable *tablegen-anonymous-counter* :: <integer> = 0;
define variable *tablegen-anonymous-value-counter* :: <integer> = 0;

define constant $EOF-token = 0;
define constant $%LBRACE-token = 1;
define constant $%RBRACE-token = 2;
define constant $%CLASS-token = 3;
define constant $%ID-token = 4;
define constant $%LANGLE-token = 5;
define constant $%RANGLE-token = 6;
define constant $%COMMA-token = 7;
define constant $%FIELD-token = 8;
define constant $%STRING-token = 9;
define constant $%BIT-token = 10;
define constant $%BITS-token = 11;
define constant $%INTVAL-token = 12;
define constant $%INT-token = 13;
define constant $%LIST-token = 14;
define constant $%CODE-token = 15;
define constant $%DAG-token = 16;
define constant $%EQ-token = 17;
define constant $%STRVAL-token = 18;
define constant $%QUEST-token = 19;
define constant $%LBRACK-token = 20;
define constant $%RBRACK-token = 21;
define constant $%DOT-token = 22;
define constant $%LPAREN-token = 23;
define constant $%RPAREN-token = 24;
define constant $%CARTOK-token = 25;
define constant $%CDRTOK-token = 26;
define constant $%NULLTOK-token = 27;
define constant $%CASTTOK-token = 28;
define constant $%CONCATTOK-token = 29;
define constant $%SHLTOK-token = 30;
define constant $%SRATOK-token = 31;
define constant $%SRLTOK-token = 32;
define constant $%EQTOK-token = 33;
define constant $%STRCONCATTOK-token = 34;
define constant $%NAMECONCATTOK-token = 35;
define constant $%IFTOK-token = 36;
define constant $%FOREACHTOK-token = 37;
define constant $%SUBSTTOK-token = 38;
define constant $%MINUS-token = 39;
define constant $%COLON-token = 40;
define constant $%VARNAME-token = 41;
define constant $%SEMI-token = 42;
define constant $%LET-token = 43;
define constant $%DEF-token = 44;
define constant $%MULTICLASS-token = 45;
define constant $%DEFM-token = 46;
define constant $%IN-token = 47;

define function tablegen-parser-action0 (arg$1, arg$2) => (value)
  #f
end tablegen-parser-action0;

define function tablegen-parser-action1 (arg$1) => (value)
  #f
end tablegen-parser-action1;

define function tablegen-parser-action3 (arg$1) => (value)
  arg$1("", make(<string-table>))
end tablegen-parser-action3;

define function tablegen-parser-action7 (arg$1, arg$2, arg$3, arg$4) => (value)
  *tablegen-overrides* := arg$1
end tablegen-parser-action7;

define function tablegen-parser-action8 (arg$1, arg$2) => (value)
  *tablegen-overrides* := arg$1
end tablegen-parser-action8;

define function tablegen-parser-action9 (arg$1, arg$2, arg$3, arg$4) => (value)
  method (prefix :: <string>, environment :: <string-table>)
         // Alpha-convert template parameters
         for (parameter in arg$3)
           environment[parameter.initializer-name]
             := make(<tablegen-named-value>,
                     name: format-to-string("%s:%s",
                                            arg$2,
                                            parameter.initializer-name));
         end for;

         let initializers = make(<stretchy-object-vector>);

         for (reference in arg$4.head)
           instantiate-class-reference
             (initializers, environment, reference, #f);
         end for;

         do(curry(add-initializer, initializers, environment), arg$4.tail);
         $tablegen-classes[arg$2]
           := make(<tablegen-class>,
                   name: arg$2,
                   template-parameters: arg$3,
                   superclasses: map(class-reference-class, arg$4.head),
                   initializers: initializers);
       end
end tablegen-parser-action9;

define function tablegen-parser-action10 (arg$1) => (value)
  arg$1
end tablegen-parser-action10;

define function tablegen-parser-action12 () => (value)
  begin
         let name
           = format-to-string("anonymous.%d", *tablegen-anonymous-counter*);
         *tablegen-anonymous-counter* := *tablegen-anonymous-counter* + 1;
         name
       end
end tablegen-parser-action12;

define function tablegen-parser-action14 () => (value)
  #()
end tablegen-parser-action14;

define function tablegen-parser-action16 (arg$1, arg$2, arg$3) => (value)
  reverse!(arg$2)
end tablegen-parser-action16;

define function tablegen-parser-action17 (arg$1) => (value)
  list(arg$1)
end tablegen-parser-action17;

define function tablegen-parser-action18 (arg$1, arg$2, arg$3) => (value)
  add(arg$1, arg$3)
end tablegen-parser-action18;

define function tablegen-parser-action19 (arg$1, arg$2, arg$3, arg$4) => (value)
  make(<tablegen-initializer>, type: arg$2, name: arg$3, value: arg$4)
end tablegen-parser-action19;

define function tablegen-parser-action20 () => (value)
  #f
end tablegen-parser-action20;

define function tablegen-parser-action22 (arg$1) => (value)
  make(<tablegen-simple-type>, kind: #"STRING")
end tablegen-parser-action22;

define function tablegen-parser-action23 (arg$1) => (value)
  make(<tablegen-simple-type>, kind: #"BIT")
end tablegen-parser-action23;

define function tablegen-parser-action24 (arg$1, arg$2, arg$3, arg$4) => (value)
  make(<tablegen-bits-type>, size: arg$3)
end tablegen-parser-action24;

define function tablegen-parser-action25 (arg$1) => (value)
  make(<tablegen-simple-type>, kind: #"INT")
end tablegen-parser-action25;

define function tablegen-parser-action26 (arg$1, arg$2, arg$3, arg$4) => (value)
  make(<tablegen-list-type>, of: arg$3)
end tablegen-parser-action26;

define function tablegen-parser-action27 (arg$1) => (value)
  make(<tablegen-simple-type>, kind: #"CODE")
end tablegen-parser-action27;

define function tablegen-parser-action28 (arg$1) => (value)
  make(<tablegen-simple-type>, kind: #"DAG")
end tablegen-parser-action28;

define function tablegen-parser-action29 (arg$1) => (value)
  make(<tablegen-class-type>, class: arg$1)
end tablegen-parser-action29;

define function tablegen-parser-action30 (arg$1) => (value)
  $tablegen-classes[arg$1]
end tablegen-parser-action30;

define function tablegen-parser-action31 () => (value)
  make(<tablegen-unknown-value>)
end tablegen-parser-action31;

define function tablegen-parser-action32 (arg$1, arg$2) => (value)
  arg$2
end tablegen-parser-action32;

define function tablegen-parser-action33 (arg$1) => (value)
  make(<tablegen-named-value>, name: arg$1)
end tablegen-parser-action33;

define function tablegen-parser-action34 (arg$1) => (value)
  make(<tablegen-simple-value>, value: arg$1)
end tablegen-parser-action34;

define function tablegen-parser-action36 (arg$1) => (value)
  make(<tablegen-unknown-value>)
end tablegen-parser-action36;

define function tablegen-parser-action37 (arg$1, arg$2, arg$3) => (value)
  error("bits-value")
end tablegen-parser-action37;

define function tablegen-parser-action38 (arg$1, arg$2, arg$3, arg$4) => (value)
  begin
         let def-name
           = format-to-string("anonymous.val.%d",
                              *tablegen-anonymous-value-counter*);
         *tablegen-anonymous-value-counter*
           := *tablegen-anonymous-value-counter* + 1;

         let initializers = make(<stretchy-object-vector>);
         let environment = make(<string-table>);
         let reference
           = make(<tablegen-class-reference>,
                  class: arg$1, template-arguments: reverse!(arg$3));
         instantiate-class-reference(initializers, environment, reference, #t);

         $tablegen-definitions[def-name]
           := make(<tablegen-definition>,
                   name: def-name,
                   superclasses: vector(arg$1),
                   initializers: initializers);

         make(<tablegen-named-value>, name: def-name);
       end
end tablegen-parser-action38;

define function tablegen-parser-action39 (arg$1, arg$2, arg$3, arg$4) => (value)
  error("bit-list value")
end tablegen-parser-action39;

define function tablegen-parser-action40 (arg$1, arg$2, arg$3) => (value)
  make(<tablegen-list-value>, list: arg$2)
end tablegen-parser-action40;

define function tablegen-parser-action41 (arg$1, arg$2, arg$3) => (value)
  error("field %s", arg$3)
end tablegen-parser-action41;

define function tablegen-parser-action42 (arg$1, arg$2, arg$3, arg$4) => (value)
  error("dag")
end tablegen-parser-action42;

define function tablegen-parser-action43 (arg$1, arg$2, arg$3, arg$4) => (value)
  error("slice")
end tablegen-parser-action43;

define function tablegen-parser-action44 (arg$1, arg$2, arg$3, arg$4) => (value)
  error("!car")
end tablegen-parser-action44;

define function tablegen-parser-action45 (arg$1, arg$2, arg$3, arg$4) => (value)
  error("!cdr")
end tablegen-parser-action45;

define function tablegen-parser-action46 (arg$1, arg$2, arg$3, arg$4) => (value)
  error("!null")
end tablegen-parser-action46;

define function tablegen-parser-action47 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  error("!cast")
end tablegen-parser-action47;

define function tablegen-parser-action48 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  error("!concat")
end tablegen-parser-action48;

define function tablegen-parser-action49 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  error("!shl")
end tablegen-parser-action49;

define function tablegen-parser-action50 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  error("!sra")
end tablegen-parser-action50;

define function tablegen-parser-action51 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  error("!srl")
end tablegen-parser-action51;

define function tablegen-parser-action52 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  error("!eq")
end tablegen-parser-action52;

define function tablegen-parser-action53 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  make(<tablegen-operator-value>,
            kind: #"strconcat",
            operands: vector(arg$3, arg$5))
end tablegen-parser-action53;

define function tablegen-parser-action54 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9) => (value)
  error("!nameconcat")
end tablegen-parser-action54;

define function tablegen-parser-action55 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8) => (value)
  error("!if")
end tablegen-parser-action55;

define function tablegen-parser-action56 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8) => (value)
  error("!foreach")
end tablegen-parser-action56;

define function tablegen-parser-action57 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8) => (value)
  error("!subst")
end tablegen-parser-action57;

define function tablegen-parser-action60 (arg$1) => (value)
  reverse!(arg$1)
end tablegen-parser-action60;

define function tablegen-parser-action65 (arg$1, arg$2, arg$3) => (value)
  #f
end tablegen-parser-action65;

define function tablegen-parser-action68 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  #f
end tablegen-parser-action68;

define function tablegen-parser-action69 (arg$1, arg$2, arg$3, arg$4) => (value)
  #f
end tablegen-parser-action69;

define function tablegen-parser-action76 (arg$1, arg$2) => (value)
  pair(arg$1, arg$2)
end tablegen-parser-action76;

define function tablegen-parser-action78 (arg$1, arg$2) => (value)
  reverse!(arg$2)
end tablegen-parser-action78;

define function tablegen-parser-action81 (arg$1) => (value)
  make(<tablegen-class-reference>, class: arg$1)
end tablegen-parser-action81;

define function tablegen-parser-action82 (arg$1, arg$2, arg$3, arg$4) => (value)
  begin
         make(<tablegen-class-reference>,
              class: arg$1,
              template-arguments: reverse!(arg$3))
       end
end tablegen-parser-action82;

define function tablegen-parser-action83 (arg$1) => (value)
  #()
end tablegen-parser-action83;

define function tablegen-parser-action86 (arg$1, arg$2) => (value)
  add(arg$1, arg$2)
end tablegen-parser-action86;

define function tablegen-parser-action87 (arg$1, arg$2) => (value)
  arg$1
end tablegen-parser-action87;

define function tablegen-parser-action88 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  error("body-item %%LET")
end tablegen-parser-action88;

define function tablegen-parser-action90 (arg$1, arg$2, arg$3) => (value)
  error("opt-bit-list")
end tablegen-parser-action90;

define function tablegen-parser-action91 (arg$1, arg$2, arg$3) => (value)
  method (prefix :: <string>, environment :: <string-table>)
         let initializers = make(<stretchy-object-vector>);
         for (reference in arg$3.head)
           instantiate-class-reference
             (initializers, environment, reference, #t);
         end for;
         do(curry(add-initializer, initializers, environment), arg$3.tail);

         $tablegen-definitions[arg$2]
           := make(<tablegen-definition>,
                   name: concatenate(prefix, arg$2),
                   superclasses: map(class-reference-class, arg$3.head),
                   initializers: initializers);
       end
end tablegen-parser-action91;

define function tablegen-parser-action93 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  begin
         $tablegen-classes[arg$2]
           := make(<tablegen-multi-class>,
                   name: arg$2,
                   template-parameters: arg$3,
                   definitions: arg$5);
       end
end tablegen-parser-action93;

define function tablegen-parser-action99 (arg$1, arg$2, arg$3, arg$4) => (value)
  arg$2
end tablegen-parser-action99;

define function tablegen-parser-action100 () => (value)
  begin
         let previous-overrides = *tablegen-overrides*;
         // FIXME
         //*tablegen-overrides* := shallow-copy(*tablegen-overrides*);
         *tablegen-overrides* := make(<string-table>);
         map-into(*tablegen-overrides*, identity, previous-overrides);
         previous-overrides
       end
end tablegen-parser-action100;

define function tablegen-parser-action103 (arg$1, arg$2, arg$3, arg$4) => (value)
  *tablegen-overrides*[arg$1] := arg$4
end tablegen-parser-action103;

define constant tablegen-parser :: <parser>
  = make(<parser>,
  action-table:
      #[#[3, -6, 43, -3, 44, -10, 45, -12, 46, -13],
	#[0, -245, 3, -6, 43, -3, 44, -10, 45, -12, 46, -13],
	#[65535, 100, 4, 100],
	#[65535, 4, 43, 4, 44, 4, 45, 4, 46, 4, 0, 4, 2, 4, 3, 4],
	#[65535, 6, 0, 6, 2, 6, 3, 6, 43, 6, 44, 6, 45, 6, 46, 6],
	#[1, 12, 4, -199, 5, 12, 40, 12, 42, 12],
	#[65535, 5, 3, 5, 2, 5, 45, 5, 43, 5, 0, 5, 46, 5, 44, 5],
	#[1, -227, 3, -6, 46, -13, 43, -3, 44, -10, 45, -12],
	#[65535, 1, 0, 1, 2, 1, 3, 1, 43, 1, 44, 1, 45, 1, 46, 1],
	#[1, 12, 4, -199, 40, 12, 42, 12],
	#[65535, 3, 0, 3, 2, 3, 3, 3, 43, 3, 44, 3, 45, 3, 46, 3],
	#[4, -176],
	#[4, -14],
	#[40, -15],
	#[4, -17],
	#[1, 81, 7, 81, 5, -20, 42, 81],
	#[65535, 30, 1, 30, 4, 30, 5, 30, 6, 30, 7, 30, 42, 30],
	#[42, -19],
	#[65535, 98, 0, 98, 2, 98, 3, 98, 46, 98, 45, 98, 44, 98, 43, 98],
	#[1, -31, 23, -23, 37, -30, 4, -39, 18, -22, 29, -24, 12, -38, 27, -45, 26, -44, 19, -35, 20, -32, 35, -42, 34, -40, 25, -28, 33, -36, 32, -33, 28, -25, 30, -27, 31, -29, 36, -34, 38, -26],
	#[5, -130],
	#[65535, 35, 7, 35, 21, 35, 47, 35, 2, 35, 1, 35, 20, 35, 6, 35, 42, 35, 22, 35, 24, 35, 40, 35],
	#[4, -109],
	#[23, -139],
	#[5, -144],
	#[23, -96],
	#[23, -70],
	#[23, -127],
	#[23, -134],
	#[23, -66],
	#[12, -38, 1, -31, 4, -39, 27, -45, 23, -23, 18, -22, 25, -28, 2, 59, 20, -32, 26, -44, 19, -35, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 35, -42, 36, -34, 37, -30, 38, -26],
	#[28, -25, 21, 59, 18, -22, 12, -38, 37, -30, 1, -31, 36, -34, 4, -39, 19, -35, 29, -24, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 38, -26],
	#[23, -68],
	#[23, -116],
	#[65535, 36, 1, 36, 2, 36, 6, 36, 7, 36, 20, 36, 21, 36, 22, 36, 24, 36, 40, 36, 42, 36, 47, 36],
	#[23, -103],
	#[65535, 33, 1, 33, 2, 33, 6, 33, 7, 33, 20, 33, 21, 33, 22, 33, 24, 33, 40, 33, 42, 33, 47, 33],
	#[65535, 34, 7, 34, 1, 34, 2, 34, 22, 34, 21, 34, 20, 34, 24, 34, 6, 34, 40, 34, 42, 34, 47, 34],
	#[47, 58, 22, 58, 1, 58, 2, 58, 5, 30, 6, 58, 7, 58, 20, 58, 21, 58, 24, 58, 40, 58, 42, 58],
	#[23, -64],
	#[1, -74, 2, 61, 6, 61, 7, 61, 20, -73, 21, 61, 22, -75],
	#[5, -48],
	#[6, -174, 7, -125],
	#[23, -93],
	#[23, -46],
	#[23, -23, 1, -31, 4, -39, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 34, -40, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 36, -34, 37, -30, 38, -26],
	#[1, -74, 20, -73, 22, -75, 24, -173],
	#[4, -17, 9, -52, 10, -53, 11, -56, 13, -50, 14, -49, 15, -57, 16, -55],
	#[5, -150],
	#[65535, 25, 4, 25, 6, 25],
	#[65535, 29, 4, 29, 6, 29],
	#[65535, 22, 4, 22, 6, 22],
	#[65535, 23, 4, 23, 6, 23],
	#[6, -61],
	#[65535, 28, 4, 28, 6, 28],
	#[5, -58],
	#[65535, 27, 4, 27, 6, 27],
	#[12, -59],
	#[6, -60],
	#[65535, 24, 4, 24, 6, 24],
	#[23, -62],
	#[23, -23, 1, -31, 4, -39, 12, -38, 18, -22, 19, -35, 20, -32, 34, -40, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 35, -42, 36, -34, 37, -30, 38, -26],
	#[7, -170, 1, -74, 20, -73, 22, -75],
	#[1, -31, 4, -39, 12, -38, 18, -22, 19, -35, 20, -32, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 35, -42, 36, -34, 37, -30, 38, -26],
	#[1, -74, 7, -167, 20, -73, 22, -75],
	#[1, -31, 4, -39, 12, -38, 18, -22, 19, -35, 20, -32, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 35, -42, 36, -34, 37, -30, 38, -26],
	#[1, -74, 7, -162, 20, -73, 22, -75],
	#[1, -31, 4, -39, 29, -24, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 36, -34, 37, -30, 38, -26],
	#[1, -74, 7, -159, 20, -73, 22, -75],
	#[1, -31, 4, -39, 29, -24, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 36, -34, 37, -30, 38, -26],
	#[1, -74, 7, -72, 20, -73, 22, -75],
	#[1, -31, 4, -39, 29, -24, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 36, -34, 37, -30, 38, -26],
	#[12, -79],
	#[12, -79],
	#[4, -76],
	#[65535, 41, 1, 41, 2, 41, 6, 41, 7, 41, 20, 41, 21, 41, 22, 41, 24, 41, 40, 41, 42, 41, 47, 41],
	#[2, 63, 7, -84, 21, 63],
	#[2, -83],
	#[2, 64, 7, 64, 12, -81, 21, 64, 39, -80],
	#[12, -82],
	#[65535, 66, 2, 66, 7, 66, 21, 66],
	#[65535, 65, 2, 65, 7, 65, 21, 65],
	#[65535, 39, 1, 39, 2, 39, 6, 39, 7, 39, 20, 39, 21, 39, 22, 39, 24, 39, 40, 39, 42, 39, 47, 39],
	#[12, -85],
	#[2, 67, 7, 67, 12, -87, 21, 67, 39, -86],
	#[12, -88],
	#[65535, 69, 2, 69, 7, 69, 21, 69],
	#[65535, 68, 2, 68, 7, 68, 21, 68],
	#[21, -90],
	#[65535, 43, 20, 43, 21, 43, 22, 43, 24, 43, 40, 43, 42, 43, 47, 43, 1, 43, 2, 43, 6, 43, 7, 43],
	#[1, -74, 20, -73, 22, -75, 24, -92],
	#[65535, 49, 1, 49, 2, 49, 6, 49, 7, 49, 20, 49, 21, 49, 22, 49, 24, 49, 40, 49, 42, 49, 47, 49],
	#[30, -27, 23, -23, 29, -24, 4, -39, 1, -31, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 34, -40, 25, -28, 26, -44, 27, -45, 28, -25, 31, -29, 32, -33, 33, -36, 36, -34, 37, -30, 38, -26],
	#[1, -74, 20, -73, 22, -75, 24, -95],
	#[65535, 45, 22, 45, 6, 45, 7, 45, 2, 45, 1, 45, 20, 45, 21, 45, 24, 45, 40, 45, 42, 45, 47, 45],
	#[1, -31, 4, -39, 12, -38, 18, -22, 19, -35, 20, -32, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 35, -42, 36, -34, 37, -30, 38, -26],
	#[1, -74, 7, -98, 20, -73, 22, -75],
	#[1, -31, 4, -39, 12, -38, 18, -22, 19, -35, 20, -32, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 35, -42, 36, -34, 37, -30, 38, -26],
	#[1, -74, 7, -100, 20, -73, 22, -75],
	#[1, -31, 4, -39, 29, -24, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 36, -34, 37, -30, 38, -26],
	#[1, -74, 20, -73, 22, -75, 24, -102],
	#[65535, 57, 7, 57, 2, 57, 6, 57, 1, 57, 47, 57, 20, 57, 21, 57, 22, 57, 24, 57, 40, 57, 42, 57],
	#[1, -31, 4, -39, 19, -35, 18, -22, 12, -38, 27, -45, 38, -26, 25, -28, 36, -34, 20, -32, 23, -23, 26, -44, 32, -33, 28, -25, 29, -24, 30, -27, 31, -29, 33, -36, 34, -40, 35, -42, 37, -30],
	#[1, -74, 7, -105, 20, -73, 22, -75],
	#[1, -31, 4, -39, 12, -38, 18, -22, 19, -35, 20, -32, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 35, -42, 36, -34, 37, -30, 38, -26],
	#[1, -74, 20, -73, 22, -75, 24, -107],
	#[65535, 52, 1, 52, 2, 52, 6, 52, 7, 52, 20, 52, 21, 52, 22, 52, 24, 52, 40, 52, 42, 52, 47, 52],
	#[34, -40, 1, -31, 37, -30, 27, -45, 4, -39, 29, -24, 12, -38, 38, -26, 18, -22, 19, -35, 20, -32, 23, -23, 24, 70, 25, -28, 26, -44, 32, -33, 28, -25, 30, -27, 31, -29, 33, -36, 35, -42, 36, -34],
	#[65535, 58, 31, 58, 23, 58, 38, 58, 1, 58, 30, 58, 4, 58, 29, 58, 12, 58, 37, 58, 34, 58, 36, 58, 26, 58, 19, 58, 18, 58, 25, 58, 33, 58, 20, 58, 24, 58, 27, 58, 28, 58, 32, 58, 35, 58],
	#[7, -156, 24, 71],
	#[24, -155],
	#[1, -74, 7, 74, 20, -73, 22, -75, 24, 74, 40, -114],
	#[65535, 72, 7, 72, 24, 72],
	#[41, -115],
	#[65535, 75, 7, 75, 24, 75],
	#[34, -40, 1, -31, 4, -39, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 36, -34, 37, -30, 38, -26],
	#[7, -118, 20, -73, 1, -74, 22, -75],
	#[34, -40, 1, -31, 4, -39, 28, -25, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 36, -34, 37, -30, 38, -26],
	#[1, -74, 7, -120, 20, -73, 22, -75],
	#[1, -31, 4, -39, 12, -38, 18, -22, 19, -35, 20, -32, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 35, -42, 36, -34, 37, -30, 38, -26],
	#[1, -74, 20, -73, 22, -75, 24, -122],
	#[65535, 55, 1, 55, 2, 55, 6, 55, 7, 55, 20, 55, 21, 55, 22, 55, 24, 55, 40, 55, 42, 55, 47, 55],
	#[21, -154],
	#[7, -125, 2, 60, 21, 60],
	#[30, -27, 4, -39, 23, -23, 1, -31, 29, -24, 25, -28, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 34, -40, 26, -44, 27, -45, 28, -25, 31, -29, 32, -33, 33, -36, 36, -34, 37, -30, 38, -26],
	#[1, -74, 2, 62, 6, 62, 7, 62, 20, -73, 21, 62, 22, -75],
	#[4, -39, 1, -31, 12, -38, 18, -22, 19, -35, 20, -32, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 34, -40, 35, -42, 36, -34, 37, -30, 38, -26],
	#[22, -75, 1, -74, 20, -73, 24, -129],
	#[65535, 44, 1, 44, 2, 44, 6, 44, 7, 44, 20, 44, 21, 44, 22, 44, 24, 44, 40, 44, 42, 44, 47, 44],
	#[30, -27, 36, -34, 4, -39, 34, -40, 38, -26, 12, -38, 1, -31, 31, -29, 26, -44, 33, -36, 18, -22, 29, -24, 19, -35, 20, -32, 32, -33, 23, -23, 25, -28, 27, -45, 28, -25, 35, -42, 37, -30],
	#[6, -132, 7, -125],
	#[65535, 38, 21, 38, 6, 38, 20, 38, 2, 38, 1, 38, 7, 38, 22, 38, 24, 38, 40, 38, 42, 38, 47, 38],
	#[2, -153],
	#[34, -40, 1, -31, 4, -39, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 32, -33, 33, -36, 36, -34, 37, -30, 38, -26],
	#[22, -75, 1, -74, 7, -136, 20, -73],
	#[12, -38, 1, -31, 31, -29, 4, -39, 33, -36, 30, -27, 18, -22, 19, -35, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 32, -33, 34, -40, 36, -34, 37, -30, 38, -26],
	#[22, -75, 1, -74, 20, -73, 24, -138],
	#[65535, 50, 1, 50, 2, 50, 20, 50, 6, 50, 7, 50, 47, 50, 21, 50, 22, 50, 24, 50, 40, 50, 42, 50],
	#[23, -23, 1, -31, 4, -39, 33, -36, 30, -27, 35, -42, 29, -24, 12, -38, 18, -22, 19, -35, 20, -32, 32, -33, 25, -28, 26, -44, 27, -45, 28, -25, 31, -29, 34, -40, 36, -34, 37, -30, 38, -26],
	#[1, -74, 7, -141, 20, -73, 22, -75],
	#[34, -40, 1, -31, 31, -29, 4, -39, 30, -27, 29, -24, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 32, -33, 33, -36, 36, -34, 37, -30, 38, -26],
	#[22, -75, 1, -74, 20, -73, 24, -143],
	#[65535, 48, 1, 48, 2, 48, 6, 48, 7, 48, 20, 48, 21, 48, 22, 48, 24, 48, 40, 48, 42, 48, 47, 48],
	#[15, -57, 4, -17, 9, -52, 10, -53, 11, -56, 13, -50, 14, -49, 16, -55],
	#[6, -146],
	#[23, -147],
	#[23, -23, 37, -30, 1, -31, 4, -39, 12, -38, 38, -26, 18, -22, 19, -35, 20, -32, 32, -33, 25, -28, 26, -44, 27, -45, 28, -25, 29, -24, 30, -27, 31, -29, 33, -36, 34, -40, 35, -42, 36, -34],
	#[20, -73, 1, -74, 22, -75, 24, -149],
	#[65535, 47, 1, 47, 2, 47, 6, 47, 7, 47, 20, 47, 21, 47, 22, 47, 24, 47, 40, 47, 42, 47, 47, 47],
	#[15, -57, 9, -52, 14, -49, 13, -50, 16, -55, 11, -56, 10, -53, 4, -17],
	#[6, -152],
	#[65535, 26, 6, 26, 4, 26],
	#[65535, 37, 20, 37, 22, 37, 1, 37, 2, 37, 6, 37, 7, 37, 24, 37, 47, 37, 42, 37, 21, 37, 40, 37],
	#[65535, 40, 6, 40, 21, 40, 20, 40, 2, 40, 1, 40, 42, 40, 7, 40, 22, 40, 24, 40, 40, 40, 47, 40],
	#[65535, 42, 1, 42, 2, 42, 6, 42, 7, 42, 42, 42, 47, 42, 20, 42, 21, 42, 22, 42, 24, 42, 40, 42],
	#[30, -27, 31, -29, 12, -38, 19, -35, 20, -32, 27, -45, 34, -40, 29, -24, 1, -31, 33, -36, 37, -30, 4, -39, 18, -22, 35, -42, 23, -23, 25, -28, 26, -44, 28, -25, 32, -33, 36, -34, 38, -26],
	#[22, -75, 1, -74, 7, 74, 20, -73, 24, 74, 40, -114],
	#[65535, 73, 7, 73, 24, 73],
	#[23, -23, 34, -40, 31, -29, 12, -38, 30, -27, 36, -34, 4, -39, 27, -45, 38, -26, 19, -35, 1, -31, 37, -30, 26, -44, 33, -36, 18, -22, 35, -42, 29, -24, 25, -28, 32, -33, 20, -32, 28, -25],
	#[1, -74, 20, -73, 22, -75, 24, -161],
	#[65535, 51, 21, 51, 1, 51, 2, 51, 6, 51, 7, 51, 42, 51, 47, 51, 20, 51, 22, 51, 24, 51, 40, 51],
	#[30, -27, 23, -23, 36, -34, 4, -39, 18, -22, 34, -40, 20, -32, 1, -31, 38, -26, 12, -38, 33, -36, 37, -30, 32, -33, 35, -42, 29, -24, 25, -28, 27, -45, 19, -35, 26, -44, 28, -25, 31, -29],
	#[1, -74, 7, -164, 20, -73, 22, -75],
	#[34, -40, 37, -30, 1, -31, 4, -39, 30, -27, 29, -24, 12, -38, 18, -22, 19, -35, 20, -32, 35, -42, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 31, -29, 32, -33, 33, -36, 36, -34, 38, -26],
	#[22, -75, 1, -74, 20, -73, 24, -166],
	#[65535, 56, 2, 56, 1, 56, 6, 56, 7, 56, 20, 56, 21, 56, 22, 56, 24, 56, 40, 56, 42, 56, 47, 56],
	#[4, -39, 1, -31, 18, -22, 35, -42, 29, -24, 12, -38, 27, -45, 38, -26, 19, -35, 20, -32, 32, -33, 23, -23, 25, -28, 26, -44, 28, -25, 30, -27, 31, -29, 33, -36, 34, -40, 36, -34, 37, -30],
	#[1, -74, 24, -169, 20, -73, 22, -75],
	#[65535, 53, 7, 53, 1, 53, 2, 53, 6, 53, 20, 53, 21, 53, 22, 53, 24, 53, 40, 53, 42, 53, 47, 53],
	#[38, -26, 37, -30, 1, -31, 4, -39, 18, -22, 35, -42, 29, -24, 12, -38, 19, -35, 20, -32, 32, -33, 23, -23, 25, -28, 26, -44, 27, -45, 28, -25, 30, -27, 31, -29, 33, -36, 34, -40, 36, -34],
	#[22, -75, 1, -74, 20, -73, 24, -172],
	#[65535, 54, 6, 54, 47, 54, 1, 54, 2, 54, 7, 54, 20, 54, 21, 54, 22, 54, 24, 54, 40, 54, 42, 54],
	#[65535, 46, 2, 46, 1, 46, 20, 46, 6, 46, 7, 46, 47, 46, 21, 46, 22, 46, 24, 46, 40, 46, 42, 46],
	#[65535, 82, 1, 82, 7, 82, 42, 82],
	#[1, 14, 5, -179],
	#[65535, 94, 1, 94, 5, 94],
	#[65535, 15, 1, 15, 40, 15, 42, 15],
	#[1, -192],
	#[15, 20, 13, 20, 14, 20, 9, 20, 16, 20, 11, 20, 8, -182, 10, 20, 4, 20],
	#[65535, 17, 7, 17, 6, 17],
	#[7, -190, 6, -189],
	#[65535, 21, 13, 21, 4, 21, 10, 21, 11, 21, 16, 21, 9, 21, 15, 21, 14, 21],
	#[11, -56, 10, -53, 9, -52, 4, -17, 13, -50, 14, -49, 15, -57, 16, -55],
	#[4, -185],
	#[17, -186, 6, 31, 7, 31, 42, 31],
	#[38, -26, 31, -29, 12, -38, 27, -45, 30, -27, 20, -32, 4, -39, 19, -35, 23, -23, 1, -31, 26, -44, 34, -40, 33, -36, 37, -30, 25, -28, 18, -22, 32, -33, 28, -25, 29, -24, 35, -42, 36, -34],
	#[65535, 19, 7, 19, 6, 19, 42, 19],
	#[7, 32, 20, -73, 1, -74, 22, -75, 6, 32, 42, 32],
	#[65535, 16, 42, 16, 1, 16, 40, 16],
	#[4, 20, 8, -182, 9, 20, 10, 20, 11, 20, 13, 20, 14, 20, 15, 20, 16, 20],
	#[65535, 18, 6, 18, 7, 18],
	#[44, -10],
	#[65535, 97, 2, 97, 44, 97],
	#[65535, 95, 44, 95, 2, 95],
	#[2, -197, 44, -10],
	#[65535, 96, 2, 96, 44, 96],
	#[65535, 93, 46, 93, 0, 93, 45, 93, 3, 93, 2, 93, 43, 93, 44, 93],
	#[65535, 92, 1, 92, 42, 92, 40, 92],
	#[65535, 13, 1, 13, 5, 13, 40, 13, 42, 13],
	#[1, 77, 42, 77, 40, -204],
	#[65535, 11, 1, 11, 5, 11, 40, 11, 42, 11],
	#[1, -211, 42, -209],
	#[65535, 91, 2, 91, 3, 91, 0, 91, 44, 91, 46, 91, 43, 91, 45, 91],
	#[4, -17],
	#[65535, 79, 1, 79, 7, 79, 42, 79],
	#[1, 78, 7, -207, 42, 78],
	#[4, -17],
	#[65535, 80, 7, 80, 1, 80, 42, 80],
	#[65535, 83, 46, 83, 43, 83, 2, 83, 0, 83, 3, 83, 45, 83, 44, 83],
	#[65535, 76, 45, 76, 0, 76, 2, 76, 3, 76, 44, 76, 43, 76, 46, 76],
	#[65535, 85, 13, 85, 14, 85, 15, 85, 16, 85, 9, 85, 11, 85, 2, 85, 8, 85, 10, 85, 4, 85, 43, 85],
	#[14, 20, 4, 20, 11, 20, 15, 20, 2, -215, 8, -182, 9, 20, 10, 20, 16, 20, 13, 20, 43, -213],
	#[4, -218],
	#[42, -217],
	#[65535, 84, 45, 84, 2, 84, 3, 84, 46, 84, 0, 84, 43, 84, 44, 84],
	#[65535, 86, 14, 86, 15, 86, 4, 86, 11, 86, 2, 86, 9, 86, 8, 86, 16, 86, 13, 86, 10, 86, 43, 86],
	#[65535, 87, 10, 87, 4, 87, 13, 87, 11, 87, 2, 87, 8, 87, 9, 87, 14, 87, 15, 87, 16, 87, 43, 87],
	#[1, -220, 17, 89],
	#[17, -223],
	#[12, -79],
	#[2, -222],
	#[65535, 90, 17, 90],
	#[30, -27, 33, -36, 23, -23, 31, -29, 36, -34, 4, -39, 34, -40, 29, -24, 38, -26, 26, -44, 1, -31, 12, -38, 37, -30, 18, -22, 19, -35, 20, -32, 25, -28, 27, -45, 28, -25, 32, -33, 35, -42],
	#[20, -73, 22, -75, 1, -74, 42, -225],
	#[65535, 88, 15, 88, 43, 88, 14, 88, 8, 88, 9, 88, 11, 88, 2, 88, 13, 88, 16, 88, 10, 88, 4, 88],
	#[65535, 8, 46, 8, 3, 8, 2, 8, 0, 8, 43, 8, 44, 8, 45, 8],
	#[44, -10, 3, -6, 46, -13, 43, -3, 45, -12],
	#[45, -12, 46, -13, 3, -6, 2, -244, 44, -10, 43, -3],
	#[65535, 10, 5, 10, 1, 10, 42, 10, 40, 10],
	#[5, -179, 1, 14, 42, 14, 40, 14],
	#[1, 77, 42, 77, 40, -204],
	#[65535, 9, 0, 9, 2, 9, 3, 9, 46, 9, 45, 9, 43, 9, 44, 9],
	#[4, -236],
	#[65535, 101, 7, 101, 47, 101],
	#[7, -240, 47, -241],
	#[17, 89, 1, -220],
	#[17, -238],
	#[31, -29, 12, -38, 4, -39, 1, -31, 33, -36, 18, -22, 35, -42, 28, -25, 26, -44, 19, -35, 20, -32, 32, -33, 23, -23, 25, -28, 27, -45, 29, -24, 30, -27, 34, -40, 36, -34, 37, -30, 38, -26],
	#[7, 103, 20, -73, 1, -74, 22, -75, 47, 103],
	#[4, -236],
	#[65535, 99, 45, 99, 3, 99, 46, 99, 1, 99, 43, 99, 44, 99],
	#[65535, 102, 47, 102, 7, 102],
	#[65535, 2, 46, 2, 3, 2, 45, 2, 2, 2, 0, 2, 44, 2, 43, 2],
	#[65535, 7, 46, 7, 2, 7, 0, 7, 3, 7, 45, 7, 44, 7, 43, 7],
	#[#"eoi", #"accept"]],
  goto-table:
      #[#[87, 7, 86, 4, 82, 6, 80, 3, 51, 10, 50, 8, 49, 1, 46, 12, 45, 11, 44, 9, 43, 2, 3, 5],
	#[87, 7, 80, 3, 82, 6, 51, 10, 50, 242, 46, 12, 45, 11, 44, 9, 43, 2, 86, 4, 3, 5, 0, 244],
	#[88, 232],
	#[],
	#[],
	#[54, 200, 53, 228, 52, 229, 4, 198],
	#[],
	#[87, 7, 86, 4, 82, 6, 80, 3, 51, 10, 50, 225, 45, 11, 44, 9, 43, 2, 46, 12, 3, 5, 1, 226],
	#[],
	#[53, 197, 54, 200, 81, 199, 4, 198],
	#[],
	#[83, 174, 4, 175],
	#[4, 13],
	#[40, 14],
	#[75, 17, 61, 15, 4, 16],
	#[5, 19],
	#[],
	#[42, 18],
	#[],
	#[66, 42, 64, 36, 63, 40, 61, 20, 38, 25, 36, 33, 35, 41, 34, 39, 33, 35, 31, 28, 30, 26, 29, 23, 28, 24, 32, 32, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 27, 44, 12, 37, 18, 21, 4, 38, 37, 29, 1, 30],
	#[5, 129],
	#[],
	#[64, 107, 4, 108],
	#[23, 138],
	#[5, 143],
	#[23, 95],
	#[23, 69],
	#[23, 126],
	#[23, 133],
	#[23, 65],
	#[66, 123, 65, 132, 64, 36, 63, 40, 61, 20, 38, 25, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 19, 34, 25, 27, 20, 31, 37, 29, 18, 21, 1, 30, 27, 44, 4, 38, 12, 37, 26, 43, 23, 22],
	#[66, 123, 65, 122, 64, 36, 63, 40, 61, 20, 38, 25, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 35, 41, 20, 31, 36, 33, 12, 37, 29, 23, 18, 21, 19, 34, 4, 38, 1, 30, 37, 29],
	#[23, 67],
	#[23, 115],
	#[],
	#[23, 102],
	#[],
	#[],
	#[],
	#[23, 63],
	#[22, 74, 20, 72, 1, 73],
	#[5, 47],
	#[7, 124, 6, 173],
	#[23, 92],
	#[23, 45],
	#[64, 36, 63, 46, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30],
	#[24, 172, 22, 74, 20, 72, 1, 73],
	#[60, 53, 61, 50, 16, 54, 15, 56, 14, 48, 10, 52, 4, 16, 9, 51, 11, 55, 13, 49],
	#[5, 149],
	#[],
	#[],
	#[],
	#[],
	#[6, 60],
	#[],
	#[5, 57],
	#[],
	#[12, 58],
	#[6, 59],
	#[],
	#[23, 61],
	#[64, 36, 63, 62, 61, 20, 38, 25, 36, 33, 35, 41, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 34, 39, 20, 31, 19, 34, 12, 37, 4, 38, 1, 30, 37, 29, 18, 21, 23, 22],
	#[22, 74, 20, 72, 1, 73, 7, 169],
	#[61, 20, 64, 36, 38, 25, 37, 29, 36, 33, 35, 41, 63, 64, 29, 23, 23, 22, 20, 31, 19, 34, 18, 21, 32, 32, 26, 43, 27, 44, 12, 37, 28, 24, 4, 38, 31, 28, 1, 30, 34, 39, 30, 26, 33, 35, 25, 27],
	#[20, 72, 1, 73, 7, 166, 22, 74],
	#[64, 36, 63, 66, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30],
	#[1, 73, 20, 72, 7, 161, 22, 74],
	#[64, 36, 63, 68, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 4, 38, 1, 30, 12, 37],
	#[22, 74, 20, 72, 1, 73, 7, 158],
	#[64, 36, 63, 70, 61, 20, 38, 25, 37, 29, 36, 33, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 35, 41, 20, 31, 19, 34, 18, 21, 12, 37, 29, 23, 4, 38, 1, 30],
	#[22, 74, 20, 72, 7, 71, 1, 73],
	#[64, 36, 63, 90, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30],
	#[68, 76, 67, 88, 12, 78],
	#[68, 76, 67, 77, 12, 78],
	#[4, 75],
	#[],
	#[7, 83],
	#[2, 82],
	#[39, 79, 12, 80],
	#[12, 81],
	#[],
	#[],
	#[],
	#[12, 84],
	#[39, 85, 12, 86],
	#[12, 87],
	#[],
	#[],
	#[21, 89],
	#[],
	#[24, 91, 22, 74, 20, 72, 1, 73],
	#[],
	#[38, 25, 37, 29, 34, 39, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 33, 35, 25, 27, 23, 22, 35, 41, 20, 31, 36, 33, 18, 21, 26, 43, 12, 37, 1, 30, 19, 34, 4, 38, 64, 36, 63, 93, 61, 20],
	#[24, 94, 22, 74, 20, 72, 1, 73],
	#[],
	#[64, 36, 63, 96, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30],
	#[22, 74, 20, 72, 7, 97, 1, 73],
	#[64, 36, 63, 98, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30],
	#[22, 74, 20, 72, 1, 73, 7, 99],
	#[64, 36, 63, 100, 61, 20, 38, 25, 37, 29, 36, 33, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 35, 41, 20, 31, 19, 34, 18, 21, 12, 37, 29, 23, 4, 38, 1, 30],
	#[24, 101, 22, 74, 20, 72, 1, 73],
	#[],
	#[64, 36, 61, 20, 37, 29, 36, 33, 35, 41, 63, 103, 33, 35, 31, 28, 30, 26, 28, 24, 32, 32, 26, 43, 25, 27, 34, 39, 20, 31, 38, 25, 27, 44, 12, 37, 29, 23, 18, 21, 19, 34, 4, 38, 1, 30, 23, 22],
	#[22, 74, 20, 72, 7, 104, 1, 73],
	#[64, 36, 63, 105, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30],
	#[24, 106, 22, 74, 20, 72, 1, 73],
	#[],
	#[70, 109, 69, 110, 64, 36, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 63, 111, 31, 28, 30, 26, 28, 24, 32, 32, 33, 35, 25, 27, 34, 39, 20, 31, 19, 34, 18, 21, 26, 43, 27, 44, 12, 37, 29, 23, 4, 38, 1, 30, 23, 22],
	#[],
	#[7, 155],
	#[24, 154],
	#[71, 112, 40, 113, 22, 74, 20, 72, 1, 73],
	#[],
	#[41, 114],
	#[],
	#[64, 36, 63, 116, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30, 34, 39],
	#[22, 74, 20, 72, 1, 73, 7, 117],
	#[64, 36, 63, 118, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 34, 39, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30, 23, 22],
	#[22, 74, 20, 72, 7, 119, 1, 73],
	#[64, 36, 63, 120, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30],
	#[24, 121, 22, 74, 20, 72, 1, 73],
	#[],
	#[21, 153],
	#[7, 124],
	#[64, 36, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 34, 39, 63, 125, 4, 38, 1, 30, 12, 37],
	#[22, 74, 20, 72, 1, 73],
	#[64, 36, 63, 127, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30],
	#[24, 128, 22, 74, 20, 72, 1, 73],
	#[],
	#[64, 36, 66, 130, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 63, 40, 30, 26, 28, 24, 32, 32, 33, 35, 25, 27, 34, 39, 20, 31, 19, 34, 18, 21, 26, 43, 27, 44, 29, 23, 4, 38, 31, 28, 1, 30, 12, 37, 23, 22],
	#[7, 124, 6, 131],
	#[],
	#[2, 152],
	#[64, 36, 63, 134, 61, 20, 38, 25, 37, 29, 36, 33, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 35, 41, 20, 31, 19, 34, 18, 21, 34, 39, 4, 38, 1, 30, 12, 37],
	#[22, 74, 20, 72, 7, 135, 1, 73],
	#[64, 36, 63, 136, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30],
	#[24, 137, 22, 74, 20, 72, 1, 73],
	#[],
	#[64, 36, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 63, 139, 33, 35, 31, 28, 30, 26, 28, 24, 32, 32, 26, 43, 25, 27, 34, 39, 20, 31, 19, 34, 18, 21, 27, 44, 12, 37, 29, 23, 4, 38, 1, 30, 23, 22],
	#[22, 74, 20, 72, 7, 140, 1, 73],
	#[64, 36, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 32, 32, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30, 33, 35, 63, 141],
	#[24, 142, 22, 74, 20, 72, 1, 73],
	#[],
	#[61, 50, 60, 144, 16, 54, 15, 56, 14, 48, 13, 49, 11, 55, 10, 52, 9, 51, 4, 16],
	#[6, 145],
	#[23, 146],
	#[64, 36, 63, 147, 61, 20, 36, 33, 35, 41, 34, 39, 33, 35, 31, 28, 30, 26, 29, 23, 28, 24, 32, 32, 26, 43, 25, 27, 20, 31, 19, 34, 18, 21, 38, 25, 27, 44, 12, 37, 4, 38, 1, 30, 37, 29, 23, 22],
	#[24, 148, 22, 74, 20, 72, 1, 73],
	#[],
	#[60, 150, 61, 50, 16, 54, 15, 56, 14, 48, 13, 49, 11, 55, 10, 52, 9, 51, 4, 16],
	#[6, 151],
	#[],
	#[],
	#[],
	#[],
	#[64, 36, 63, 156, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 18, 21, 12, 37, 4, 38, 23, 22, 20, 31, 1, 30, 19, 34],
	#[40, 113, 20, 72, 71, 157, 1, 73, 22, 74],
	#[],
	#[27, 44, 33, 35, 25, 27, 64, 36, 20, 31, 36, 33, 18, 21, 32, 32, 28, 24, 29, 23, 35, 41, 30, 26, 19, 34, 63, 159, 61, 20, 31, 28, 37, 29, 1, 30, 26, 43, 38, 25, 34, 39, 4, 38, 12, 37, 23, 22],
	#[24, 160, 22, 74, 20, 72, 1, 73],
	#[],
	#[31, 28, 28, 24, 25, 27, 64, 36, 32, 32, 27, 44, 12, 37, 35, 41, 30, 26, 33, 35, 61, 20, 20, 31, 36, 33, 1, 30, 37, 29, 18, 21, 38, 25, 29, 23, 34, 39, 4, 38, 26, 43, 19, 34, 63, 162, 23, 22],
	#[22, 74, 20, 72, 7, 163, 1, 73],
	#[63, 164, 64, 36, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 30, 26, 29, 23, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 4, 38, 1, 30],
	#[24, 165, 20, 72, 1, 73, 22, 74],
	#[],
	#[64, 36, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 63, 167, 33, 35, 31, 28, 30, 26, 28, 24, 27, 44, 26, 43, 25, 27, 23, 22, 32, 32, 20, 31, 19, 34, 18, 21, 12, 37, 29, 23, 4, 38, 1, 30, 34, 39],
	#[24, 168, 22, 74, 20, 72, 1, 73],
	#[],
	#[64, 36, 61, 20, 37, 29, 36, 33, 35, 41, 63, 170, 33, 35, 31, 28, 30, 26, 28, 24, 27, 44, 26, 43, 23, 22, 32, 32, 19, 34, 38, 25, 12, 37, 25, 27, 29, 23, 4, 38, 20, 31, 1, 30, 18, 21, 34, 39],
	#[24, 171, 20, 72, 1, 73, 22, 74],
	#[],
	#[],
	#[],
	#[56, 176, 55, 177, 5, 178],
	#[],
	#[],
	#[1, 191],
	#[8, 181, 59, 182, 58, 179, 57, 180],
	#[],
	#[7, 189, 6, 188],
	#[],
	#[60, 183, 61, 50, 15, 56, 14, 48, 13, 49, 16, 54, 10, 52, 9, 51, 4, 16, 11, 55],
	#[4, 184],
	#[17, 185, 62, 186],
	#[64, 36, 36, 33, 35, 41, 63, 187, 30, 26, 29, 23, 28, 24, 27, 44, 25, 27, 61, 20, 32, 32, 18, 21, 37, 29, 33, 35, 1, 30, 34, 39, 26, 43, 19, 34, 4, 38, 20, 31, 12, 37, 38, 25, 31, 28, 23, 22],
	#[],
	#[22, 74, 20, 72, 1, 73],
	#[],
	#[58, 190, 59, 182, 8, 181],
	#[],
	#[80, 192, 84, 194, 85, 193, 44, 9],
	#[],
	#[],
	#[80, 192, 85, 195, 44, 9, 2, 196],
	#[],
	#[],
	#[],
	#[],
	#[73, 201, 72, 202, 40, 203],
	#[],
	#[76, 209, 42, 208, 1, 210],
	#[],
	#[74, 205, 75, 204, 61, 15, 4, 16],
	#[],
	#[7, 206],
	#[75, 207, 61, 15, 4, 16],
	#[],
	#[],
	#[],
	#[77, 211],
	#[78, 215, 58, 213, 59, 182, 43, 212, 8, 181, 2, 214],
	#[4, 217],
	#[42, 216],
	#[],
	#[],
	#[],
	#[79, 218, 1, 219],
	#[17, 222],
	#[67, 220, 68, 76, 12, 78],
	#[2, 221],
	#[],
	#[64, 36, 63, 223, 61, 20, 35, 41, 32, 32, 28, 24, 27, 44, 33, 35, 25, 27, 20, 31, 36, 33, 18, 21, 31, 28, 1, 30, 37, 29, 12, 37, 23, 22, 26, 43, 29, 23, 34, 39, 4, 38, 19, 34, 38, 25, 30, 26],
	#[22, 74, 42, 224, 20, 72, 1, 73],
	#[],
	#[],
	#[80, 3, 86, 4, 82, 6, 49, 227, 45, 11, 43, 2, 50, 8, 51, 10, 46, 12, 87, 7, 44, 9, 3, 5],
	#[87, 7, 86, 4, 82, 6, 80, 3, 50, 242, 44, 9, 43, 2, 46, 12, 51, 10, 2, 243, 3, 5, 45, 11],
	#[],
	#[56, 176, 55, 230, 5, 178],
	#[73, 201, 72, 231, 40, 203],
	#[],
	#[90, 233, 89, 234, 4, 235],
	#[],
	#[47, 240, 7, 239],
	#[79, 236, 1, 219],
	#[17, 237],
	#[64, 36, 63, 238, 61, 20, 38, 25, 37, 29, 36, 33, 35, 41, 34, 39, 33, 35, 32, 32, 31, 28, 28, 24, 26, 43, 25, 27, 23, 22, 20, 31, 19, 34, 18, 21, 12, 37, 29, 23, 30, 26, 1, 30, 27, 44, 4, 38],
	#[20, 72, 22, 74, 1, 73],
	#[90, 241, 4, 235],
	#[],
	#[],
	#[],
	#[],
	#[]],
  action-function-table:
	 vector(tablegen-parser-action0,
		tablegen-parser-action1,
		tablegen-parser-action0,
		tablegen-parser-action3,
		tablegen-parser-action3,
		tablegen-parser-action1,
		tablegen-parser-action1,
		tablegen-parser-action7,
		tablegen-parser-action8,
		tablegen-parser-action9,
		tablegen-parser-action10,
		tablegen-parser-action10,
		tablegen-parser-action12,
		tablegen-parser-action10,
		tablegen-parser-action14,
		tablegen-parser-action10,
		tablegen-parser-action16,
		tablegen-parser-action17,
		tablegen-parser-action18,
		tablegen-parser-action19,
		tablegen-parser-action20,
		tablegen-parser-action1,
		tablegen-parser-action22,
		tablegen-parser-action23,
		tablegen-parser-action24,
		tablegen-parser-action25,
		tablegen-parser-action26,
		tablegen-parser-action27,
		tablegen-parser-action28,
		tablegen-parser-action29,
		tablegen-parser-action30,
		tablegen-parser-action31,
		tablegen-parser-action32,
		tablegen-parser-action33,
		tablegen-parser-action34,
		tablegen-parser-action34,
		tablegen-parser-action36,
		tablegen-parser-action37,
		tablegen-parser-action38,
		tablegen-parser-action39,
		tablegen-parser-action40,
		tablegen-parser-action41,
		tablegen-parser-action42,
		tablegen-parser-action43,
		tablegen-parser-action44,
		tablegen-parser-action45,
		tablegen-parser-action46,
		tablegen-parser-action47,
		tablegen-parser-action48,
		tablegen-parser-action49,
		tablegen-parser-action50,
		tablegen-parser-action51,
		tablegen-parser-action52,
		tablegen-parser-action53,
		tablegen-parser-action54,
		tablegen-parser-action55,
		tablegen-parser-action56,
		tablegen-parser-action57,
		tablegen-parser-action10,
		tablegen-parser-action14,
		tablegen-parser-action60,
		tablegen-parser-action17,
		tablegen-parser-action18,
		tablegen-parser-action1,
		tablegen-parser-action1,
		tablegen-parser-action65,
		tablegen-parser-action0,
		tablegen-parser-action65,
		tablegen-parser-action68,
		tablegen-parser-action69,
		tablegen-parser-action20,
		tablegen-parser-action1,
		tablegen-parser-action0,
		tablegen-parser-action69,
		tablegen-parser-action20,
		tablegen-parser-action0,
		tablegen-parser-action76,
		tablegen-parser-action14,
		tablegen-parser-action78,
		tablegen-parser-action17,
		tablegen-parser-action18,
		tablegen-parser-action81,
		tablegen-parser-action82,
		tablegen-parser-action83,
		tablegen-parser-action16,
		tablegen-parser-action14,
		tablegen-parser-action86,
		tablegen-parser-action87,
		tablegen-parser-action88,
		tablegen-parser-action20,
		tablegen-parser-action90,
		tablegen-parser-action91,
		tablegen-parser-action10,
		tablegen-parser-action93,
		tablegen-parser-action10,
		tablegen-parser-action17,
		tablegen-parser-action86,
		tablegen-parser-action10,
		tablegen-parser-action68,
		tablegen-parser-action99,
		tablegen-parser-action100,
		tablegen-parser-action1,
		tablegen-parser-action65,
		tablegen-parser-action103),
  action-nargs-table: #[2, 1, 2, 1, 1, 1, 1, 4, 2, 4, 1, 1, 0, 1, 0, 1, 3, 1, 3, 4, 0, 1, 1, 1, 4, 1, 4, 1, 1, 1, 1, 0, 2, 1, 1, 1, 1, 3, 4, 4, 3, 3, 4, 4, 4, 4, 4, 7, 6, 6, 6, 6, 6, 6, 9, 8, 8, 8, 1, 0, 1, 1, 3, 1, 1, 3, 2, 3, 5, 4, 0, 1, 2, 4, 0, 2, 2, 0, 2, 1, 3, 1, 4, 1, 3, 0, 2, 2, 6, 0, 3, 3, 1, 6, 1, 1, 2, 1, 5, 4, 0, 1, 3, 4],
  action-nt-table: #[48, 49, 49, 50, 50, 50, 50, 50, 50, 51, 52, 53, 54, 54, 55, 55, 56, 57, 57, 58, 59, 59, 60, 60, 60, 60, 60, 60, 60, 60, 61, 62, 62, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 64, 65, 65, 66, 66, 67, 68, 68, 68, 68, 68, 68, 69, 69, 70, 70, 71, 71, 72, 73, 73, 74, 74, 75, 75, 76, 76, 77, 77, 78, 78, 79, 79, 80, 81, 82, 83, 84, 84, 85, 86, 87, 88, 89, 89, 90]);
