Module:       llvm-asm-parser-internals
Author:       Peter S. Housel
Copyright:    Original Code is Copyright 2009-2010 Gwydion Dylan Maintainers
              All rights reserved.
License:      See License.txt in this distribution for details.
Warranty:     Distributed WITHOUT WARRANTY OF ANY KIND

define class <llvm-parse-state> (<object>)
  constant slot parse-module :: <llvm-module>,
    required-init-keyword: module:;

  slot parse-unnamed-type-index :: <integer> = 0;
  constant slot parse-unnamed-types :: <stretchy-object-vector>
    = make(<stretchy-object-vector>);

  slot parse-unnamed-global-index :: <integer> = 0;
  constant slot parse-unnamed-global-values :: <stretchy-object-vector>
    = make(<stretchy-object-vector>);

  slot parse-function :: false-or(<llvm-function>) = #f;
  slot parse-unnamed-local-index :: <integer> = 0;
  constant slot parse-unnamed-local-values :: <stretchy-object-vector>
    = make(<stretchy-object-vector>);

  constant slot parse-unnamed-metadata-values :: <stretchy-object-vector>
    = make(<stretchy-object-vector>);
end class;

define thread variable *llvm-parse-state* :: <llvm-parse-state>
  = make(<llvm-parse-state>, module: make(<llvm-module>, name: "<dummy>"));

define method do-define-type (name :: <string>, type :: <llvm-type>) => ();
  let definition
    = element(*llvm-parse-state*.parse-module.llvm-type-table, name,
              default: #f);
  if (definition)
    if (instance?(definition, <llvm-symbolic-type>))
      definition.llvm-placeholder-type-forward := type;
    else
      error("type %%%s is multiply defined", name);
    end if;
  end if;
  
  // Record this type 
  element(*llvm-parse-state*.parse-module.llvm-type-table, name) := type
end method;

define method do-define-type
    (supplied-index :: false-or(<integer>), type :: <llvm-type>) => ();
  let new-index = *llvm-parse-state*.parse-unnamed-type-index;
  if (supplied-index & supplied-index ~= new-index)
    error("Expected %%%d rather than %%%d for unnamed type",
          new-index, supplied-index);
  end;

  let definition
    = element(*llvm-parse-state*.parse-unnamed-types, new-index, default: #f);
  if (definition)
    if (instance?(definition, <llvm-symbolic-type>))
      definition.llvm-placeholder-type-forward := type;
    else
      error("type %%%d is multiply defined", new-index);
    end if;
  end if;

  // Record this type
  element(*llvm-parse-state*.parse-unnamed-types, new-index) := type;
  *llvm-parse-state*.parse-unnamed-type-index := new-index + 1;
  type
end method;

define method do-define-global-value
    (name :: <string>, value :: <llvm-constant-value>) => ();
  let definition
    = element(*llvm-parse-state*.parse-module.llvm-global-table, name,
              default: #f);
  if (definition)
    if (instance?(definition, <llvm-symbolic-constant>))
      definition.llvm-placeholder-value-forward := value;
      llvm-constrain-type(definition.llvm-value-type, llvm-value-type(value));
    else
      error("value @%s is multiply defined", name);
    end if;
  end if;
  
  // Record this value
  element(*llvm-parse-state*.parse-module.llvm-global-table, name) := value
end method;

define method do-define-global-value
    (supplied-index :: false-or(<integer>), value :: <llvm-constant-value>)
 => ();
  let new-index = *llvm-parse-state*.parse-unnamed-global-index;
  if (supplied-index & supplied-index ~= new-index)
    error("Expected @%d rather than @%d for unnamed global value",
          new-index, supplied-index);
  end;
  let definition
    = element(*llvm-parse-state*.parse-unnamed-global-values, new-index,
              default: #f);
  if (definition)
    if (instance?(definition, <llvm-symbolic-constant>))
      definition.llvm-placeholder-value-forward := value;
      llvm-constrain-type(definition.llvm-value-type, llvm-value-type(value));
    else
      error("value @%d is multiply defined", new-index);
    end if;
  end if;
  
  // Record this value
  element(*llvm-parse-state*.parse-unnamed-global-values, new-index) := value;
  *llvm-parse-state*.parse-unnamed-global-index := new-index + 1;
end method;

define method do-define-local-value
    (name :: <string>, value :: <llvm-value>) => (value :: <llvm-value>);
  let definition
    = element(*llvm-parse-state*.parse-function.llvm-function-value-table,
              name, default: #f);
  if (definition)
    if (instance?(definition, <llvm-symbolic-value>))
      definition.llvm-placeholder-value-forward := value;
      llvm-constrain-type(definition.llvm-value-type, llvm-value-type(value));
    else
      error("value %%%s is multiply defined", name);
    end if;
  end if;
  
  // Record this value
  element(*llvm-parse-state*.parse-function.llvm-function-value-table, name)
    := value
end method;

define method do-define-local-value
    (supplied-index :: false-or(<integer>), value :: <llvm-value>)
 => (value :: <llvm-value>);
  unless (llvm-void-type?(llvm-value-type(value)))
    let new-index = *llvm-parse-state*.parse-unnamed-local-index;
    if (supplied-index & supplied-index ~= new-index)
      error("Expected %%%d rather than %%%d for unnamed local value",
            new-index, supplied-index);
    end;
    let definition
      = element(*llvm-parse-state*.parse-unnamed-local-values, new-index,
                default: #f);
    if (definition)
      if (instance?(definition, <llvm-symbolic-value>))
        definition.llvm-placeholder-value-forward := value;
        llvm-constrain-type(definition.llvm-value-type, llvm-value-type(value));
      else
        error("value %%%d is multiply defined", new-index);
      end if;
    end if;
    
    // Record this value
    element(*llvm-parse-state*.parse-unnamed-local-values, new-index) := value;
    *llvm-parse-state*.parse-unnamed-local-index := new-index + 1;
  end unless;
  value
end method;

define method do-define-metadata-value
    (supplied-index :: <integer>, value :: <llvm-metadata-value>)
 => ();
  let definition
    = element(*llvm-parse-state*.parse-unnamed-metadata-values, supplied-index,
              default: #f);
  if (definition)
    if (instance?(definition, <llvm-symbolic-metadata>))
      definition.llvm-placeholder-value-forward := value;
      llvm-constrain-type(definition.llvm-value-type, llvm-value-type(value));
    else
      error("value !%d is multiply defined", supplied-index);
    end if;
  end if;

  // Record this value
  element(*llvm-parse-state*.parse-unnamed-metadata-values, supplied-index)
    := value;
end method;

define class <arg-entry> (<object>)
  constant slot arg-attributes :: <llvm-attributes>,
    init-value: $llvm-attribute-none, init-keyword: attributes:;
  constant slot arg-type :: <llvm-type>,
    init-keyword: type:;
end class;

define class <param-entry> (<object>)
  constant slot param-attributes :: <llvm-attributes>,
    init-value: $llvm-attribute-none, init-keyword: attributes:;
  constant slot param-value :: <llvm-value>,
    init-keyword: value:;
end class;

define function string-constant-contents
    (string-constant :: <string>)
 => (contents :: <sequence>);
  let contents = make(<stretchy-object-vector>);
  let string-constant-size = string-constant.size;

  iterate loop (i = 0)
    if (i < string-constant-size)
      let ch = string-constant[i];
      add!(contents, make(<llvm-integer-constant>,
                          type: $llvm-i8-type,
                          integer: as(<integer>, ch)));
      loop(i + 1);
    end if;
  end iterate;
  
  contents
end function;

// Change some attributes (that used to be function attributes) to
// return attributes
define constant $return-attributes-mask
  = %logior($llvm-attribute-zext, $llvm-attribute-sext, $llvm-attribute-inreg);
define function munge-attributes
    (return-attributes :: <llvm-attributes>,
     function-attributes :: <llvm-attributes>)
 => (return-attributes :: <llvm-attributes>,
     function-attributes :: <llvm-attributes>);
  values(%logior(return-attributes,
                 %logand(function-attributes, $return-attributes-mask)),
         %logand(function-attributes, %lognot($return-attributes-mask)))
end function;

define constant $EOF-token = 0;
define constant $%MODULE-token = 1;
define constant $%ASM-token = 2;
define constant $%ALIAS-token = 3;
define constant $%TARGET-token = 4;
define constant $%TRIPLE-token = 5;
define constant $%EQUALS-token = 6;
define constant $%STRINGCONSTANT-token = 7;
define constant $%DATALAYOUT-token = 8;
define constant $%DEPLIBS-token = 9;
define constant $%GLOBALVAL_ID-token = 10;
define constant $%LBRACK-token = 11;
define constant $%RBRACK-token = 12;
define constant $%COMMA-token = 13;
define constant $%PRIVATE-token = 14;
define constant $%LINKER_PRIVATE-token = 15;
define constant $%LINKER_PRIVATE_WEAK-token = 16;
define constant $%LINKER_PRIVATE_WEAK_DEF_AUTO-token = 17;
define constant $%INTERNAL-token = 18;
define constant $%WEAK-token = 19;
define constant $%WEAK_ODR-token = 20;
define constant $%LINKONCE-token = 21;
define constant $%LINKONCE_ODR-token = 22;
define constant $%AVAILABLE_EXTERNALLY-token = 23;
define constant $%APPENDING-token = 24;
define constant $%DLLEXPORT-token = 25;
define constant $%COMMON-token = 26;
define constant $%DLLIMPORT-token = 27;
define constant $%EXTERN_WEAK-token = 28;
define constant $%EXTERNAL-token = 29;
define constant $%TYPE-token = 30;
define constant $%LBRACE-token = 31;
define constant $%RBRACE-token = 32;
define constant $%LANGLE-token = 33;
define constant $%RANGLE-token = 34;
define constant $%OPAQUE-token = 35;
define constant $%INTTYPE-token = 36;
define constant $%STAR-token = 37;
define constant $%EUINT64VAL-token = 38;
define constant $%x-token = 39;
define constant $%DEFINE-token = 40;
define constant $%DECLARE-token = 41;
define constant $%LPAREN-token = 42;
define constant $%RPAREN-token = 43;
define constant $%CCC-token = 44;
define constant $%FASTCC-token = 45;
define constant $%COLDCC-token = 46;
define constant $%X86_STDCALLCC-token = 47;
define constant $%X86_FASTCALLCC-token = 48;
define constant $%ARM_APCSCC-token = 49;
define constant $%ARM_AAPCSCC-token = 50;
define constant $%ARM_AAPCS_VFPCC-token = 51;
define constant $%MSP430_INTRCC-token = 52;
define constant $%X86_THISCALLCC-token = 53;
define constant $%PTX_KERNELCC-token = 54;
define constant $%PTX_DEVICECC-token = 55;
define constant $%CC-token = 56;
define constant $%INREG-token = 57;
define constant $%ZEROEXT-token = 58;
define constant $%SIGNEXT-token = 59;
define constant $%NOALIAS-token = 60;
define constant $%BEGIN-token = 61;
define constant $%END-token = 62;
define constant $%LOCALVAL_ID-token = 63;
define constant $%FLOAT-token = 64;
define constant $%DOUBLE-token = 65;
define constant $%PPC_FP128-token = 66;
define constant $%FP128-token = 67;
define constant $%X86_FP80-token = 68;
define constant $%LABEL-token = 69;
define constant $%METADATA-token = 70;
define constant $%X86_MMX-token = 71;
define constant $%ADDRSPACE-token = 72;
define constant $%VOID-token = 73;
define constant $%DEFAULT-token = 74;
define constant $%HIDDEN-token = 75;
define constant $%PROTECTED-token = 76;
define constant $%THREAD_LOCAL-token = 77;
define constant $%UNNAMED_ADDR-token = 78;
define constant $%GLOBAL-token = 79;
define constant $%CONSTANT-token = 80;
define constant $%C-token = 81;
define constant $%NULL-token = 82;
define constant $%UNDEF-token = 83;
define constant $%ZEROINITIALIZER-token = 84;
define constant $%ESINT64VAL-token = 85;
define constant $%FPVAL-token = 86;
define constant $%TRUE-token = 87;
define constant $%FALSE-token = 88;
define constant $%BLOCKADDRESS-token = 89;
define constant $%ALIGN-token = 90;
define constant $%SECTION-token = 91;
define constant $%BITCAST-token = 92;
define constant $%TO-token = 93;
define constant $%GETELEMENTPTR-token = 94;
define constant $%DOTDOTDOT-token = 95;
define constant $%SELECT-token = 96;
define constant $%ICMP-token = 97;
define constant $%FCMP-token = 98;
define constant $%EXTRACTELEMENT-token = 99;
define constant $%INSERTELEMENT-token = 100;
define constant $%SHUFFLEVECTOR-token = 101;
define constant $%EXTRACTVALUE-token = 102;
define constant $%INSERTVALUE-token = 103;
define constant $%TRUNC-token = 104;
define constant $%ZEXT-token = 105;
define constant $%SEXT-token = 106;
define constant $%FPTRUNC-token = 107;
define constant $%FPEXT-token = 108;
define constant $%UITOFP-token = 109;
define constant $%SITOFP-token = 110;
define constant $%FPTOUI-token = 111;
define constant $%FPTOSI-token = 112;
define constant $%INTTOPTR-token = 113;
define constant $%PTRTOINT-token = 114;
define constant $%LOCALVAR-token = 115;
define constant $%GLOBALVAR-token = 116;
define constant $%GC-token = 117;
define constant $%ADD-token = 118;
define constant $%FADD-token = 119;
define constant $%SUB-token = 120;
define constant $%FSUB-token = 121;
define constant $%MUL-token = 122;
define constant $%FMUL-token = 123;
define constant $%UDIV-token = 124;
define constant $%EXACT-token = 125;
define constant $%SDIV-token = 126;
define constant $%FDIV-token = 127;
define constant $%UREM-token = 128;
define constant $%SREM-token = 129;
define constant $%FREM-token = 130;
define constant $%NUW-token = 131;
define constant $%NSW-token = 132;
define constant $%SHL-token = 133;
define constant $%LSHR-token = 134;
define constant $%ASHR-token = 135;
define constant $%AND-token = 136;
define constant $%OR-token = 137;
define constant $%XOR-token = 138;
define constant $%EQ-token = 139;
define constant $%NE-token = 140;
define constant $%SLT-token = 141;
define constant $%SGT-token = 142;
define constant $%SLE-token = 143;
define constant $%SGE-token = 144;
define constant $%ULT-token = 145;
define constant $%UGT-token = 146;
define constant $%ULE-token = 147;
define constant $%UGE-token = 148;
define constant $%OEQ-token = 149;
define constant $%ONE-token = 150;
define constant $%OLT-token = 151;
define constant $%OGT-token = 152;
define constant $%OLE-token = 153;
define constant $%OGE-token = 154;
define constant $%ORD-token = 155;
define constant $%UNO-token = 156;
define constant $%UEQ-token = 157;
define constant $%UNE-token = 158;
define constant $%LABELSTR-token = 159;
define constant $%RET-token = 160;
define constant $%BR-token = 161;
define constant $%SWITCH-token = 162;
define constant $%INDIRECTBR-token = 163;
define constant $%INVOKE-token = 164;
define constant $%UNWIND-token = 165;
define constant $%UNREACHABLE-token = 166;
define constant $%RESUME-token = 167;
define constant $%NORETURN-token = 168;
define constant $%NOUNWIND-token = 169;
define constant $%READNONE-token = 170;
define constant $%READONLY-token = 171;
define constant $%NOINLINE-token = 172;
define constant $%ALWAYSINLINE-token = 173;
define constant $%INLINEHINT-token = 174;
define constant $%OPTSIZE-token = 175;
define constant $%SSP-token = 176;
define constant $%SSPREQ-token = 177;
define constant $%NOREDZONE-token = 178;
define constant $%NOIMPLICITFLOAT-token = 179;
define constant $%NAKED-token = 180;
define constant $%ALIGNSTACK-token = 181;
define constant $%UWTABLE-token = 182;
define constant $%RETURNS_TWICE-token = 183;
define constant $%NONLAZYBIND-token = 184;
define constant $%VAARG-token = 185;
define constant $%PHI-token = 186;
define constant $%LANDINGPAD-token = 187;
define constant $%PERSONALITY-token = 188;
define constant $%ALLOCA-token = 189;
define constant $%LOAD-token = 190;
define constant $%ATOMIC-token = 191;
define constant $%VOLATILE-token = 192;
define constant $%STORE-token = 193;
define constant $%CMPXCHG-token = 194;
define constant $%ATOMICRMW-token = 195;
define constant $%FENCE-token = 196;
define constant $%GETRESULT-token = 197;
define constant $%CLEANUP-token = 198;
define constant $%CATCH-token = 199;
define constant $%FILTER-token = 200;
define constant $%TAIL-token = 201;
define constant $%CALL-token = 202;
define constant $%SINGLETHREAD-token = 203;
define constant $%UNORDERED-token = 204;
define constant $%MONOTONIC-token = 205;
define constant $%ACQUIRE-token = 206;
define constant $%RELEASE-token = 207;
define constant $%ACQ_REL-token = 208;
define constant $%SEQ_CST-token = 209;
define constant $%XCHG-token = 210;
define constant $%NAND-token = 211;
define constant $%MAX-token = 212;
define constant $%MIN-token = 213;
define constant $%UMAX-token = 214;
define constant $%UMIN-token = 215;
define constant $%SIDEEFFECT-token = 216;
define constant $%SRET-token = 217;
define constant $%NOCAPTURE-token = 218;
define constant $%BYVAL-token = 219;
define constant $%NEST-token = 220;
define constant $%INBOUNDS-token = 221;
define constant $%METADATAVAR-token = 222;
define constant $%EXCLAIM-token = 223;

define function llvm-parser-action0 (arg$1, arg$2) => (value)
  #f
end llvm-parser-action0;

define function llvm-parser-action1 () => (value)
  #f
end llvm-parser-action1;

define function llvm-parser-action3 (arg$1, arg$2, arg$3) => (value)
  if (empty?(*llvm-parse-state*.parse-module.llvm-module-asm))
         *llvm-parse-state*.parse-module.llvm-module-asm := arg$3;
       else
         *llvm-parse-state*.parse-module.llvm-module-asm
           := concatenate(*llvm-parse-state*.parse-module.llvm-module-asm,
                          "\n", arg$3);
       end if
end llvm-parser-action3;

define function llvm-parser-action4 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9) => (value)
  begin
         let pointer-type
           = make(<llvm-pointer-type>,
                  pointee: llvm-value-type(arg$8),
                  address-space: arg$5);
         let global
           = apply(make, <llvm-global-variable>,
                   name: arg$1,
                   type: pointer-type,
                   initializer: arg$8,
                   constant?: arg$7 == #"constant",
                   linkage: arg$2,
                   visibility: arg$3,
                   thread-local?: arg$4,
		   unnamed-address?: arg$6,
                   arg$9);
         add!(*llvm-parse-state*.parse-module.llvm-module-globals, global);
         do-define-global-value(arg$1, global);
       end
end llvm-parser-action4;

define function llvm-parser-action5 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  begin
         let pointer-type
           = make(<llvm-pointer-type>,
                  pointee: arg$5.llvm-value-type);
         let global
           = apply(make, <llvm-global-variable>,
                   type: pointer-type,
                   initializer: arg$5,
                   constant?: arg$4 == #"constant",
                   linkage: arg$1,
                   visibility: arg$2,
                   thread-local?: arg$3,
                   arg$6);
         add!(*llvm-parse-state*.parse-module.llvm-module-globals, global);
         do-define-global-value(#f, global);
       end
end llvm-parser-action5;

define function llvm-parser-action6 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9) => (value)
  begin
         let pointer-type
           = make(<llvm-pointer-type>,
                  pointee: arg$8,
                  address-space: arg$6);
         let global
           = apply(make, <llvm-global-variable>,
                   name: arg$1,
                   type: pointer-type,
                   constant?: arg$7 == #"constant",
                   linkage: arg$2,
                   visibility: arg$3,
                   thread-local?: arg$4,
		   unnamed-address?: arg$5,
                   arg$9);
         add!(*llvm-parse-state*.parse-module.llvm-module-globals, global);
         do-define-global-value(arg$1, global);
       end
end llvm-parser-action6;

define function llvm-parser-action7 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  begin
         let pointer-type = make(<llvm-pointer-type>, pointee: arg$5);
         let global
           = apply(make, <llvm-global-variable>,
                   type: pointer-type,
                   constant?: arg$4 == #"constant",
                   linkage: arg$1,
                   visibility: arg$2,
                   thread-local?: arg$3,
                   arg$6);
         add!(*llvm-parse-state*.parse-module.llvm-module-globals, global);
         do-define-global-value(#f, global);
       end
end llvm-parser-action7;

define function llvm-parser-action8 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  begin
         let alias
           = make(<llvm-global-alias>,
                  name: arg$1,
                  type: llvm-value-type(arg$6),
                  linkage: arg$5,
                  visibility: arg$3,
                  aliasee: arg$6);
         add!(*llvm-parse-state*.parse-module.llvm-module-aliases, alias);
         do-define-global-value(arg$1, alias);
       end
end llvm-parser-action8;

define function llvm-parser-action9 (arg$1, arg$2, arg$3, arg$4) => (value)
  *llvm-parse-state*.parse-module.llvm-module-target-triple := arg$4
end llvm-parser-action9;

define function llvm-parser-action10 (arg$1, arg$2, arg$3, arg$4) => (value)
  *llvm-parse-state*.parse-module.llvm-module-data-layout := arg$4
end llvm-parser-action10;

define function llvm-parser-action11 (arg$1, arg$2, arg$3) => (value)
  concatenate!(*llvm-parse-state*.parse-module.llvm-module-dependent-libraries, arg$3)
end llvm-parser-action11;

define function llvm-parser-action12 (arg$1) => (value)
  arg$1
end llvm-parser-action12;

define function llvm-parser-action13 (arg$1, arg$2) => (value)
  arg$1
end llvm-parser-action13;

define function llvm-parser-action15 (arg$1, arg$2, arg$3) => (value)
  reverse!(arg$2)
end llvm-parser-action15;

define function llvm-parser-action16 () => (value)
  #()
end llvm-parser-action16;

define function llvm-parser-action17 (arg$1) => (value)
  list(arg$1)
end llvm-parser-action17;

define function llvm-parser-action18 (arg$1, arg$2, arg$3) => (value)
  pair(arg$3, arg$1)
end llvm-parser-action18;

define function llvm-parser-action19 () => (value)
  #"external"
end llvm-parser-action19;

define function llvm-parser-action20 (arg$1) => (value)
  #"private"
end llvm-parser-action20;

define function llvm-parser-action21 (arg$1) => (value)
  #"linker-private"
end llvm-parser-action21;

define function llvm-parser-action22 (arg$1) => (value)
  #"linker-private-weak"
end llvm-parser-action22;

define function llvm-parser-action23 (arg$1) => (value)
  #"linker-private-weak-def-auto"
end llvm-parser-action23;

define function llvm-parser-action24 (arg$1) => (value)
  #"internal"
end llvm-parser-action24;

define function llvm-parser-action25 (arg$1) => (value)
  #"weak"
end llvm-parser-action25;

define function llvm-parser-action26 (arg$1) => (value)
  #"weak-odr"
end llvm-parser-action26;

define function llvm-parser-action27 (arg$1) => (value)
  #"linkonce"
end llvm-parser-action27;

define function llvm-parser-action28 (arg$1) => (value)
  #"linkonce-odr"
end llvm-parser-action28;

define function llvm-parser-action29 (arg$1) => (value)
  #"available-externally"
end llvm-parser-action29;

define function llvm-parser-action30 (arg$1) => (value)
  #"appending"
end llvm-parser-action30;

define function llvm-parser-action31 (arg$1) => (value)
  #"dllexport"
end llvm-parser-action31;

define function llvm-parser-action32 (arg$1) => (value)
  #"common"
end llvm-parser-action32;

define function llvm-parser-action33 (arg$1) => (value)
  #"dllimport"
end llvm-parser-action33;

define function llvm-parser-action34 (arg$1) => (value)
  #"extern-weak"
end llvm-parser-action34;

define function llvm-parser-action35 (arg$1) => (value)
  #"external"
end llvm-parser-action35;

define function llvm-parser-action36 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  begin
         let type
           = make(<llvm-struct-type>, name: arg$1, packed?: #f,
                  elements: reverse!(arg$4));
         do-define-type(arg$1, type);
       end
end llvm-parser-action36;

define function llvm-parser-action37 (arg$1, arg$2, arg$3, arg$4) => (value)
  begin
         let type
           = make(<llvm-struct-type>, name: arg$1, packed?: #f,
                  elements: #[]);
         do-define-type(arg$1, type);
       end
end llvm-parser-action37;

define function llvm-parser-action38 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  begin
         let type
           = make(<llvm-struct-type>, name: arg$1, packed?: #t,
                  elements: reverse!(arg$5));
         do-define-type(arg$1, type);
       end
end llvm-parser-action38;

define function llvm-parser-action39 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  begin
         let type
           = make(<llvm-struct-type>, name: arg$1, packed?: #t,
                  elements: #[]);
         do-define-type(arg$1, type);
       end
end llvm-parser-action39;

define function llvm-parser-action40 (arg$1, arg$2, arg$3) => (value)
  begin
         let type = make(<llvm-opaque-type>, name: arg$1);
         do-define-type(arg$1, type);
       end
end llvm-parser-action40;

define function llvm-parser-action41 (arg$1, arg$2, arg$3) => (value)
  begin
         let type = make(<llvm-integer-type>, width: arg$3);
       end
end llvm-parser-action41;

define function llvm-parser-action42 (arg$1, arg$2, arg$3) => (value)
  begin
         let type = make(<llvm-primitive-type>, kind: arg$3);
         do-define-type(arg$1, type);
       end
end llvm-parser-action42;

define function llvm-parser-action43 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  begin
         let type
           = make(<llvm-pointer-type>, pointee: arg$3, address-space: arg$4);
         do-define-type(arg$1, type);
       end
end llvm-parser-action43;

define function llvm-parser-action44 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  begin
         let type = make(<llvm-array-type>, size: arg$4, element-type: arg$6);
         do-define-type(arg$1, type);
       end
end llvm-parser-action44;

define function llvm-parser-action45 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  begin
         let type = make(<llvm-vector-type>, size: arg$4, element-type: arg$6);
         do-define-type(arg$1, type);
       end
end llvm-parser-action45;

define function llvm-parser-action46 (arg$1, arg$2, arg$3) => (value)
  do-define-type(arg$1, arg$3)
end llvm-parser-action46;

define function llvm-parser-action51 (arg$1, arg$2) => (value)
  begin
         *llvm-parse-state*.parse-function := #f;
         *llvm-parse-state*.parse-unnamed-local-index := 0;
         *llvm-parse-state*.parse-unnamed-local-values.size := 0;
       end
end llvm-parser-action51;

define function llvm-parser-action52 (arg$1, arg$2, arg$3) => (value)
  begin
         let function
           = apply(make, <llvm-function>,
                   linkage: arg$1,
                   visibility: arg$2,
                   arg$3);
         add!(*llvm-parse-state*.parse-module.llvm-module-functions, function);
         do-define-global-value(function.llvm-global-name, function);

         // Install this function as the current one while parsing the
         // function-body
         *llvm-parse-state*.parse-function := function;

         // Install the function arguments as local values
         for (argument in function.llvm-function-arguments)
           do-define-local-value(argument.llvm-argument-name, argument);
         end for;
       end
end llvm-parser-action52;

define function llvm-parser-action53 (arg$1, arg$2, arg$3) => (value)
  begin
         let function
           = apply(make, <llvm-function>,
                   linkage: arg$1,
                   visibility: arg$2,
                   arg$3);
         add!(*llvm-parse-state*.parse-module.llvm-module-functions, function);
         do-define-global-value(function.llvm-global-name, function);
       end
end llvm-parser-action53;

define function llvm-parser-action54 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9, arg$10, arg$11, arg$12) => (value)
  begin
         let function-type
           = make(<llvm-function-type>,
                  return-type: arg$3,
                  parameter-types: map(llvm-value-type, arg$6.head),
                  varargs?: arg$6.tail);
         let pointer-type
           = make(<llvm-pointer-type>, pointee: function-type);
         let (return-attributes, function-attributes)
           = munge-attributes(arg$2, arg$9);
         let attribute-list
           = make(<llvm-attribute-list>,
                  return-attributes: return-attributes,
                  function-attributes: function-attributes,
                  parameter-attributes:
                    map(llvm-argument-attributes, arg$6.head));
         vector(calling-convention: arg$1,
                type: pointer-type,
                name: arg$4,
                arguments: arg$6.head,
                attribute-list: attribute-list,
		unnamed-address?: arg$8,
                section: arg$10,
                alignment: arg$11,
                garbage-collector: arg$12)
       end
end llvm-parser-action54;

define function llvm-parser-action59 () => (value)
  $llvm-calling-convention-c
end llvm-parser-action59;

define function llvm-parser-action60 (arg$1) => (value)
  $llvm-calling-convention-c
end llvm-parser-action60;

define function llvm-parser-action61 (arg$1) => (value)
  $llvm-calling-convention-fast
end llvm-parser-action61;

define function llvm-parser-action62 (arg$1) => (value)
  $llvm-calling-convention-cold
end llvm-parser-action62;

define function llvm-parser-action63 (arg$1) => (value)
  $llvm-calling-convention-x86-stdcall
end llvm-parser-action63;

define function llvm-parser-action64 (arg$1) => (value)
  $llvm-calling-convention-x86-fastcall
end llvm-parser-action64;

define function llvm-parser-action65 (arg$1) => (value)
  $llvm-calling-convention-arm-apcs
end llvm-parser-action65;

define function llvm-parser-action66 (arg$1) => (value)
  $llvm-calling-convention-arm-aapcs
end llvm-parser-action66;

define function llvm-parser-action67 (arg$1) => (value)
  $llvm-calling-convention-arm-vfp
end llvm-parser-action67;

define function llvm-parser-action68 (arg$1) => (value)
  $llvm-calling-convention-msp430-intr
end llvm-parser-action68;

define function llvm-parser-action69 (arg$1) => (value)
  $llvm-calling-convention-x86-thiscall
end llvm-parser-action69;

define function llvm-parser-action70 (arg$1) => (value)
  $llvm-calling-convention-ptx-kernel
end llvm-parser-action70;

define function llvm-parser-action71 (arg$1) => (value)
  $llvm-calling-convention-ptx-device
end llvm-parser-action71;

define function llvm-parser-action72 (arg$1, arg$2) => (value)
  arg$2
end llvm-parser-action72;

define function llvm-parser-action73 () => (value)
  $llvm-attribute-none
end llvm-parser-action73;

define function llvm-parser-action74 (arg$1, arg$2) => (value)
  llvm-attribute-merge(arg$1, arg$2)
end llvm-parser-action74;

define function llvm-parser-action75 (arg$1) => (value)
  $llvm-attribute-inreg
end llvm-parser-action75;

define function llvm-parser-action76 (arg$1) => (value)
  $llvm-attribute-zext
end llvm-parser-action76;

define function llvm-parser-action77 (arg$1) => (value)
  $llvm-attribute-sext
end llvm-parser-action77;

define function llvm-parser-action78 (arg$1) => (value)
  $llvm-attribute-noalias
end llvm-parser-action78;

define function llvm-parser-action79 (arg$1, arg$2, arg$3) => (value)
  #f
end llvm-parser-action79;

define function llvm-parser-action81 (arg$1) => (value)
  add!(*llvm-parse-state*.parse-function.llvm-function-basic-blocks,
            arg$1)
end llvm-parser-action81;

define function llvm-parser-action82 (arg$1, arg$2) => (value)
  add!(*llvm-parse-state*.parse-function.llvm-function-basic-blocks,
            arg$2)
end llvm-parser-action82;

define function llvm-parser-action85 (arg$1) => (value)
  make(<llvm-opaque-type>)
end llvm-parser-action85;

define function llvm-parser-action86 (arg$1) => (value)
  make(<llvm-integer-type>, width: arg$1)
end llvm-parser-action86;

define function llvm-parser-action87 (arg$1) => (value)
  make(<llvm-primitive-type>, kind: arg$1)
end llvm-parser-action87;

define function llvm-parser-action88 (arg$1, arg$2, arg$3) => (value)
  make(<llvm-pointer-type>, pointee: arg$1, address-space: arg$2)
end llvm-parser-action88;

define function llvm-parser-action89 (arg$1) => (value)
  begin
         element(*llvm-parse-state*.parse-unnamed-types, arg$1, default: #f)
           | begin
               let placeholder = make(<llvm-symbolic-type>, name: arg$1);
               element(*llvm-parse-state*.parse-unnamed-types, arg$1)
                 := placeholder;
               placeholder;
             end
       end
end llvm-parser-action89;

define function llvm-parser-action90 (arg$1) => (value)
  begin
         element(*llvm-parse-state*.parse-module.llvm-type-table,
                 arg$1, default: #f)
           | begin
               let placeholder = make(<llvm-symbolic-type>, name: arg$1);
               element(*llvm-parse-state*.parse-module.llvm-type-table, arg$1)
                 := placeholder;
               placeholder
             end
       end
end llvm-parser-action90;

define function llvm-parser-action91 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-array-type>, size: arg$2, element-type: arg$4)
end llvm-parser-action91;

define function llvm-parser-action92 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-vector-type>, size: arg$2, element-type: arg$4)
end llvm-parser-action92;

define function llvm-parser-action93 (arg$1, arg$2, arg$3) => (value)
  make(<llvm-struct-type>, packed?: #f, elements: reverse!(arg$2))
end llvm-parser-action93;

define function llvm-parser-action94 (arg$1, arg$2) => (value)
  make(<llvm-struct-type>, packed?: #f, elements: #[])
end llvm-parser-action94;

define function llvm-parser-action95 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-struct-type>, packed?: #t, elements: reverse!(arg$3))
end llvm-parser-action95;

define function llvm-parser-action96 (arg$1, arg$2, arg$3, arg$4) => (value)
  make(<llvm-struct-type>, packed?: #t, elements: #[])
end llvm-parser-action96;

define function llvm-parser-action97 (arg$1) => (value)
  #"FLOAT"
end llvm-parser-action97;

define function llvm-parser-action98 (arg$1) => (value)
  #"DOUBLE"
end llvm-parser-action98;

define function llvm-parser-action99 (arg$1) => (value)
  #"PPC_FP128"
end llvm-parser-action99;

define function llvm-parser-action100 (arg$1) => (value)
  #"FP128"
end llvm-parser-action100;

define function llvm-parser-action101 (arg$1) => (value)
  #"X86_FP80"
end llvm-parser-action101;

define function llvm-parser-action102 (arg$1) => (value)
  #"LABEL"
end llvm-parser-action102;

define function llvm-parser-action103 (arg$1) => (value)
  #"METADATA"
end llvm-parser-action103;

define function llvm-parser-action104 (arg$1) => (value)
  #"X86_MMX"
end llvm-parser-action104;

define function llvm-parser-action105 () => (value)
  0
end llvm-parser-action105;

define function llvm-parser-action106 (arg$1, arg$2, arg$3, arg$4) => (value)
  arg$3
end llvm-parser-action106;

define function llvm-parser-action107 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-function-type>,
            return-type: arg$1,
            parameter-types: map(arg-type, arg$3.head),
            varargs?: arg$3.tail)
end llvm-parser-action107;

define function llvm-parser-action108 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-function-type>,
            return-type: $llvm-void-type,
            parameter-types: map(arg-type, arg$3.head),
            varargs?: arg$3.tail)
end llvm-parser-action108;

define function llvm-parser-action109 (arg$1) => (value)
  make(<arg-entry>, type: arg$1)
end llvm-parser-action109;

define function llvm-parser-action111 (arg$1, arg$2) => (value)
  make(<arg-entry>, type: arg$1, attributes: arg$2)
end llvm-parser-action111;

define function llvm-parser-action112 (arg$1, arg$2) => (value)
  make(<arg-entry>,
            type: arg$1.arg-type,
            attributes: llvm-attribute-merge(arg$1.arg-attributes, arg$2))
end llvm-parser-action112;

define function llvm-parser-action113 () => (value)
  #"default"
end llvm-parser-action113;

define function llvm-parser-action114 (arg$1) => (value)
  #"default"
end llvm-parser-action114;

define function llvm-parser-action115 (arg$1) => (value)
  #"hidden"
end llvm-parser-action115;

define function llvm-parser-action116 (arg$1) => (value)
  #"protected"
end llvm-parser-action116;

define function llvm-parser-action118 (arg$1) => (value)
  #t
end llvm-parser-action118;

define function llvm-parser-action121 (arg$1) => (value)
  #"global"
end llvm-parser-action121;

define function llvm-parser-action122 (arg$1) => (value)
  #"constant"
end llvm-parser-action122;

define function llvm-parser-action123 (arg$1, arg$2, arg$3, arg$4) => (value)
  make(<llvm-aggregate-constant>,
            type: arg$1,
            aggregate-values: reverse!(arg$3))
end llvm-parser-action123;

define function llvm-parser-action124 (arg$1, arg$2, arg$3) => (value)
  make(<llvm-aggregate-constant>,
            type: arg$1,
            aggregate-values: #[])
end llvm-parser-action124;

define function llvm-parser-action125 (arg$1, arg$2, arg$3) => (value)
  make(<llvm-aggregate-constant>,
            type: arg$1,
            aggregate-values: string-constant-contents(arg$3))
end llvm-parser-action125;

define function llvm-parser-action129 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  make(<llvm-aggregate-constant>,
            type: arg$1,
            aggregate-values: reverse!(arg$4))
end llvm-parser-action129;

define function llvm-parser-action130 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-aggregate-constant>,
            type: arg$1,
            aggregate-values: #[])
end llvm-parser-action130;

define function llvm-parser-action131 (arg$1, arg$2) => (value)
  make(<llvm-null-constant>, type: arg$1)
end llvm-parser-action131;

define function llvm-parser-action132 (arg$1, arg$2) => (value)
  make(<llvm-undef-constant>, type: arg$1)
end llvm-parser-action132;

define function llvm-parser-action133 (arg$1, arg$2) => (value)
  begin
         llvm-constrain-type(llvm-value-type(arg$2),
                             arg$1);
         arg$2
       end
end llvm-parser-action133;

define function llvm-parser-action136 (arg$1, arg$2) => (value)
  make(<llvm-integer-constant>,
            type: arg$1,
            integer: arg$2)
end llvm-parser-action136;

define function llvm-parser-action138 (arg$1, arg$2) => (value)
  make(<llvm-float-constant>,
            type: arg$1,
            float: arg$2)
end llvm-parser-action138;

define function llvm-parser-action139 (arg$1, arg$2) => (value)
  make(<llvm-integer-constant>,
            type: arg$1,
            integer: 1)
end llvm-parser-action139;

define function llvm-parser-action140 (arg$1, arg$2) => (value)
  make(<llvm-integer-constant>,
            type: arg$1,
            integer: 0)
end llvm-parser-action140;

define function llvm-parser-action141 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  error("%%BLOCKADDRESS")
end llvm-parser-action141;

define function llvm-parser-action143 () => (value)
  #[]
end llvm-parser-action143;

define function llvm-parser-action144 (arg$1, arg$2, arg$3) => (value)
  concatenate(arg$2, arg$3)
end llvm-parser-action144;

define function llvm-parser-action145 (arg$1) => (value)
  vector(section:, arg$1)
end llvm-parser-action145;

define function llvm-parser-action146 (arg$1, arg$2) => (value)
  vector(alignment:, arg$2)
end llvm-parser-action146;

define function llvm-parser-action149 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  make(<llvm-cast-constant>,
            operator: #"BITCAST",
            type: arg$5,
            operands: vector(arg$3))
end llvm-parser-action149;

define function llvm-parser-action150 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  make(<llvm-gep-constant>,
            in-bounds?: arg$2,
            operands: pair(arg$4, reverse!(arg$5)))
end llvm-parser-action150;

define function llvm-parser-action151 (arg$1) => (value)
  begin
         element(*llvm-parse-state*.parse-unnamed-local-values, arg$1,
                 default: #f)
           | begin
               let placeholder = make(<llvm-symbolic-value>, name: arg$1);
               element(*llvm-parse-state*.parse-unnamed-local-values, arg$1)
                 := placeholder
             end
       end
end llvm-parser-action151;

define function llvm-parser-action152 (arg$1) => (value)
  begin
         element(*llvm-parse-state*.parse-unnamed-global-values, arg$1,
                 default: #f)
           | begin
               let placeholder = make(<llvm-symbolic-constant>, name: arg$1);
               element(*llvm-parse-state*.parse-unnamed-global-values, arg$1)
                 := placeholder
             end
       end
end llvm-parser-action152;

define function llvm-parser-action153 (arg$1) => (value)
  begin
         element(*llvm-parse-state*.parse-function.llvm-function-value-table,
                 arg$1, default: #f)
           | begin
               let placeholder = make(<llvm-symbolic-value>, name: arg$1);
               element(*llvm-parse-state*
                         .parse-function.llvm-function-value-table, arg$1)
                 := placeholder
             end
       end
end llvm-parser-action153;

define function llvm-parser-action154 (arg$1) => (value)
  begin
         element(*llvm-parse-state*.parse-module.llvm-global-table, arg$1,
                 default: #f)
           | begin
               let placeholder = make(<llvm-symbolic-constant>, name: arg$1);
               element(*llvm-parse-state*
                         .parse-module.llvm-global-table, arg$1)
                 := placeholder
             end
       end
end llvm-parser-action154;

define function llvm-parser-action155 () => (value)
  pair(#(), #f)
end llvm-parser-action155;

define function llvm-parser-action156 (arg$1) => (value)
  pair(reverse!(arg$1), #f)
end llvm-parser-action156;

define function llvm-parser-action157 (arg$1, arg$2, arg$3) => (value)
  pair(reverse!(arg$1), #t)
end llvm-parser-action157;

define function llvm-parser-action158 (arg$1) => (value)
  pair(#(), #t)
end llvm-parser-action158;

define function llvm-parser-action165 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  make(<llvm-cast-constant>,
            type: arg$5,
            operator: arg$1,
            operands: vector(arg$3))
end llvm-parser-action165;

define function llvm-parser-action167 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8) => (value)
  error("const-expr %%SELECT")
end llvm-parser-action167;

define function llvm-parser-action168 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  apply(make, <llvm-binop-constant>,
             operator: arg$1.head,
             operands: vector(arg$3, arg$5),
             arg$1.tail)
end llvm-parser-action168;

define function llvm-parser-action169 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  apply(make, <llvm-binop-constant>,
            operator: arg$1.head,
            operands: vector(arg$3, arg$5),
	    arg$1.tail)
end llvm-parser-action169;

define function llvm-parser-action170 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  begin
         make(<llvm-icmp-constant>,
            predicate: arg$2,
            operands: vector(arg$4, arg$6))
       end
end llvm-parser-action170;

define function llvm-parser-action171 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  begin
         make(<llvm-fcmp-constant>,
            predicate: arg$2,
            operands: vector(arg$4, arg$6))
       end
end llvm-parser-action171;

define function llvm-parser-action172 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  error("const-expr %%EXTRACTELEMENT")
end llvm-parser-action172;

define function llvm-parser-action173 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8) => (value)
  error("const-expr %%INSERTELEMENT")
end llvm-parser-action173;

define function llvm-parser-action174 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8) => (value)
  error("const-expr %%SHUFFLEVECTOR")
end llvm-parser-action174;

define function llvm-parser-action175 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  error("const-expr %%EXTRACTVALUE")
end llvm-parser-action175;

define function llvm-parser-action176 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  error("const-expr %%INSERTVALUE")
end llvm-parser-action176;

define function llvm-parser-action177 (arg$1) => (value)
  #"TRUNC"
end llvm-parser-action177;

define function llvm-parser-action178 (arg$1) => (value)
  #"ZEXT"
end llvm-parser-action178;

define function llvm-parser-action179 (arg$1) => (value)
  #"SEXT"
end llvm-parser-action179;

define function llvm-parser-action180 (arg$1) => (value)
  #"FPTRUNC"
end llvm-parser-action180;

define function llvm-parser-action181 (arg$1) => (value)
  #"FPEXT"
end llvm-parser-action181;

define function llvm-parser-action182 (arg$1) => (value)
  #"BITCAST"
end llvm-parser-action182;

define function llvm-parser-action183 (arg$1) => (value)
  #"UITOFP"
end llvm-parser-action183;

define function llvm-parser-action184 (arg$1) => (value)
  #"SITOFP"
end llvm-parser-action184;

define function llvm-parser-action185 (arg$1) => (value)
  #"FPTOUI"
end llvm-parser-action185;

define function llvm-parser-action186 (arg$1) => (value)
  #"FPTOSI"
end llvm-parser-action186;

define function llvm-parser-action187 (arg$1) => (value)
  #"INTTOPTR"
end llvm-parser-action187;

define function llvm-parser-action188 (arg$1) => (value)
  #"PTRTOINT"
end llvm-parser-action188;

define function llvm-parser-action189 (arg$1, arg$2) => (value)
  list(arg$2)
end llvm-parser-action189;

define function llvm-parser-action192 (arg$1) => (value)
  $llvm-void-type
end llvm-parser-action192;

define function llvm-parser-action193 (arg$1, arg$2, arg$3) => (value)
  begin
         do-define-local-value(arg$2, arg$3);
         add!(arg$1.llvm-basic-block-instructions, arg$3);
         arg$1
       end
end llvm-parser-action193;

define function llvm-parser-action205 (arg$1, arg$2) => (value)
  list(make(<llvm-argument>,
                 type: arg$1.arg-type,
                 name: arg$2,
                 attributes: arg$1.arg-attributes,
                 index: 0))
end llvm-parser-action205;

define function llvm-parser-action206 (arg$1, arg$2, arg$3, arg$4) => (value)
  pair(make(<llvm-argument>,
                 type: arg$3.arg-type,
                 name: arg$4,
                 attributes: arg$3.arg-attributes,
                 index: arg$1.head.llvm-argument-index + 1),
            arg$1)
end llvm-parser-action206;

define function llvm-parser-action217 (arg$1, arg$2) => (value)
  pair(#"ADD", arg$2)
end llvm-parser-action217;

define function llvm-parser-action218 (arg$1) => (value)
  #(#"FADD")
end llvm-parser-action218;

define function llvm-parser-action219 (arg$1, arg$2) => (value)
  pair(#"SUB", arg$2)
end llvm-parser-action219;

define function llvm-parser-action220 (arg$1) => (value)
  #(#"FSUB")
end llvm-parser-action220;

define function llvm-parser-action221 (arg$1, arg$2) => (value)
  pair(#"MUL", arg$2)
end llvm-parser-action221;

define function llvm-parser-action222 (arg$1) => (value)
  #(#"FMUL")
end llvm-parser-action222;

define function llvm-parser-action223 (arg$1) => (value)
  #(#"UDIV")
end llvm-parser-action223;

define function llvm-parser-action224 (arg$1, arg$2) => (value)
  #(#"UDIV", exact?:, #t)
end llvm-parser-action224;

define function llvm-parser-action225 (arg$1) => (value)
  #(#"SDIV")
end llvm-parser-action225;

define function llvm-parser-action226 (arg$1, arg$2) => (value)
  #(#"SDIV", exact?:, #t)
end llvm-parser-action226;

define function llvm-parser-action227 (arg$1) => (value)
  #(#"FDIV")
end llvm-parser-action227;

define function llvm-parser-action228 (arg$1) => (value)
  #(#"UREM")
end llvm-parser-action228;

define function llvm-parser-action229 (arg$1) => (value)
  #(#"SREM")
end llvm-parser-action229;

define function llvm-parser-action230 (arg$1) => (value)
  #(#"FREM")
end llvm-parser-action230;

define function llvm-parser-action232 (arg$1) => (value)
  #(no-unsigned-wrap?:, #t)
end llvm-parser-action232;

define function llvm-parser-action233 (arg$1) => (value)
  #(no-signed-wrap?:, #t)
end llvm-parser-action233;

define function llvm-parser-action234 (arg$1, arg$2) => (value)
  #(no-unsigned-wrap?:, #t, no-signed-wrap?:, #t)
end llvm-parser-action234;

define function llvm-parser-action236 (arg$1, arg$2) => (value)
  pair(#"SHL", arg$2)
end llvm-parser-action236;

define function llvm-parser-action237 (arg$1) => (value)
  #(#"LSHR")
end llvm-parser-action237;

define function llvm-parser-action238 (arg$1, arg$2) => (value)
  #(#"LSHR", exact?:, #t)
end llvm-parser-action238;

define function llvm-parser-action239 (arg$1) => (value)
  #(#"ASHR")
end llvm-parser-action239;

define function llvm-parser-action240 (arg$1, arg$2) => (value)
  #(#"ASHR", exact?:, #t)
end llvm-parser-action240;

define function llvm-parser-action241 (arg$1) => (value)
  #(#"AND")
end llvm-parser-action241;

define function llvm-parser-action242 (arg$1) => (value)
  #(#"OR")
end llvm-parser-action242;

define function llvm-parser-action243 (arg$1) => (value)
  #(#"XOR")
end llvm-parser-action243;

define function llvm-parser-action244 (arg$1) => (value)
  #"EQ"
end llvm-parser-action244;

define function llvm-parser-action245 (arg$1) => (value)
  #"NE"
end llvm-parser-action245;

define function llvm-parser-action246 (arg$1) => (value)
  #"SLT"
end llvm-parser-action246;

define function llvm-parser-action247 (arg$1) => (value)
  #"SGT"
end llvm-parser-action247;

define function llvm-parser-action248 (arg$1) => (value)
  #"SLE"
end llvm-parser-action248;

define function llvm-parser-action249 (arg$1) => (value)
  #"SGE"
end llvm-parser-action249;

define function llvm-parser-action250 (arg$1) => (value)
  #"ULT"
end llvm-parser-action250;

define function llvm-parser-action251 (arg$1) => (value)
  #"UGT"
end llvm-parser-action251;

define function llvm-parser-action252 (arg$1) => (value)
  #"ULE"
end llvm-parser-action252;

define function llvm-parser-action253 (arg$1) => (value)
  #"UGE"
end llvm-parser-action253;

define function llvm-parser-action254 (arg$1) => (value)
  #"OEQ"
end llvm-parser-action254;

define function llvm-parser-action255 (arg$1) => (value)
  #"ONE"
end llvm-parser-action255;

define function llvm-parser-action256 (arg$1) => (value)
  #"OLT"
end llvm-parser-action256;

define function llvm-parser-action257 (arg$1) => (value)
  #"OGT"
end llvm-parser-action257;

define function llvm-parser-action258 (arg$1) => (value)
  #"OLE"
end llvm-parser-action258;

define function llvm-parser-action259 (arg$1) => (value)
  #"OGE"
end llvm-parser-action259;

define function llvm-parser-action260 (arg$1) => (value)
  #"ORD"
end llvm-parser-action260;

define function llvm-parser-action261 (arg$1) => (value)
  #"UNO"
end llvm-parser-action261;

define function llvm-parser-action262 (arg$1) => (value)
  #"UEQ"
end llvm-parser-action262;

define function llvm-parser-action263 (arg$1) => (value)
  #"UNE"
end llvm-parser-action263;

define function llvm-parser-action268 (arg$1) => (value)
  #"TRUE"
end llvm-parser-action268;

define function llvm-parser-action269 (arg$1) => (value)
  #"FALSE"
end llvm-parser-action269;

define function llvm-parser-action270 () => (value)
  begin
         let basic-block = make(<llvm-basic-block>);
         do-define-local-value(#f, basic-block);
         basic-block
       end
end llvm-parser-action270;

define function llvm-parser-action271 (arg$1) => (value)
  begin
         let basic-block = make(<llvm-basic-block>, name: arg$1);
         do-define-local-value(arg$1, basic-block);
         basic-block
       end
end llvm-parser-action271;

define function llvm-parser-action272 (arg$1, arg$2) => (value)
  begin
         add!(arg$1.llvm-basic-block-instructions, arg$2);
         arg$1
       end
end llvm-parser-action272;

define function llvm-parser-action273 (arg$1, arg$2) => (value)
  do-define-local-value(arg$1, arg$2)
end llvm-parser-action273;

define function llvm-parser-action275 (arg$1, arg$2, arg$3) => (value)
  make(<llvm-return-instruction>,
            operands: reverse!(arg$2),
            metadata: arg$3)
end llvm-parser-action275;

define function llvm-parser-action276 (arg$1, arg$2, arg$3) => (value)
  make(<llvm-return-instruction>, operands: #[], metadata: arg$3)
end llvm-parser-action276;

define function llvm-parser-action277 (arg$1, arg$2, arg$3, arg$4) => (value)
  begin
         llvm-constrain-type(llvm-value-type(arg$3), $llvm-label-type);
         make(<llvm-branch-instruction>,
              operands: vector(arg$3),
              metadata: arg$4);
       end
end llvm-parser-action277;

define function llvm-parser-action278 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9, arg$10) => (value)
  begin
         llvm-constrain-type(llvm-value-type(arg$3),
                             make(<llvm-integer-type>, width: arg$2));
         llvm-constrain-type(llvm-value-type(arg$6), $llvm-label-type);
         llvm-constrain-type(llvm-value-type(arg$9), $llvm-label-type);
         make(<llvm-branch-instruction>,
              operands: vector(arg$3, arg$6, arg$9),
              metadata: arg$10);
       end
end llvm-parser-action278;

define function llvm-parser-action279 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9, arg$10) => (value)
  begin
         llvm-constrain-type(llvm-value-type(arg$3),
                             make(<llvm-integer-type>, width: arg$2));
         llvm-constrain-type(llvm-value-type(arg$6), $llvm-label-type);
         make(<llvm-switch-instruction>,
              operands: concatenate(vector(arg$3, arg$6), arg$8),
              metadata: arg$10)
       end
end llvm-parser-action279;

define function llvm-parser-action280 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9) => (value)
  begin
         llvm-constrain-type(llvm-value-type(arg$3),
                             make(<llvm-integer-type>, width: arg$2));
         llvm-constrain-type(llvm-value-type(arg$6), $llvm-label-type);
         make(<llvm-switch-instruction>,
              operands: vector(arg$3, arg$6),
              metadata: arg$9)
       end
end llvm-parser-action280;

define function llvm-parser-action281 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  error("%%INDIRECTBR")
end llvm-parser-action281;

define function llvm-parser-action282 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  error("%%INDIRECTBR")
end llvm-parser-action282;

define function llvm-parser-action283 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9, arg$10, arg$11, arg$12, arg$13, arg$14, arg$15, arg$16) => (value)
  begin
         let type = arg$4;
         let return-type
           = if (instance?(type, <llvm-pointer-type>))
               let pointee
                 = llvm-type-forward(type.llvm-pointer-type-pointee);
               if (instance?(pointee, <llvm-function-type>))
                 llvm-constrain-type(llvm-value-type(arg$5), type);
                 llvm-type-forward(pointee.llvm-function-type-return-type)
               else
                 type
               end if
             else
               type
             end if;
         llvm-constrain-type(llvm-value-type(arg$12), $llvm-label-type);
         llvm-constrain-type(llvm-value-type(arg$15), $llvm-label-type);
         let param-list = reverse!(arg$7);
         let (return-attributes, function-attributes)
           = munge-attributes(arg$3, arg$9);
         let attribute-list
           = make(<llvm-attribute-list>,
                  return-attributes: return-attributes,
                  function-attributes: function-attributes,
                  parameter-attributes: map(param-attributes, param-list));
         make(<llvm-invoke-instruction>,
              type: return-type,
              attribute-list: attribute-list,
              calling-convention: arg$2,
              operands: concatenate(vector(arg$12, arg$15, arg$5),
                                    map(param-value, param-list)),
              metadata: arg$16)
       end
end llvm-parser-action283;

define function llvm-parser-action284 (arg$1, arg$2) => (value)
  make(<llvm-unreachable-instruction>, metadata: arg$2)
end llvm-parser-action284;

define function llvm-parser-action285 (arg$1, arg$2, arg$3) => (value)
  make(<llvm-resume-instruction>,
            operands: vector(arg$2),
            metadata: arg$3)
end llvm-parser-action285;

define function llvm-parser-action286 (arg$1) => (value)
  $llvm-attribute-noreturn
end llvm-parser-action286;

define function llvm-parser-action287 (arg$1) => (value)
  $llvm-attribute-nounwind
end llvm-parser-action287;

define function llvm-parser-action291 (arg$1) => (value)
  $llvm-attribute-readnone
end llvm-parser-action291;

define function llvm-parser-action292 (arg$1) => (value)
  $llvm-attribute-readonly
end llvm-parser-action292;

define function llvm-parser-action293 (arg$1) => (value)
  $llvm-attribute-noinline
end llvm-parser-action293;

define function llvm-parser-action294 (arg$1) => (value)
  $llvm-attribute-alwaysinline
end llvm-parser-action294;

define function llvm-parser-action295 (arg$1) => (value)
  $llvm-attribute-inlinehint
end llvm-parser-action295;

define function llvm-parser-action296 (arg$1) => (value)
  $llvm-attribute-optsize
end llvm-parser-action296;

define function llvm-parser-action297 (arg$1) => (value)
  $llvm-attribute-ssp
end llvm-parser-action297;

define function llvm-parser-action298 (arg$1) => (value)
  $llvm-attribute-sspreq
end llvm-parser-action298;

define function llvm-parser-action299 (arg$1) => (value)
  $llvm-attribute-noredzone
end llvm-parser-action299;

define function llvm-parser-action300 (arg$1) => (value)
  $llvm-attribute-noimplicitfloat
end llvm-parser-action300;

define function llvm-parser-action301 (arg$1) => (value)
  $llvm-attribute-naked
end llvm-parser-action301;

define function llvm-parser-action302 (arg$1, arg$2, arg$3, arg$4) => (value)
  llvm-attribute-stack-alignment(arg$3)
end llvm-parser-action302;

define function llvm-parser-action303 (arg$1) => (value)
  $llvm-attribute-uwtable
end llvm-parser-action303;

define function llvm-parser-action304 (arg$1) => (value)
  $llvm-attribute-returns-twice
end llvm-parser-action304;

define function llvm-parser-action305 (arg$1) => (value)
  $llvm-attribute-non-lazy-bind
end llvm-parser-action305;

define function llvm-parser-action306 (arg$1, arg$2) => (value)
  begin
         llvm-constrain-type(llvm-value-type(arg$2), arg$1);
         arg$2
       end
end llvm-parser-action306;

define function llvm-parser-action311 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  begin
         llvm-constrain-type(llvm-value-type(arg$2),
                             make(<llvm-integer-type>, width: arg$1));
         llvm-constrain-type(llvm-value-type(arg$5), $llvm-label-type);
         vector(arg$2, arg$5)
       end
end llvm-parser-action311;

define function llvm-parser-action312 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  begin
         llvm-constrain-type(llvm-value-type(arg$3),
                             make(<llvm-integer-type>, width: arg$2));
         llvm-constrain-type(llvm-value-type(arg$6), $llvm-label-type);
         concatenate(arg$1, vector(arg$3, arg$6))
       end
end llvm-parser-action312;

define function llvm-parser-action314 (arg$1, arg$2, arg$3, arg$4) => (value)
  #f
end llvm-parser-action314;

define function llvm-parser-action316 (arg$1, arg$2, arg$3) => (value)
  begin
         llvm-constrain-type(llvm-value-type(arg$2), arg$1.arg-type);
         let entry
           = make(<param-entry>,
                  attributes: llvm-attribute-merge(arg$1.arg-attributes,
                                                   arg$3),
                  value: arg$2);
         list(entry)
       end
end llvm-parser-action316;

define function llvm-parser-action317 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  begin
         llvm-constrain-type(llvm-value-type(arg$4), arg$3.arg-type);
         let entry
           = make(<param-entry>,
                  attributes: llvm-attribute-merge(arg$3.arg-attributes,
                                                   arg$5),
                  value: arg$4);
         pair(entry, arg$1)
       end
end llvm-parser-action317;

define function llvm-parser-action322 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  begin
         let type = arg$2;
         llvm-constrain-type(llvm-value-type(arg$3), type);
         llvm-constrain-type(llvm-value-type(arg$5), type);
         apply(make, <llvm-binop-instruction>,
               operator: arg$1.head,
               operands: vector(arg$3, arg$5),
               metadata: arg$6,
               arg$1.tail)
       end
end llvm-parser-action322;

define function llvm-parser-action323 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  begin
         let type = arg$2;
         llvm-constrain-type(llvm-value-type(arg$3), type);
         llvm-constrain-type(llvm-value-type(arg$5), type);
         apply(make, <llvm-binop-instruction>,
              operator: arg$1.head,
              operands: vector(arg$3, arg$5),
              metadata: arg$6,
	      arg$1.tail)
       end
end llvm-parser-action323;

define function llvm-parser-action324 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  begin
         let type = arg$3;
         llvm-constrain-type(llvm-value-type(arg$4), type);
         llvm-constrain-type(llvm-value-type(arg$6), type);
         make(<llvm-icmp-instruction>,
              predicate: arg$2,
              operands: vector(arg$4, arg$6),
              metadata: arg$7)
       end
end llvm-parser-action324;

define function llvm-parser-action325 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  begin
         let type = arg$3;
         llvm-constrain-type(llvm-value-type(arg$4), type);
         llvm-constrain-type(llvm-value-type(arg$6), type);
         make(<llvm-fcmp-instruction>,
              predicate: arg$2,
              operands: vector(arg$4, arg$6),
              metadata: arg$7)
       end
end llvm-parser-action325;

define function llvm-parser-action326 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-cast-instruction>,
            type: arg$4,
            operator: arg$1,
            operands: vector(arg$2),
            metadata: arg$5)
end llvm-parser-action326;

define function llvm-parser-action327 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  make(<llvm-select-instruction>,
            operands: vector(arg$2, arg$4, arg$6),
            metadata: arg$7)
end llvm-parser-action327;

define function llvm-parser-action328 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-va-arg-instruction>,
            type: arg$4,
            operands: vector(arg$2),
            metadata: arg$5)
end llvm-parser-action328;

define function llvm-parser-action329 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-extractelement-instruction>,
            operands: vector(arg$2, arg$4),
            metadata: arg$5)
end llvm-parser-action329;

define function llvm-parser-action330 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  make(<llvm-insertelement-instruction>,
            operands: vector(arg$2, arg$4, arg$6),
            metadata: arg$7)
end llvm-parser-action330;

define function llvm-parser-action331 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  make(<llvm-shufflevector-instruction>,
            operands: vector(arg$2, arg$4, arg$6),
            metadata: arg$7)
end llvm-parser-action331;

define function llvm-parser-action332 (arg$1, arg$2, arg$3, arg$4) => (value)
  begin
         let type = arg$2;
         for (i from 0 below arg$3.size by 2)
           llvm-constrain-type(llvm-value-type(arg$3[i]), type);
           llvm-constrain-type(llvm-value-type(arg$3[i + 1]), $llvm-label-type)
         end for;
         make(<llvm-phi-node>, operands: arg$3, metadata: arg$4)
       end
end llvm-parser-action332;

define function llvm-parser-action333 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  make(<llvm-landingpad-instruction>,
            type: arg$2,
            operands: pair(arg$4, reverse!(arg$6)),
            cleanup?: arg$5,
            metadata: arg$7)
end llvm-parser-action333;

define function llvm-parser-action334 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9, arg$10) => (value)
  begin
         let param-list = reverse!(arg$7);
         let type = arg$4;
         let return-type
           = if (instance?(type, <llvm-pointer-type>))
               let pointee
                 = llvm-type-forward(type.llvm-pointer-type-pointee);
               if (instance?(pointee, <llvm-function-type>))
                 llvm-constrain-type(llvm-value-type(arg$5), type);
                 llvm-type-forward(pointee.llvm-function-type-return-type)
               else
                 type
               end if
             else
               type
             end if;
         let (return-attributes, function-attributes)
           = munge-attributes(arg$3, arg$9);
         let attribute-list
           = make(<llvm-attribute-list>,
                  return-attributes: return-attributes,
                  function-attributes: function-attributes,
                  parameter-attributes: map(param-attributes, param-list));
         make(<llvm-call-instruction>,
              type: return-type,
              attribute-list: attribute-list,
              tail-call?: arg$1,
              calling-convention: arg$2,
              operands: pair(arg$5, map(param-value, param-list)),
              metadata: arg$10)
       end
end llvm-parser-action334;

define function llvm-parser-action335 (arg$1, arg$2, arg$3) => (value)
  begin
         let pointer-type
           = make(<llvm-pointer-type>, pointee: arg$2);
         let size-operand
           = make(<llvm-integer-constant>,
                  type: $llvm-i32-type,
                  integer: 1);
         apply(make, <llvm-alloca-instruction>,
               type: pointer-type,
               operands: vector(size-operand),
               arg$3)
       end
end llvm-parser-action335;

define function llvm-parser-action336 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  begin
         let pointer-type
           = make(<llvm-pointer-type>, pointee: arg$2);
         llvm-constrain-type(llvm-value-type(arg$5),
                             make(<llvm-integer-type>, width: arg$4));
         apply(make, <llvm-alloca-instruction>,
               type: pointer-type,
               operands: vector(arg$5),
               arg$6)
       end
end llvm-parser-action336;

define function llvm-parser-action337 (arg$1, arg$2, arg$3, arg$4) => (value)
  apply(make, <llvm-load-instruction>,
             volatile?: arg$2,
             operands: vector(arg$3),
             arg$4)
end llvm-parser-action337;

define function llvm-parser-action338 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  apply(make, <llvm-atomic-load-instruction>,
             volatile?: arg$3,
             operands: vector(arg$4),
             scope: arg$5,
             ordering: arg$6,
             arg$7)
end llvm-parser-action338;

define function llvm-parser-action339 (arg$1, arg$2, arg$3, arg$4) => (value)
  apply(make, <llvm-load-instruction>,
            volatile?: #t,
            operands: vector(arg$3),
            arg$4)
end llvm-parser-action339;

define function llvm-parser-action340 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  apply(make, <llvm-store-instruction>,
             volatile?: arg$2,
             operands: vector(arg$3, arg$5),
             arg$6)
end llvm-parser-action340;

define function llvm-parser-action341 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9) => (value)
  apply(make, <llvm-atomic-store-instruction>,
             volatile?: arg$3,
             operands: vector(arg$4, arg$6),
             scope: arg$7,
             ordering: arg$8,
             arg$9)
end llvm-parser-action341;

define function llvm-parser-action342 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  apply(make, <llvm-store-instruction>,
             volatile?: #t,
             operands: vector(arg$3, arg$5),
             arg$6)
end llvm-parser-action342;

define function llvm-parser-action343 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9, arg$10) => (value)
  make(<llvm-cmpxchg-instruction>,
            volatile?: arg$2,
            operands: vector(arg$3, arg$5, arg$7),
            scope: arg$8,
            ordering: arg$9,
            metadata: arg$10)
end llvm-parser-action343;

define function llvm-parser-action344 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8, arg$9) => (value)
  make(<llvm-atomicrmw-instruction>,
            volatile?: arg$2,
            operation: arg$3,
            operands: vector(arg$4, arg$6),
            scope: arg$7,
            ordering: arg$8,
            metadata: arg$9)
end llvm-parser-action344;

define function llvm-parser-action345 (arg$1, arg$2, arg$3, arg$4) => (value)
  make(<llvm-fence-instruction>,
            scope: arg$2,
            ordering: arg$3,
            metadata: arg$4)
end llvm-parser-action345;

define function llvm-parser-action346 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-gep-instruction>,
            in-bounds?: arg$2,
            operands: pair(arg$3, reverse!(arg$4)),
            metadata: arg$5)
end llvm-parser-action346;

define function llvm-parser-action347 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  make(<llvm-insert-value-instruction>,
            operands: vector(arg$2, arg$4),
            indices: reverse!(arg$5),
            metadata: arg$6)
end llvm-parser-action347;

define function llvm-parser-action348 (arg$1, arg$2, arg$3, arg$4) => (value)
  make(<llvm-extract-value-instruction>,
            operands: vector(arg$2),
            indices: reverse!(arg$3),
            metadata: arg$4)
end llvm-parser-action348;

define function llvm-parser-action349 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-extract-value-instruction>,
            operands: vector(arg$2),
            indices: vector(arg$4),
            metadata: arg$5)
end llvm-parser-action349;

define function llvm-parser-action353 (arg$1, arg$2, arg$3) => (value)
  add(arg$1, arg$3)
end llvm-parser-action353;

define function llvm-parser-action355 (arg$1, arg$2) => (value)
  #t
end llvm-parser-action355;

define function llvm-parser-action356 (arg$1) => (value)
  #f
end llvm-parser-action356;

define function llvm-parser-action357 () => (value)
  #"cross-thread"
end llvm-parser-action357;

define function llvm-parser-action358 (arg$1) => (value)
  #"single-thread"
end llvm-parser-action358;

define function llvm-parser-action359 (arg$1) => (value)
  #"unordered"
end llvm-parser-action359;

define function llvm-parser-action360 (arg$1) => (value)
  #"monotonic"
end llvm-parser-action360;

define function llvm-parser-action361 (arg$1) => (value)
  #"acquire"
end llvm-parser-action361;

define function llvm-parser-action362 (arg$1) => (value)
  #"release"
end llvm-parser-action362;

define function llvm-parser-action363 (arg$1) => (value)
  #"acquire-release"
end llvm-parser-action363;

define function llvm-parser-action364 (arg$1) => (value)
  #"sequentially-consistent"
end llvm-parser-action364;

define function llvm-parser-action365 (arg$1) => (value)
  #"xchg"
end llvm-parser-action365;

define function llvm-parser-action366 (arg$1) => (value)
  #"add"
end llvm-parser-action366;

define function llvm-parser-action367 (arg$1) => (value)
  #"sub"
end llvm-parser-action367;

define function llvm-parser-action368 (arg$1) => (value)
  #"and"
end llvm-parser-action368;

define function llvm-parser-action369 (arg$1) => (value)
  #"nand"
end llvm-parser-action369;

define function llvm-parser-action370 (arg$1) => (value)
  #"or"
end llvm-parser-action370;

define function llvm-parser-action371 (arg$1) => (value)
  #"xor"
end llvm-parser-action371;

define function llvm-parser-action372 (arg$1) => (value)
  #"max"
end llvm-parser-action372;

define function llvm-parser-action373 (arg$1) => (value)
  #"min"
end llvm-parser-action373;

define function llvm-parser-action374 (arg$1) => (value)
  #"umax"
end llvm-parser-action374;

define function llvm-parser-action375 (arg$1) => (value)
  #"umin"
end llvm-parser-action375;

define function llvm-parser-action376 (arg$1) => (value)
  make(<llvm-integer-constant>, integer: arg$1)
end llvm-parser-action376;

define function llvm-parser-action378 (arg$1) => (value)
  make(<llvm-float-constant>, float: arg$1)
end llvm-parser-action378;

define function llvm-parser-action379 (arg$1) => (value)
  make(<llvm-integer-constant>, integer: 1)
end llvm-parser-action379;

define function llvm-parser-action380 (arg$1) => (value)
  make(<llvm-integer-constant>, integer: 0)
end llvm-parser-action380;

define function llvm-parser-action381 (arg$1) => (value)
  make(<llvm-null-constant>)
end llvm-parser-action381;

define function llvm-parser-action382 (arg$1) => (value)
  make(<llvm-undef-constant>)
end llvm-parser-action382;

define function llvm-parser-action384 (arg$1, arg$2, arg$3) => (value)
  make(<llvm-aggregate-constant>, aggregate-values: reverse!(arg$2))
end llvm-parser-action384;

define function llvm-parser-action386 (arg$1, arg$2) => (value)
  make(<llvm-aggregate-constant>, aggregate-values: #[])
end llvm-parser-action386;

define function llvm-parser-action387 (arg$1, arg$2) => (value)
  make(<llvm-aggregate-constant>,
            aggregate-values: string-constant-contents(arg$2))
end llvm-parser-action387;

define function llvm-parser-action390 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  make(<llvm-aggregate-constant>, aggregate-values: reverse!(arg$3))
end llvm-parser-action390;

define function llvm-parser-action391 (arg$1, arg$2, arg$3, arg$4) => (value)
  make(<llvm-aggregate-constant>, aggregate-values: #[])
end llvm-parser-action391;

define function llvm-parser-action392 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  error("%%BLOCKADDRESS")
end llvm-parser-action392;

define function llvm-parser-action394 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  apply(make, <llvm-asm-constant>,
             asm-string: arg$3,
             constraint: arg$5,
             arg$2)
end llvm-parser-action394;

define function llvm-parser-action397 (arg$1) => (value)
  #[side-effect?:, #t]
end llvm-parser-action397;

define function llvm-parser-action398 (arg$1) => (value)
  #[align-stack?:, #t]
end llvm-parser-action398;

define function llvm-parser-action399 (arg$1, arg$2) => (value)
  #[side-effect?:, #t, align-stack?:, #t]
end llvm-parser-action399;

define function llvm-parser-action407 (arg$1) => (value)
  $llvm-attribute-sret
end llvm-parser-action407;

define function llvm-parser-action409 (arg$1) => (value)
  $llvm-attribute-nocapture
end llvm-parser-action409;

define function llvm-parser-action410 (arg$1) => (value)
  $llvm-attribute-byval
end llvm-parser-action410;

define function llvm-parser-action411 (arg$1) => (value)
  $llvm-attribute-nest
end llvm-parser-action411;

define function llvm-parser-action412 (arg$1, arg$2) => (value)
  llvm-attribute-alignment(arg$2)
end llvm-parser-action412;

define function llvm-parser-action413 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  vector(arg$2, arg$4)
end llvm-parser-action413;

define function llvm-parser-action414 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  concatenate(arg$1, vector(arg$4, arg$6))
end llvm-parser-action414;

define function llvm-parser-action419 (arg$1) => (value)
  vector(alignment: 0, metadata: arg$1)
end llvm-parser-action419;

define function llvm-parser-action420 (arg$1, arg$2, arg$3, arg$4) => (value)
  vector(alignment: arg$3, metadata: arg$4)
end llvm-parser-action420;

define function llvm-parser-action421 (arg$1, arg$2, arg$3, arg$4, arg$5) => (value)
  add!(*llvm-parse-state*.parse-module.llvm-module-named-metadata,
            make(<llvm-named-metadata>, name: arg$1, operands: #()))
end llvm-parser-action421;

define function llvm-parser-action422 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6) => (value)
  add!(*llvm-parse-state*.parse-module.llvm-module-named-metadata,
            make(<llvm-named-metadata>, name: arg$1,
	         operands: reverse!(arg$5)))
end llvm-parser-action422;

define function llvm-parser-action423 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7) => (value)
  do-define-metadata-value(arg$2,
                                make(<llvm-metadata-node>,
                                     node-values: #[],
                                     function-local?: #f))
end llvm-parser-action423;

define function llvm-parser-action424 (arg$1, arg$2, arg$3, arg$4, arg$5, arg$6, arg$7, arg$8) => (value)
  do-define-metadata-value(arg$2,
                                make(<llvm-metadata-node>,
                                     node-values: reverse!(arg$7),
                                     function-local?: #f))
end llvm-parser-action424;

define function llvm-parser-action428 (arg$1) => (value)
  list(#f)
end llvm-parser-action428;

define function llvm-parser-action430 (arg$1, arg$2, arg$3) => (value)
  pair(#f, arg$1)
end llvm-parser-action430;

define function llvm-parser-action432 (arg$1, arg$2, arg$3) => (value)
  make(<llvm-metadata-node>, node-values: #[], function-local?: #f)
end llvm-parser-action432;

define function llvm-parser-action433 (arg$1, arg$2, arg$3, arg$4) => (value)
  make(<llvm-metadata-node>, node-values: reverse!(arg$3))
end llvm-parser-action433;

define function llvm-parser-action434 (arg$1, arg$2) => (value)
  make(<llvm-metadata-string>, string: arg$2)
end llvm-parser-action434;

define function llvm-parser-action435 (arg$1, arg$2) => (value)
  begin
         element(*llvm-parse-state*.parse-unnamed-metadata-values, arg$2,
                 default: #f)
           | begin
               let placeholder = make(<llvm-symbolic-metadata>, name: arg$2);
               element(*llvm-parse-state*.parse-unnamed-metadata-values, arg$2)
                 := placeholder
             end
       end
end llvm-parser-action435;

define function llvm-parser-action438 (arg$1, arg$2) => (value)
  list(make(<llvm-named-metadata>, name: arg$1, operands: list(arg$2)))
end llvm-parser-action438;

define function llvm-parser-action439 (arg$1, arg$2, arg$3, arg$4) => (value)
  pair(make(<llvm-named-metadata>, name: arg$3, operands: list(arg$4)),
            arg$1)
end llvm-parser-action439;

define constant llvm-parser :: <parser>
  = make(<parser>,
  action-table:
      #[#[65535, 1, 0, 1, 1, 1, 4, 1, 7, 1, 9, 1, 10, 1, 14, 1, 15, 1, 16, 1, 17, 1, 18, 1, 24, 1, 41, 1, 21, 1, 22, 1, 23, 1, 27, 1, 28, 1, 63, 1, 40, 1, 77, 1, 75, 1, 79, 1, 80, 1, 116, 1, 74, 1, 115, 1, 222, 1, 223, 1, 25, 1, 19, 1, 26, 1, 20, 1, 76, 1, 29, 1],
	#[7, -37, 1, -16, 14, -4, 0, -13, 16, -3, 4, -27, 19, -25, 18, -5, 9, -29, 10, -19, 25, -14, 15, -24, 17, -17, 20, -11, 21, -36, 22, -20, 23, -6, 24, -28, 26, -38, 27, -23, 28, -7, 29, -31, 40, -30, 41, -8, 63, -33, 74, 19, 75, 19, 76, 19, 77, 19, 79, 19, 80, 19, 115, -10, 116, -9, 222, -35, 223, -22],
	#[65535, 22, 47, 22, 31, 22, 3, 22, 7, 22, 11, 22, 35, 22, 46, 22, 54, 22, 33, 22, 53, 22, 52, 22, 36, 22, 51, 22, 50, 22, 56, 22, 49, 22, 48, 22, 44, 22, 45, 22, 55, 22, 57, 22, 58, 22, 59, 22, 60, 22, 63, 22, 64, 22, 65, 22, 66, 22, 67, 22, 68, 22, 69, 22, 70, 22, 71, 22, 72, 22, 73, 22, 74, 22, 75, 22, 76, 22, 77, 22, 78, 22, 79, 22, 80, 22, 92, 22, 94, 22, 115, 22],
	#[65535, 20, 3, 20, 7, 20, 11, 20, 31, 20, 33, 20, 35, 20, 36, 20, 44, 20, 45, 20, 46, 20, 47, 20, 48, 20, 49, 20, 50, 20, 51, 20, 52, 20, 53, 20, 54, 20, 55, 20, 56, 20, 57, 20, 58, 20, 59, 20, 60, 20, 63, 20, 64, 20, 65, 20, 66, 20, 67, 20, 68, 20, 69, 20, 70, 20, 71, 20, 72, 20, 73, 20, 74, 20, 75, 20, 76, 20, 77, 20, 78, 20, 79, 20, 80, 20, 92, 20, 94, 20, 115, 20],
	#[65535, 24, 47, 24, 31, 24, 3, 24, 7, 24, 11, 24, 35, 24, 46, 24, 54, 24, 33, 24, 53, 24, 52, 24, 36, 24, 51, 24, 50, 24, 56, 24, 49, 24, 48, 24, 44, 24, 45, 24, 55, 24, 57, 24, 58, 24, 59, 24, 60, 24, 63, 24, 64, 24, 65, 24, 66, 24, 67, 24, 68, 24, 69, 24, 70, 24, 71, 24, 72, 24, 73, 24, 74, 24, 75, 24, 76, 24, 77, 24, 78, 24, 79, 24, 80, 24, 92, 24, 94, 24, 115, 24],
	#[65535, 29, 3, 29, 7, 29, 11, 29, 31, 29, 33, 29, 35, 29, 36, 29, 44, 29, 45, 29, 46, 29, 47, 29, 48, 29, 49, 29, 50, 29, 51, 29, 52, 29, 53, 29, 54, 29, 55, 29, 56, 29, 57, 29, 58, 29, 59, 29, 60, 29, 63, 29, 64, 29, 65, 29, 66, 29, 67, 29, 68, 29, 69, 29, 70, 29, 71, 29, 72, 29, 73, 29, 74, 29, 75, 29, 76, 29, 77, 29, 78, 29, 79, 29, 80, 29, 92, 29, 94, 29, 115, 29],
	#[65535, 34, 54, 34, 45, 34, 11, 34, 31, 34, 46, 34, 7, 34, 59, 34, 48, 34, 52, 34, 49, 34, 47, 34, 33, 34, 58, 34, 69, 34, 35, 34, 55, 34, 51, 34, 68, 34, 70, 34, 36, 34, 63, 34, 50, 34, 76, 34, 78, 34, 44, 34, 56, 34, 72, 34, 71, 34, 53, 34, 79, 34, 75, 34, 60, 34, 67, 34, 57, 34, 77, 34, 66, 34, 64, 34, 65, 34, 73, 34, 74, 34, 80, 34, 115, 34],
	#[23, -6, 7, 19, 47, 19, 14, -4, 11, 19, 22, -20, 16, -3, 31, 19, 19, -25, 18, -5, 25, -14, 15, -24, 17, -17, 20, -11, 21, -36, 35, 19, 46, 19, 24, -28, 26, -38, 27, -23, 28, -7, 29, -31, 54, 19, 33, 19, 53, 19, 52, 19, 36, 19, 51, 19, 50, 19, 56, 19, 49, 19, 48, 19, 44, 19, 45, 19, 55, 19, 57, 19, 58, 19, 59, 19, 60, 19, 63, 19, 64, 19, 65, 19, 66, 19, 67, 19, 68, 19, 69, 19, 70, 19, 71, 19, 73, 19, 74, 19, 75, 19, 76, 19, 115, 19],
	#[65535, 200, 13, 200, 15, 200, 22, 200, 29, 200, 20, 200, 4, 200, 111, 200, 92, 200, 17, 200, 12, 200, 14, 200, 16, 200, 62, 200, 7, 200, 36, 200, 127, 200, 19, 200, 10, 200, 34, 200, 1, 200, 6, 200, 79, 200, 21, 200, 23, 200, 25, 200, 26, 200, 110, 200, 9, 200, 28, 200, 41, 200, 24, 200, 18, 200, 32, 200, 11, 200, 98, 200, 0, 200, 27, 200, 94, 200, 58, 200, 43, 200, 63, 200, 76, 200, 102, 200, 90, 200, 40, 200, 59, 200, 99, 200, 57, 200, 75, 200, 60, 200, 42, 200, 77, 200, 74, 200, 80, 200, 134, 200, 109, 200, 108, 200, 113, 200, 93, 200, 107, 200, 106, 200, 96, 200, 97, 200, 116, 200, 100, 200, 101, 200, 115, 200, 103, 200, 104, 200, 105, 200, 135, 200, 112, 200, 114, 200, 118, 200, 119, 200, 120, 200, 121, 200, 122, 200, 123, 200, 124, 200, 126, 200, 138, 200, 128, 200, 129, 200, 130, 200, 136, 200, 133, 200, 137, 200, 159, 200, 160, 200, 161, 200, 162, 200, 163, 200, 164, 200, 165, 200, 166, 200, 167, 200, 189, 200, 185, 200, 186, 200, 187, 200, 190, 200, 192, 200, 193, 200, 194, 200, 195, 200, 196, 200, 197, 200, 198, 200, 199, 200, 200, 200, 201, 200, 202, 200, 203, 200, 204, 200, 205, 200, 206, 200, 207, 200, 208, 200, 209, 200, 217, 200, 218, 200, 219, 200, 220, 200, 222, 200, 223, 200],
	#[65535, 197, 23, 197, 4, 197, 6, 197, 14, 197, 7, 197, 1, 197, 22, 197, 15, 197, 2, 197, 11, 197, 28, 197, 21, 197, 0, 197, 16, 197, 9, 197, 17, 197, 25, 197, 62, 197, 75, 197, 29, 197, 31, 197, 18, 197, 60, 197, 42, 197, 33, 197, 12, 197, 34, 197, 19, 197, 59, 197, 26, 197, 40, 197, 20, 197, 10, 197, 24, 197, 43, 197, 86, 197, 13, 197, 27, 197, 84, 197, 37, 197, 36, 197, 77, 197, 79, 197, 57, 197, 58, 197, 41, 197, 85, 197, 87, 197, 80, 197, 83, 197, 32, 197, 102, 197, 63, 197, 90, 197, 93, 197, 72, 197, 38, 197, 81, 197, 89, 197, 92, 197, 76, 197, 88, 197, 101, 197, 103, 197, 99, 197, 94, 197, 82, 197, 115, 197, 74, 197, 111, 197, 98, 197, 97, 197, 105, 197, 119, 197, 110, 197, 116, 197, 96, 197, 112, 197, 107, 197, 104, 197, 114, 197, 109, 197, 118, 197, 106, 197, 108, 197, 113, 197, 100, 197, 130, 197, 121, 197, 120, 197, 122, 197, 123, 197, 124, 197, 126, 197, 127, 197, 128, 197, 129, 197, 164, 197, 133, 197, 134, 197, 135, 197, 136, 197, 137, 197, 138, 197, 189, 197, 160, 197, 159, 197, 206, 197, 161, 197, 162, 197, 163, 197, 186, 197, 165, 197, 166, 197, 167, 197, 185, 197, 205, 197, 187, 197, 188, 197, 190, 197, 192, 197, 193, 197, 194, 197, 195, 197, 196, 197, 197, 197, 198, 197, 199, 197, 200, 197, 201, 197, 202, 197, 203, 197, 204, 197, 207, 197, 208, 197, 209, 197, 217, 197, 218, 197, 219, 197, 220, 197, 222, 197, 223, 197],
	#[65535, 26, 3, 26, 7, 26, 11, 26, 31, 26, 33, 26, 35, 26, 36, 26, 44, 26, 45, 26, 46, 26, 47, 26, 48, 26, 49, 26, 50, 26, 51, 26, 52, 26, 53, 26, 54, 26, 55, 26, 56, 26, 57, 26, 58, 26, 59, 26, 60, 26, 63, 26, 64, 26, 65, 26, 66, 26, 67, 26, 68, 26, 69, 26, 70, 26, 71, 26, 72, 26, 73, 26, 74, 26, 75, 26, 76, 26, 77, 26, 78, 26, 79, 26, 80, 26, 92, 26, 94, 26, 115, 26],
	#[6, -906],
	#[#"eoi", #"accept"],
	#[65535, 31, 3, 31, 7, 31, 11, 31, 31, 31, 33, 31, 35, 31, 36, 31, 44, 31, 45, 31, 46, 31, 47, 31, 48, 31, 49, 31, 50, 31, 51, 31, 52, 31, 53, 31, 54, 31, 55, 31, 56, 31, 57, 31, 58, 31, 59, 31, 60, 31, 63, 31, 64, 31, 65, 31, 66, 31, 67, 31, 68, 31, 69, 31, 70, 31, 71, 31, 72, 31, 73, 31, 74, 31, 75, 31, 76, 31, 77, 31, 78, 31, 79, 31, 80, 31, 92, 31, 94, 31, 115, 31],
	#[74, -43, 75, -44, 76, -42, 77, 113, 79, 113, 80, 113],
	#[2, -898],
	#[65535, 23, 3, 23, 11, 23, 46, 23, 7, 23, 33, 23, 71, 23, 47, 23, 36, 23, 35, 23, 50, 23, 57, 23, 31, 23, 59, 23, 63, 23, 49, 23, 58, 23, 77, 23, 92, 23, 76, 23, 60, 23, 44, 23, 45, 23, 70, 23, 48, 23, 75, 23, 79, 23, 51, 23, 52, 23, 53, 23, 54, 23, 55, 23, 56, 23, 64, 23, 65, 23, 66, 23, 67, 23, 68, 23, 69, 23, 80, 23, 94, 23, 72, 23, 73, 23, 74, 23, 78, 23, 115, 23],
	#[74, -43, 75, -44, 76, -42, 77, 113, 79, 113, 80, 113],
	#[6, -892],
	#[65535, 28, 55, 28, 7, 28, 47, 28, 3, 28, 79, 28, 49, 28, 33, 28, 46, 28, 31, 28, 48, 28, 63, 28, 45, 28, 54, 28, 11, 28, 71, 28, 58, 28, 53, 28, 56, 28, 50, 28, 52, 28, 36, 28, 75, 28, 70, 28, 76, 28, 60, 28, 44, 28, 67, 28, 64, 28, 69, 28, 78, 28, 59, 28, 68, 28, 35, 28, 57, 28, 51, 28, 77, 28, 72, 28, 66, 28, 65, 28, 73, 28, 74, 28, 80, 28, 92, 28, 94, 28, 115, 28],
	#[65535, 2, 7, 2, 14, 2, 0, 2, 1, 2, 4, 2, 19, 2, 18, 2, 9, 2, 10, 2, 16, 2, 15, 2, 17, 2, 20, 2, 21, 2, 22, 2, 23, 2, 24, 2, 25, 2, 26, 2, 27, 2, 28, 2, 29, 2, 40, 2, 41, 2, 63, 2, 74, 2, 75, 2, 76, 2, 77, 2, 79, 2, 80, 2, 115, 2, 116, 2, 222, 2, 223, 2],
	#[38, -884],
	#[65535, 33, 7, 33, 11, 33, 46, 33, 31, 33, 33, 33, 35, 33, 36, 33, 44, 33, 45, 33, 59, 33, 47, 33, 48, 33, 49, 33, 50, 33, 51, 33, 52, 33, 53, 33, 54, 33, 55, 33, 56, 33, 57, 33, 58, 33, 64, 33, 60, 33, 63, 33, 65, 33, 66, 33, 67, 33, 68, 33, 69, 33, 70, 33, 71, 33, 72, 33, 73, 33, 74, 33, 75, 33, 76, 33, 77, 33, 78, 33, 79, 33, 80, 33, 115, 33],
	#[65535, 21, 3, 21, 7, 21, 11, 21, 31, 21, 33, 21, 35, 21, 36, 21, 44, 21, 45, 21, 46, 21, 47, 21, 48, 21, 49, 21, 50, 21, 51, 21, 52, 21, 53, 21, 54, 21, 55, 21, 56, 21, 57, 21, 58, 21, 59, 21, 60, 21, 63, 21, 64, 21, 65, 21, 66, 21, 67, 21, 68, 21, 69, 21, 70, 21, 71, 21, 72, 21, 73, 21, 74, 21, 75, 21, 76, 21, 77, 21, 78, 21, 79, 21, 80, 21, 92, 21, 94, 21, 115, 21],
	#[65535, 25, 47, 25, 71, 25, 3, 25, 45, 25, 7, 25, 51, 25, 33, 25, 11, 25, 79, 25, 46, 25, 58, 25, 50, 25, 31, 25, 59, 25, 35, 25, 36, 25, 77, 25, 44, 25, 70, 25, 48, 25, 49, 25, 69, 25, 65, 25, 52, 25, 53, 25, 54, 25, 55, 25, 56, 25, 57, 25, 92, 25, 64, 25, 60, 25, 75, 25, 63, 25, 80, 25, 66, 25, 67, 25, 68, 25, 94, 25, 72, 25, 73, 25, 74, 25, 76, 25, 78, 25, 115, 25],
	#[6, -883],
	#[5, -877, 8, -878],
	#[65535, 30, 31, 30, 77, 30, 36, 30, 7, 30, 35, 30, 47, 30, 59, 30, 3, 30, 33, 30, 11, 30, 46, 30, 58, 30, 50, 30, 51, 30, 71, 30, 44, 30, 45, 30, 70, 30, 48, 30, 49, 30, 79, 30, 65, 30, 52, 30, 53, 30, 54, 30, 55, 30, 56, 30, 57, 30, 92, 30, 64, 30, 60, 30, 75, 30, 63, 30, 80, 30, 66, 30, 67, 30, 68, 30, 69, 30, 94, 30, 72, 30, 73, 30, 74, 30, 76, 30, 78, 30, 115, 30],
	#[6, -869],
	#[7, 19, 11, 19, 14, -4, 15, -24, 16, -3, 17, -17, 18, -5, 19, -25, 20, -11, 21, -36, 22, -20, 23, -6, 24, -28, 25, -14, 26, -38, 31, 19, 33, 19, 35, 19, 36, 19, 44, 19, 45, 19, 46, 19, 47, 19, 48, 19, 49, 19, 50, 19, 51, 19, 52, 19, 53, 19, 54, 19, 55, 19, 56, 19, 57, 19, 58, 19, 59, 19, 60, 19, 63, 19, 64, 19, 65, 19, 66, 19, 67, 19, 68, 19, 69, 19, 70, 19, 71, 19, 73, 19, 74, 19, 75, 19, 76, 19, 115, 19],
	#[65535, 35, 47, 35, 31, 35, 7, 35, 11, 35, 35, 35, 46, 35, 54, 35, 33, 35, 53, 35, 52, 35, 36, 35, 51, 35, 50, 35, 56, 35, 49, 35, 48, 35, 44, 35, 45, 35, 55, 35, 57, 35, 58, 35, 59, 35, 60, 35, 63, 35, 64, 35, 65, 35, 66, 35, 67, 35, 68, 35, 69, 35, 70, 35, 71, 35, 72, 35, 73, 35, 74, 35, 75, 35, 76, 35, 77, 35, 78, 35, 79, 35, 80, 35, 115, 35],
	#[65535, 48, 30, 48],
	#[6, -488],
	#[30, -460],
	#[6, -450],
	#[65535, 27, 31, 27, 47, 27, 36, 27, 3, 27, 7, 27, 35, 27, 33, 27, 46, 27, 11, 27, 48, 27, 51, 27, 45, 27, 49, 27, 54, 27, 59, 27, 53, 27, 52, 27, 56, 27, 50, 27, 44, 27, 55, 27, 57, 27, 58, 27, 64, 27, 60, 27, 63, 27, 65, 27, 66, 27, 67, 27, 68, 27, 69, 27, 70, 27, 71, 27, 72, 27, 73, 27, 74, 27, 75, 27, 76, 27, 77, 27, 78, 27, 79, 27, 80, 27, 92, 27, 94, 27, 115, 27],
	#[65535, 198, 4, 198, 0, 198, 1, 198, 2, 198, 26, 198, 6, 198, 7, 198, 9, 198, 10, 198, 11, 198, 12, 198, 13, 198, 14, 198, 15, 198, 16, 198, 17, 198, 18, 198, 19, 198, 20, 198, 21, 198, 22, 198, 23, 198, 24, 198, 25, 198, 27, 198, 28, 198, 29, 198, 31, 198, 32, 198, 33, 198, 34, 198, 36, 198, 37, 198, 38, 198, 40, 198, 41, 198, 42, 198, 43, 198, 57, 198, 58, 198, 59, 198, 60, 198, 62, 198, 63, 198, 72, 198, 74, 198, 75, 198, 76, 198, 77, 198, 79, 198, 80, 198, 81, 198, 82, 198, 83, 198, 84, 198, 85, 198, 86, 198, 87, 198, 88, 198, 89, 198, 90, 198, 92, 198, 93, 198, 94, 198, 96, 198, 97, 198, 98, 198, 99, 198, 100, 198, 101, 198, 102, 198, 103, 198, 104, 198, 105, 198, 106, 198, 107, 198, 108, 198, 109, 198, 110, 198, 111, 198, 112, 198, 113, 198, 114, 198, 115, 198, 116, 198, 118, 198, 119, 198, 120, 198, 121, 198, 122, 198, 123, 198, 124, 198, 126, 198, 127, 198, 128, 198, 129, 198, 130, 198, 133, 198, 134, 198, 135, 198, 136, 198, 137, 198, 138, 198, 159, 198, 160, 198, 161, 198, 162, 198, 163, 198, 164, 198, 165, 198, 166, 198, 167, 198, 185, 198, 186, 198, 187, 198, 188, 198, 189, 198, 190, 198, 192, 198, 193, 198, 194, 198, 195, 198, 196, 198, 197, 198, 198, 198, 199, 198, 200, 198, 201, 198, 202, 198, 203, 198, 204, 198, 205, 198, 206, 198, 207, 198, 208, 198, 209, 198, 217, 198, 218, 198, 219, 198, 220, 198, 222, 198, 223, 198],
	#[65535, 32, 3, 32, 7, 32, 11, 32, 31, 32, 33, 32, 35, 32, 36, 32, 44, 32, 45, 32, 46, 32, 47, 32, 48, 32, 49, 32, 50, 32, 51, 32, 52, 32, 53, 32, 54, 32, 55, 32, 56, 32, 57, 32, 58, 32, 59, 32, 60, 32, 63, 32, 64, 32, 65, 32, 66, 32, 67, 32, 68, 32, 69, 32, 70, 32, 71, 32, 72, 32, 73, 32, 74, 32, 75, 32, 76, 32, 77, 32, 78, 32, 79, 32, 80, 32, 92, 32, 94, 32, 115, 32],
	#[15, -24, 17, -17, 3, 19, 29, -31, 14, -4, 16, -3, 18, -5, 19, -25, 20, -11, 21, -36, 22, -20, 23, -6, 24, -28, 25, -14, 26, -38, 27, -23, 28, -7, 72, 19, 74, 19, 75, 19, 76, 19, 77, 19, 78, 19, 79, 19, 80, 19],
	#[3, 113, 77, 113, 76, -42, 75, -44, 79, 113, 80, 113, 72, 113, 74, -43, 78, 113],
	#[72, 113, 74, -43, 75, -44, 76, -42, 77, 113, 78, 113, 79, 113, 80, 113],
	#[65535, 116, 54, 116, 7, 116, 3, 116, 55, 116, 31, 116, 11, 116, 33, 116, 45, 116, 46, 116, 60, 116, 47, 116, 36, 116, 77, 116, 57, 116, 50, 116, 69, 116, 59, 116, 63, 116, 35, 116, 58, 116, 51, 116, 44, 116, 70, 116, 48, 116, 49, 116, 79, 116, 65, 116, 52, 116, 53, 116, 56, 116, 64, 116, 80, 116, 66, 116, 67, 116, 68, 116, 71, 116, 72, 116, 73, 116, 78, 116, 115, 116],
	#[65535, 114, 47, 114, 31, 114, 3, 114, 7, 114, 11, 114, 35, 114, 46, 114, 54, 114, 33, 114, 53, 114, 52, 114, 36, 114, 51, 114, 50, 114, 56, 114, 49, 114, 48, 114, 44, 114, 45, 114, 55, 114, 57, 114, 58, 114, 59, 114, 60, 114, 63, 114, 64, 114, 65, 114, 66, 114, 67, 114, 68, 114, 69, 114, 70, 114, 71, 114, 72, 114, 73, 114, 77, 114, 78, 114, 79, 114, 80, 114, 115, 114],
	#[65535, 115, 31, 115, 7, 115, 36, 115, 55, 115, 11, 115, 35, 115, 54, 115, 33, 115, 3, 115, 44, 115, 45, 115, 46, 115, 47, 115, 48, 115, 49, 115, 50, 115, 51, 115, 52, 115, 53, 115, 56, 115, 57, 115, 58, 115, 59, 115, 60, 115, 63, 115, 64, 115, 65, 115, 66, 115, 67, 115, 68, 115, 69, 115, 70, 115, 71, 115, 72, 115, 73, 115, 77, 115, 78, 115, 79, 115, 80, 115, 115, 115],
	#[72, 117, 77, -47, 78, 117, 79, 117, 80, 117],
	#[72, 119, 78, -49, 79, 119, 80, 119],
	#[65535, 118, 72, 118, 78, 118, 79, 118, 80, 118],
	#[72, -50, 79, 105, 80, 105],
	#[65535, 120, 21, 120, 29, 120, 15, 120, 20, 120, 4, 120, 23, 120, 61, 120, 0, 120, 10, 120, 31, 120, 59, 120, 63, 120, 18, 120, 7, 120, 14, 120, 1, 120, 90, 120, 28, 120, 26, 120, 116, 120, 22, 120, 16, 120, 17, 120, 79, 120, 91, 120, 58, 120, 57, 120, 9, 120, 40, 120, 25, 120, 74, 120, 41, 120, 77, 120, 75, 120, 117, 120, 27, 120, 80, 120, 19, 120, 76, 120, 72, 120, 175, 120, 24, 120, 115, 120, 173, 120, 170, 120, 168, 120, 169, 120, 174, 120, 183, 120, 172, 120, 177, 120, 171, 120, 182, 120, 181, 120, 180, 120, 179, 120, 178, 120, 176, 120, 184, 120, 222, 120, 223, 120],
	#[42, -96],
	#[79, -53, 80, -54],
	#[31, -59, 7, -37, 36, -65, 11, -66, 70, -72, 33, -60, 69, -68, 68, -64, 63, -63, 64, -73, 71, -55, 35, -67, 67, -61, 66, -58, 65, -71, 73, -70, 115, -10],
	#[65535, 121, 31, 121, 36, 121, 7, 121, 11, 121, 33, 121, 35, 121, 63, 121, 64, 121, 65, 121, 66, 121, 67, 121, 68, 121, 69, 121, 70, 121, 71, 121, 73, 121, 115, 121],
	#[65535, 122, 7, 122, 11, 122, 31, 122, 33, 122, 35, 122, 36, 122, 63, 122, 64, 122, 65, 122, 66, 122, 67, 122, 68, 122, 69, 122, 70, 122, 71, 122, 73, 122, 115, 122],
	#[65535, 104, 23, 104, 14, 104, 21, 104, 12, 104, 22, 104, 11, 104, 4, 104, 13, 104, 1, 104, 37, 104, 31, 104, 57, 104, 9, 104, 86, 104, 7, 104, 20, 104, 29, 104, 17, 104, 10, 104, 15, 104, 2, 104, 27, 104, 77, 104, 41, 104, 84, 104, 16, 104, 32, 104, 59, 104, 38, 104, 33, 104, 87, 104, 72, 104, 18, 104, 60, 104, 63, 104, 43, 104, 34, 104, 58, 104, 25, 104, 75, 104, 24, 104, 76, 104, 79, 104, 19, 104, 0, 104, 42, 104, 81, 104, 82, 104, 98, 104, 26, 104, 28, 104, 40, 104, 83, 104, 85, 104, 74, 104, 80, 104, 88, 104, 89, 104, 90, 104, 92, 104, 94, 104, 96, 104, 97, 104, 99, 104, 100, 104, 101, 104, 102, 104, 103, 104, 104, 104, 105, 104, 106, 104, 107, 104, 108, 104, 109, 104, 110, 104, 111, 104, 112, 104, 113, 104, 114, 104, 115, 104, 116, 104, 118, 104, 119, 104, 120, 104, 121, 104, 122, 104, 123, 104, 124, 104, 126, 104, 127, 104, 128, 104, 129, 104, 130, 104, 133, 104, 134, 104, 135, 104, 136, 104, 137, 104, 138, 104, 160, 104, 161, 104, 162, 104, 163, 104, 164, 104, 166, 104, 167, 104, 185, 104, 186, 104, 187, 104, 188, 104, 189, 104, 190, 104, 192, 104, 193, 104, 194, 104, 195, 104, 196, 104, 197, 104, 201, 104, 202, 104, 217, 104, 218, 104, 219, 104, 220, 104, 222, 104, 223, 104],
	#[65535, 84, 7, 84, 0, 84, 1, 84, 2, 84, 4, 84, 18, 84, 9, 84, 10, 84, 11, 84, 12, 84, 13, 84, 14, 84, 15, 84, 16, 84, 17, 84, 25, 84, 19, 84, 20, 84, 21, 84, 22, 84, 23, 84, 24, 84, 26, 84, 27, 84, 28, 84, 29, 84, 31, 84, 32, 84, 33, 84, 34, 84, 37, 84, 38, 84, 40, 84, 41, 84, 42, 84, 43, 84, 63, 84, 72, 84, 74, 84, 75, 84, 76, 84, 77, 84, 79, 84, 80, 84, 81, 84, 82, 84, 83, 84, 84, 84, 85, 84, 86, 84, 87, 84, 88, 84, 89, 84, 92, 84, 94, 84, 96, 84, 97, 84, 98, 84, 99, 84, 100, 84, 101, 84, 102, 84, 103, 84, 104, 84, 105, 84, 106, 84, 107, 84, 108, 84, 109, 84, 110, 84, 111, 84, 112, 84, 113, 84, 114, 84, 115, 84, 116, 84, 118, 84, 119, 84, 120, 84, 121, 84, 122, 84, 123, 84, 124, 84, 126, 84, 127, 84, 128, 84, 129, 84, 130, 84, 133, 84, 134, 84, 135, 84, 136, 84, 137, 84, 138, 84, 160, 84, 161, 84, 162, 84, 163, 84, 164, 84, 166, 84, 167, 84, 185, 84, 186, 84, 187, 84, 188, 84, 189, 84, 190, 84, 192, 84, 193, 84, 194, 84, 195, 84, 196, 84, 197, 84, 201, 84, 202, 84, 222, 84, 223, 84],
	#[65535, 87, 7, 87, 1, 87, 4, 87, 14, 87, 11, 87, 0, 87, 16, 87, 2, 87, 19, 87, 18, 87, 9, 87, 10, 87, 25, 87, 12, 87, 13, 87, 15, 87, 17, 87, 20, 87, 21, 87, 22, 87, 23, 87, 24, 87, 26, 87, 27, 87, 28, 87, 29, 87, 31, 87, 32, 87, 33, 87, 34, 87, 58, 87, 37, 87, 38, 87, 40, 87, 41, 87, 42, 87, 43, 87, 57, 87, 87, 87, 59, 87, 60, 87, 63, 87, 72, 87, 74, 87, 75, 87, 76, 87, 77, 87, 79, 87, 80, 87, 81, 87, 82, 87, 83, 87, 84, 87, 85, 87, 86, 87, 98, 87, 88, 87, 89, 87, 90, 87, 92, 87, 94, 87, 96, 87, 97, 87, 99, 87, 100, 87, 101, 87, 102, 87, 103, 87, 104, 87, 105, 87, 106, 87, 107, 87, 108, 87, 109, 87, 110, 87, 111, 87, 112, 87, 113, 87, 114, 87, 115, 87, 116, 87, 118, 87, 119, 87, 120, 87, 121, 87, 122, 87, 123, 87, 124, 87, 126, 87, 127, 87, 128, 87, 129, 87, 130, 87, 133, 87, 134, 87, 135, 87, 136, 87, 137, 87, 138, 87, 160, 87, 161, 87, 162, 87, 163, 87, 164, 87, 166, 87, 167, 87, 185, 87, 186, 87, 187, 87, 188, 87, 189, 87, 190, 87, 192, 87, 193, 87, 194, 87, 195, 87, 196, 87, 197, 87, 201, 87, 202, 87, 217, 87, 218, 87, 219, 87, 220, 87, 222, 87, 223, 87],
	#[65535, 99, 31, 99, 1, 99, 23, 99, 10, 99, 2, 99, 11, 99, 7, 99, 37, 99, 9, 99, 15, 99, 27, 99, 14, 99, 12, 99, 16, 99, 33, 99, 0, 99, 58, 99, 4, 99, 63, 99, 19, 99, 18, 99, 24, 99, 29, 99, 25, 99, 34, 99, 13, 99, 26, 99, 17, 99, 20, 99, 21, 99, 22, 99, 32, 99, 28, 99, 38, 99, 40, 99, 41, 99, 42, 99, 43, 99, 57, 99, 87, 99, 59, 99, 60, 99, 74, 99, 72, 99, 81, 99, 75, 99, 76, 99, 77, 99, 79, 99, 80, 99, 82, 99, 83, 99, 84, 99, 85, 99, 86, 99, 98, 99, 88, 99, 89, 99, 90, 99, 92, 99, 94, 99, 96, 99, 97, 99, 99, 99, 100, 99, 101, 99, 102, 99, 103, 99, 104, 99, 105, 99, 106, 99, 107, 99, 108, 99, 109, 99, 110, 99, 111, 99, 112, 99, 113, 99, 114, 99, 115, 99, 116, 99, 118, 99, 119, 99, 120, 99, 121, 99, 122, 99, 123, 99, 124, 99, 126, 99, 127, 99, 128, 99, 129, 99, 130, 99, 133, 99, 134, 99, 135, 99, 136, 99, 137, 99, 138, 99, 160, 99, 161, 99, 162, 99, 163, 99, 164, 99, 166, 99, 167, 99, 185, 99, 186, 99, 187, 99, 188, 99, 189, 99, 190, 99, 192, 99, 193, 99, 194, 99, 195, 99, 196, 99, 197, 99, 201, 99, 202, 99, 217, 99, 218, 99, 219, 99, 220, 99, 222, 99, 223, 99],
	#[7, -37, 11, -66, 31, -59, 32, -111, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[31, -101, 38, -100],
	#[65535, 100, 31, 100, 23, 100, 10, 100, 2, 100, 7, 100, 37, 100, 9, 100, 11, 100, 15, 100, 1, 100, 27, 100, 14, 100, 12, 100, 16, 100, 33, 100, 0, 100, 58, 100, 4, 100, 63, 100, 19, 100, 18, 100, 24, 100, 29, 100, 25, 100, 34, 100, 13, 100, 26, 100, 17, 100, 20, 100, 21, 100, 22, 100, 32, 100, 28, 100, 38, 100, 40, 100, 41, 100, 42, 100, 43, 100, 57, 100, 87, 100, 59, 100, 60, 100, 74, 100, 72, 100, 81, 100, 75, 100, 76, 100, 77, 100, 79, 100, 80, 100, 82, 100, 83, 100, 84, 100, 85, 100, 86, 100, 98, 100, 88, 100, 89, 100, 90, 100, 92, 100, 94, 100, 96, 100, 97, 100, 99, 100, 100, 100, 101, 100, 102, 100, 103, 100, 104, 100, 105, 100, 106, 100, 107, 100, 108, 100, 109, 100, 110, 100, 111, 100, 112, 100, 113, 100, 114, 100, 115, 100, 116, 100, 118, 100, 119, 100, 120, 100, 121, 100, 122, 100, 123, 100, 124, 100, 126, 100, 127, 100, 128, 100, 129, 100, 130, 100, 133, 100, 134, 100, 135, 100, 136, 100, 137, 100, 138, 100, 160, 100, 161, 100, 162, 100, 163, 100, 164, 100, 166, 100, 167, 100, 185, 100, 186, 100, 187, 100, 188, 100, 189, 100, 190, 100, 192, 100, 193, 100, 194, 100, 195, 100, 196, 100, 197, 100, 201, 100, 202, 100, 217, 100, 218, 100, 219, 100, 220, 100, 222, 100, 223, 100],
	#[65535, 83, 0, 83, 1, 83, 2, 83, 4, 83, 7, 83, 9, 83, 10, 83, 11, 83, 12, 83, 13, 83, 14, 83, 15, 83, 16, 83, 17, 83, 18, 83, 19, 83, 20, 83, 21, 83, 22, 83, 23, 83, 24, 83, 25, 83, 26, 83, 27, 83, 28, 83, 29, 83, 31, 83, 32, 83, 33, 83, 34, 83, 37, 83, 38, 83, 40, 83, 41, 83, 42, 83, 43, 83, 63, 83, 72, 83, 74, 83, 75, 83, 76, 83, 77, 83, 79, 83, 80, 83, 81, 83, 82, 83, 83, 83, 84, 83, 85, 83, 86, 83, 87, 83, 88, 83, 89, 83, 92, 83, 94, 83, 96, 83, 97, 83, 98, 83, 99, 83, 100, 83, 101, 83, 102, 83, 103, 83, 104, 83, 105, 83, 106, 83, 107, 83, 108, 83, 109, 83, 110, 83, 111, 83, 112, 83, 113, 83, 114, 83, 115, 83, 116, 83, 118, 83, 119, 83, 120, 83, 121, 83, 122, 83, 123, 83, 124, 83, 126, 83, 127, 83, 128, 83, 129, 83, 130, 83, 133, 83, 134, 83, 135, 83, 136, 83, 137, 83, 138, 83, 160, 83, 161, 83, 162, 83, 163, 83, 164, 83, 166, 83, 167, 83, 185, 83, 186, 83, 187, 83, 188, 83, 189, 83, 190, 83, 192, 83, 193, 83, 194, 83, 195, 83, 196, 83, 197, 83, 201, 83, 202, 83, 222, 83, 223, 83],
	#[65535, 89, 22, 89, 23, 89, 13, 89, 1, 89, 92, 89, 31, 89, 14, 89, 17, 89, 7, 89, 20, 89, 29, 89, 19, 89, 10, 89, 15, 89, 2, 89, 11, 89, 28, 89, 37, 89, 18, 89, 21, 89, 0, 89, 16, 89, 32, 89, 34, 89, 38, 89, 4, 89, 86, 89, 110, 89, 9, 89, 27, 89, 63, 89, 12, 89, 94, 89, 40, 89, 59, 89, 84, 89, 25, 89, 75, 89, 74, 89, 72, 89, 24, 89, 33, 89, 79, 89, 26, 89, 111, 89, 42, 89, 87, 89, 58, 89, 43, 89, 82, 89, 41, 89, 76, 89, 60, 89, 102, 89, 90, 89, 57, 89, 77, 89, 99, 89, 88, 89, 127, 89, 83, 89, 98, 89, 85, 89, 81, 89, 80, 89, 134, 89, 89, 89, 109, 89, 113, 89, 107, 89, 106, 89, 96, 89, 97, 89, 116, 89, 100, 89, 101, 89, 115, 89, 103, 89, 104, 89, 105, 89, 135, 89, 112, 89, 108, 89, 114, 89, 118, 89, 119, 89, 120, 89, 121, 89, 122, 89, 123, 89, 124, 89, 126, 89, 138, 89, 128, 89, 129, 89, 130, 89, 136, 89, 133, 89, 137, 89, 160, 89, 161, 89, 162, 89, 163, 89, 164, 89, 166, 89, 167, 89, 185, 89, 186, 89, 187, 89, 188, 89, 189, 89, 190, 89, 192, 89, 193, 89, 194, 89, 195, 89, 196, 89, 197, 89, 201, 89, 202, 89, 217, 89, 218, 89, 219, 89, 220, 89, 222, 89, 223, 89],
	#[65535, 101, 15, 101, 0, 101, 1, 101, 2, 101, 4, 101, 7, 101, 9, 101, 10, 101, 11, 101, 12, 101, 13, 101, 14, 101, 26, 101, 16, 101, 17, 101, 18, 101, 19, 101, 20, 101, 21, 101, 22, 101, 23, 101, 24, 101, 25, 101, 27, 101, 28, 101, 29, 101, 31, 101, 32, 101, 33, 101, 34, 101, 37, 101, 38, 101, 40, 101, 41, 101, 42, 101, 43, 101, 57, 101, 58, 101, 59, 101, 60, 101, 63, 101, 72, 101, 74, 101, 75, 101, 76, 101, 77, 101, 79, 101, 80, 101, 81, 101, 82, 101, 83, 101, 84, 101, 85, 101, 86, 101, 87, 101, 88, 101, 89, 101, 90, 101, 92, 101, 94, 101, 96, 101, 97, 101, 98, 101, 99, 101, 100, 101, 101, 101, 102, 101, 103, 101, 104, 101, 105, 101, 106, 101, 107, 101, 108, 101, 109, 101, 110, 101, 111, 101, 112, 101, 113, 101, 114, 101, 115, 101, 116, 101, 118, 101, 119, 101, 120, 101, 121, 101, 122, 101, 123, 101, 124, 101, 126, 101, 127, 101, 128, 101, 129, 101, 130, 101, 133, 101, 134, 101, 135, 101, 136, 101, 137, 101, 138, 101, 160, 101, 161, 101, 162, 101, 163, 101, 164, 101, 166, 101, 167, 101, 185, 101, 186, 101, 187, 101, 188, 101, 189, 101, 190, 101, 192, 101, 193, 101, 194, 101, 195, 101, 196, 101, 197, 101, 201, 101, 202, 101, 217, 101, 218, 101, 219, 101, 220, 101, 222, 101, 223, 101],
	#[65535, 86, 13, 86, 7, 86, 29, 86, 15, 86, 4, 86, 0, 86, 12, 86, 10, 86, 17, 86, 31, 86, 20, 86, 18, 86, 11, 86, 14, 86, 37, 86, 1, 86, 2, 86, 21, 86, 25, 86, 19, 86, 24, 86, 22, 86, 74, 86, 9, 86, 94, 86, 79, 86, 33, 86, 23, 86, 58, 86, 26, 86, 92, 86, 40, 86, 60, 86, 87, 86, 28, 86, 77, 86, 75, 86, 34, 86, 63, 86, 86, 86, 41, 86, 16, 86, 57, 86, 42, 86, 59, 86, 76, 86, 103, 86, 89, 86, 102, 86, 84, 86, 83, 86, 32, 86, 38, 86, 110, 86, 80, 86, 81, 86, 43, 86, 27, 86, 90, 86, 98, 86, 72, 86, 85, 86, 88, 86, 101, 86, 99, 86, 114, 86, 82, 86, 115, 86, 106, 86, 190, 86, 111, 86, 104, 86, 97, 86, 105, 86, 119, 86, 116, 86, 96, 86, 112, 86, 107, 86, 127, 86, 109, 86, 118, 86, 108, 86, 113, 86, 100, 86, 130, 86, 121, 86, 120, 86, 122, 86, 123, 86, 124, 86, 126, 86, 138, 86, 128, 86, 129, 86, 164, 86, 133, 86, 134, 86, 135, 86, 136, 86, 137, 86, 167, 86, 189, 86, 161, 86, 185, 86, 202, 86, 160, 86, 194, 86, 162, 86, 163, 86, 186, 86, 166, 86, 192, 86, 188, 86, 187, 86, 193, 86, 201, 86, 195, 86, 196, 86, 197, 86, 217, 86, 218, 86, 219, 86, 220, 86, 222, 86, 223, 86],
	#[38, -106],
	#[65535, 85, 0, 85, 1, 85, 2, 85, 4, 85, 7, 85, 9, 85, 10, 85, 11, 85, 12, 85, 13, 85, 14, 85, 15, 85, 16, 85, 17, 85, 18, 85, 19, 85, 20, 85, 21, 85, 22, 85, 23, 85, 24, 85, 25, 85, 26, 85, 27, 85, 28, 85, 29, 85, 31, 85, 32, 85, 33, 85, 34, 85, 37, 85, 38, 85, 40, 85, 41, 85, 42, 85, 43, 85, 57, 85, 58, 85, 59, 85, 60, 85, 63, 85, 72, 85, 74, 85, 75, 85, 76, 85, 77, 85, 79, 85, 80, 85, 81, 85, 82, 85, 83, 85, 84, 85, 85, 85, 86, 85, 87, 85, 88, 85, 89, 85, 90, 85, 92, 85, 94, 85, 96, 85, 97, 85, 98, 85, 99, 85, 100, 85, 101, 85, 102, 85, 103, 85, 104, 85, 105, 85, 106, 85, 107, 85, 108, 85, 109, 85, 110, 85, 111, 85, 112, 85, 113, 85, 114, 85, 115, 85, 116, 85, 118, 85, 119, 85, 120, 85, 121, 85, 122, 85, 123, 85, 124, 85, 126, 85, 127, 85, 128, 85, 129, 85, 130, 85, 133, 85, 134, 85, 135, 85, 136, 85, 137, 85, 138, 85, 160, 85, 161, 85, 162, 85, 163, 85, 164, 85, 166, 85, 167, 85, 185, 85, 186, 85, 187, 85, 188, 85, 189, 85, 190, 85, 192, 85, 193, 85, 194, 85, 195, 85, 196, 85, 197, 85, 201, 85, 202, 85, 217, 85, 218, 85, 219, 85, 220, 85, 222, 85, 223, 85],
	#[65535, 102, 7, 102, 22, 102, 15, 102, 2, 102, 21, 102, 23, 102, 1, 102, 14, 102, 38, 102, 0, 102, 86, 102, 29, 102, 31, 102, 9, 102, 13, 102, 63, 102, 12, 102, 37, 102, 16, 102, 25, 102, 11, 102, 20, 102, 4, 102, 10, 102, 17, 102, 76, 102, 79, 102, 26, 102, 110, 102, 18, 102, 32, 102, 33, 102, 84, 102, 40, 102, 27, 102, 94, 102, 58, 102, 43, 102, 82, 102, 19, 102, 92, 102, 24, 102, 102, 102, 28, 102, 41, 102, 57, 102, 59, 102, 99, 102, 72, 102, 34, 102, 111, 102, 60, 102, 88, 102, 87, 102, 42, 102, 90, 102, 75, 102, 77, 102, 127, 102, 83, 102, 98, 102, 85, 102, 74, 102, 80, 102, 81, 102, 134, 102, 89, 102, 109, 102, 113, 102, 107, 102, 106, 102, 96, 102, 97, 102, 116, 102, 100, 102, 101, 102, 115, 102, 103, 102, 104, 102, 105, 102, 135, 102, 112, 102, 108, 102, 114, 102, 118, 102, 119, 102, 120, 102, 121, 102, 122, 102, 123, 102, 124, 102, 126, 102, 138, 102, 128, 102, 129, 102, 130, 102, 136, 102, 133, 102, 137, 102, 160, 102, 161, 102, 162, 102, 163, 102, 164, 102, 166, 102, 167, 102, 185, 102, 186, 102, 187, 102, 188, 102, 189, 102, 190, 102, 192, 102, 193, 102, 194, 102, 195, 102, 196, 102, 197, 102, 201, 102, 202, 102, 217, 102, 218, 102, 219, 102, 220, 102, 222, 102, 223, 102],
	#[7, 143, 15, 143, 0, 143, 1, 143, 20, 143, 4, 143, 18, 143, 9, 143, 10, 143, 13, -156, 14, 143, 26, 143, 16, 143, 17, 143, 25, 143, 19, 143, 21, 143, 22, 143, 23, 143, 24, 143, 27, 143, 28, 143, 29, 143, 37, 105, 40, 143, 41, 143, 42, -93, 63, 143, 72, -50, 74, 143, 75, 143, 76, 143, 77, 143, 79, 143, 80, 143, 115, 143, 116, 143, 222, 143, 223, 143],
	#[42, -75],
	#[65535, 98, 31, 98, 14, 98, 7, 98, 22, 98, 1, 98, 37, 98, 23, 98, 10, 98, 2, 98, 18, 98, 13, 98, 63, 98, 12, 98, 9, 98, 15, 98, 4, 98, 27, 98, 17, 98, 16, 98, 19, 98, 11, 98, 0, 98, 20, 98, 26, 98, 29, 98, 25, 98, 21, 98, 34, 98, 24, 98, 58, 98, 33, 98, 32, 98, 28, 98, 38, 98, 40, 98, 41, 98, 42, 98, 43, 98, 57, 98, 87, 98, 59, 98, 60, 98, 74, 98, 72, 98, 81, 98, 75, 98, 76, 98, 77, 98, 79, 98, 80, 98, 82, 98, 83, 98, 84, 98, 85, 98, 86, 98, 98, 98, 88, 98, 89, 98, 90, 98, 92, 98, 94, 98, 96, 98, 97, 98, 99, 98, 100, 98, 101, 98, 102, 98, 103, 98, 104, 98, 105, 98, 106, 98, 107, 98, 108, 98, 109, 98, 110, 98, 111, 98, 112, 98, 113, 98, 114, 98, 115, 98, 116, 98, 118, 98, 119, 98, 120, 98, 121, 98, 122, 98, 123, 98, 124, 98, 126, 98, 127, 98, 128, 98, 129, 98, 130, 98, 133, 98, 134, 98, 135, 98, 136, 98, 137, 98, 138, 98, 160, 98, 161, 98, 162, 98, 163, 98, 164, 98, 166, 98, 167, 98, 185, 98, 186, 98, 187, 98, 188, 98, 189, 98, 190, 98, 192, 98, 193, 98, 194, 98, 195, 98, 196, 98, 197, 98, 201, 98, 202, 98, 217, 98, 218, 98, 219, 98, 220, 98, 222, 98, 223, 98],
	#[65535, 103, 0, 103, 1, 103, 2, 103, 4, 103, 7, 103, 9, 103, 10, 103, 11, 103, 12, 103, 13, 103, 14, 103, 15, 103, 16, 103, 17, 103, 18, 103, 19, 103, 20, 103, 21, 103, 22, 103, 23, 103, 24, 103, 25, 103, 26, 103, 27, 103, 28, 103, 29, 103, 31, 103, 32, 103, 33, 103, 34, 103, 37, 103, 38, 103, 40, 103, 41, 103, 42, 103, 43, 103, 57, 103, 58, 103, 59, 103, 60, 103, 63, 103, 72, 103, 74, 103, 75, 103, 76, 103, 77, 103, 79, 103, 80, 103, 81, 103, 82, 103, 83, 103, 84, 103, 85, 103, 86, 103, 87, 103, 88, 103, 89, 103, 90, 103, 92, 103, 94, 103, 96, 103, 97, 103, 98, 103, 99, 103, 100, 103, 101, 103, 102, 103, 103, 103, 104, 103, 105, 103, 106, 103, 107, 103, 108, 103, 109, 103, 110, 103, 111, 103, 112, 103, 113, 103, 114, 103, 115, 103, 116, 103, 118, 103, 119, 103, 120, 103, 121, 103, 122, 103, 123, 103, 124, 103, 126, 103, 127, 103, 128, 103, 129, 103, 130, 103, 133, 103, 134, 103, 135, 103, 136, 103, 137, 103, 138, 103, 160, 103, 161, 103, 162, 103, 163, 103, 164, 103, 166, 103, 167, 103, 185, 103, 186, 103, 187, 103, 188, 103, 189, 103, 190, 103, 192, 103, 193, 103, 194, 103, 195, 103, 196, 103, 197, 103, 201, 103, 202, 103, 217, 103, 218, 103, 219, 103, 220, 103, 222, 103, 223, 103],
	#[65535, 97, 14, 97, 7, 97, 2, 97, 11, 97, 0, 97, 1, 97, 9, 97, 4, 97, 19, 97, 18, 97, 10, 97, 16, 97, 12, 97, 13, 97, 15, 97, 17, 97, 20, 97, 21, 97, 22, 97, 23, 97, 24, 97, 25, 97, 26, 97, 27, 97, 28, 97, 29, 97, 31, 97, 32, 97, 33, 97, 34, 97, 58, 97, 37, 97, 38, 97, 40, 97, 41, 97, 42, 97, 43, 97, 57, 97, 87, 97, 59, 97, 60, 97, 63, 97, 72, 97, 74, 97, 75, 97, 76, 97, 77, 97, 79, 97, 80, 97, 81, 97, 82, 97, 83, 97, 84, 97, 85, 97, 86, 97, 98, 97, 88, 97, 89, 97, 90, 97, 92, 97, 94, 97, 96, 97, 97, 97, 99, 97, 100, 97, 101, 97, 102, 97, 103, 97, 104, 97, 105, 97, 106, 97, 107, 97, 108, 97, 109, 97, 110, 97, 111, 97, 112, 97, 113, 97, 114, 97, 115, 97, 116, 97, 118, 97, 119, 97, 120, 97, 121, 97, 122, 97, 123, 97, 124, 97, 126, 97, 127, 97, 128, 97, 129, 97, 130, 97, 133, 97, 134, 97, 135, 97, 136, 97, 137, 97, 138, 97, 160, 97, 161, 97, 162, 97, 163, 97, 164, 97, 166, 97, 167, 97, 185, 97, 186, 97, 187, 97, 188, 97, 189, 97, 190, 97, 192, 97, 193, 97, 194, 97, 195, 97, 196, 97, 197, 97, 201, 97, 202, 97, 217, 97, 218, 97, 219, 97, 220, 97, 222, 97, 223, 97],
	#[65535, 90, 2, 90, 0, 90, 1, 90, 9, 90, 4, 90, 7, 90, 10, 90, 11, 90, 12, 90, 13, 90, 14, 90, 15, 90, 16, 90, 17, 90, 18, 90, 19, 90, 20, 90, 21, 90, 22, 90, 23, 90, 24, 90, 25, 90, 26, 90, 27, 90, 28, 90, 29, 90, 31, 90, 32, 90, 33, 90, 34, 90, 37, 90, 38, 90, 40, 90, 41, 90, 42, 90, 43, 90, 57, 90, 58, 90, 59, 90, 60, 90, 63, 90, 72, 90, 74, 90, 75, 90, 76, 90, 77, 90, 79, 90, 80, 90, 81, 90, 82, 90, 83, 90, 84, 90, 85, 90, 86, 90, 87, 90, 88, 90, 89, 90, 90, 90, 92, 90, 94, 90, 96, 90, 97, 90, 98, 90, 99, 90, 100, 90, 101, 90, 102, 90, 103, 90, 104, 90, 105, 90, 106, 90, 107, 90, 108, 90, 109, 90, 110, 90, 111, 90, 112, 90, 113, 90, 114, 90, 115, 90, 116, 90, 118, 90, 119, 90, 120, 90, 121, 90, 122, 90, 123, 90, 124, 90, 126, 90, 127, 90, 128, 90, 129, 90, 130, 90, 133, 90, 134, 90, 135, 90, 136, 90, 137, 90, 138, 90, 160, 90, 161, 90, 162, 90, 163, 90, 164, 90, 166, 90, 167, 90, 185, 90, 186, 90, 187, 90, 188, 90, 189, 90, 190, 90, 192, 90, 193, 90, 194, 90, 195, 90, 196, 90, 197, 90, 201, 90, 202, 90, 217, 90, 218, 90, 219, 90, 220, 90, 222, 90, 223, 90],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 43, 155, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 95, -80, 115, -10],
	#[7, 110, 2, 110, 31, 110, 10, 110, 11, 110, 13, 110, 43, 110, 42, 83, 33, 110, 37, 83, 38, 110, 60, 110, 58, -149, 57, -152, 87, 110, 59, -150, 82, 110, 63, 110, 84, 110, 90, 110, 83, 110, 94, 110, 72, 83, 103, 110, 92, 110, 98, 110, 105, 110, 102, 110, 81, 110, 111, 110, 100, 110, 106, 110, 85, 110, 86, 110, 110, 110, 88, 110, 89, 110, 119, 110, 108, 110, 114, 110, 107, 110, 118, 110, 96, 110, 97, 110, 127, 110, 99, 110, 121, 110, 101, 110, 115, 110, 126, 110, 104, 110, 120, 110, 113, 110, 124, 110, 130, 110, 109, 110, 123, 110, 122, 110, 112, 110, 128, 110, 133, 110, 129, 110, 116, 110, 134, 110, 135, 110, 136, 110, 137, 110, 138, 110, 217, 110, 218, 110, 219, 110, 220, 110, 223, 110],
	#[13, -146, 43, 156],
	#[2, 109, 7, 109, 10, 109, 11, 109, 13, 109, 31, 109, 33, 109, 37, 84, 38, 109, 42, 84, 43, 109, 60, 109, 63, 109, 72, 84, 81, 109, 82, 109, 83, 109, 84, 109, 85, 109, 86, 109, 87, 109, 88, 109, 89, 109, 90, 109, 92, 109, 94, 109, 96, 109, 97, 109, 98, 109, 99, 109, 100, 109, 101, 109, 102, 109, 103, 109, 104, 109, 105, 109, 106, 109, 107, 109, 108, 109, 109, 109, 110, 109, 111, 109, 112, 109, 113, 109, 114, 109, 115, 109, 116, 109, 118, 109, 119, 109, 120, 109, 121, 109, 122, 109, 123, 109, 124, 109, 126, 109, 127, 109, 128, 109, 129, 109, 130, 109, 133, 109, 134, 109, 135, 109, 136, 109, 137, 109, 138, 109, 217, 109, 218, 109, 219, 109, 220, 109, 223, 109],
	#[43, -153],
	#[65535, 158, 43, 158],
	#[42, -93, 37, 105, 72, -50],
	#[13, 159, 43, 159, 60, -88, 90, -86, 105, -91, 106, -84, 217, -90, 218, -85, 219, -87, 220, -89],
	#[65535, 112, 2, 112, 7, 112, 10, 112, 11, 112, 13, 112, 31, 112, 33, 112, 38, 112, 43, 112, 60, 112, 63, 112, 81, 112, 82, 112, 83, 112, 84, 112, 85, 112, 86, 112, 87, 112, 88, 112, 89, 112, 90, 112, 92, 112, 94, 112, 96, 112, 97, 112, 98, 112, 99, 112, 100, 112, 101, 112, 102, 112, 103, 112, 104, 112, 105, 112, 106, 112, 107, 112, 108, 112, 109, 112, 110, 112, 111, 112, 112, 112, 113, 112, 114, 112, 115, 112, 116, 112, 118, 112, 119, 112, 120, 112, 121, 112, 122, 112, 123, 112, 124, 112, 126, 112, 127, 112, 128, 112, 129, 112, 130, 112, 133, 112, 134, 112, 135, 112, 136, 112, 137, 112, 138, 112, 217, 112, 218, 112, 219, 112, 220, 112, 223, 112],
	#[65535, 406, 13, 406, 7, 406, 60, 406, 58, 406, 43, 406, 115, 406, 57, 406, 90, 406, 106, 406, 59, 406, 105, 406, 217, 406, 218, 406, 219, 406, 220, 406],
	#[65535, 409, 7, 409, 2, 409, 10, 409, 11, 409, 13, 409, 43, 409, 31, 409, 33, 409, 58, 409, 38, 409, 57, 409, 87, 409, 59, 409, 60, 409, 63, 409, 83, 409, 82, 409, 81, 409, 84, 409, 85, 409, 86, 409, 98, 409, 88, 409, 89, 409, 90, 409, 92, 409, 94, 409, 96, 409, 97, 409, 99, 409, 100, 409, 101, 409, 102, 409, 103, 409, 104, 409, 105, 409, 106, 409, 107, 409, 108, 409, 109, 409, 110, 409, 111, 409, 112, 409, 113, 409, 114, 409, 115, 409, 116, 409, 118, 409, 119, 409, 120, 409, 121, 409, 122, 409, 123, 409, 124, 409, 126, 409, 127, 409, 128, 409, 129, 409, 130, 409, 133, 409, 134, 409, 135, 409, 136, 409, 137, 409, 138, 409, 217, 409, 218, 409, 219, 409, 220, 409, 223, 409],
	#[38, -92],
	#[65535, 410, 31, 410, 13, 410, 2, 410, 38, 410, 33, 410, 94, 410, 63, 410, 7, 410, 57, 410, 10, 410, 11, 410, 60, 410, 85, 410, 87, 410, 43, 410, 83, 410, 59, 410, 97, 410, 90, 410, 58, 410, 82, 410, 81, 410, 86, 410, 110, 410, 107, 410, 88, 410, 101, 410, 103, 410, 89, 410, 116, 410, 118, 410, 84, 410, 96, 410, 109, 410, 111, 410, 98, 410, 102, 410, 92, 410, 114, 410, 99, 410, 119, 410, 112, 410, 105, 410, 106, 410, 104, 410, 115, 410, 113, 410, 100, 410, 108, 410, 121, 410, 120, 410, 122, 410, 123, 410, 124, 410, 126, 410, 127, 410, 128, 410, 129, 410, 130, 410, 133, 410, 134, 410, 135, 410, 136, 410, 137, 410, 138, 410, 217, 410, 218, 410, 219, 410, 220, 410, 223, 410],
	#[65535, 408, 2, 408, 7, 408, 10, 408, 11, 408, 13, 408, 31, 408, 33, 408, 38, 408, 43, 408, 57, 408, 58, 408, 59, 408, 60, 408, 63, 408, 81, 408, 82, 408, 83, 408, 84, 408, 85, 408, 86, 408, 87, 408, 88, 408, 89, 408, 90, 408, 92, 408, 94, 408, 96, 408, 97, 408, 98, 408, 99, 408, 100, 408, 101, 408, 102, 408, 103, 408, 104, 408, 105, 408, 106, 408, 107, 408, 108, 408, 109, 408, 110, 408, 111, 408, 112, 408, 113, 408, 114, 408, 115, 408, 116, 408, 118, 408, 119, 408, 120, 408, 121, 408, 122, 408, 123, 408, 124, 408, 126, 408, 127, 408, 128, 408, 129, 408, 130, 408, 133, 408, 134, 408, 135, 408, 136, 408, 137, 408, 138, 408, 217, 408, 218, 408, 219, 408, 220, 408, 223, 408],
	#[65535, 411, 63, 411, 7, 411, 10, 411, 43, 411, 82, 411, 101, 411, 84, 411, 11, 411, 33, 411, 2, 411, 13, 411, 92, 411, 102, 411, 59, 411, 31, 411, 57, 411, 38, 411, 100, 411, 86, 411, 110, 411, 134, 411, 85, 411, 83, 411, 98, 411, 103, 411, 58, 411, 135, 411, 106, 411, 107, 411, 126, 411, 111, 411, 90, 411, 89, 411, 87, 411, 115, 411, 119, 411, 60, 411, 81, 411, 97, 411, 94, 411, 118, 411, 121, 411, 99, 411, 88, 411, 104, 411, 113, 411, 108, 411, 96, 411, 112, 411, 128, 411, 105, 411, 114, 411, 123, 411, 127, 411, 109, 411, 120, 411, 116, 411, 122, 411, 130, 411, 133, 411, 129, 411, 124, 411, 138, 411, 137, 411, 136, 411, 217, 411, 218, 411, 219, 411, 220, 411, 223, 411],
	#[65535, 407, 38, 407, 2, 407, 60, 407, 13, 407, 84, 407, 31, 407, 7, 407, 10, 407, 11, 407, 103, 407, 33, 407, 59, 407, 57, 407, 87, 407, 43, 407, 83, 407, 102, 407, 63, 407, 58, 407, 82, 407, 81, 407, 86, 407, 105, 407, 88, 407, 101, 407, 85, 407, 94, 407, 118, 407, 115, 407, 109, 407, 111, 407, 89, 407, 90, 407, 92, 407, 114, 407, 99, 407, 119, 407, 110, 407, 98, 407, 96, 407, 112, 407, 107, 407, 97, 407, 106, 407, 104, 407, 113, 407, 116, 407, 100, 407, 108, 407, 121, 407, 120, 407, 122, 407, 123, 407, 124, 407, 126, 407, 127, 407, 128, 407, 129, 407, 130, 407, 133, 407, 134, 407, 135, 407, 136, 407, 137, 407, 138, 407, 217, 407, 218, 407, 219, 407, 220, 407, 223, 407],
	#[65535, 405, 7, 405, 13, 405, 43, 405, 57, 405, 58, 405, 59, 405, 60, 405, 90, 405, 105, 405, 106, 405, 115, 405, 217, 405, 218, 405, 219, 405, 220, 405],
	#[65535, 412, 38, 412, 86, 412, 110, 412, 13, 412, 103, 412, 2, 412, 60, 412, 10, 412, 58, 412, 7, 412, 11, 412, 87, 412, 43, 412, 31, 412, 98, 412, 33, 412, 63, 412, 90, 412, 57, 412, 82, 412, 81, 412, 101, 412, 85, 412, 94, 412, 100, 412, 84, 412, 111, 412, 89, 412, 102, 412, 83, 412, 92, 412, 59, 412, 99, 412, 119, 412, 106, 412, 115, 412, 116, 412, 96, 412, 112, 412, 107, 412, 97, 412, 105, 412, 109, 412, 118, 412, 104, 412, 88, 412, 113, 412, 121, 412, 108, 412, 114, 412, 120, 412, 122, 412, 123, 412, 124, 412, 126, 412, 127, 412, 128, 412, 129, 412, 130, 412, 133, 412, 134, 412, 135, 412, 136, 412, 137, 412, 138, 412, 217, 412, 218, 412, 219, 412, 220, 412, 223, 412],
	#[31, -59, 71, -55, 36, -65, 33, -60, 70, -72, 67, -61, 43, 155, 63, -63, 7, -37, 11, -66, 69, -68, 64, -73, 66, -58, 65, -71, 73, -70, 68, -64, 35, -67, 95, -80, 115, -10],
	#[37, -95],
	#[65535, 88, 2, 88, 14, 88, 31, 88, 29, 88, 12, 88, 63, 88, 22, 88, 7, 88, 19, 88, 0, 88, 10, 88, 4, 88, 15, 88, 43, 88, 18, 88, 85, 88, 38, 88, 23, 88, 24, 88, 13, 88, 11, 88, 1, 88, 9, 88, 27, 88, 25, 88, 33, 88, 77, 88, 21, 88, 76, 88, 40, 88, 34, 88, 28, 88, 75, 88, 26, 88, 127, 88, 57, 88, 84, 88, 17, 88, 42, 88, 103, 88, 20, 88, 86, 88, 59, 88, 37, 88, 79, 88, 82, 88, 60, 88, 41, 88, 74, 88, 94, 88, 16, 88, 32, 88, 87, 88, 104, 88, 58, 88, 119, 88, 99, 88, 102, 88, 126, 88, 80, 88, 108, 88, 92, 88, 88, 88, 133, 88, 90, 88, 110, 88, 81, 88, 89, 88, 116, 88, 101, 88, 96, 88, 135, 88, 98, 88, 118, 88, 167, 88, 97, 88, 72, 88, 109, 88, 83, 88, 106, 88, 189, 88, 105, 88, 120, 88, 121, 88, 100, 88, 114, 88, 134, 88, 107, 88, 112, 88, 128, 88, 129, 88, 122, 88, 164, 88, 194, 88, 160, 88, 111, 88, 123, 88, 137, 88, 124, 88, 115, 88, 161, 88, 190, 88, 138, 88, 113, 88, 130, 88, 202, 88, 185, 88, 163, 88, 136, 88, 162, 88, 192, 88, 186, 88, 166, 88, 188, 88, 187, 88, 193, 88, 201, 88, 195, 88, 196, 88, 197, 88, 217, 88, 218, 88, 219, 88, 220, 88, 222, 88, 223, 88],
	#[38, -97],
	#[43, -98],
	#[65535, 106, 79, 106, 78, 106, 80, 106, 37, 106],
	#[43, -120],
	#[39, -117],
	#[31, -59, 71, -55, 36, -65, 33, -60, 70, -72, 67, -61, 63, -63, 7, -37, 11, -66, 69, -68, 66, -58, 73, -70, 64, -73, 32, -104, 68, -64, 35, -67, 65, -71, 115, -10],
	#[13, -113, 32, -115],
	#[13, 161, 42, -93, 32, 161, 37, 105, 72, -50],
	#[34, -105],
	#[65535, 96, 31, 96, 2, 96, 7, 96, 37, 96, 15, 96, 1, 96, 10, 96, 14, 96, 23, 96, 11, 96, 0, 96, 4, 96, 63, 96, 18, 96, 9, 96, 29, 96, 25, 96, 12, 96, 13, 96, 27, 96, 26, 96, 16, 96, 17, 96, 19, 96, 20, 96, 21, 96, 22, 96, 34, 96, 24, 96, 33, 96, 32, 96, 28, 96, 38, 96, 40, 96, 41, 96, 42, 96, 43, 96, 57, 96, 58, 96, 59, 96, 60, 96, 74, 96, 72, 96, 81, 96, 75, 96, 76, 96, 77, 96, 79, 96, 80, 96, 82, 96, 83, 96, 84, 96, 85, 96, 86, 96, 87, 96, 88, 96, 89, 96, 90, 96, 92, 96, 94, 96, 96, 96, 97, 96, 98, 96, 99, 96, 100, 96, 101, 96, 102, 96, 103, 96, 104, 96, 105, 96, 106, 96, 107, 96, 108, 96, 109, 96, 110, 96, 111, 96, 112, 96, 113, 96, 114, 96, 115, 96, 116, 96, 118, 96, 119, 96, 120, 96, 121, 96, 122, 96, 123, 96, 124, 96, 126, 96, 127, 96, 128, 96, 129, 96, 130, 96, 133, 96, 134, 96, 135, 96, 136, 96, 137, 96, 138, 96, 160, 96, 161, 96, 162, 96, 163, 96, 164, 96, 166, 96, 167, 96, 185, 96, 186, 96, 187, 96, 188, 96, 189, 96, 190, 96, 192, 96, 193, 96, 194, 96, 195, 96, 196, 96, 197, 96, 201, 96, 202, 96, 217, 96, 218, 96, 219, 96, 220, 96, 222, 96, 223, 96],
	#[39, -107],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[12, -109, 37, 105, 42, -93, 72, -50],
	#[65535, 91, 23, 91, 0, 91, 1, 91, 2, 91, 4, 91, 7, 91, 9, 91, 10, 91, 11, 91, 12, 91, 13, 91, 14, 91, 15, 91, 16, 91, 17, 91, 18, 91, 19, 91, 20, 91, 21, 91, 22, 91, 34, 91, 24, 91, 25, 91, 26, 91, 27, 91, 28, 91, 29, 91, 31, 91, 32, 91, 33, 91, 63, 91, 37, 91, 38, 91, 40, 91, 41, 91, 42, 91, 43, 91, 58, 91, 79, 91, 74, 91, 57, 91, 87, 91, 59, 91, 60, 91, 86, 91, 84, 91, 72, 91, 99, 91, 103, 91, 75, 91, 76, 91, 77, 91, 102, 91, 80, 91, 81, 91, 82, 91, 83, 91, 85, 91, 98, 91, 88, 91, 89, 91, 90, 91, 92, 91, 94, 91, 96, 91, 97, 91, 104, 91, 100, 91, 101, 91, 105, 91, 106, 91, 107, 91, 108, 91, 109, 91, 110, 91, 111, 91, 112, 91, 113, 91, 114, 91, 115, 91, 116, 91, 118, 91, 119, 91, 120, 91, 121, 91, 122, 91, 123, 91, 124, 91, 126, 91, 127, 91, 128, 91, 129, 91, 130, 91, 133, 91, 134, 91, 135, 91, 136, 91, 137, 91, 138, 91, 160, 91, 161, 91, 162, 91, 163, 91, 164, 91, 166, 91, 167, 91, 185, 91, 186, 91, 187, 91, 188, 91, 189, 91, 190, 91, 192, 91, 193, 91, 194, 91, 195, 91, 196, 91, 197, 91, 201, 91, 202, 91, 217, 91, 218, 91, 219, 91, 220, 91, 222, 91, 223, 91],
	#[13, -113, 32, -112],
	#[65535, 94, 23, 94, 14, 94, 22, 94, 4, 94, 13, 94, 87, 94, 31, 94, 21, 94, 7, 94, 16, 94, 1, 94, 29, 94, 19, 94, 10, 94, 15, 94, 2, 94, 11, 94, 37, 94, 77, 94, 102, 94, 0, 94, 57, 94, 32, 94, 20, 94, 33, 94, 86, 94, 110, 94, 18, 94, 40, 94, 63, 94, 12, 94, 34, 94, 9, 94, 25, 94, 74, 94, 103, 94, 27, 94, 17, 94, 43, 94, 79, 94, 26, 94, 42, 94, 80, 94, 58, 94, 81, 94, 83, 94, 41, 94, 59, 94, 76, 94, 60, 94, 127, 94, 98, 94, 75, 94, 24, 94, 72, 94, 28, 94, 38, 94, 101, 94, 85, 94, 94, 94, 82, 94, 84, 94, 190, 94, 111, 94, 89, 94, 97, 94, 105, 94, 90, 94, 92, 94, 114, 94, 99, 94, 119, 94, 106, 94, 115, 94, 116, 94, 96, 94, 112, 94, 107, 94, 104, 94, 109, 94, 118, 94, 108, 94, 88, 94, 113, 94, 100, 94, 130, 94, 121, 94, 120, 94, 122, 94, 123, 94, 124, 94, 126, 94, 138, 94, 128, 94, 129, 94, 164, 94, 133, 94, 134, 94, 135, 94, 136, 94, 137, 94, 167, 94, 161, 94, 189, 94, 188, 94, 202, 94, 160, 94, 194, 94, 162, 94, 163, 94, 186, 94, 166, 94, 192, 94, 185, 94, 187, 94, 193, 94, 201, 94, 195, 94, 196, 94, 197, 94, 217, 94, 218, 94, 219, 94, 220, 94, 222, 94, 223, 94],
	#[65535, 93, 0, 93, 1, 93, 2, 93, 4, 93, 7, 93, 9, 93, 10, 93, 11, 93, 12, 93, 13, 93, 14, 93, 15, 93, 16, 93, 17, 93, 18, 93, 19, 93, 20, 93, 21, 93, 22, 93, 23, 93, 24, 93, 25, 93, 26, 93, 27, 93, 28, 93, 29, 93, 31, 93, 32, 93, 33, 93, 34, 93, 37, 93, 38, 93, 40, 93, 41, 93, 42, 93, 43, 93, 57, 93, 58, 93, 59, 93, 60, 93, 63, 93, 72, 93, 74, 93, 75, 93, 76, 93, 77, 93, 79, 93, 80, 93, 81, 93, 82, 93, 83, 93, 84, 93, 85, 93, 86, 93, 87, 93, 88, 93, 89, 93, 90, 93, 92, 93, 94, 93, 96, 93, 97, 93, 98, 93, 99, 93, 100, 93, 101, 93, 102, 93, 103, 93, 104, 93, 105, 93, 106, 93, 107, 93, 108, 93, 109, 93, 110, 93, 111, 93, 112, 93, 113, 93, 114, 93, 115, 93, 116, 93, 118, 93, 119, 93, 120, 93, 121, 93, 122, 93, 123, 93, 124, 93, 126, 93, 127, 93, 128, 93, 129, 93, 130, 93, 133, 93, 134, 93, 135, 93, 136, 93, 137, 93, 138, 93, 160, 93, 161, 93, 162, 93, 163, 93, 164, 93, 166, 93, 167, 93, 185, 93, 186, 93, 187, 93, 188, 93, 189, 93, 190, 93, 192, 93, 193, 93, 194, 93, 195, 93, 196, 93, 197, 93, 201, 93, 202, 93, 217, 93, 218, 93, 219, 93, 220, 93, 222, 93, 223, 93],
	#[31, -59, 36, -65, 69, -68, 63, -63, 7, -37, 11, -66, 68, -64, 64, -73, 35, -67, 70, -72, 33, -60, 67, -61, 71, -55, 66, -58, 65, -71, 73, -70, 115, -10],
	#[13, 162, 37, 105, 32, 162, 42, -93, 72, -50],
	#[34, -116],
	#[65535, 95, 31, 95, 23, 95, 7, 95, 2, 95, 15, 95, 10, 95, 14, 95, 1, 95, 0, 95, 4, 95, 63, 95, 18, 95, 9, 95, 29, 95, 11, 95, 12, 95, 13, 95, 27, 95, 26, 95, 16, 95, 17, 95, 37, 95, 19, 95, 20, 95, 21, 95, 22, 95, 34, 95, 24, 95, 25, 95, 33, 95, 32, 95, 28, 95, 38, 95, 40, 95, 41, 95, 42, 95, 43, 95, 57, 95, 58, 95, 59, 95, 60, 95, 74, 95, 72, 95, 81, 95, 75, 95, 76, 95, 77, 95, 79, 95, 80, 95, 82, 95, 83, 95, 84, 95, 85, 95, 86, 95, 87, 95, 88, 95, 89, 95, 90, 95, 92, 95, 94, 95, 96, 95, 97, 95, 98, 95, 99, 95, 100, 95, 101, 95, 102, 95, 103, 95, 104, 95, 105, 95, 106, 95, 107, 95, 108, 95, 109, 95, 110, 95, 111, 95, 112, 95, 113, 95, 114, 95, 115, 95, 116, 95, 118, 95, 119, 95, 120, 95, 121, 95, 122, 95, 123, 95, 124, 95, 126, 95, 127, 95, 128, 95, 129, 95, 130, 95, 133, 95, 134, 95, 135, 95, 136, 95, 137, 95, 138, 95, 160, 95, 161, 95, 162, 95, 163, 95, 164, 95, 166, 95, 167, 95, 185, 95, 186, 95, 187, 95, 188, 95, 189, 95, 190, 95, 192, 95, 193, 95, 194, 95, 195, 95, 196, 95, 197, 95, 201, 95, 202, 95, 217, 95, 218, 95, 219, 95, 220, 95, 222, 95, 223, 95],
	#[7, -37, 11, -66, 31, -59, 33, -60, 63, -63, 35, -67, 36, -65, 71, -55, 65, -71, 70, -72, 64, -73, 69, -68, 68, -64, 67, -61, 66, -58, 73, -70, 115, -10],
	#[34, -119, 37, 105, 42, -93, 72, -50],
	#[65535, 92, 14, 92, 22, 92, 4, 92, 13, 92, 28, 92, 31, 92, 21, 92, 9, 92, 7, 92, 0, 92, 1, 92, 29, 92, 19, 92, 10, 92, 15, 92, 2, 92, 11, 92, 37, 92, 77, 92, 102, 92, 23, 92, 57, 92, 32, 92, 20, 92, 40, 92, 86, 92, 110, 92, 18, 92, 80, 92, 63, 92, 12, 92, 34, 92, 16, 92, 25, 92, 74, 92, 103, 92, 27, 92, 17, 92, 33, 92, 38, 92, 26, 92, 42, 92, 87, 92, 58, 92, 43, 92, 83, 92, 41, 92, 59, 92, 76, 92, 60, 92, 81, 92, 79, 92, 75, 92, 24, 92, 72, 92, 85, 92, 88, 92, 101, 92, 89, 92, 94, 92, 82, 92, 84, 92, 190, 92, 111, 92, 104, 92, 97, 92, 105, 92, 90, 92, 92, 92, 114, 92, 99, 92, 119, 92, 106, 92, 115, 92, 98, 92, 96, 92, 112, 92, 107, 92, 127, 92, 109, 92, 118, 92, 108, 92, 113, 92, 116, 92, 100, 92, 130, 92, 121, 92, 120, 92, 122, 92, 123, 92, 124, 92, 126, 92, 138, 92, 128, 92, 129, 92, 164, 92, 133, 92, 134, 92, 135, 92, 136, 92, 137, 92, 167, 92, 189, 92, 161, 92, 185, 92, 202, 92, 160, 92, 194, 92, 162, 92, 163, 92, 186, 92, 166, 92, 192, 92, 188, 92, 187, 92, 193, 92, 201, 92, 195, 92, 196, 92, 197, 92, 217, 92, 218, 92, 219, 92, 220, 92, 222, 92, 223, 92],
	#[65535, 207, 15, 207, 0, 207, 1, 207, 2, 207, 4, 207, 7, 207, 9, 207, 10, 207, 11, 207, 12, 207, 13, 207, 14, 207, 16, 207, 17, 207, 18, 207, 19, 207, 20, 207, 21, 207, 22, 207, 23, 207, 24, 207, 25, 207, 26, 207, 27, 207, 28, 207, 29, 207, 31, 207, 32, 207, 33, 207, 34, 207, 37, 207, 38, 207, 40, 207, 41, 207, 42, 207, 43, 207, 57, 207, 58, 207, 59, 207, 60, 207, 63, 207, 72, 207, 74, 207, 75, 207, 76, 207, 77, 207, 79, 207, 80, 207, 81, 207, 82, 207, 83, 207, 84, 207, 85, 207, 86, 207, 87, 207, 88, 207, 89, 207, 90, 207, 92, 207, 94, 207, 96, 207, 97, 207, 98, 207, 99, 207, 100, 207, 101, 207, 102, 207, 103, 207, 104, 207, 105, 207, 106, 207, 107, 207, 108, 207, 109, 207, 110, 207, 111, 207, 112, 207, 113, 207, 114, 207, 115, 207, 116, 207, 118, 207, 119, 207, 120, 207, 121, 207, 122, 207, 123, 207, 124, 207, 126, 207, 127, 207, 128, 207, 129, 207, 130, 207, 133, 207, 134, 207, 135, 207, 136, 207, 137, 207, 138, 207, 160, 207, 161, 207, 162, 207, 163, 207, 164, 207, 166, 207, 167, 207, 168, 207, 169, 207, 170, 207, 171, 207, 172, 207, 173, 207, 174, 207, 175, 207, 176, 207, 177, 207, 178, 207, 179, 207, 180, 207, 181, 207, 182, 207, 183, 207, 184, 207, 185, 207, 186, 207, 187, 207, 188, 207, 189, 207, 190, 207, 192, 207, 193, 207, 194, 207, 195, 207, 196, 207, 197, 207, 201, 207, 202, 207, 217, 207, 218, 207, 219, 207, 220, 207, 222, 207, 223, 207],
	#[14, 107, 7, 107, 10, 107, 15, 107, 18, 107, 19, 107, 38, 107, 23, 107, 13, 107, 11, 107, 1, 107, 22, 107, 4, 107, 31, 107, 21, 107, 24, 107, 9, 107, 34, 107, 28, 107, 12, 107, 26, 107, 0, 107, 29, 107, 27, 107, 17, 107, 25, 107, 40, 107, 20, 107, 86, 107, 2, 107, 37, 107, 79, 107, 77, 107, 33, 107, 41, 107, 74, 107, 94, 107, 16, 107, 32, 107, 43, 107, 85, 107, 58, -124, 59, -125, 57, -128, 63, 107, 75, 107, 80, 107, 42, 107, 98, 107, 76, 107, 60, 107, 90, 107, 110, 107, 134, 107, 82, 107, 103, 107, 101, 107, 84, 107, 135, 107, 115, 107, 81, 107, 167, 107, 96, 107, 72, 107, 109, 107, 92, 107, 83, 107, 102, 107, 87, 107, 99, 107, 105, 107, 173, -136, 116, 107, 100, 107, 88, 107, 114, 107, 171, -130, 107, 107, 106, 107, 127, 107, 104, 107, 108, 107, 118, 107, 166, 107, 190, 107, 133, 107, 126, 107, 111, 107, 174, -140, 137, 107, 89, 107, 124, 107, 175, -123, 119, 107, 120, 107, 128, 107, 161, 107, 123, 107, 138, 107, 113, 107, 136, 107, 97, 107, 112, 107, 182, -131, 121, 107, 130, 107, 177, -129, 122, 107, 163, 107, 170, -127, 169, -137, 129, 107, 164, 107, 176, -126, 189, 107, 162, 107, 168, -142, 183, -134, 172, -141, 178, -132, 185, 107, 202, 107, 184, -138, 160, 107, 194, 107, 181, -133, 180, -139, 186, 107, 179, -135, 192, 107, 188, 107, 187, 107, 193, 107, 201, 107, 195, 107, 196, 107, 197, 107, 217, 107, 218, 107, 219, 107, 220, 107, 222, 107, 223, 107],
	#[65535, 208, 21, 208, 7, 208, 15, 208, 4, 208, 23, 208, 0, 208, 10, 208, 31, 208, 20, 208, 18, 208, 11, 208, 14, 208, 2, 208, 28, 208, 12, 208, 19, 208, 22, 208, 1, 208, 9, 208, 17, 208, 34, 208, 27, 208, 13, 208, 26, 208, 24, 208, 40, 208, 25, 208, 37, 208, 41, 208, 77, 208, 57, 208, 38, 208, 63, 208, 86, 208, 33, 208, 82, 208, 16, 208, 85, 208, 29, 208, 42, 208, 108, 208, 92, 208, 76, 208, 75, 208, 58, 208, 93, 208, 32, 208, 79, 208, 102, 208, 84, 208, 83, 208, 72, 208, 59, 208, 81, 208, 89, 208, 87, 208, 175, 208, 60, 208, 109, 208, 43, 208, 94, 208, 90, 208, 114, 208, 61, 208, 74, 208, 98, 208, 96, 208, 91, 208, 116, 208, 103, 208, 110, 208, 80, 208, 99, 208, 117, 208, 97, 208, 105, 208, 88, 208, 101, 208, 112, 208, 135, 208, 118, 208, 115, 208, 106, 208, 190, 208, 111, 208, 104, 208, 107, 208, 124, 208, 119, 208, 113, 208, 134, 208, 100, 208, 182, 208, 128, 208, 127, 208, 123, 208, 121, 208, 166, 208, 173, 208, 122, 208, 130, 208, 133, 208, 120, 208, 129, 208, 137, 208, 163, 208, 126, 208, 138, 208, 174, 208, 185, 208, 164, 208, 136, 208, 171, 208, 161, 208, 169, 208, 177, 208, 202, 208, 170, 208, 167, 208, 162, 208, 176, 208, 189, 208, 168, 208, 183, 208, 172, 208, 178, 208, 184, 208, 160, 208, 194, 208, 181, 208, 180, 208, 186, 208, 179, 208, 192, 208, 188, 208, 187, 208, 193, 208, 201, 208, 195, 208, 196, 208, 197, 208, 217, 208, 218, 208, 219, 208, 220, 208, 222, 208, 223, 208],
	#[65535, 296, 0, 296, 1, 296, 2, 296, 4, 296, 7, 296, 9, 296, 10, 296, 11, 296, 12, 296, 13, 296, 14, 296, 15, 296, 16, 296, 17, 296, 18, 296, 19, 296, 20, 296, 21, 296, 22, 296, 23, 296, 24, 296, 25, 296, 26, 296, 27, 296, 28, 296, 29, 296, 31, 296, 32, 296, 33, 296, 34, 296, 37, 296, 38, 296, 40, 296, 41, 296, 42, 296, 43, 296, 57, 296, 58, 296, 59, 296, 60, 296, 61, 296, 63, 296, 72, 296, 74, 296, 75, 296, 76, 296, 77, 296, 79, 296, 80, 296, 81, 296, 82, 296, 83, 296, 84, 296, 85, 296, 86, 296, 87, 296, 88, 296, 89, 296, 90, 296, 91, 296, 92, 296, 93, 296, 94, 296, 96, 296, 97, 296, 98, 296, 99, 296, 100, 296, 101, 296, 102, 296, 103, 296, 104, 296, 105, 296, 106, 296, 107, 296, 108, 296, 109, 296, 110, 296, 111, 296, 112, 296, 113, 296, 114, 296, 115, 296, 116, 296, 117, 296, 118, 296, 119, 296, 120, 296, 121, 296, 122, 296, 123, 296, 124, 296, 126, 296, 127, 296, 128, 296, 129, 296, 130, 296, 133, 296, 134, 296, 135, 296, 136, 296, 137, 296, 138, 296, 160, 296, 161, 296, 162, 296, 163, 296, 164, 296, 166, 296, 167, 296, 168, 296, 169, 296, 170, 296, 171, 296, 172, 296, 173, 296, 174, 296, 175, 296, 176, 296, 177, 296, 178, 296, 179, 296, 180, 296, 181, 296, 182, 296, 183, 296, 184, 296, 185, 296, 186, 296, 187, 296, 188, 296, 189, 296, 190, 296, 192, 296, 193, 296, 194, 296, 195, 296, 196, 296, 197, 296, 201, 296, 202, 296, 217, 296, 218, 296, 219, 296, 220, 296, 222, 296, 223, 296],
	#[65535, 289, 13, 289, 4, 289, 21, 289, 19, 289, 7, 289, 11, 289, 29, 289, 15, 289, 34, 289, 0, 289, 77, 289, 23, 289, 18, 289, 28, 289, 12, 289, 10, 289, 87, 289, 2, 289, 31, 289, 1, 289, 20, 289, 24, 289, 25, 289, 33, 289, 14, 289, 37, 289, 60, 289, 86, 289, 43, 289, 26, 289, 61, 289, 59, 289, 22, 289, 84, 289, 9, 289, 17, 289, 79, 289, 91, 289, 27, 289, 32, 289, 108, 289, 92, 289, 40, 289, 81, 289, 89, 289, 41, 289, 75, 289, 38, 289, 63, 289, 110, 289, 82, 289, 16, 289, 57, 289, 74, 289, 42, 289, 94, 289, 76, 289, 103, 289, 58, 289, 93, 289, 102, 289, 117, 289, 83, 289, 72, 289, 90, 289, 114, 289, 175, 289, 80, 289, 109, 289, 116, 289, 98, 289, 96, 289, 97, 289, 85, 289, 88, 289, 101, 289, 99, 289, 135, 289, 105, 289, 115, 289, 106, 289, 190, 289, 111, 289, 104, 289, 107, 289, 124, 289, 119, 289, 113, 289, 134, 289, 100, 289, 112, 289, 128, 289, 127, 289, 123, 289, 118, 289, 166, 289, 173, 289, 121, 289, 130, 289, 133, 289, 120, 289, 122, 289, 137, 289, 163, 289, 183, 289, 126, 289, 138, 289, 174, 289, 129, 289, 164, 289, 136, 289, 171, 289, 161, 289, 169, 289, 177, 289, 182, 289, 170, 289, 167, 289, 162, 289, 176, 289, 189, 289, 168, 289, 188, 289, 172, 289, 178, 289, 185, 289, 202, 289, 184, 289, 160, 289, 194, 289, 181, 289, 180, 289, 186, 289, 179, 289, 192, 289, 187, 289, 193, 289, 201, 289, 195, 289, 196, 289, 197, 289, 217, 289, 218, 289, 219, 289, 220, 289, 222, 289, 223, 289],
	#[65535, 290, 31, 290, 1, 290, 7, 290, 2, 290, 15, 290, 4, 290, 10, 290, 14, 290, 23, 290, 0, 290, 26, 290, 63, 290, 18, 290, 9, 290, 29, 290, 11, 290, 12, 290, 13, 290, 27, 290, 16, 290, 17, 290, 37, 290, 19, 290, 20, 290, 21, 290, 22, 290, 34, 290, 24, 290, 25, 290, 33, 290, 32, 290, 28, 290, 38, 290, 40, 290, 41, 290, 42, 290, 43, 290, 57, 290, 58, 290, 59, 290, 60, 290, 61, 290, 74, 290, 72, 290, 81, 290, 75, 290, 76, 290, 77, 290, 79, 290, 80, 290, 82, 290, 83, 290, 84, 290, 85, 290, 86, 290, 87, 290, 88, 290, 89, 290, 90, 290, 91, 290, 92, 290, 93, 290, 94, 290, 96, 290, 97, 290, 98, 290, 99, 290, 100, 290, 101, 290, 102, 290, 103, 290, 104, 290, 105, 290, 106, 290, 107, 290, 108, 290, 109, 290, 110, 290, 111, 290, 112, 290, 113, 290, 114, 290, 115, 290, 116, 290, 117, 290, 118, 290, 119, 290, 120, 290, 121, 290, 122, 290, 123, 290, 124, 290, 126, 290, 127, 290, 128, 290, 129, 290, 130, 290, 133, 290, 134, 290, 135, 290, 136, 290, 137, 290, 138, 290, 160, 290, 161, 290, 162, 290, 163, 290, 164, 290, 166, 290, 167, 290, 168, 290, 169, 290, 170, 290, 171, 290, 172, 290, 173, 290, 174, 290, 175, 290, 176, 290, 177, 290, 178, 290, 179, 290, 180, 290, 181, 290, 182, 290, 183, 290, 184, 290, 185, 290, 186, 290, 187, 290, 188, 290, 189, 290, 190, 290, 192, 290, 193, 290, 194, 290, 195, 290, 196, 290, 197, 290, 201, 290, 202, 290, 217, 290, 218, 290, 219, 290, 220, 290, 222, 290, 223, 290],
	#[65535, 297, 0, 297, 1, 297, 2, 297, 4, 297, 7, 297, 9, 297, 10, 297, 11, 297, 12, 297, 13, 297, 14, 297, 15, 297, 16, 297, 17, 297, 18, 297, 19, 297, 20, 297, 21, 297, 22, 297, 23, 297, 24, 297, 25, 297, 26, 297, 27, 297, 28, 297, 29, 297, 31, 297, 32, 297, 33, 297, 34, 297, 37, 297, 38, 297, 40, 297, 41, 297, 42, 297, 43, 297, 57, 297, 58, 297, 59, 297, 60, 297, 61, 297, 63, 297, 72, 297, 74, 297, 75, 297, 76, 297, 77, 297, 79, 297, 80, 297, 81, 297, 82, 297, 83, 297, 84, 297, 85, 297, 86, 297, 87, 297, 88, 297, 89, 297, 90, 297, 91, 297, 92, 297, 93, 297, 94, 297, 96, 297, 97, 297, 98, 297, 99, 297, 100, 297, 101, 297, 102, 297, 103, 297, 104, 297, 105, 297, 106, 297, 107, 297, 108, 297, 109, 297, 110, 297, 111, 297, 112, 297, 113, 297, 114, 297, 115, 297, 116, 297, 117, 297, 118, 297, 119, 297, 120, 297, 121, 297, 122, 297, 123, 297, 124, 297, 126, 297, 127, 297, 128, 297, 129, 297, 130, 297, 133, 297, 134, 297, 135, 297, 136, 297, 137, 297, 138, 297, 160, 297, 161, 297, 162, 297, 163, 297, 164, 297, 166, 297, 167, 297, 168, 297, 169, 297, 170, 297, 171, 297, 172, 297, 173, 297, 174, 297, 175, 297, 176, 297, 177, 297, 178, 297, 179, 297, 180, 297, 181, 297, 182, 297, 183, 297, 184, 297, 185, 297, 186, 297, 187, 297, 188, 297, 189, 297, 190, 297, 192, 297, 193, 297, 194, 297, 195, 297, 196, 297, 197, 297, 201, 297, 202, 297, 217, 297, 218, 297, 219, 297, 220, 297, 222, 297, 223, 297],
	#[65535, 291, 15, 291, 1, 291, 14, 291, 31, 291, 13, 291, 22, 291, 10, 291, 7, 291, 2, 291, 9, 291, 21, 291, 18, 291, 16, 291, 4, 291, 27, 291, 29, 291, 38, 291, 23, 291, 12, 291, 11, 291, 37, 291, 0, 291, 34, 291, 20, 291, 26, 291, 63, 291, 19, 291, 92, 291, 24, 291, 40, 291, 25, 291, 41, 291, 58, 291, 84, 291, 17, 291, 110, 291, 33, 291, 59, 291, 79, 291, 57, 291, 61, 291, 111, 291, 94, 291, 60, 291, 82, 291, 28, 291, 87, 291, 43, 291, 42, 291, 32, 291, 102, 291, 99, 291, 90, 291, 75, 291, 72, 291, 88, 291, 77, 291, 86, 291, 76, 291, 83, 291, 98, 291, 85, 291, 74, 291, 80, 291, 81, 291, 134, 291, 89, 291, 109, 291, 91, 291, 113, 291, 93, 291, 107, 291, 106, 291, 96, 291, 97, 291, 127, 291, 116, 291, 100, 291, 101, 291, 115, 291, 103, 291, 104, 291, 105, 291, 135, 291, 112, 291, 108, 291, 114, 291, 117, 291, 118, 291, 119, 291, 120, 291, 121, 291, 122, 291, 123, 291, 124, 291, 126, 291, 138, 291, 128, 291, 129, 291, 130, 291, 136, 291, 133, 291, 137, 291, 160, 291, 161, 291, 162, 291, 163, 291, 164, 291, 166, 291, 167, 291, 168, 291, 169, 291, 170, 291, 171, 291, 172, 291, 173, 291, 174, 291, 175, 291, 176, 291, 177, 291, 178, 291, 179, 291, 180, 291, 181, 291, 182, 291, 183, 291, 184, 291, 185, 291, 186, 291, 187, 291, 188, 291, 189, 291, 190, 291, 192, 291, 193, 291, 194, 291, 195, 291, 196, 291, 197, 291, 201, 291, 202, 291, 217, 291, 218, 291, 219, 291, 220, 291, 222, 291, 223, 291],
	#[65535, 288, 4, 288, 7, 288, 14, 288, 2, 288, 12, 288, 22, 288, 23, 288, 13, 288, 1, 288, 9, 288, 18, 288, 28, 288, 31, 288, 21, 288, 11, 288, 63, 288, 26, 288, 0, 288, 20, 288, 29, 288, 17, 288, 10, 288, 16, 288, 15, 288, 75, 288, 24, 288, 93, 288, 37, 288, 25, 288, 77, 288, 102, 288, 42, 288, 83, 288, 19, 288, 34, 288, 38, 288, 40, 288, 86, 288, 32, 288, 92, 288, 76, 288, 60, 288, 81, 288, 33, 288, 43, 288, 94, 288, 57, 288, 59, 288, 84, 288, 61, 288, 74, 288, 27, 288, 108, 288, 89, 288, 41, 288, 58, 288, 103, 288, 110, 288, 80, 288, 79, 288, 117, 288, 82, 288, 85, 288, 87, 288, 175, 288, 116, 288, 98, 288, 109, 288, 97, 288, 90, 288, 96, 288, 72, 288, 88, 288, 91, 288, 101, 288, 99, 288, 135, 288, 105, 288, 115, 288, 106, 288, 190, 288, 111, 288, 104, 288, 107, 288, 124, 288, 119, 288, 113, 288, 134, 288, 100, 288, 112, 288, 128, 288, 127, 288, 114, 288, 123, 288, 118, 288, 166, 288, 173, 288, 121, 288, 130, 288, 133, 288, 120, 288, 122, 288, 137, 288, 163, 288, 126, 288, 138, 288, 174, 288, 129, 288, 164, 288, 136, 288, 171, 288, 161, 288, 169, 288, 177, 288, 182, 288, 170, 288, 167, 288, 162, 288, 176, 288, 189, 288, 168, 288, 183, 288, 172, 288, 178, 288, 185, 288, 202, 288, 184, 288, 160, 288, 194, 288, 181, 288, 180, 288, 186, 288, 179, 288, 192, 288, 188, 288, 187, 288, 193, 288, 201, 288, 195, 288, 196, 288, 197, 288, 217, 288, 218, 288, 219, 288, 220, 288, 222, 288, 223, 288],
	#[65535, 298, 15, 298, 23, 298, 18, 298, 12, 298, 10, 298, 2, 298, 31, 298, 63, 298, 24, 298, 7, 298, 14, 298, 37, 298, 60, 298, 86, 298, 21, 298, 25, 298, 19, 298, 59, 298, 22, 298, 1, 298, 9, 298, 11, 298, 29, 298, 4, 298, 13, 298, 26, 298, 92, 298, 17, 298, 93, 298, 87, 298, 28, 298, 77, 298, 75, 298, 34, 298, 61, 298, 84, 298, 33, 298, 0, 298, 16, 298, 20, 298, 127, 298, 42, 298, 98, 298, 41, 298, 103, 298, 57, 298, 27, 298, 79, 298, 102, 298, 117, 298, 83, 298, 32, 298, 38, 298, 40, 298, 110, 298, 76, 298, 80, 298, 81, 298, 43, 298, 94, 298, 58, 298, 74, 298, 126, 298, 108, 298, 90, 298, 96, 298, 82, 298, 85, 298, 106, 298, 175, 298, 116, 298, 123, 298, 109, 298, 105, 298, 113, 298, 72, 298, 88, 298, 91, 298, 101, 298, 99, 298, 135, 298, 118, 298, 115, 298, 167, 298, 190, 298, 111, 298, 89, 298, 97, 298, 124, 298, 119, 298, 134, 298, 100, 298, 112, 298, 107, 298, 104, 298, 114, 298, 136, 298, 121, 298, 166, 298, 128, 298, 122, 298, 130, 298, 133, 298, 120, 298, 129, 298, 137, 298, 163, 298, 183, 298, 170, 298, 138, 298, 174, 298, 162, 298, 164, 298, 186, 298, 171, 298, 161, 298, 178, 298, 187, 298, 202, 298, 188, 298, 168, 298, 184, 298, 179, 298, 169, 298, 189, 298, 173, 298, 172, 298, 177, 298, 182, 298, 160, 298, 194, 298, 181, 298, 180, 298, 185, 298, 176, 298, 192, 298, 193, 298, 201, 298, 195, 298, 196, 298, 197, 298, 217, 298, 218, 298, 219, 298, 220, 298, 222, 298, 223, 298],
	#[65535, 292, 31, 292, 23, 292, 10, 292, 2, 292, 7, 292, 37, 292, 15, 292, 1, 292, 27, 292, 14, 292, 16, 292, 11, 292, 0, 292, 4, 292, 63, 292, 19, 292, 18, 292, 9, 292, 29, 292, 25, 292, 12, 292, 13, 292, 26, 292, 17, 292, 20, 292, 21, 292, 22, 292, 34, 292, 24, 292, 58, 292, 33, 292, 32, 292, 28, 292, 38, 292, 40, 292, 41, 292, 42, 292, 43, 292, 57, 292, 87, 292, 59, 292, 60, 292, 61, 292, 74, 292, 72, 292, 81, 292, 75, 292, 76, 292, 77, 292, 79, 292, 80, 292, 82, 292, 83, 292, 84, 292, 85, 292, 86, 292, 98, 292, 88, 292, 89, 292, 90, 292, 91, 292, 92, 292, 93, 292, 94, 292, 96, 292, 97, 292, 99, 292, 100, 292, 101, 292, 102, 292, 103, 292, 104, 292, 105, 292, 106, 292, 107, 292, 108, 292, 109, 292, 110, 292, 111, 292, 112, 292, 113, 292, 114, 292, 115, 292, 116, 292, 117, 292, 118, 292, 119, 292, 120, 292, 121, 292, 122, 292, 123, 292, 124, 292, 126, 292, 127, 292, 128, 292, 129, 292, 130, 292, 133, 292, 134, 292, 135, 292, 136, 292, 137, 292, 138, 292, 160, 292, 161, 292, 162, 292, 163, 292, 164, 292, 166, 292, 167, 292, 168, 292, 169, 292, 170, 292, 171, 292, 172, 292, 173, 292, 174, 292, 175, 292, 176, 292, 177, 292, 178, 292, 179, 292, 180, 292, 181, 292, 182, 292, 183, 292, 184, 292, 185, 292, 186, 292, 187, 292, 188, 292, 189, 292, 190, 292, 192, 292, 193, 292, 194, 292, 195, 292, 196, 292, 197, 292, 201, 292, 202, 292, 217, 292, 218, 292, 219, 292, 220, 292, 222, 292, 223, 292],
	#[65535, 303, 0, 303, 1, 303, 2, 303, 4, 303, 7, 303, 9, 303, 10, 303, 11, 303, 12, 303, 13, 303, 14, 303, 15, 303, 16, 303, 17, 303, 18, 303, 19, 303, 20, 303, 21, 303, 22, 303, 23, 303, 24, 303, 25, 303, 26, 303, 27, 303, 28, 303, 29, 303, 31, 303, 32, 303, 33, 303, 34, 303, 37, 303, 38, 303, 40, 303, 41, 303, 42, 303, 43, 303, 57, 303, 58, 303, 59, 303, 60, 303, 61, 303, 63, 303, 72, 303, 74, 303, 75, 303, 76, 303, 77, 303, 79, 303, 80, 303, 81, 303, 82, 303, 83, 303, 84, 303, 85, 303, 86, 303, 87, 303, 88, 303, 89, 303, 90, 303, 91, 303, 92, 303, 93, 303, 94, 303, 96, 303, 97, 303, 98, 303, 99, 303, 100, 303, 101, 303, 102, 303, 103, 303, 104, 303, 105, 303, 106, 303, 107, 303, 108, 303, 109, 303, 110, 303, 111, 303, 112, 303, 113, 303, 114, 303, 115, 303, 116, 303, 117, 303, 118, 303, 119, 303, 120, 303, 121, 303, 122, 303, 123, 303, 124, 303, 126, 303, 127, 303, 128, 303, 129, 303, 130, 303, 133, 303, 134, 303, 135, 303, 136, 303, 137, 303, 138, 303, 160, 303, 161, 303, 162, 303, 163, 303, 164, 303, 166, 303, 167, 303, 168, 303, 169, 303, 170, 303, 171, 303, 172, 303, 173, 303, 174, 303, 175, 303, 176, 303, 177, 303, 178, 303, 179, 303, 180, 303, 181, 303, 182, 303, 183, 303, 184, 303, 185, 303, 186, 303, 187, 303, 188, 303, 189, 303, 190, 303, 192, 303, 193, 303, 194, 303, 195, 303, 196, 303, 197, 303, 201, 303, 202, 303, 217, 303, 218, 303, 219, 303, 220, 303, 222, 303, 223, 303],
	#[65535, 299, 31, 299, 7, 299, 15, 299, 2, 299, 1, 299, 0, 299, 23, 299, 10, 299, 19, 299, 14, 299, 38, 299, 4, 299, 11, 299, 13, 299, 22, 299, 12, 299, 37, 299, 9, 299, 17, 299, 21, 299, 18, 299, 16, 299, 32, 299, 27, 299, 24, 299, 34, 299, 29, 299, 79, 299, 26, 299, 59, 299, 40, 299, 33, 299, 87, 299, 20, 299, 61, 299, 63, 299, 41, 299, 92, 299, 57, 299, 42, 299, 102, 299, 25, 299, 90, 299, 58, 299, 84, 299, 43, 299, 110, 299, 74, 299, 77, 299, 99, 299, 72, 299, 75, 299, 111, 299, 94, 299, 60, 299, 82, 299, 28, 299, 88, 299, 86, 299, 76, 299, 83, 299, 98, 299, 85, 299, 81, 299, 80, 299, 134, 299, 89, 299, 109, 299, 91, 299, 113, 299, 93, 299, 107, 299, 106, 299, 96, 299, 97, 299, 127, 299, 116, 299, 100, 299, 101, 299, 115, 299, 103, 299, 104, 299, 105, 299, 135, 299, 112, 299, 108, 299, 114, 299, 117, 299, 118, 299, 119, 299, 120, 299, 121, 299, 122, 299, 123, 299, 124, 299, 126, 299, 138, 299, 128, 299, 129, 299, 130, 299, 136, 299, 133, 299, 137, 299, 160, 299, 161, 299, 162, 299, 163, 299, 164, 299, 166, 299, 167, 299, 168, 299, 169, 299, 170, 299, 171, 299, 172, 299, 173, 299, 174, 299, 175, 299, 176, 299, 177, 299, 178, 299, 179, 299, 180, 299, 181, 299, 182, 299, 183, 299, 184, 299, 185, 299, 186, 299, 187, 299, 188, 299, 189, 299, 190, 299, 192, 299, 193, 299, 194, 299, 195, 299, 196, 299, 197, 299, 201, 299, 202, 299, 217, 299, 218, 299, 219, 299, 220, 299, 222, 299, 223, 299],
	#[42, -143],
	#[65535, 304, 0, 304, 1, 304, 2, 304, 4, 304, 7, 304, 9, 304, 10, 304, 11, 304, 12, 304, 13, 304, 14, 304, 15, 304, 16, 304, 17, 304, 18, 304, 19, 304, 20, 304, 21, 304, 22, 304, 23, 304, 24, 304, 25, 304, 26, 304, 27, 304, 28, 304, 29, 304, 31, 304, 32, 304, 33, 304, 34, 304, 37, 304, 38, 304, 40, 304, 41, 304, 42, 304, 43, 304, 57, 304, 58, 304, 59, 304, 60, 304, 61, 304, 63, 304, 72, 304, 74, 304, 75, 304, 76, 304, 77, 304, 79, 304, 80, 304, 81, 304, 82, 304, 83, 304, 84, 304, 85, 304, 86, 304, 87, 304, 88, 304, 89, 304, 90, 304, 91, 304, 92, 304, 93, 304, 94, 304, 96, 304, 97, 304, 98, 304, 99, 304, 100, 304, 101, 304, 102, 304, 103, 304, 104, 304, 105, 304, 106, 304, 107, 304, 108, 304, 109, 304, 110, 304, 111, 304, 112, 304, 113, 304, 114, 304, 115, 304, 116, 304, 117, 304, 118, 304, 119, 304, 120, 304, 121, 304, 122, 304, 123, 304, 124, 304, 126, 304, 127, 304, 128, 304, 129, 304, 130, 304, 133, 304, 134, 304, 135, 304, 136, 304, 137, 304, 138, 304, 160, 304, 161, 304, 162, 304, 163, 304, 164, 304, 166, 304, 167, 304, 168, 304, 169, 304, 170, 304, 171, 304, 172, 304, 173, 304, 174, 304, 175, 304, 176, 304, 177, 304, 178, 304, 179, 304, 180, 304, 181, 304, 182, 304, 183, 304, 184, 304, 185, 304, 186, 304, 187, 304, 188, 304, 189, 304, 190, 304, 192, 304, 193, 304, 194, 304, 195, 304, 196, 304, 197, 304, 201, 304, 202, 304, 217, 304, 218, 304, 219, 304, 220, 304, 222, 304, 223, 304],
	#[65535, 300, 31, 300, 7, 300, 2, 300, 15, 300, 1, 300, 10, 300, 14, 300, 23, 300, 0, 300, 4, 300, 63, 300, 19, 300, 18, 300, 9, 300, 29, 300, 11, 300, 12, 300, 13, 300, 27, 300, 26, 300, 16, 300, 17, 300, 37, 300, 33, 300, 20, 300, 21, 300, 22, 300, 34, 300, 24, 300, 25, 300, 32, 300, 28, 300, 38, 300, 40, 300, 41, 300, 42, 300, 43, 300, 57, 300, 58, 300, 59, 300, 60, 300, 61, 300, 74, 300, 72, 300, 81, 300, 75, 300, 76, 300, 77, 300, 79, 300, 80, 300, 82, 300, 83, 300, 84, 300, 85, 300, 86, 300, 87, 300, 88, 300, 89, 300, 90, 300, 91, 300, 92, 300, 93, 300, 94, 300, 96, 300, 97, 300, 98, 300, 99, 300, 100, 300, 101, 300, 102, 300, 103, 300, 104, 300, 105, 300, 106, 300, 107, 300, 108, 300, 109, 300, 110, 300, 111, 300, 112, 300, 113, 300, 114, 300, 115, 300, 116, 300, 117, 300, 118, 300, 119, 300, 120, 300, 121, 300, 122, 300, 123, 300, 124, 300, 126, 300, 127, 300, 128, 300, 129, 300, 130, 300, 133, 300, 134, 300, 135, 300, 136, 300, 137, 300, 138, 300, 160, 300, 161, 300, 162, 300, 163, 300, 164, 300, 166, 300, 167, 300, 168, 300, 169, 300, 170, 300, 171, 300, 172, 300, 173, 300, 174, 300, 175, 300, 176, 300, 177, 300, 178, 300, 179, 300, 180, 300, 181, 300, 182, 300, 183, 300, 184, 300, 185, 300, 186, 300, 187, 300, 188, 300, 189, 300, 190, 300, 192, 300, 193, 300, 194, 300, 195, 300, 196, 300, 197, 300, 201, 300, 202, 300, 217, 300, 218, 300, 219, 300, 220, 300, 222, 300, 223, 300],
	#[65535, 294, 4, 294, 7, 294, 14, 294, 2, 294, 12, 294, 22, 294, 23, 294, 13, 294, 1, 294, 9, 294, 18, 294, 28, 294, 31, 294, 21, 294, 11, 294, 63, 294, 26, 294, 0, 294, 20, 294, 29, 294, 17, 294, 10, 294, 16, 294, 15, 294, 75, 294, 24, 294, 93, 294, 37, 294, 25, 294, 77, 294, 102, 294, 42, 294, 83, 294, 19, 294, 34, 294, 38, 294, 40, 294, 86, 294, 32, 294, 92, 294, 76, 294, 60, 294, 81, 294, 33, 294, 43, 294, 94, 294, 57, 294, 59, 294, 84, 294, 61, 294, 74, 294, 27, 294, 108, 294, 89, 294, 41, 294, 58, 294, 103, 294, 110, 294, 80, 294, 79, 294, 117, 294, 82, 294, 85, 294, 87, 294, 175, 294, 116, 294, 98, 294, 109, 294, 97, 294, 90, 294, 96, 294, 72, 294, 88, 294, 91, 294, 101, 294, 99, 294, 135, 294, 105, 294, 115, 294, 106, 294, 190, 294, 111, 294, 104, 294, 107, 294, 124, 294, 119, 294, 113, 294, 134, 294, 100, 294, 112, 294, 128, 294, 127, 294, 114, 294, 123, 294, 118, 294, 166, 294, 173, 294, 121, 294, 130, 294, 133, 294, 120, 294, 122, 294, 137, 294, 163, 294, 126, 294, 138, 294, 174, 294, 129, 294, 164, 294, 136, 294, 171, 294, 161, 294, 169, 294, 177, 294, 182, 294, 170, 294, 167, 294, 162, 294, 176, 294, 189, 294, 168, 294, 183, 294, 172, 294, 178, 294, 185, 294, 202, 294, 184, 294, 160, 294, 194, 294, 181, 294, 180, 294, 186, 294, 179, 294, 192, 294, 188, 294, 187, 294, 193, 294, 201, 294, 195, 294, 196, 294, 197, 294, 217, 294, 218, 294, 219, 294, 220, 294, 222, 294, 223, 294],
	#[65535, 287, 31, 287, 15, 287, 1, 287, 21, 287, 10, 287, 14, 287, 2, 287, 13, 287, 22, 287, 7, 287, 37, 287, 9, 287, 25, 287, 11, 287, 16, 287, 4, 287, 27, 287, 17, 287, 34, 287, 29, 287, 19, 287, 23, 287, 12, 287, 33, 287, 0, 287, 20, 287, 26, 287, 63, 287, 18, 287, 24, 287, 32, 287, 28, 287, 38, 287, 40, 287, 41, 287, 42, 287, 43, 287, 57, 287, 58, 287, 59, 287, 60, 287, 61, 287, 74, 287, 72, 287, 81, 287, 75, 287, 76, 287, 77, 287, 79, 287, 80, 287, 82, 287, 83, 287, 84, 287, 85, 287, 86, 287, 87, 287, 88, 287, 89, 287, 90, 287, 91, 287, 92, 287, 93, 287, 94, 287, 96, 287, 97, 287, 98, 287, 99, 287, 100, 287, 101, 287, 102, 287, 103, 287, 104, 287, 105, 287, 106, 287, 107, 287, 108, 287, 109, 287, 110, 287, 111, 287, 112, 287, 113, 287, 114, 287, 115, 287, 116, 287, 117, 287, 118, 287, 119, 287, 120, 287, 121, 287, 122, 287, 123, 287, 124, 287, 126, 287, 127, 287, 128, 287, 129, 287, 130, 287, 133, 287, 134, 287, 135, 287, 136, 287, 137, 287, 138, 287, 160, 287, 161, 287, 162, 287, 163, 287, 164, 287, 166, 287, 167, 287, 168, 287, 169, 287, 170, 287, 171, 287, 172, 287, 173, 287, 174, 287, 175, 287, 176, 287, 177, 287, 178, 287, 179, 287, 180, 287, 181, 287, 182, 287, 183, 287, 184, 287, 185, 287, 186, 287, 187, 287, 188, 287, 189, 287, 190, 287, 192, 287, 193, 287, 194, 287, 195, 287, 196, 287, 197, 287, 201, 287, 202, 287, 217, 287, 218, 287, 219, 287, 220, 287, 222, 287, 223, 287],
	#[65535, 305, 31, 305, 1, 305, 23, 305, 10, 305, 2, 305, 7, 305, 37, 305, 15, 305, 4, 305, 27, 305, 14, 305, 16, 305, 11, 305, 0, 305, 26, 305, 63, 305, 19, 305, 18, 305, 9, 305, 29, 305, 25, 305, 12, 305, 13, 305, 17, 305, 20, 305, 21, 305, 22, 305, 34, 305, 24, 305, 58, 305, 33, 305, 32, 305, 28, 305, 38, 305, 40, 305, 41, 305, 42, 305, 43, 305, 57, 305, 87, 305, 59, 305, 60, 305, 61, 305, 74, 305, 72, 305, 81, 305, 75, 305, 76, 305, 77, 305, 79, 305, 80, 305, 82, 305, 83, 305, 84, 305, 85, 305, 86, 305, 98, 305, 88, 305, 89, 305, 90, 305, 91, 305, 92, 305, 93, 305, 94, 305, 96, 305, 97, 305, 99, 305, 100, 305, 101, 305, 102, 305, 103, 305, 104, 305, 105, 305, 106, 305, 107, 305, 108, 305, 109, 305, 110, 305, 111, 305, 112, 305, 113, 305, 114, 305, 115, 305, 116, 305, 117, 305, 118, 305, 119, 305, 120, 305, 121, 305, 122, 305, 123, 305, 124, 305, 126, 305, 127, 305, 128, 305, 129, 305, 130, 305, 133, 305, 134, 305, 135, 305, 136, 305, 137, 305, 138, 305, 160, 305, 161, 305, 162, 305, 163, 305, 164, 305, 166, 305, 167, 305, 168, 305, 169, 305, 170, 305, 171, 305, 172, 305, 173, 305, 174, 305, 175, 305, 176, 305, 177, 305, 178, 305, 179, 305, 180, 305, 181, 305, 182, 305, 183, 305, 184, 305, 185, 305, 186, 305, 187, 305, 188, 305, 189, 305, 190, 305, 192, 305, 193, 305, 194, 305, 195, 305, 196, 305, 197, 305, 201, 305, 202, 305, 217, 305, 218, 305, 219, 305, 220, 305, 222, 305, 223, 305],
	#[65535, 301, 0, 301, 1, 301, 2, 301, 4, 301, 7, 301, 9, 301, 10, 301, 11, 301, 12, 301, 13, 301, 14, 301, 15, 301, 16, 301, 17, 301, 18, 301, 19, 301, 20, 301, 21, 301, 22, 301, 23, 301, 24, 301, 25, 301, 26, 301, 27, 301, 28, 301, 29, 301, 31, 301, 32, 301, 33, 301, 34, 301, 37, 301, 38, 301, 40, 301, 41, 301, 42, 301, 43, 301, 57, 301, 58, 301, 59, 301, 60, 301, 61, 301, 63, 301, 72, 301, 74, 301, 75, 301, 76, 301, 77, 301, 79, 301, 80, 301, 81, 301, 82, 301, 83, 301, 84, 301, 85, 301, 86, 301, 87, 301, 88, 301, 89, 301, 90, 301, 91, 301, 92, 301, 93, 301, 94, 301, 96, 301, 97, 301, 98, 301, 99, 301, 100, 301, 101, 301, 102, 301, 103, 301, 104, 301, 105, 301, 106, 301, 107, 301, 108, 301, 109, 301, 110, 301, 111, 301, 112, 301, 113, 301, 114, 301, 115, 301, 116, 301, 117, 301, 118, 301, 119, 301, 120, 301, 121, 301, 122, 301, 123, 301, 124, 301, 126, 301, 127, 301, 128, 301, 129, 301, 130, 301, 133, 301, 134, 301, 135, 301, 136, 301, 137, 301, 138, 301, 160, 301, 161, 301, 162, 301, 163, 301, 164, 301, 166, 301, 167, 301, 168, 301, 169, 301, 170, 301, 171, 301, 172, 301, 173, 301, 174, 301, 175, 301, 176, 301, 177, 301, 178, 301, 179, 301, 180, 301, 181, 301, 182, 301, 183, 301, 184, 301, 185, 301, 186, 301, 187, 301, 188, 301, 189, 301, 190, 301, 192, 301, 193, 301, 194, 301, 195, 301, 196, 301, 197, 301, 201, 301, 202, 301, 217, 301, 218, 301, 219, 301, 220, 301, 222, 301, 223, 301],
	#[65535, 295, 14, 295, 10, 295, 13, 295, 22, 295, 4, 295, 31, 295, 21, 295, 19, 295, 7, 295, 12, 295, 11, 295, 29, 295, 9, 295, 15, 295, 38, 295, 20, 295, 34, 295, 2, 295, 37, 295, 0, 295, 25, 295, 23, 295, 18, 295, 28, 295, 27, 295, 16, 295, 17, 295, 43, 295, 85, 295, 40, 295, 59, 295, 1, 295, 63, 295, 24, 295, 74, 295, 33, 295, 93, 295, 76, 295, 60, 295, 86, 295, 110, 295, 26, 295, 61, 295, 116, 295, 101, 295, 84, 295, 32, 295, 94, 295, 79, 295, 91, 295, 90, 295, 72, 295, 57, 295, 92, 295, 83, 295, 102, 295, 87, 295, 41, 295, 77, 295, 75, 295, 117, 295, 100, 295, 82, 295, 96, 295, 106, 295, 127, 295, 42, 295, 98, 295, 97, 295, 103, 295, 114, 295, 135, 295, 81, 295, 107, 295, 111, 295, 112, 295, 99, 295, 58, 295, 115, 295, 119, 295, 80, 295, 109, 295, 104, 295, 118, 295, 105, 295, 113, 295, 134, 295, 89, 295, 88, 295, 120, 295, 108, 295, 121, 295, 122, 295, 123, 295, 124, 295, 126, 295, 138, 295, 128, 295, 129, 295, 130, 295, 136, 295, 133, 295, 137, 295, 160, 295, 161, 295, 162, 295, 163, 295, 164, 295, 166, 295, 167, 295, 168, 295, 169, 295, 170, 295, 171, 295, 172, 295, 173, 295, 174, 295, 175, 295, 176, 295, 177, 295, 178, 295, 179, 295, 180, 295, 181, 295, 182, 295, 183, 295, 184, 295, 185, 295, 186, 295, 187, 295, 188, 295, 189, 295, 190, 295, 192, 295, 193, 295, 194, 295, 195, 295, 196, 295, 197, 295, 201, 295, 202, 295, 217, 295, 218, 295, 219, 295, 220, 295, 222, 295, 223, 295],
	#[65535, 293, 15, 293, 0, 293, 1, 293, 2, 293, 4, 293, 7, 293, 9, 293, 10, 293, 11, 293, 12, 293, 13, 293, 14, 293, 16, 293, 17, 293, 18, 293, 19, 293, 20, 293, 21, 293, 22, 293, 23, 293, 24, 293, 25, 293, 26, 293, 27, 293, 28, 293, 29, 293, 31, 293, 32, 293, 33, 293, 34, 293, 37, 293, 38, 293, 40, 293, 41, 293, 42, 293, 43, 293, 57, 293, 58, 293, 59, 293, 60, 293, 61, 293, 63, 293, 72, 293, 74, 293, 75, 293, 76, 293, 77, 293, 79, 293, 80, 293, 81, 293, 82, 293, 83, 293, 84, 293, 85, 293, 86, 293, 87, 293, 88, 293, 89, 293, 90, 293, 91, 293, 92, 293, 93, 293, 94, 293, 96, 293, 97, 293, 98, 293, 99, 293, 100, 293, 101, 293, 102, 293, 103, 293, 104, 293, 105, 293, 106, 293, 107, 293, 108, 293, 109, 293, 110, 293, 111, 293, 112, 293, 113, 293, 114, 293, 115, 293, 116, 293, 117, 293, 118, 293, 119, 293, 120, 293, 121, 293, 122, 293, 123, 293, 124, 293, 126, 293, 127, 293, 128, 293, 129, 293, 130, 293, 133, 293, 134, 293, 135, 293, 136, 293, 137, 293, 138, 293, 160, 293, 161, 293, 162, 293, 163, 293, 164, 293, 166, 293, 167, 293, 168, 293, 169, 293, 170, 293, 171, 293, 172, 293, 173, 293, 174, 293, 175, 293, 176, 293, 177, 293, 178, 293, 179, 293, 180, 293, 181, 293, 182, 293, 183, 293, 184, 293, 185, 293, 186, 293, 187, 293, 188, 293, 189, 293, 190, 293, 192, 293, 193, 293, 194, 293, 195, 293, 196, 293, 197, 293, 201, 293, 202, 293, 217, 293, 218, 293, 219, 293, 220, 293, 222, 293, 223, 293],
	#[65535, 286, 0, 286, 1, 286, 2, 286, 4, 286, 7, 286, 9, 286, 10, 286, 11, 286, 12, 286, 13, 286, 14, 286, 15, 286, 16, 286, 17, 286, 18, 286, 19, 286, 20, 286, 21, 286, 22, 286, 23, 286, 24, 286, 25, 286, 26, 286, 27, 286, 28, 286, 29, 286, 31, 286, 32, 286, 33, 286, 34, 286, 37, 286, 38, 286, 40, 286, 41, 286, 42, 286, 43, 286, 57, 286, 58, 286, 59, 286, 60, 286, 61, 286, 63, 286, 72, 286, 74, 286, 75, 286, 76, 286, 77, 286, 79, 286, 80, 286, 81, 286, 82, 286, 83, 286, 84, 286, 85, 286, 86, 286, 87, 286, 88, 286, 89, 286, 90, 286, 91, 286, 92, 286, 93, 286, 94, 286, 96, 286, 97, 286, 98, 286, 99, 286, 100, 286, 101, 286, 102, 286, 103, 286, 104, 286, 105, 286, 106, 286, 107, 286, 108, 286, 109, 286, 110, 286, 111, 286, 112, 286, 113, 286, 114, 286, 115, 286, 116, 286, 117, 286, 118, 286, 119, 286, 120, 286, 121, 286, 122, 286, 123, 286, 124, 286, 126, 286, 127, 286, 128, 286, 129, 286, 130, 286, 133, 286, 134, 286, 135, 286, 136, 286, 137, 286, 138, 286, 160, 286, 161, 286, 162, 286, 163, 286, 164, 286, 166, 286, 167, 286, 168, 286, 169, 286, 170, 286, 171, 286, 172, 286, 173, 286, 174, 286, 175, 286, 176, 286, 177, 286, 178, 286, 179, 286, 180, 286, 181, 286, 182, 286, 183, 286, 184, 286, 185, 286, 186, 286, 187, 286, 188, 286, 189, 286, 190, 286, 192, 286, 193, 286, 194, 286, 195, 286, 196, 286, 197, 286, 201, 286, 202, 286, 217, 286, 218, 286, 219, 286, 220, 286, 222, 286, 223, 286],
	#[38, -144],
	#[43, -145],
	#[65535, 302, 23, 302, 14, 302, 1, 302, 9, 302, 13, 302, 22, 302, 10, 302, 7, 302, 2, 302, 21, 302, 18, 302, 15, 302, 4, 302, 27, 302, 17, 302, 16, 302, 38, 302, 59, 302, 12, 302, 25, 302, 11, 302, 37, 302, 0, 302, 34, 302, 31, 302, 20, 302, 26, 302, 63, 302, 19, 302, 92, 302, 24, 302, 29, 302, 41, 302, 58, 302, 40, 302, 84, 302, 110, 302, 33, 302, 77, 302, 79, 302, 57, 302, 61, 302, 111, 302, 94, 302, 60, 302, 82, 302, 28, 302, 87, 302, 43, 302, 42, 302, 32, 302, 102, 302, 99, 302, 90, 302, 75, 302, 72, 302, 88, 302, 91, 302, 86, 302, 76, 302, 83, 302, 98, 302, 85, 302, 74, 302, 80, 302, 81, 302, 134, 302, 89, 302, 109, 302, 108, 302, 113, 302, 93, 302, 107, 302, 106, 302, 96, 302, 97, 302, 127, 302, 116, 302, 100, 302, 101, 302, 115, 302, 103, 302, 104, 302, 105, 302, 135, 302, 112, 302, 114, 302, 117, 302, 118, 302, 119, 302, 120, 302, 121, 302, 122, 302, 123, 302, 124, 302, 126, 302, 138, 302, 128, 302, 129, 302, 130, 302, 136, 302, 133, 302, 137, 302, 160, 302, 161, 302, 162, 302, 163, 302, 164, 302, 166, 302, 167, 302, 168, 302, 169, 302, 170, 302, 171, 302, 172, 302, 173, 302, 174, 302, 175, 302, 176, 302, 177, 302, 178, 302, 179, 302, 180, 302, 181, 302, 182, 302, 183, 302, 184, 302, 185, 302, 186, 302, 187, 302, 188, 302, 189, 302, 190, 302, 192, 302, 193, 302, 194, 302, 195, 302, 196, 302, 197, 302, 201, 302, 202, 302, 217, 302, 218, 302, 219, 302, 220, 302, 222, 302, 223, 302],
	#[31, -59, 36, -65, 7, -37, 11, -66, 35, -67, 33, -60, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 95, -148, 115, -10],
	#[13, 160, 43, 160, 60, -88, 90, -86, 105, -91, 106, -84, 217, -90, 218, -85, 219, -87, 220, -89],
	#[65535, 157, 43, 157],
	#[65535, 402, 2, 402, 7, 402, 10, 402, 11, 402, 13, 402, 31, 402, 33, 402, 38, 402, 43, 402, 57, 402, 58, 402, 59, 402, 60, 402, 63, 402, 81, 402, 82, 402, 83, 402, 84, 402, 85, 402, 86, 402, 87, 402, 88, 402, 89, 402, 90, 402, 92, 402, 94, 402, 96, 402, 97, 402, 98, 402, 99, 402, 100, 402, 101, 402, 102, 402, 103, 402, 104, 402, 105, 402, 106, 402, 107, 402, 108, 402, 109, 402, 110, 402, 111, 402, 112, 402, 113, 402, 114, 402, 115, 402, 116, 402, 118, 402, 119, 402, 120, 402, 121, 402, 122, 402, 123, 402, 124, 402, 126, 402, 127, 402, 128, 402, 129, 402, 130, 402, 133, 402, 134, 402, 135, 402, 136, 402, 137, 402, 138, 402, 217, 402, 218, 402, 219, 402, 220, 402, 223, 402],
	#[65535, 403, 7, 403, 13, 403, 85, 403, 63, 403, 11, 403, 10, 403, 110, 403, 59, 403, 101, 403, 94, 403, 81, 403, 33, 403, 2, 403, 111, 403, 114, 403, 92, 403, 83, 403, 102, 403, 87, 403, 99, 403, 31, 403, 57, 403, 38, 403, 90, 403, 86, 403, 106, 403, 88, 403, 98, 403, 118, 403, 103, 403, 58, 403, 82, 403, 116, 403, 107, 403, 84, 403, 123, 403, 122, 403, 104, 403, 89, 403, 112, 403, 60, 403, 109, 403, 97, 403, 100, 403, 108, 403, 96, 403, 113, 403, 105, 403, 43, 403, 126, 403, 121, 403, 127, 403, 115, 403, 119, 403, 130, 403, 133, 403, 120, 403, 129, 403, 128, 403, 124, 403, 134, 403, 135, 403, 136, 403, 137, 403, 138, 403, 217, 403, 218, 403, 219, 403, 220, 403, 223, 403],
	#[65535, 111, 102, 111, 82, 111, 38, 111, 10, 111, 13, 111, 87, 111, 31, 111, 43, 111, 92, 111, 63, 111, 7, 111, 2, 111, 33, 111, 11, 111, 85, 111, 84, 111, 107, 111, 103, 111, 83, 111, 86, 111, 90, 111, 114, 111, 96, 111, 60, 111, 88, 111, 94, 111, 101, 111, 116, 111, 100, 111, 104, 111, 97, 111, 119, 111, 99, 111, 138, 111, 109, 111, 81, 111, 108, 111, 98, 111, 105, 111, 127, 111, 121, 111, 110, 111, 134, 111, 106, 111, 113, 111, 135, 111, 115, 111, 118, 111, 89, 111, 111, 111, 128, 111, 126, 111, 120, 111, 122, 111, 112, 111, 123, 111, 137, 111, 130, 111, 133, 111, 129, 111, 124, 111, 136, 111, 217, 111, 218, 111, 219, 111, 220, 111, 223, 111],
	#[65535, 404, 38, 404, 31, 404, 60, 404, 7, 404, 11, 404, 10, 404, 2, 404, 13, 404, 33, 404, 59, 404, 94, 404, 87, 404, 43, 404, 83, 404, 119, 404, 84, 404, 63, 404, 90, 404, 58, 404, 82, 404, 81, 404, 86, 404, 92, 404, 57, 404, 101, 404, 103, 404, 85, 404, 88, 404, 105, 404, 115, 404, 106, 404, 111, 404, 98, 404, 102, 404, 89, 404, 110, 404, 116, 404, 96, 404, 112, 404, 107, 404, 97, 404, 114, 404, 109, 404, 118, 404, 99, 404, 104, 404, 113, 404, 100, 404, 108, 404, 121, 404, 120, 404, 122, 404, 123, 404, 124, 404, 126, 404, 127, 404, 128, 404, 129, 404, 130, 404, 133, 404, 134, 404, 135, 404, 136, 404, 137, 404, 138, 404, 217, 404, 218, 404, 219, 404, 220, 404, 223, 404],
	#[65535, 207, 94, 207, 4, 207, 13, 207, 28, 207, 31, 207, 7, 207, 22, 207, 15, 207, 2, 207, 1, 207, 12, 207, 23, 207, 10, 207, 19, 207, 14, 207, 38, 207, 0, 207, 86, 207, 29, 207, 9, 207, 60, 207, 63, 207, 33, 207, 41, 207, 16, 207, 21, 207, 20, 207, 57, 207, 27, 207, 17, 207, 76, 207, 83, 207, 26, 207, 110, 207, 18, 207, 25, 207, 42, 207, 84, 207, 87, 207, 77, 207, 58, 207, 43, 207, 101, 207, 59, 207, 74, 207, 40, 207, 11, 207, 79, 207, 34, 207, 24, 207, 89, 207, 32, 207, 85, 207, 116, 207, 98, 207, 102, 207, 99, 207, 90, 207, 75, 207, 37, 207, 82, 207, 81, 207, 127, 207, 92, 207, 72, 207, 88, 207, 104, 207, 103, 207, 80, 207, 126, 207, 114, 207, 119, 207, 111, 207, 100, 207, 96, 207, 97, 207, 105, 207, 106, 207, 107, 207, 108, 207, 109, 207, 123, 207, 122, 207, 112, 207, 113, 207, 133, 207, 115, 207, 138, 207, 118, 207, 130, 207, 120, 207, 121, 207, 129, 207, 128, 207, 124, 207, 134, 207, 135, 207, 136, 207, 137, 207, 167, 207, 162, 207, 160, 207, 168, 207, 163, 207, 161, 207, 164, 207, 166, 207, 169, 207, 170, 207, 171, 207, 172, 207, 173, 207, 174, 207, 175, 207, 176, 207, 177, 207, 178, 207, 179, 207, 180, 207, 181, 207, 182, 207, 183, 207, 184, 207, 185, 207, 186, 207, 187, 207, 188, 207, 189, 207, 190, 207, 192, 207, 193, 207, 194, 207, 195, 207, 196, 207, 197, 207, 201, 207, 202, 207, 217, 207, 218, 207, 219, 207, 220, 207, 222, 207, 223, 207],
	#[31, 108, 15, 108, 23, 108, 1, 108, 19, 108, 14, 108, 2, 108, 0, 108, 9, 108, 13, 108, 22, 108, 7, 108, 37, 108, 59, -125, 17, 108, 21, 108, 11, 108, 20, 108, 4, 108, 10, 108, 24, 108, 16, 108, 38, 108, 26, 108, 12, 108, 18, 108, 25, 108, 42, 108, 84, 108, 87, 108, 27, 108, 94, 108, 58, -124, 43, 108, 63, 108, 101, 108, 92, 108, 60, 108, 29, 108, 28, 108, 33, 108, 41, 108, 40, 108, 86, 108, 77, 108, 79, 108, 57, -128, 34, 108, 111, 108, 89, 108, 82, 108, 85, 108, 74, 108, 83, 108, 32, 108, 102, 108, 99, 108, 90, 108, 75, 108, 72, 108, 88, 108, 81, 108, 127, 108, 110, 108, 76, 108, 98, 108, 104, 108, 103, 108, 80, 108, 126, 108, 114, 108, 119, 108, 116, 108, 100, 108, 96, 108, 97, 108, 105, 108, 106, 108, 107, 108, 108, 108, 109, 108, 123, 108, 122, 108, 112, 108, 113, 108, 133, 108, 115, 108, 138, 108, 118, 108, 130, 108, 120, 108, 121, 108, 129, 108, 128, 108, 124, 108, 134, 108, 135, 108, 136, 108, 137, 108, 167, 108, 162, 108, 160, 108, 168, -142, 163, 108, 161, 108, 164, 108, 166, 108, 169, -137, 170, -127, 171, -130, 172, -141, 173, -136, 174, -140, 175, -123, 176, -126, 177, -129, 178, -132, 179, -135, 180, -139, 181, -133, 182, -131, 183, -134, 184, -138, 185, 108, 186, 108, 187, 108, 188, 108, 189, 108, 190, 108, 192, 108, 193, 108, 194, 108, 195, 108, 196, 108, 197, 108, 201, 108, 202, 108, 217, 108, 218, 108, 219, 108, 220, 108, 222, 108, 223, 108],
	#[65535, 6, 0, 6, 1, 6, 4, 6, 7, 6, 9, 6, 10, 6, 14, 6, 15, 6, 16, 6, 17, 6, 18, 6, 19, 6, 20, 6, 21, 6, 22, 6, 23, 6, 24, 6, 25, 6, 26, 6, 27, 6, 28, 6, 29, 6, 40, 6, 41, 6, 63, 6, 74, 6, 75, 6, 76, 6, 77, 6, 79, 6, 80, 6, 115, 6, 116, 6, 222, 6, 223, 6],
	#[90, -159, 91, -158],
	#[4, 143, 1, 143, 7, 143, 19, 143, 9, 143, 26, 143, 0, 143, 15, 143, 10, 143, 17, 143, 23, 143, 13, -156, 14, 143, 16, 143, 18, 143, 24, 143, 20, 143, 21, 143, 22, 143, 25, 143, 27, 143, 28, 143, 29, 143, 63, 143, 77, 143, 40, 143, 41, 143, 76, 143, 75, 143, 79, 143, 116, 143, 80, 143, 74, 143, 115, 143, 222, 143, 223, 143],
	#[7, -162],
	#[38, -161],
	#[65535, 145, 7, 145, 14, 145, 0, 145, 1, 145, 4, 145, 19, 145, 18, 145, 9, 145, 10, 145, 16, 145, 13, 145, 15, 145, 17, 145, 20, 145, 21, 145, 22, 145, 23, 145, 24, 145, 25, 145, 26, 145, 27, 145, 28, 145, 29, 145, 40, 145, 41, 145, 63, 145, 74, 145, 75, 145, 76, 145, 77, 145, 79, 145, 80, 145, 115, 145, 116, 145, 222, 145, 223, 145],
	#[65535, 146, 7, 146, 15, 146, 1, 146, 10, 146, 17, 146, 14, 146, 13, 146, 22, 146, 0, 146, 16, 146, 20, 146, 4, 146, 19, 146, 18, 146, 9, 146, 29, 146, 28, 146, 21, 146, 23, 146, 24, 146, 25, 146, 26, 146, 27, 146, 63, 146, 77, 146, 40, 146, 41, 146, 75, 146, 79, 146, 74, 146, 80, 146, 76, 146, 115, 146, 116, 146, 222, 146, 223, 146],
	#[65535, 147, 7, 147, 14, 147, 0, 147, 1, 147, 4, 147, 19, 147, 18, 147, 9, 147, 10, 147, 16, 147, 13, 147, 15, 147, 17, 147, 20, 147, 21, 147, 22, 147, 23, 147, 24, 147, 25, 147, 26, 147, 27, 147, 28, 147, 29, 147, 31, 147, 40, 147, 41, 147, 61, 147, 63, 147, 74, 147, 75, 147, 76, 147, 77, 147, 79, 147, 80, 147, 90, 147, 115, 147, 116, 147, 117, 147, 222, 147, 223, 147],
	#[65535, 144, 0, 144, 1, 144, 4, 144, 7, 144, 9, 144, 10, 144, 14, 144, 15, 144, 16, 144, 17, 144, 18, 144, 19, 144, 20, 144, 21, 144, 22, 144, 23, 144, 24, 144, 25, 144, 26, 144, 27, 144, 28, 144, 29, 144, 40, 144, 41, 144, 63, 144, 74, 144, 75, 144, 76, 144, 77, 144, 79, 144, 80, 144, 115, 144, 116, 144, 222, 144, 223, 144],
	#[77, -47, 3, -165, 78, 117, 80, 117, 79, 117, 72, 117],
	#[7, 19, 14, -4, 11, 19, 16, -3, 19, -25, 18, -5, 25, -14, 15, -24, 17, -17, 20, -11, 21, -36, 22, -20, 23, -6, 24, -28, 26, -38, 31, 19, 33, 19, 35, 19, 36, 19, 63, 19, 64, 19, 65, 19, 66, 19, 67, 19, 68, 19, 69, 19, 70, 19, 71, 19, 73, 19, 92, 19, 94, 19, 115, 19],
	#[72, -50, 78, 105, 79, 105, 80, 105],
	#[79, 119, 78, -49, 80, 119],
	#[79, -53, 80, -54],
	#[31, -59, 7, -37, 36, -65, 70, -72, 69, -68, 35, -67, 68, -64, 63, -63, 64, -73, 11, -66, 71, -55, 33, -60, 67, -61, 66, -58, 65, -71, 73, -70, 115, -10],
	#[31, -220, 63, -219, 37, 105, 84, -204, 10, -212, 87, -221, 94, -210, 82, -186, 7, -37, 85, -205, 102, -178, 11, -195, 38, -200, 99, -218, 92, -189, 33, -202, 88, -208, 42, -93, 72, -50, 81, -176, 86, -224, 98, -192, 101, -214, 103, -228, 83, -236, 104, -185, 89, -235, 100, -193, 96, -182, 97, -217, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[0, 143, 1, 143, 4, 143, 7, 143, 9, 143, 10, 143, 13, -156, 14, 143, 15, 143, 16, 143, 17, 143, 18, 143, 19, 143, 20, 143, 21, 143, 22, 143, 23, 143, 24, 143, 25, 143, 26, 143, 27, 143, 28, 143, 29, 143, 40, 143, 41, 143, 63, 143, 74, 143, 75, 143, 76, 143, 77, 143, 79, 143, 80, 143, 115, 143, 116, 143, 222, 143, 223, 143],
	#[65535, 4, 7, 4, 14, 4, 15, 4, 22, 4, 4, 4, 0, 4, 28, 4, 10, 4, 21, 4, 9, 4, 63, 4, 26, 4, 41, 4, 1, 4, 29, 4, 17, 4, 27, 4, 79, 4, 74, 4, 23, 4, 16, 4, 19, 4, 20, 4, 40, 4, 75, 4, 76, 4, 80, 4, 25, 4, 24, 4, 18, 4, 77, 4, 115, 4, 116, 4, 222, 4, 223, 4],
	#[65535, 178, 7, 178, 11, 178, 31, 178, 33, 178, 35, 178, 36, 178, 42, 178, 63, 178, 64, 178, 65, 178, 66, 178, 67, 178, 68, 178, 69, 178, 70, 178, 71, 178, 73, 178, 115, 178],
	#[65535, 222, 31, 222, 7, 222, 42, 222, 36, 222, 35, 222, 11, 222, 69, 222, 63, 222, 33, 222, 66, 222, 68, 222, 67, 222, 70, 222, 73, 222, 64, 222, 71, 222, 65, 222, 115, 222],
	#[65535, 242, 31, 242, 7, 242, 36, 242, 70, 242, 68, 242, 69, 242, 63, 242, 64, 242, 11, 242, 71, 242, 67, 242, 42, 242, 35, 242, 33, 242, 66, 242, 65, 242, 73, 242, 115, 242],
	#[7, -426],
	#[65535, 154, 15, 154, 23, 154, 1, 154, 14, 154, 7, 154, 19, 154, 9, 154, 21, 154, 36, 154, 20, 154, 4, 154, 10, 154, 17, 154, 16, 154, 62, 154, 12, 154, 18, 154, 11, 154, 22, 154, 0, 154, 27, 154, 94, 154, 58, 154, 26, 154, 63, 154, 101, 154, 92, 154, 74, 154, 24, 154, 29, 154, 25, 154, 32, 154, 13, 154, 40, 154, 43, 154, 59, 154, 79, 154, 57, 154, 34, 154, 111, 154, 60, 154, 41, 154, 28, 154, 80, 154, 42, 154, 98, 154, 102, 154, 99, 154, 90, 154, 75, 154, 77, 154, 127, 154, 110, 154, 76, 154, 159, 154, 104, 154, 103, 154, 93, 154, 126, 154, 114, 154, 119, 154, 116, 154, 100, 154, 96, 154, 97, 154, 105, 154, 106, 154, 107, 154, 108, 154, 109, 154, 123, 154, 122, 154, 112, 154, 113, 154, 133, 154, 115, 154, 138, 154, 118, 154, 130, 154, 120, 154, 121, 154, 129, 154, 128, 154, 124, 154, 134, 154, 135, 154, 136, 154, 137, 154, 167, 154, 162, 154, 160, 154, 163, 154, 161, 154, 164, 154, 165, 154, 166, 154, 185, 154, 186, 154, 187, 154, 189, 154, 190, 154, 192, 154, 193, 154, 194, 154, 195, 154, 196, 154, 197, 154, 198, 154, 199, 154, 200, 154, 201, 154, 202, 154, 203, 154, 204, 154, 205, 154, 206, 154, 207, 154, 208, 154, 209, 154, 217, 154, 218, 154, 219, 154, 220, 154, 222, 154, 223, 154],
	#[42, -357],
	#[42, -421],
	#[65535, 183, 7, 183, 11, 183, 31, 183, 33, 183, 35, 183, 36, 183, 42, 183, 64, 183, 63, 183, 65, 183, 66, 183, 67, 183, 68, 183, 69, 183, 70, 183, 71, 183, 73, 183, 115, 183],
	#[7, 231, 11, 231, 31, 231, 33, 231, 35, 231, 36, 231, 42, 231, 63, 231, 64, 231, 65, 231, 66, 231, 67, 231, 68, 231, 69, 231, 70, 231, 71, 231, 73, 231, 115, 231, 131, -254, 132, -255],
	#[42, -413],
	#[65535, 188, 31, 188, 35, 188, 11, 188, 7, 188, 68, 188, 33, 188, 36, 188, 66, 188, 69, 188, 42, 188, 70, 188, 67, 188, 73, 188, 63, 188, 64, 188, 71, 188, 65, 188, 115, 188],
	#[7, 237, 11, 237, 31, 237, 33, 237, 35, 237, 36, 237, 42, 237, 63, 237, 64, 237, 65, 237, 66, 237, 67, 237, 68, 237, 69, 237, 70, 237, 71, 237, 73, 237, 115, 237, 125, -412],
	#[65535, 177, 31, 177, 70, 177, 36, 177, 69, 177, 33, 177, 68, 177, 63, 177, 7, 177, 42, 177, 11, 177, 64, 177, 35, 177, 65, 177, 67, 177, 71, 177, 66, 177, 73, 177, 115, 177],
	#[65535, 131, 13, 131, 21, 131, 14, 131, 4, 131, 23, 131, 0, 131, 10, 131, 17, 131, 29, 131, 63, 131, 18, 131, 7, 131, 93, 131, 9, 131, 28, 131, 12, 131, 15, 131, 22, 131, 1, 131, 135, 131, 94, 131, 118, 131, 20, 131, 27, 131, 111, 131, 26, 131, 92, 131, 76, 131, 102, 131, 41, 131, 77, 131, 75, 131, 34, 131, 100, 131, 25, 131, 101, 131, 97, 131, 16, 131, 43, 131, 74, 131, 19, 131, 98, 131, 99, 131, 122, 131, 103, 131, 24, 131, 190, 131, 32, 131, 79, 131, 107, 131, 126, 131, 113, 131, 123, 131, 104, 131, 40, 131, 114, 131, 110, 131, 119, 131, 80, 131, 109, 131, 112, 131, 128, 131, 189, 131, 127, 131, 106, 131, 96, 131, 108, 131, 195, 131, 121, 131, 105, 131, 164, 131, 116, 131, 161, 131, 115, 131, 138, 131, 130, 131, 133, 131, 120, 131, 129, 131, 137, 131, 124, 131, 136, 131, 134, 131, 160, 131, 202, 131, 163, 131, 167, 131, 194, 131, 162, 131, 192, 131, 186, 131, 166, 131, 185, 131, 187, 131, 193, 131, 201, 131, 200, 131, 196, 131, 197, 131, 199, 131, 222, 131, 223, 131],
	#[65535, 227, 7, 227, 11, 227, 31, 227, 33, 227, 35, 227, 36, 227, 42, 227, 63, 227, 64, 227, 65, 227, 66, 227, 67, 227, 68, 227, 69, 227, 70, 227, 71, 227, 73, 227, 115, 227],
	#[65535, 241, 31, 241, 63, 241, 7, 241, 11, 241, 71, 241, 68, 241, 42, 241, 65, 241, 70, 241, 33, 241, 67, 241, 69, 241, 35, 241, 36, 241, 64, 241, 66, 241, 73, 241, 115, 241],
	#[65535, 182, 7, 182, 11, 182, 31, 182, 33, 182, 35, 182, 36, 182, 42, 182, 63, 182, 64, 182, 65, 182, 66, 182, 67, 182, 68, 182, 69, 182, 70, 182, 71, 182, 73, 182, 115, 182],
	#[31, 231, 63, 231, 68, 231, 11, 231, 71, 231, 36, 231, 66, 231, 33, 231, 69, 231, 64, 231, 42, 231, 70, 231, 65, 231, 7, 231, 115, 231, 73, 231, 67, 231, 35, 231, 131, -254, 132, -255],
	#[65535, 230, 31, 230, 11, 230, 68, 230, 36, 230, 115, 230, 63, 230, 7, 230, 42, 230, 33, 230, 69, 230, 35, 230, 70, 230, 73, 230, 64, 230, 71, 230, 65, 230, 67, 230, 66, 230],
	#[87, -398, 88, -396, 145, -397, 146, -400, 147, -402, 148, -404, 149, -390, 150, -393, 151, -405, 152, -399, 153, -401, 154, -403, 155, -389, 156, -391, 157, -394, 158, -392],
	#[42, -382],
	#[65535, 187, 31, 187, 63, 187, 7, 187, 11, 187, 42, 187, 70, 187, 69, 187, 33, 187, 68, 187, 35, 187, 36, 187, 65, 187, 64, 187, 67, 187, 71, 187, 66, 187, 73, 187, 115, 187],
	#[7, -37, 11, -66, 12, -379, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 153, 7, 153, 14, 153, 12, 153, 22, 153, 23, 153, 0, 153, 21, 153, 16, 153, 36, 153, 1, 153, 10, 153, 15, 153, 9, 153, 19, 153, 59, 153, 17, 153, 25, 153, 20, 153, 4, 153, 27, 153, 29, 153, 62, 153, 111, 153, 18, 153, 32, 153, 11, 153, 24, 153, 34, 153, 94, 153, 58, 153, 26, 153, 63, 153, 101, 153, 92, 153, 74, 153, 60, 153, 102, 153, 28, 153, 13, 153, 40, 153, 43, 153, 80, 153, 79, 153, 57, 153, 41, 153, 42, 153, 98, 153, 99, 153, 90, 153, 75, 153, 77, 153, 127, 153, 110, 153, 76, 153, 159, 153, 104, 153, 103, 153, 93, 153, 126, 153, 114, 153, 119, 153, 116, 153, 100, 153, 96, 153, 97, 153, 105, 153, 106, 153, 107, 153, 108, 153, 109, 153, 123, 153, 122, 153, 112, 153, 113, 153, 133, 153, 115, 153, 138, 153, 118, 153, 130, 153, 120, 153, 121, 153, 129, 153, 128, 153, 124, 153, 134, 153, 135, 153, 136, 153, 137, 153, 167, 153, 162, 153, 160, 153, 163, 153, 161, 153, 164, 153, 165, 153, 166, 153, 185, 153, 186, 153, 187, 153, 189, 153, 190, 153, 192, 153, 193, 153, 194, 153, 195, 153, 196, 153, 197, 153, 198, 153, 199, 153, 200, 153, 201, 153, 202, 153, 203, 153, 204, 153, 205, 153, 206, 153, 207, 153, 208, 153, 209, 153, 217, 153, 218, 153, 219, 153, 220, 153, 222, 153, 223, 153],
	#[7, 231, 11, 231, 31, 231, 33, 231, 35, 231, 36, 231, 42, 231, 63, 231, 64, 231, 65, 231, 66, 231, 67, 231, 68, 231, 69, 231, 70, 231, 71, 231, 73, 231, 115, 231, 131, -254, 132, -255],
	#[70, 225, 31, 225, 63, 225, 7, 225, 11, 225, 42, 225, 69, 225, 33, 225, 68, 225, 35, 225, 36, 225, 65, 225, 64, 225, 67, 225, 71, 225, 66, 225, 73, 225, 115, 225, 125, -377],
	#[65535, 181, 7, 181, 11, 181, 31, 181, 33, 181, 35, 181, 36, 181, 42, 181, 63, 181, 64, 181, 65, 181, 66, 181, 67, 181, 68, 181, 69, 181, 70, 181, 71, 181, 73, 181, 115, 181],
	#[65535, 137, 15, 137, 0, 137, 1, 137, 4, 137, 7, 137, 9, 137, 10, 137, 12, 137, 13, 137, 14, 137, 26, 137, 16, 137, 17, 137, 18, 137, 19, 137, 20, 137, 21, 137, 22, 137, 23, 137, 24, 137, 25, 137, 27, 137, 28, 137, 29, 137, 32, 137, 34, 137, 40, 137, 41, 137, 43, 137, 63, 137, 74, 137, 75, 137, 76, 137, 77, 137, 79, 137, 80, 137, 92, 137, 93, 137, 94, 137, 96, 137, 97, 137, 98, 137, 99, 137, 100, 137, 101, 137, 102, 137, 103, 137, 104, 137, 105, 137, 106, 137, 107, 137, 108, 137, 109, 137, 110, 137, 111, 137, 112, 137, 113, 137, 114, 137, 115, 137, 116, 137, 118, 137, 119, 137, 120, 137, 121, 137, 122, 137, 123, 137, 124, 137, 126, 137, 127, 137, 128, 137, 129, 137, 130, 137, 133, 137, 134, 137, 135, 137, 136, 137, 137, 137, 138, 137, 160, 137, 161, 137, 162, 137, 163, 137, 164, 137, 166, 137, 167, 137, 185, 137, 186, 137, 187, 137, 189, 137, 190, 137, 192, 137, 193, 137, 194, 137, 195, 137, 196, 137, 197, 137, 199, 137, 200, 137, 201, 137, 202, 137, 222, 137, 223, 137],
	#[65535, 220, 7, 220, 31, 220, 11, 220, 42, 220, 33, 220, 63, 220, 35, 220, 36, 220, 66, 220, 64, 220, 65, 220, 73, 220, 67, 220, 68, 220, 69, 220, 70, 220, 71, 220, 115, 220],
	#[31, -370, 63, -63, 7, -37, 70, -72, 66, -58, 35, -67, 11, -66, 67, -61, 36, -65, 73, -70, 33, -60, 64, -73, 71, -55, 68, -64, 65, -71, 69, -68, 115, -10],
	#[65535, 229, 7, 229, 11, 229, 31, 229, 33, 229, 35, 229, 36, 229, 42, 229, 63, 229, 64, 229, 65, 229, 66, 229, 67, 229, 68, 229, 69, 229, 70, 229, 71, 229, 73, 229, 115, 229],
	#[65535, 135, 4, 135, 7, 135, 14, 135, 12, 135, 15, 135, 22, 135, 27, 135, 13, 135, 102, 135, 77, 135, 21, 135, 9, 135, 63, 135, 101, 135, 0, 135, 18, 135, 29, 135, 17, 135, 10, 135, 92, 135, 109, 135, 1, 135, 28, 135, 116, 135, 107, 135, 23, 135, 111, 135, 19, 135, 20, 135, 40, 135, 26, 135, 32, 135, 119, 135, 74, 135, 43, 135, 93, 135, 118, 135, 16, 135, 99, 135, 25, 135, 75, 135, 80, 135, 103, 135, 98, 135, 24, 135, 76, 135, 79, 135, 114, 135, 100, 135, 96, 135, 94, 135, 166, 135, 190, 135, 106, 135, 113, 135, 104, 135, 127, 135, 34, 135, 41, 135, 110, 135, 97, 135, 105, 135, 108, 135, 112, 135, 115, 135, 130, 135, 121, 135, 120, 135, 122, 135, 123, 135, 124, 135, 126, 135, 138, 135, 128, 135, 129, 135, 164, 135, 133, 135, 134, 135, 135, 135, 136, 135, 137, 135, 167, 135, 189, 135, 161, 135, 195, 135, 202, 135, 160, 135, 194, 135, 162, 135, 163, 135, 186, 135, 192, 135, 185, 135, 187, 135, 193, 135, 201, 135, 200, 135, 196, 135, 197, 135, 199, 135, 222, 135, 223, 135],
	#[65535, 136, 0, 136, 1, 136, 4, 136, 7, 136, 9, 136, 10, 136, 12, 136, 13, 136, 14, 136, 15, 136, 16, 136, 17, 136, 18, 136, 19, 136, 20, 136, 21, 136, 22, 136, 23, 136, 24, 136, 25, 136, 26, 136, 27, 136, 28, 136, 29, 136, 32, 136, 34, 136, 40, 136, 41, 136, 43, 136, 63, 136, 74, 136, 75, 136, 76, 136, 77, 136, 79, 136, 80, 136, 92, 136, 93, 136, 94, 136, 96, 136, 97, 136, 98, 136, 99, 136, 100, 136, 101, 136, 102, 136, 103, 136, 104, 136, 105, 136, 106, 136, 107, 136, 108, 136, 109, 136, 110, 136, 111, 136, 112, 136, 113, 136, 114, 136, 115, 136, 116, 136, 118, 136, 119, 136, 120, 136, 121, 136, 122, 136, 123, 136, 124, 136, 126, 136, 127, 136, 128, 136, 129, 136, 130, 136, 133, 136, 134, 136, 135, 136, 136, 136, 137, 136, 138, 136, 160, 136, 161, 136, 162, 136, 163, 136, 164, 136, 166, 136, 167, 136, 185, 136, 186, 136, 187, 136, 189, 136, 190, 136, 192, 136, 193, 136, 194, 136, 195, 136, 196, 136, 197, 136, 199, 136, 200, 136, 201, 136, 202, 136, 222, 136, 223, 136],
	#[65535, 186, 31, 186, 42, 186, 68, 186, 70, 186, 36, 186, 63, 186, 7, 186, 35, 186, 11, 186, 33, 186, 71, 186, 66, 186, 64, 186, 67, 186, 73, 186, 65, 186, 69, 186, 115, 186],
	#[65535, 431, 12, 431, 21, 431, 18, 431, 15, 431, 4, 431, 9, 431, 14, 431, 62, 431, 11, 431, 10, 431, 0, 431, 1, 431, 36, 431, 20, 431, 26, 431, 63, 431, 19, 431, 7, 431, 76, 431, 60, 431, 17, 431, 16, 431, 34, 431, 13, 431, 27, 431, 103, 431, 43, 431, 25, 431, 59, 431, 79, 431, 22, 431, 23, 431, 24, 431, 41, 431, 28, 431, 29, 431, 74, 431, 42, 431, 32, 431, 102, 431, 97, 431, 40, 431, 57, 431, 77, 431, 101, 431, 58, 431, 94, 431, 75, 431, 80, 431, 190, 431, 111, 431, 93, 431, 105, 431, 90, 431, 92, 431, 104, 431, 99, 431, 119, 431, 106, 431, 115, 431, 110, 431, 98, 431, 96, 431, 112, 431, 107, 431, 127, 431, 114, 431, 109, 431, 118, 431, 108, 431, 113, 431, 116, 431, 100, 431, 130, 431, 121, 431, 120, 431, 122, 431, 123, 431, 124, 431, 126, 431, 138, 431, 128, 431, 129, 431, 164, 431, 133, 431, 134, 431, 135, 431, 136, 431, 137, 431, 167, 431, 189, 431, 186, 431, 161, 431, 195, 431, 202, 431, 159, 431, 160, 431, 194, 431, 162, 431, 163, 431, 203, 431, 165, 431, 166, 431, 192, 431, 185, 431, 187, 431, 193, 431, 201, 431, 200, 431, 196, 431, 197, 431, 198, 431, 199, 431, 204, 431, 205, 431, 206, 431, 207, 431, 208, 431, 209, 431, 217, 431, 218, 431, 219, 431, 220, 431, 222, 431, 223, 431],
	#[65535, 140, 7, 140, 14, 140, 0, 140, 1, 140, 4, 140, 19, 140, 18, 140, 9, 140, 10, 140, 16, 140, 12, 140, 13, 140, 15, 140, 17, 140, 20, 140, 21, 140, 22, 140, 23, 140, 24, 140, 25, 140, 26, 140, 27, 140, 28, 140, 29, 140, 32, 140, 34, 140, 40, 140, 41, 140, 43, 140, 77, 140, 75, 140, 63, 140, 74, 140, 76, 140, 79, 140, 80, 140, 98, 140, 92, 140, 93, 140, 94, 140, 96, 140, 97, 140, 99, 140, 100, 140, 101, 140, 102, 140, 103, 140, 104, 140, 105, 140, 106, 140, 107, 140, 108, 140, 109, 140, 110, 140, 111, 140, 112, 140, 113, 140, 114, 140, 115, 140, 116, 140, 118, 140, 119, 140, 120, 140, 121, 140, 122, 140, 123, 140, 124, 140, 126, 140, 127, 140, 128, 140, 129, 140, 130, 140, 133, 140, 134, 140, 135, 140, 136, 140, 137, 140, 138, 140, 160, 140, 161, 140, 162, 140, 163, 140, 164, 140, 166, 140, 167, 140, 185, 140, 186, 140, 187, 140, 189, 140, 190, 140, 192, 140, 193, 140, 194, 140, 195, 140, 196, 140, 197, 140, 199, 140, 200, 140, 201, 140, 202, 140, 222, 140, 223, 140],
	#[7, -348, 31, -347, 38, -346],
	#[42, 417, 221, -303],
	#[65535, 134, 0, 134, 1, 134, 4, 134, 7, 134, 9, 134, 10, 134, 12, 134, 13, 134, 14, 134, 15, 134, 16, 134, 17, 134, 18, 134, 19, 134, 20, 134, 21, 134, 22, 134, 23, 134, 24, 134, 25, 134, 26, 134, 27, 134, 28, 134, 29, 134, 32, 134, 34, 134, 40, 134, 41, 134, 43, 134, 63, 134, 74, 134, 75, 134, 76, 134, 77, 134, 79, 134, 80, 134, 92, 134, 93, 134, 94, 134, 96, 134, 97, 134, 98, 134, 99, 134, 100, 134, 101, 134, 102, 134, 103, 134, 104, 134, 105, 134, 106, 134, 107, 134, 108, 134, 109, 134, 110, 134, 111, 134, 112, 134, 113, 134, 114, 134, 115, 134, 116, 134, 118, 134, 119, 134, 120, 134, 121, 134, 122, 134, 123, 134, 124, 134, 126, 134, 127, 134, 128, 134, 129, 134, 130, 134, 133, 134, 134, 134, 135, 134, 136, 134, 137, 134, 138, 134, 160, 134, 161, 134, 162, 134, 163, 134, 164, 134, 166, 134, 167, 134, 185, 134, 186, 134, 187, 134, 189, 134, 190, 134, 192, 134, 193, 134, 194, 134, 195, 134, 196, 134, 197, 134, 199, 134, 200, 134, 201, 134, 202, 134, 222, 134, 223, 134],
	#[65535, 152, 4, 152, 11, 152, 1, 152, 0, 152, 10, 152, 7, 152, 34, 152, 19, 152, 9, 152, 23, 152, 12, 152, 13, 152, 14, 152, 15, 152, 16, 152, 17, 152, 18, 152, 24, 152, 20, 152, 21, 152, 22, 152, 25, 152, 26, 152, 27, 152, 28, 152, 29, 152, 43, 152, 32, 152, 59, 152, 63, 152, 36, 152, 62, 152, 40, 152, 41, 152, 42, 152, 60, 152, 58, 152, 57, 152, 127, 152, 74, 152, 75, 152, 76, 152, 77, 152, 79, 152, 80, 152, 99, 152, 98, 152, 90, 152, 92, 152, 93, 152, 94, 152, 96, 152, 97, 152, 100, 152, 101, 152, 102, 152, 103, 152, 104, 152, 105, 152, 106, 152, 107, 152, 108, 152, 109, 152, 110, 152, 111, 152, 112, 152, 113, 152, 114, 152, 115, 152, 116, 152, 118, 152, 119, 152, 120, 152, 121, 152, 122, 152, 123, 152, 124, 152, 126, 152, 138, 152, 128, 152, 129, 152, 130, 152, 133, 152, 134, 152, 135, 152, 136, 152, 137, 152, 167, 152, 159, 152, 160, 152, 161, 152, 162, 152, 163, 152, 164, 152, 165, 152, 166, 152, 205, 152, 206, 152, 185, 152, 186, 152, 187, 152, 189, 152, 190, 152, 192, 152, 193, 152, 194, 152, 195, 152, 196, 152, 197, 152, 198, 152, 199, 152, 200, 152, 201, 152, 202, 152, 203, 152, 204, 152, 207, 152, 208, 152, 209, 152, 217, 152, 218, 152, 219, 152, 220, 152, 222, 152, 223, 152],
	#[31, 239, 7, 239, 11, 239, 42, 239, 33, 239, 35, 239, 36, 239, 71, 239, 66, 239, 70, 239, 69, 239, 68, 239, 67, 239, 65, 239, 64, 239, 63, 239, 73, 239, 115, 239, 125, -302],
	#[42, -295],
	#[42, -290],
	#[65535, 228, 7, 228, 11, 228, 31, 228, 33, 228, 35, 228, 36, 228, 42, 228, 63, 228, 64, 228, 65, 228, 66, 228, 67, 228, 68, 228, 69, 228, 70, 228, 71, 228, 73, 228, 115, 228],
	#[139, -277, 140, -279, 141, -282, 142, -284, 143, -274, 144, -275, 145, -276, 146, -278, 147, -281, 148, -283],
	#[42, -269],
	#[65535, 151, 0, 151, 1, 151, 4, 151, 7, 151, 9, 151, 10, 151, 11, 151, 12, 151, 13, 151, 14, 151, 15, 151, 16, 151, 17, 151, 18, 151, 19, 151, 20, 151, 21, 151, 22, 151, 23, 151, 24, 151, 25, 151, 26, 151, 27, 151, 28, 151, 29, 151, 32, 151, 34, 151, 36, 151, 40, 151, 41, 151, 42, 151, 43, 151, 57, 151, 58, 151, 59, 151, 60, 151, 62, 151, 63, 151, 74, 151, 75, 151, 76, 151, 77, 151, 79, 151, 80, 151, 90, 151, 92, 151, 93, 151, 94, 151, 96, 151, 97, 151, 98, 151, 99, 151, 100, 151, 101, 151, 102, 151, 103, 151, 104, 151, 105, 151, 106, 151, 107, 151, 108, 151, 109, 151, 110, 151, 111, 151, 112, 151, 113, 151, 114, 151, 115, 151, 116, 151, 118, 151, 119, 151, 120, 151, 121, 151, 122, 151, 123, 151, 124, 151, 126, 151, 127, 151, 128, 151, 129, 151, 130, 151, 133, 151, 134, 151, 135, 151, 136, 151, 137, 151, 138, 151, 159, 151, 160, 151, 161, 151, 162, 151, 163, 151, 164, 151, 165, 151, 166, 151, 167, 151, 185, 151, 186, 151, 187, 151, 189, 151, 190, 151, 192, 151, 193, 151, 194, 151, 195, 151, 196, 151, 197, 151, 198, 151, 199, 151, 200, 151, 201, 151, 202, 151, 203, 151, 204, 151, 205, 151, 206, 151, 207, 151, 208, 151, 209, 151, 217, 151, 218, 151, 219, 151, 220, 151, 222, 151, 223, 151],
	#[31, -59, 63, -63, 7, -37, 32, -264, 11, -66, 33, -60, 36, -65, 35, -67, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 139, 18, 139, 0, 139, 1, 139, 4, 139, 7, 139, 9, 139, 10, 139, 12, 139, 13, 139, 14, 139, 15, 139, 16, 139, 17, 139, 25, 139, 19, 139, 20, 139, 21, 139, 22, 139, 23, 139, 24, 139, 26, 139, 27, 139, 28, 139, 29, 139, 32, 139, 34, 139, 40, 139, 41, 139, 43, 139, 63, 139, 74, 139, 75, 139, 76, 139, 77, 139, 79, 139, 80, 139, 92, 139, 93, 139, 94, 139, 96, 139, 97, 139, 98, 139, 99, 139, 100, 139, 101, 139, 102, 139, 103, 139, 104, 139, 105, 139, 106, 139, 107, 139, 108, 139, 109, 139, 110, 139, 111, 139, 112, 139, 113, 139, 114, 139, 115, 139, 116, 139, 118, 139, 119, 139, 120, 139, 121, 139, 122, 139, 123, 139, 124, 139, 126, 139, 127, 139, 128, 139, 129, 139, 130, 139, 133, 139, 134, 139, 135, 139, 136, 139, 137, 139, 138, 139, 160, 139, 161, 139, 162, 139, 163, 139, 164, 139, 166, 139, 167, 139, 185, 139, 186, 139, 187, 139, 189, 139, 190, 139, 192, 139, 193, 139, 194, 139, 195, 139, 196, 139, 197, 139, 199, 139, 200, 139, 201, 139, 202, 139, 222, 139, 223, 139],
	#[65535, 185, 7, 185, 11, 185, 31, 185, 33, 185, 35, 185, 36, 185, 42, 185, 63, 185, 64, 185, 65, 185, 66, 185, 67, 185, 68, 185, 69, 185, 70, 185, 71, 185, 73, 185, 115, 185],
	#[42, -258],
	#[65535, 138, 0, 138, 1, 138, 4, 138, 7, 138, 9, 138, 10, 138, 12, 138, 13, 138, 14, 138, 15, 138, 16, 138, 17, 138, 18, 138, 19, 138, 20, 138, 21, 138, 22, 138, 23, 138, 24, 138, 25, 138, 26, 138, 27, 138, 28, 138, 29, 138, 32, 138, 34, 138, 40, 138, 41, 138, 43, 138, 63, 138, 74, 138, 75, 138, 76, 138, 77, 138, 79, 138, 80, 138, 92, 138, 93, 138, 94, 138, 96, 138, 97, 138, 98, 138, 99, 138, 100, 138, 101, 138, 102, 138, 103, 138, 104, 138, 105, 138, 106, 138, 107, 138, 108, 138, 109, 138, 110, 138, 111, 138, 112, 138, 113, 138, 114, 138, 115, 138, 116, 138, 118, 138, 119, 138, 120, 138, 121, 138, 122, 138, 123, 138, 124, 138, 126, 138, 127, 138, 128, 138, 129, 138, 130, 138, 133, 138, 134, 138, 135, 138, 136, 138, 137, 138, 138, 138, 160, 138, 161, 138, 162, 138, 163, 138, 164, 138, 166, 138, 167, 138, 185, 138, 186, 138, 187, 138, 189, 138, 190, 138, 192, 138, 193, 138, 194, 138, 195, 138, 196, 138, 197, 138, 199, 138, 200, 138, 201, 138, 202, 138, 222, 138, 223, 138],
	#[65535, 142, 24, 142, 20, 142, 32, 142, 40, 142, 41, 142, 76, 142, 43, 142, 74, 142, 75, 142, 77, 142, 79, 142, 80, 142, 99, 142, 98, 142, 92, 142, 93, 142, 94, 142, 96, 142, 97, 142, 100, 142, 101, 142, 102, 142, 103, 142, 104, 142, 105, 142, 106, 142, 107, 142, 108, 142, 109, 142, 110, 142, 111, 142, 112, 142, 113, 142, 114, 142, 115, 142, 116, 142, 118, 142, 119, 142, 120, 142, 121, 142, 122, 142, 123, 142, 124, 142, 126, 142, 127, 142, 128, 142, 129, 142, 130, 142, 133, 142, 134, 142, 135, 142, 136, 142, 137, 142, 138, 142, 160, 142, 161, 142, 162, 142, 163, 142, 164, 142, 166, 142, 167, 142, 185, 142, 186, 142, 187, 142, 189, 142, 190, 142, 192, 142, 193, 142, 194, 142, 195, 142, 196, 142, 197, 142, 199, 142, 200, 142, 201, 142, 202, 142, 222, 142, 223, 142, 4, 142, 7, 142, 14, 142, 22, 142, 23, 142, 13, 142, 18, 142, 10, 142, 21, 142, 63, 142, 1, 142, 28, 142, 25, 142, 34, 142, 19, 142, 0, 142, 15, 142, 9, 142, 29, 142, 12, 142, 17, 142, 27, 142, 26, 142, 16, 142],
	#[7, 231, 11, 231, 31, 231, 33, 231, 35, 231, 36, 231, 42, 231, 63, 231, 64, 231, 65, 231, 66, 231, 67, 231, 68, 231, 69, 231, 70, 231, 71, 231, 73, 231, 115, 231, 131, -254, 132, -255],
	#[65535, 179, 7, 179, 11, 179, 31, 179, 33, 179, 35, 179, 36, 179, 42, 179, 63, 179, 64, 179, 65, 179, 66, 179, 67, 179, 68, 179, 69, 179, 70, 179, 71, 179, 73, 179, 115, 179],
	#[42, -243],
	#[65535, 180, 7, 180, 11, 180, 31, 180, 33, 180, 35, 180, 36, 180, 42, 180, 63, 180, 64, 180, 65, 180, 66, 180, 67, 180, 68, 180, 69, 180, 70, 180, 71, 180, 73, 180, 115, 180],
	#[65535, 243, 69, 243, 31, 243, 7, 243, 71, 243, 42, 243, 35, 243, 11, 243, 63, 243, 70, 243, 66, 243, 33, 243, 73, 243, 64, 243, 67, 243, 65, 243, 68, 243, 36, 243, 115, 243],
	#[65535, 218, 31, 218, 63, 218, 7, 218, 11, 218, 42, 218, 70, 218, 69, 218, 33, 218, 68, 218, 35, 218, 36, 218, 65, 218, 64, 218, 67, 218, 71, 218, 66, 218, 73, 218, 115, 218],
	#[7, 223, 11, 223, 31, 223, 33, 223, 35, 223, 36, 223, 42, 223, 63, 223, 64, 223, 65, 223, 66, 223, 67, 223, 68, 223, 69, 223, 70, 223, 71, 223, 73, 223, 115, 223, 125, -242],
	#[65535, 184, 31, 184, 7, 184, 11, 184, 33, 184, 42, 184, 63, 184, 35, 184, 36, 184, 71, 184, 65, 184, 70, 184, 64, 184, 69, 184, 68, 184, 73, 184, 67, 184, 66, 184, 115, 184],
	#[65535, 133, 0, 133, 1, 133, 4, 133, 7, 133, 9, 133, 10, 133, 12, 133, 13, 133, 14, 133, 15, 133, 16, 133, 17, 133, 18, 133, 19, 133, 20, 133, 21, 133, 22, 133, 23, 133, 24, 133, 25, 133, 26, 133, 27, 133, 28, 133, 29, 133, 32, 133, 34, 133, 40, 133, 41, 133, 43, 133, 63, 133, 74, 133, 75, 133, 76, 133, 77, 133, 79, 133, 80, 133, 92, 133, 93, 133, 94, 133, 96, 133, 97, 133, 98, 133, 99, 133, 100, 133, 101, 133, 102, 133, 103, 133, 104, 133, 105, 133, 106, 133, 107, 133, 108, 133, 109, 133, 110, 133, 111, 133, 112, 133, 113, 133, 114, 133, 115, 133, 116, 133, 118, 133, 119, 133, 120, 133, 121, 133, 122, 133, 123, 133, 124, 133, 126, 133, 127, 133, 128, 133, 129, 133, 130, 133, 133, 133, 134, 133, 135, 133, 136, 133, 137, 133, 138, 133, 160, 133, 161, 133, 162, 133, 163, 133, 164, 133, 166, 133, 167, 133, 185, 133, 186, 133, 187, 133, 189, 133, 190, 133, 192, 133, 193, 133, 194, 133, 195, 133, 196, 133, 197, 133, 199, 133, 200, 133, 201, 133, 202, 133, 222, 133, 223, 133],
	#[42, -237],
	#[65535, 132, 14, 132, 23, 132, 10, 132, 4, 132, 7, 132, 1, 132, 21, 132, 74, 132, 103, 132, 9, 132, 29, 132, 79, 132, 12, 132, 16, 132, 0, 132, 27, 132, 94, 132, 17, 132, 63, 132, 19, 132, 18, 132, 76, 132, 24, 132, 102, 132, 25, 132, 34, 132, 13, 132, 15, 132, 43, 132, 20, 132, 99, 132, 22, 132, 75, 132, 92, 132, 26, 132, 41, 132, 28, 132, 32, 132, 93, 132, 77, 132, 40, 132, 98, 132, 101, 132, 80, 132, 104, 132, 100, 132, 96, 132, 97, 132, 105, 132, 106, 132, 107, 132, 108, 132, 109, 132, 110, 132, 111, 132, 112, 132, 113, 132, 114, 132, 115, 132, 116, 132, 118, 132, 119, 132, 120, 132, 121, 132, 122, 132, 123, 132, 124, 132, 126, 132, 127, 132, 128, 132, 129, 132, 130, 132, 133, 132, 134, 132, 135, 132, 136, 132, 137, 132, 138, 132, 160, 132, 161, 132, 162, 132, 163, 132, 164, 132, 166, 132, 167, 132, 189, 132, 185, 132, 186, 132, 187, 132, 190, 132, 192, 132, 193, 132, 194, 132, 195, 132, 196, 132, 197, 132, 199, 132, 200, 132, 201, 132, 202, 132, 222, 132, 223, 132],
	#[116, -9],
	#[13, -239],
	#[7, -37, 115, -10],
	#[43, -241],
	#[65535, 141, 21, 141, 0, 141, 1, 141, 4, 141, 7, 141, 9, 141, 10, 141, 12, 141, 13, 141, 14, 141, 15, 141, 16, 141, 17, 141, 18, 141, 19, 141, 20, 141, 22, 141, 23, 141, 24, 141, 25, 141, 26, 141, 27, 141, 28, 141, 29, 141, 32, 141, 34, 141, 40, 141, 41, 141, 43, 141, 63, 141, 74, 141, 75, 141, 76, 141, 77, 141, 79, 141, 80, 141, 92, 141, 93, 141, 94, 141, 96, 141, 97, 141, 98, 141, 99, 141, 100, 141, 101, 141, 102, 141, 103, 141, 104, 141, 105, 141, 106, 141, 107, 141, 108, 141, 109, 141, 110, 141, 111, 141, 112, 141, 113, 141, 114, 141, 115, 141, 116, 141, 118, 141, 119, 141, 120, 141, 121, 141, 122, 141, 123, 141, 124, 141, 126, 141, 127, 141, 128, 141, 129, 141, 130, 141, 133, 141, 134, 141, 135, 141, 136, 141, 137, 141, 138, 141, 160, 141, 161, 141, 162, 141, 163, 141, 164, 141, 166, 141, 167, 141, 185, 141, 186, 141, 187, 141, 189, 141, 190, 141, 192, 141, 193, 141, 194, 141, 195, 141, 196, 141, 197, 141, 199, 141, 200, 141, 201, 141, 202, 141, 222, 141, 223, 141],
	#[65535, 224, 31, 224, 7, 224, 11, 224, 42, 224, 33, 224, 63, 224, 35, 224, 36, 224, 64, 224, 65, 224, 66, 224, 67, 224, 68, 224, 69, 224, 70, 224, 71, 224, 73, 224, 115, 224],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -245],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -248],
	#[13, -251, 43, -250],
	#[38, -249],
	#[65535, 189, 63, 189, 7, 189, 13, 189, 43, 189, 99, 189, 98, 189, 92, 189, 94, 189, 96, 189, 97, 189, 100, 189, 101, 189, 102, 189, 103, 189, 104, 189, 105, 189, 106, 189, 107, 189, 108, 189, 109, 189, 110, 189, 111, 189, 112, 189, 113, 189, 114, 189, 115, 189, 118, 189, 119, 189, 120, 189, 121, 189, 122, 189, 123, 189, 124, 189, 126, 189, 127, 189, 128, 189, 129, 189, 130, 189, 133, 189, 134, 189, 135, 189, 136, 189, 137, 189, 138, 189, 160, 189, 161, 189, 162, 189, 163, 189, 164, 189, 166, 189, 167, 189, 185, 189, 186, 189, 187, 189, 189, 189, 190, 189, 192, 189, 193, 189, 194, 189, 195, 189, 196, 189, 197, 189, 201, 189, 202, 189],
	#[65535, 176, 9, 176, 15, 176, 10, 176, 14, 176, 23, 176, 12, 176, 11, 176, 22, 176, 0, 176, 1, 176, 21, 176, 17, 176, 4, 176, 19, 176, 7, 176, 24, 176, 29, 176, 16, 176, 32, 176, 13, 176, 27, 176, 26, 176, 62, 176, 18, 176, 36, 176, 20, 176, 79, 176, 57, 176, 34, 176, 92, 176, 25, 176, 60, 176, 41, 176, 28, 176, 43, 176, 42, 176, 98, 176, 59, 176, 63, 176, 58, 176, 77, 176, 40, 176, 110, 176, 76, 176, 159, 176, 101, 176, 103, 176, 90, 176, 99, 176, 94, 176, 75, 176, 80, 176, 74, 176, 111, 176, 104, 176, 93, 176, 102, 176, 126, 176, 114, 176, 119, 176, 116, 176, 100, 176, 96, 176, 97, 176, 127, 176, 105, 176, 106, 176, 107, 176, 108, 176, 109, 176, 123, 176, 122, 176, 112, 176, 113, 176, 133, 176, 115, 176, 138, 176, 118, 176, 130, 176, 120, 176, 121, 176, 129, 176, 128, 176, 124, 176, 134, 176, 135, 176, 136, 176, 137, 176, 167, 176, 162, 176, 160, 176, 163, 176, 161, 176, 164, 176, 165, 176, 166, 176, 185, 176, 186, 176, 187, 176, 189, 176, 190, 176, 192, 176, 193, 176, 194, 176, 195, 176, 196, 176, 197, 176, 198, 176, 199, 176, 200, 176, 201, 176, 202, 176, 203, 176, 204, 176, 205, 176, 206, 176, 207, 176, 208, 176, 209, 176, 217, 176, 218, 176, 219, 176, 220, 176, 222, 176, 223, 176],
	#[38, -252],
	#[65535, 190, 7, 190, 13, 190, 43, 190, 63, 190, 98, 190, 92, 190, 94, 190, 96, 190, 97, 190, 99, 190, 100, 190, 101, 190, 102, 190, 103, 190, 104, 190, 105, 190, 106, 190, 107, 190, 108, 190, 109, 190, 110, 190, 111, 190, 112, 190, 113, 190, 114, 190, 115, 190, 118, 190, 119, 190, 120, 190, 121, 190, 122, 190, 123, 190, 124, 190, 126, 190, 127, 190, 128, 190, 129, 190, 130, 190, 133, 190, 134, 190, 135, 190, 136, 190, 137, 190, 138, 190, 160, 190, 161, 190, 162, 190, 163, 190, 164, 190, 166, 190, 167, 190, 185, 190, 186, 190, 187, 190, 189, 190, 190, 190, 192, 190, 193, 190, 194, 190, 195, 190, 196, 190, 197, 190, 201, 190, 202, 190],
	#[65535, 219, 7, 219, 11, 219, 31, 219, 33, 219, 35, 219, 36, 219, 42, 219, 63, 219, 64, 219, 65, 219, 66, 219, 67, 219, 68, 219, 69, 219, 70, 219, 71, 219, 73, 219, 115, 219],
	#[31, 232, 70, 232, 11, 232, 36, 232, 69, 232, 67, 232, 73, 232, 35, 232, 63, 232, 7, 232, 64, 232, 71, 232, 66, 232, 42, 232, 68, 232, 65, 232, 115, 232, 33, 232, 132, -257],
	#[7, 233, 11, 233, 31, 233, 33, 233, 35, 233, 36, 233, 42, 233, 63, 233, 64, 233, 65, 233, 66, 233, 67, 233, 68, 233, 69, 233, 70, 233, 71, 233, 73, 233, 115, 233, 131, -256],
	#[65535, 235, 7, 235, 11, 235, 31, 235, 33, 235, 35, 235, 36, 235, 42, 235, 63, 235, 64, 235, 65, 235, 66, 235, 67, 235, 68, 235, 69, 235, 70, 235, 71, 235, 73, 235, 115, 235],
	#[65535, 234, 31, 234, 63, 234, 7, 234, 11, 234, 42, 234, 70, 234, 69, 234, 33, 234, 68, 234, 35, 234, 36, 234, 65, 234, 64, 234, 67, 234, 71, 234, 66, 234, 73, 234, 115, 234],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -260],
	#[31, -59, 63, -63, 7, -37, 11, -66, 36, -65, 35, -67, 33, -60, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[43, -262],
	#[65535, 169, 23, 169, 13, 169, 22, 169, 21, 169, 14, 169, 20, 169, 4, 169, 0, 169, 60, 169, 28, 169, 12, 169, 10, 169, 43, 169, 29, 169, 1, 169, 63, 169, 18, 169, 7, 169, 92, 169, 62, 169, 77, 169, 58, 169, 25, 169, 15, 169, 24, 169, 206, 169, 16, 169, 9, 169, 11, 169, 79, 169, 40, 169, 90, 169, 111, 169, 26, 169, 76, 169, 93, 169, 59, 169, 41, 169, 98, 169, 75, 169, 34, 169, 100, 169, 101, 169, 80, 169, 36, 169, 57, 169, 127, 169, 42, 169, 94, 169, 109, 169, 17, 169, 27, 169, 102, 169, 19, 169, 32, 169, 97, 169, 74, 169, 96, 169, 103, 169, 99, 169, 104, 169, 105, 169, 106, 169, 107, 169, 108, 169, 110, 169, 112, 169, 113, 169, 114, 169, 115, 169, 116, 169, 118, 169, 119, 169, 120, 169, 121, 169, 122, 169, 123, 169, 124, 169, 126, 169, 138, 169, 128, 169, 129, 169, 130, 169, 133, 169, 134, 169, 135, 169, 136, 169, 137, 169, 167, 169, 159, 169, 160, 169, 161, 169, 162, 169, 163, 169, 164, 169, 165, 169, 166, 169, 205, 169, 189, 169, 186, 169, 185, 169, 187, 169, 190, 169, 192, 169, 193, 169, 194, 169, 195, 169, 196, 169, 197, 169, 198, 169, 199, 169, 200, 169, 201, 169, 202, 169, 203, 169, 204, 169, 207, 169, 208, 169, 209, 169, 217, 169, 218, 169, 219, 169, 220, 169, 222, 169, 223, 169],
	#[65535, 163, 12, 163, 13, 163, 32, 163, 34, 163],
	#[65535, 128, 0, 128, 1, 128, 4, 128, 7, 128, 9, 128, 10, 128, 12, 128, 13, 128, 14, 128, 15, 128, 16, 128, 17, 128, 18, 128, 19, 128, 20, 128, 21, 128, 22, 128, 23, 128, 24, 128, 25, 128, 26, 128, 27, 128, 28, 128, 29, 128, 32, 128, 34, 128, 40, 128, 41, 128, 43, 128, 63, 128, 74, 128, 75, 128, 76, 128, 77, 128, 79, 128, 80, 128, 92, 128, 93, 128, 94, 128, 96, 128, 97, 128, 98, 128, 99, 128, 100, 128, 101, 128, 102, 128, 103, 128, 104, 128, 105, 128, 106, 128, 107, 128, 108, 128, 109, 128, 110, 128, 111, 128, 112, 128, 113, 128, 114, 128, 115, 128, 116, 128, 118, 128, 119, 128, 120, 128, 121, 128, 122, 128, 123, 128, 124, 128, 126, 128, 127, 128, 128, 128, 129, 128, 130, 128, 133, 128, 134, 128, 135, 128, 136, 128, 137, 128, 138, 128, 160, 128, 161, 128, 162, 128, 163, 128, 164, 128, 166, 128, 167, 128, 185, 128, 186, 128, 187, 128, 189, 128, 190, 128, 192, 128, 193, 128, 194, 128, 195, 128, 196, 128, 197, 128, 199, 128, 200, 128, 201, 128, 202, 128, 222, 128, 223, 128],
	#[13, -267, 32, -266],
	#[65535, 127, 34, 127, 4, 127, 1, 127, 25, 127, 7, 127, 19, 127, 9, 127, 0, 127, 15, 127, 10, 127, 17, 127, 23, 127, 12, 127, 18, 127, 13, 127, 14, 127, 26, 127, 16, 127, 24, 127, 20, 127, 21, 127, 22, 127, 27, 127, 28, 127, 29, 127, 32, 127, 63, 127, 40, 127, 41, 127, 43, 127, 74, 127, 75, 127, 76, 127, 77, 127, 79, 127, 80, 127, 99, 127, 98, 127, 92, 127, 93, 127, 94, 127, 96, 127, 97, 127, 100, 127, 101, 127, 102, 127, 103, 127, 104, 127, 105, 127, 106, 127, 107, 127, 108, 127, 109, 127, 110, 127, 111, 127, 112, 127, 113, 127, 114, 127, 115, 127, 116, 127, 118, 127, 119, 127, 120, 127, 121, 127, 122, 127, 123, 127, 124, 127, 126, 127, 127, 127, 128, 127, 129, 127, 130, 127, 133, 127, 134, 127, 135, 127, 136, 127, 137, 127, 138, 127, 160, 127, 161, 127, 162, 127, 163, 127, 164, 127, 166, 127, 167, 127, 186, 127, 185, 127, 187, 127, 189, 127, 190, 127, 192, 127, 193, 127, 194, 127, 195, 127, 196, 127, 197, 127, 199, 127, 200, 127, 201, 127, 202, 127, 222, 127, 223, 127],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 164, 12, 164, 34, 164, 13, 164, 32, 164],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -271],
	#[7, -37, 11, -66, 71, -55, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 73, -70, 115, -10],
	#[43, -273],
	#[65535, 172, 14, 172, 7, 172, 22, 172, 21, 172, 23, 172, 20, 172, 4, 172, 29, 172, 13, 172, 43, 172, 34, 172, 1, 172, 58, 172, 26, 172, 25, 172, 36, 172, 74, 172, 103, 172, 10, 172, 76, 172, 79, 172, 62, 172, 9, 172, 11, 172, 0, 172, 27, 172, 94, 172, 63, 172, 19, 172, 18, 172, 24, 172, 102, 172, 16, 172, 12, 172, 41, 172, 15, 172, 17, 172, 59, 172, 99, 172, 57, 172, 75, 172, 92, 172, 60, 172, 32, 172, 28, 172, 42, 172, 98, 172, 40, 172, 93, 172, 77, 172, 101, 172, 80, 172, 90, 172, 104, 172, 100, 172, 96, 172, 97, 172, 105, 172, 106, 172, 107, 172, 108, 172, 109, 172, 110, 172, 111, 172, 112, 172, 113, 172, 114, 172, 115, 172, 116, 172, 118, 172, 119, 172, 120, 172, 121, 172, 122, 172, 123, 172, 124, 172, 126, 172, 127, 172, 128, 172, 129, 172, 130, 172, 133, 172, 134, 172, 135, 172, 136, 172, 137, 172, 138, 172, 159, 172, 160, 172, 161, 172, 162, 172, 163, 172, 164, 172, 165, 172, 166, 172, 167, 172, 189, 172, 185, 172, 186, 172, 187, 172, 190, 172, 192, 172, 193, 172, 194, 172, 195, 172, 196, 172, 197, 172, 198, 172, 199, 172, 200, 172, 201, 172, 202, 172, 203, 172, 204, 172, 205, 172, 206, 172, 207, 172, 208, 172, 209, 172, 217, 172, 218, 172, 219, 172, 220, 172, 222, 172, 223, 172],
	#[65535, 248, 31, 248, 63, 248, 7, 248, 11, 248, 71, 248, 68, 248, 42, 248, 65, 248, 70, 248, 33, 248, 67, 248, 69, 248, 35, 248, 36, 248, 64, 248, 66, 248, 73, 248, 115, 248],
	#[65535, 249, 7, 249, 11, 249, 31, 249, 33, 249, 35, 249, 36, 249, 42, 249, 63, 249, 64, 249, 65, 249, 66, 249, 67, 249, 68, 249, 69, 249, 70, 249, 71, 249, 73, 249, 115, 249],
	#[65535, 250, 31, 250, 63, 250, 7, 250, 11, 250, 71, 250, 67, 250, 42, 250, 70, 250, 69, 250, 33, 250, 68, 250, 35, 250, 36, 250, 65, 250, 64, 250, 66, 250, 73, 250, 115, 250],
	#[65535, 244, 31, 244, 7, 244, 11, 244, 42, 244, 33, 244, 35, 244, 36, 244, 63, 244, 64, 244, 65, 244, 66, 244, 67, 244, 68, 244, 69, 244, 70, 244, 71, 244, 73, 244, 115, 244],
	#[65535, 251, 31, 251, 7, 251, 11, 251, 35, 251, 33, 251, 42, 251, 36, 251, 63, 251, 64, 251, 65, 251, 66, 251, 67, 251, 68, 251, 69, 251, 70, 251, 71, 251, 73, 251, 115, 251],
	#[65535, 245, 7, 245, 31, 245, 63, 245, 11, 245, 71, 245, 68, 245, 42, 245, 65, 245, 70, 245, 33, 245, 67, 245, 69, 245, 35, 245, 36, 245, 64, 245, 66, 245, 73, 245, 115, 245],
	#[42, -285],
	#[65535, 252, 7, 252, 11, 252, 35, 252, 31, 252, 33, 252, 36, 252, 42, 252, 63, 252, 64, 252, 65, 252, 66, 252, 67, 252, 68, 252, 69, 252, 70, 252, 71, 252, 73, 252, 115, 252],
	#[65535, 246, 31, 246, 7, 246, 11, 246, 42, 246, 33, 246, 35, 246, 36, 246, 63, 246, 64, 246, 65, 246, 66, 246, 67, 246, 68, 246, 69, 246, 70, 246, 71, 246, 73, 246, 115, 246],
	#[65535, 253, 7, 253, 11, 253, 31, 253, 33, 253, 35, 253, 36, 253, 42, 253, 63, 253, 64, 253, 65, 253, 66, 253, 67, 253, 68, 253, 69, 253, 70, 253, 71, 253, 73, 253, 115, 253],
	#[65535, 247, 31, 247, 63, 247, 7, 247, 11, 247, 71, 247, 67, 247, 42, 247, 70, 247, 69, 247, 33, 247, 68, 247, 35, 247, 36, 247, 65, 247, 64, 247, 66, 247, 73, 247, 115, 247],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -287],
	#[31, -59, 63, -63, 7, -37, 11, -66, 71, -55, 70, -72, 67, -61, 69, -68, 33, -60, 68, -64, 35, -67, 36, -65, 64, -73, 65, -71, 66, -58, 73, -70, 115, -10],
	#[43, -289],
	#[65535, 170, 10, 170, 4, 170, 22, 170, 43, 170, 34, 170, 36, 170, 1, 170, 27, 170, 14, 170, 18, 170, 11, 170, 0, 170, 20, 170, 25, 170, 19, 170, 7, 170, 9, 170, 29, 170, 16, 170, 12, 170, 13, 170, 15, 170, 17, 170, 42, 170, 21, 170, 32, 170, 23, 170, 24, 170, 40, 170, 26, 170, 41, 170, 28, 170, 57, 170, 58, 170, 59, 170, 60, 170, 62, 170, 63, 170, 74, 170, 75, 170, 76, 170, 77, 170, 79, 170, 80, 170, 98, 170, 90, 170, 92, 170, 93, 170, 94, 170, 96, 170, 97, 170, 99, 170, 100, 170, 101, 170, 102, 170, 103, 170, 104, 170, 105, 170, 106, 170, 107, 170, 108, 170, 109, 170, 110, 170, 111, 170, 112, 170, 113, 170, 114, 170, 115, 170, 116, 170, 118, 170, 119, 170, 120, 170, 121, 170, 122, 170, 123, 170, 124, 170, 126, 170, 127, 170, 128, 170, 129, 170, 130, 170, 133, 170, 134, 170, 135, 170, 136, 170, 137, 170, 138, 170, 159, 170, 160, 170, 161, 170, 162, 170, 163, 170, 164, 170, 165, 170, 166, 170, 167, 170, 185, 170, 186, 170, 187, 170, 189, 170, 190, 170, 192, 170, 193, 170, 194, 170, 195, 170, 196, 170, 197, 170, 198, 170, 199, 170, 200, 170, 201, 170, 202, 170, 203, 170, 204, 170, 205, 170, 206, 170, 207, 170, 208, 170, 209, 170, 217, 170, 218, 170, 219, 170, 220, 170, 222, 170, 223, 170],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[93, -292],
	#[7, -37, 11, -66, 31, -59, 33, -60, 63, -63, 35, -67, 36, -65, 71, -55, 65, -71, 70, -72, 64, -73, 69, -68, 68, -64, 67, -61, 66, -58, 73, -70, 115, -10],
	#[37, 105, 42, -93, 43, -294, 72, -50],
	#[65535, 165, 10, 165, 36, 165, 15, 165, 1, 165, 27, 165, 34, 165, 14, 165, 23, 165, 18, 165, 0, 165, 4, 165, 19, 165, 7, 165, 9, 165, 29, 165, 11, 165, 12, 165, 13, 165, 26, 165, 16, 165, 17, 165, 25, 165, 24, 165, 20, 165, 21, 165, 22, 165, 32, 165, 28, 165, 40, 165, 41, 165, 42, 165, 43, 165, 59, 165, 57, 165, 58, 165, 60, 165, 62, 165, 63, 165, 74, 165, 75, 165, 76, 165, 77, 165, 79, 165, 80, 165, 90, 165, 92, 165, 93, 165, 94, 165, 96, 165, 97, 165, 98, 165, 99, 165, 100, 165, 101, 165, 102, 165, 103, 165, 104, 165, 105, 165, 106, 165, 107, 165, 108, 165, 109, 165, 110, 165, 111, 165, 112, 165, 113, 165, 114, 165, 115, 165, 116, 165, 118, 165, 119, 165, 120, 165, 121, 165, 122, 165, 123, 165, 124, 165, 126, 165, 127, 165, 128, 165, 129, 165, 130, 165, 133, 165, 134, 165, 135, 165, 136, 165, 137, 165, 138, 165, 159, 165, 160, 165, 161, 165, 162, 165, 163, 165, 164, 165, 165, 165, 166, 165, 167, 165, 185, 165, 186, 165, 187, 165, 189, 165, 190, 165, 192, 165, 193, 165, 194, 165, 195, 165, 196, 165, 197, 165, 198, 165, 199, 165, 200, 165, 201, 165, 202, 165, 203, 165, 204, 165, 205, 165, 206, 165, 207, 165, 208, 165, 209, 165, 217, 165, 218, 165, 219, 165, 220, 165, 222, 165, 223, 165],
	#[31, -59, 36, -65, 7, -37, 11, -66, 33, -60, 35, -67, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -297],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -299],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 66, -58, 63, -63, 64, -73, 65, -71, 73, -70, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 115, -10],
	#[43, -301],
	#[65535, 174, 15, 174, 0, 174, 1, 174, 4, 174, 7, 174, 9, 174, 10, 174, 11, 174, 12, 174, 13, 174, 14, 174, 26, 174, 16, 174, 17, 174, 18, 174, 19, 174, 20, 174, 21, 174, 22, 174, 23, 174, 24, 174, 25, 174, 27, 174, 28, 174, 29, 174, 32, 174, 34, 174, 36, 174, 40, 174, 41, 174, 42, 174, 43, 174, 57, 174, 58, 174, 59, 174, 60, 174, 62, 174, 63, 174, 74, 174, 75, 174, 76, 174, 77, 174, 79, 174, 80, 174, 90, 174, 92, 174, 93, 174, 94, 174, 96, 174, 97, 174, 98, 174, 99, 174, 100, 174, 101, 174, 102, 174, 103, 174, 104, 174, 105, 174, 106, 174, 107, 174, 108, 174, 109, 174, 110, 174, 111, 174, 112, 174, 113, 174, 114, 174, 115, 174, 116, 174, 118, 174, 119, 174, 120, 174, 121, 174, 122, 174, 123, 174, 124, 174, 126, 174, 127, 174, 128, 174, 129, 174, 130, 174, 133, 174, 134, 174, 135, 174, 136, 174, 137, 174, 138, 174, 159, 174, 160, 174, 161, 174, 162, 174, 163, 174, 164, 174, 165, 174, 166, 174, 167, 174, 185, 174, 186, 174, 187, 174, 189, 174, 190, 174, 192, 174, 193, 174, 194, 174, 195, 174, 196, 174, 197, 174, 198, 174, 199, 174, 200, 174, 201, 174, 202, 174, 203, 174, 204, 174, 205, 174, 206, 174, 207, 174, 208, 174, 209, 174, 217, 174, 218, 174, 219, 174, 220, 174, 222, 174, 223, 174],
	#[65535, 240, 31, 240, 7, 240, 11, 240, 35, 240, 33, 240, 42, 240, 66, 240, 63, 240, 36, 240, 115, 240, 64, 240, 65, 240, 73, 240, 67, 240, 68, 240, 69, 240, 70, 240, 71, 240],
	#[65535, 418, 31, 418, 63, 418, 7, 418, 69, 418, 11, 418, 70, 418, 71, 418, 73, 418, 33, 418, 42, 418, 66, 418, 67, 418, 68, 418, 35, 418, 36, 418, 65, 418, 64, 418, 115, 418],
	#[42, -305],
	#[7, -37, 11, -66, 35, -67, 31, -59, 33, -60, 63, -63, 36, -65, 66, -58, 64, -73, 65, -71, 73, -70, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 115, -10],
	#[65535, 215, 13, 215, 43, 215],
	#[13, -309, 43, -308],
	#[65535, 166, 11, 166, 15, 166, 1, 166, 10, 166, 14, 166, 79, 166, 23, 166, 9, 166, 22, 166, 0, 166, 16, 166, 21, 166, 17, 166, 4, 166, 19, 166, 7, 166, 24, 166, 29, 166, 28, 166, 12, 166, 13, 166, 27, 166, 26, 166, 62, 166, 32, 166, 18, 166, 36, 166, 20, 166, 57, 166, 34, 166, 92, 166, 25, 166, 60, 166, 41, 166, 43, 166, 42, 166, 98, 166, 59, 166, 63, 166, 58, 166, 77, 166, 40, 166, 110, 166, 76, 166, 159, 166, 101, 166, 103, 166, 90, 166, 99, 166, 94, 166, 75, 166, 80, 166, 74, 166, 111, 166, 104, 166, 93, 166, 102, 166, 126, 166, 114, 166, 119, 166, 116, 166, 100, 166, 96, 166, 97, 166, 127, 166, 105, 166, 106, 166, 107, 166, 108, 166, 109, 166, 123, 166, 122, 166, 112, 166, 113, 166, 133, 166, 115, 166, 138, 166, 118, 166, 130, 166, 120, 166, 121, 166, 129, 166, 128, 166, 124, 166, 134, 166, 135, 166, 136, 166, 137, 166, 167, 166, 162, 166, 160, 166, 163, 166, 161, 166, 164, 166, 165, 166, 166, 166, 185, 166, 186, 166, 187, 166, 189, 166, 190, 166, 192, 166, 193, 166, 194, 166, 195, 166, 196, 166, 197, 166, 198, 166, 199, 166, 200, 166, 201, 166, 202, 166, 203, 166, 204, 166, 205, 166, 206, 166, 207, 166, 208, 166, 209, 166, 217, 166, 218, 166, 219, 166, 220, 166, 222, 166, 223, 166],
	#[7, -37, 11, -66, 31, -59, 33, -60, 63, -63, 35, -67, 36, -65, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 216, 7, 216, 13, 216, 63, 216, 43, 216, 99, 216, 98, 216, 92, 216, 94, 216, 96, 216, 97, 216, 100, 216, 101, 216, 102, 216, 103, 216, 104, 216, 105, 216, 106, 216, 107, 216, 108, 216, 109, 216, 110, 216, 111, 216, 112, 216, 113, 216, 114, 216, 115, 216, 118, 216, 119, 216, 120, 216, 121, 216, 122, 216, 123, 216, 124, 216, 126, 216, 127, 216, 128, 216, 129, 216, 130, 216, 133, 216, 134, 216, 135, 216, 136, 216, 137, 216, 138, 216, 160, 216, 161, 216, 162, 216, 163, 216, 164, 216, 166, 216, 167, 216, 185, 216, 186, 216, 187, 216, 189, 216, 190, 216, 192, 216, 193, 216, 194, 216, 195, 216, 196, 216, 197, 216, 201, 216, 202, 216],
	#[31, -326, 2, -327, 7, -37, 10, -212, 11, -318, 37, 105, 33, -321, 38, -316, 42, -93, 63, -219, 72, -50, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[7, -431],
	#[65535, 380, 36, 380, 94, 380, 63, 380, 7, 380, 11, 380, 12, 380, 13, 380, 57, 380, 59, 380, 167, 380, 92, 380, 101, 380, 60, 380, 32, 380, 43, 380, 98, 380, 102, 380, 99, 380, 90, 380, 58, 380, 62, 380, 127, 380, 110, 380, 42, 380, 159, 380, 104, 380, 103, 380, 93, 380, 126, 380, 114, 380, 119, 380, 111, 380, 100, 380, 96, 380, 97, 380, 105, 380, 106, 380, 107, 380, 108, 380, 109, 380, 123, 380, 122, 380, 112, 380, 113, 380, 133, 380, 115, 380, 118, 380, 130, 380, 120, 380, 121, 380, 129, 380, 128, 380, 124, 380, 134, 380, 135, 380, 136, 380, 137, 380, 138, 380, 162, 380, 166, 380, 160, 380, 165, 380, 163, 380, 161, 380, 164, 380, 185, 380, 186, 380, 187, 380, 189, 380, 190, 380, 192, 380, 193, 380, 194, 380, 195, 380, 196, 380, 197, 380, 198, 380, 199, 380, 200, 380, 201, 380, 202, 380, 203, 380, 204, 380, 205, 380, 206, 380, 207, 380, 208, 380, 209, 380, 217, 380, 218, 380, 219, 380, 220, 380],
	#[65535, 306, 7, 306, 13, 306, 43, 306, 32, 306, 62, 306, 63, 306, 167, 306, 98, 306, 92, 306, 93, 306, 94, 306, 96, 306, 97, 306, 99, 306, 100, 306, 101, 306, 102, 306, 103, 306, 104, 306, 105, 306, 106, 306, 107, 306, 108, 306, 109, 306, 110, 306, 111, 306, 112, 306, 113, 306, 114, 306, 115, 306, 118, 306, 119, 306, 120, 306, 121, 306, 122, 306, 123, 306, 124, 306, 126, 306, 127, 306, 128, 306, 129, 306, 130, 306, 133, 306, 134, 306, 135, 306, 136, 306, 137, 306, 138, 306, 159, 306, 160, 306, 161, 306, 162, 306, 163, 306, 164, 306, 166, 306, 190, 306, 195, 306, 194, 306, 185, 306, 186, 306, 187, 306, 189, 306, 203, 306, 202, 306, 192, 306, 193, 306, 201, 306, 200, 306, 196, 306, 197, 306, 198, 306, 199, 306, 204, 306, 205, 306, 206, 306, 207, 306, 208, 306, 209, 306],
	#[65535, 381, 7, 381, 63, 381, 92, 381, 60, 381, 11, 381, 12, 381, 13, 381, 58, 381, 43, 381, 36, 381, 59, 381, 57, 381, 94, 381, 32, 381, 98, 381, 102, 381, 126, 381, 93, 381, 62, 381, 110, 381, 42, 381, 159, 381, 101, 381, 103, 381, 90, 381, 99, 381, 118, 381, 115, 381, 109, 381, 104, 381, 107, 381, 114, 381, 119, 381, 111, 381, 100, 381, 96, 381, 97, 381, 127, 381, 105, 381, 106, 381, 108, 381, 123, 381, 122, 381, 112, 381, 113, 381, 133, 381, 129, 381, 138, 381, 128, 381, 130, 381, 120, 381, 121, 381, 124, 381, 134, 381, 135, 381, 136, 381, 137, 381, 167, 381, 162, 381, 166, 381, 160, 381, 165, 381, 163, 381, 161, 381, 164, 381, 185, 381, 186, 381, 187, 381, 189, 381, 190, 381, 192, 381, 193, 381, 194, 381, 195, 381, 196, 381, 197, 381, 198, 381, 199, 381, 200, 381, 201, 381, 202, 381, 203, 381, 204, 381, 205, 381, 206, 381, 207, 381, 208, 381, 209, 381, 217, 381, 218, 381, 219, 381, 220, 381],
	#[65535, 377, 7, 377, 11, 377, 12, 377, 13, 377, 36, 377, 32, 377, 42, 377, 43, 377, 57, 377, 58, 377, 59, 377, 60, 377, 62, 377, 63, 377, 90, 377, 92, 377, 93, 377, 94, 377, 96, 377, 97, 377, 98, 377, 99, 377, 100, 377, 101, 377, 102, 377, 103, 377, 104, 377, 105, 377, 106, 377, 107, 377, 108, 377, 109, 377, 110, 377, 111, 377, 112, 377, 113, 377, 114, 377, 115, 377, 118, 377, 119, 377, 120, 377, 121, 377, 122, 377, 123, 377, 124, 377, 126, 377, 127, 377, 128, 377, 129, 377, 130, 377, 133, 377, 134, 377, 135, 377, 136, 377, 137, 377, 138, 377, 159, 377, 160, 377, 161, 377, 162, 377, 163, 377, 164, 377, 165, 377, 166, 377, 167, 377, 185, 377, 186, 377, 187, 377, 189, 377, 190, 377, 192, 377, 193, 377, 194, 377, 195, 377, 196, 377, 197, 377, 198, 377, 199, 377, 200, 377, 201, 377, 202, 377, 203, 377, 204, 377, 205, 377, 206, 377, 207, 377, 208, 377, 209, 377, 217, 377, 218, 377, 219, 377, 220, 377],
	#[65535, 376, 7, 376, 11, 376, 12, 376, 13, 376, 43, 376, 32, 376, 63, 376, 36, 376, 42, 376, 60, 376, 101, 376, 103, 376, 58, 376, 94, 376, 57, 376, 111, 376, 93, 376, 102, 376, 90, 376, 92, 376, 59, 376, 99, 376, 119, 376, 62, 376, 105, 376, 110, 376, 98, 376, 96, 376, 112, 376, 107, 376, 97, 376, 114, 376, 109, 376, 118, 376, 106, 376, 104, 376, 115, 376, 113, 376, 100, 376, 108, 376, 121, 376, 120, 376, 122, 376, 123, 376, 124, 376, 126, 376, 127, 376, 128, 376, 129, 376, 130, 376, 133, 376, 134, 376, 135, 376, 136, 376, 137, 376, 138, 376, 159, 376, 160, 376, 161, 376, 162, 376, 163, 376, 164, 376, 165, 376, 166, 376, 167, 376, 185, 376, 186, 376, 187, 376, 189, 376, 190, 376, 192, 376, 193, 376, 194, 376, 195, 376, 196, 376, 197, 376, 198, 376, 199, 376, 200, 376, 201, 376, 202, 376, 203, 376, 204, 376, 205, 376, 206, 376, 207, 376, 208, 376, 209, 376, 217, 376, 218, 376, 219, 376, 220, 376],
	#[31, -59, 63, -63, 7, -37, 11, -66, 12, -428, 36, -65, 35, -67, 33, -60, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 310, 92, 310, 93, 310, 42, 310, 11, 310, 13, 310, 63, 310, 7, 310, 58, 310, 36, 310, 103, 310, 90, 310, 60, 310, 62, 310, 59, 310, 12, 310, 32, 310, 100, 310, 98, 310, 94, 310, 119, 310, 166, 310, 108, 310, 106, 310, 57, 310, 104, 310, 102, 310, 96, 310, 112, 310, 97, 310, 159, 310, 114, 310, 107, 310, 110, 310, 135, 310, 130, 310, 99, 310, 113, 310, 122, 310, 111, 310, 101, 310, 118, 310, 43, 310, 109, 310, 126, 310, 105, 310, 121, 310, 128, 310, 127, 310, 161, 310, 123, 310, 115, 310, 138, 310, 165, 310, 133, 310, 120, 310, 129, 310, 137, 310, 124, 310, 134, 310, 160, 310, 136, 310, 163, 310, 167, 310, 162, 310, 164, 310, 185, 310, 186, 310, 187, 310, 189, 310, 190, 310, 192, 310, 193, 310, 194, 310, 195, 310, 196, 310, 197, 310, 198, 310, 199, 310, 200, 310, 201, 310, 202, 310, 203, 310, 204, 310, 205, 310, 206, 310, 207, 310, 208, 310, 209, 310, 217, 310, 218, 310, 219, 310, 220, 310],
	#[65535, 379, 7, 379, 11, 379, 12, 379, 13, 379, 32, 379, 36, 379, 42, 379, 43, 379, 57, 379, 58, 379, 59, 379, 60, 379, 62, 379, 63, 379, 90, 379, 92, 379, 93, 379, 94, 379, 96, 379, 97, 379, 98, 379, 99, 379, 100, 379, 101, 379, 102, 379, 103, 379, 104, 379, 105, 379, 106, 379, 107, 379, 108, 379, 109, 379, 110, 379, 111, 379, 112, 379, 113, 379, 114, 379, 115, 379, 118, 379, 119, 379, 120, 379, 121, 379, 122, 379, 123, 379, 124, 379, 126, 379, 127, 379, 128, 379, 129, 379, 130, 379, 133, 379, 134, 379, 135, 379, 136, 379, 137, 379, 138, 379, 159, 379, 160, 379, 161, 379, 162, 379, 163, 379, 164, 379, 165, 379, 166, 379, 167, 379, 185, 379, 186, 379, 187, 379, 189, 379, 190, 379, 192, 379, 193, 379, 194, 379, 195, 379, 196, 379, 197, 379, 198, 379, 199, 379, 200, 379, 201, 379, 202, 379, 203, 379, 204, 379, 205, 379, 206, 379, 207, 379, 208, 379, 209, 379, 217, 379, 218, 379, 219, 379, 220, 379],
	#[7, -37, 11, -66, 31, -362, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 383, 7, 383, 11, 383, 12, 383, 13, 383, 103, 383, 43, 383, 32, 383, 63, 383, 36, 383, 42, 383, 60, 383, 101, 383, 58, 383, 94, 383, 57, 383, 111, 383, 93, 383, 102, 383, 90, 383, 92, 383, 59, 383, 99, 383, 119, 383, 62, 383, 105, 383, 110, 383, 98, 383, 96, 383, 112, 383, 107, 383, 97, 383, 114, 383, 109, 383, 118, 383, 106, 383, 104, 383, 115, 383, 113, 383, 100, 383, 108, 383, 121, 383, 120, 383, 122, 383, 123, 383, 124, 383, 126, 383, 127, 383, 128, 383, 129, 383, 130, 383, 133, 383, 134, 383, 135, 383, 136, 383, 137, 383, 138, 383, 159, 383, 160, 383, 161, 383, 162, 383, 163, 383, 164, 383, 165, 383, 166, 383, 167, 383, 185, 383, 186, 383, 187, 383, 189, 383, 190, 383, 192, 383, 193, 383, 194, 383, 195, 383, 196, 383, 197, 383, 198, 383, 199, 383, 200, 383, 201, 383, 202, 383, 203, 383, 204, 383, 205, 383, 206, 383, 207, 383, 208, 383, 209, 383, 217, 383, 218, 383, 219, 383, 220, 383],
	#[65535, 395, 7, 395, 11, 395, 12, 395, 13, 395, 36, 395, 43, 395, 32, 395, 63, 395, 58, 395, 42, 395, 60, 395, 101, 395, 103, 395, 99, 395, 94, 395, 57, 395, 111, 395, 93, 395, 102, 395, 90, 395, 92, 395, 59, 395, 119, 395, 62, 395, 105, 395, 110, 395, 98, 395, 96, 395, 112, 395, 107, 395, 97, 395, 114, 395, 109, 395, 118, 395, 106, 395, 104, 395, 115, 395, 113, 395, 100, 395, 108, 395, 121, 395, 120, 395, 122, 395, 123, 395, 124, 395, 126, 395, 127, 395, 128, 395, 129, 395, 130, 395, 133, 395, 134, 395, 135, 395, 136, 395, 137, 395, 138, 395, 159, 395, 160, 395, 161, 395, 162, 395, 163, 395, 164, 395, 165, 395, 166, 395, 167, 395, 185, 395, 186, 395, 187, 395, 189, 395, 190, 395, 192, 395, 193, 395, 194, 395, 195, 395, 196, 395, 197, 395, 198, 395, 199, 395, 200, 395, 201, 395, 202, 395, 203, 395, 204, 395, 205, 395, 206, 395, 207, 395, 208, 395, 209, 395, 217, 395, 218, 395, 219, 395, 220, 395],
	#[65535, 393, 7, 393, 11, 393, 12, 393, 13, 393, 43, 393, 32, 393, 63, 393, 36, 393, 42, 393, 60, 393, 103, 393, 58, 393, 94, 393, 57, 393, 111, 393, 93, 393, 102, 393, 90, 393, 92, 393, 59, 393, 99, 393, 119, 393, 62, 393, 101, 393, 110, 393, 98, 393, 96, 393, 112, 393, 107, 393, 97, 393, 114, 393, 109, 393, 118, 393, 106, 393, 104, 393, 115, 393, 105, 393, 113, 393, 100, 393, 108, 393, 121, 393, 120, 393, 122, 393, 123, 393, 124, 393, 126, 393, 127, 393, 128, 393, 129, 393, 130, 393, 133, 393, 134, 393, 135, 393, 136, 393, 137, 393, 138, 393, 159, 393, 160, 393, 161, 393, 162, 393, 163, 393, 164, 393, 165, 393, 166, 393, 167, 393, 185, 393, 186, 393, 187, 393, 189, 393, 190, 393, 192, 393, 193, 393, 194, 393, 195, 393, 196, 393, 197, 393, 198, 393, 199, 393, 200, 393, 201, 393, 202, 393, 203, 393, 204, 393, 205, 393, 206, 393, 207, 393, 208, 393, 209, 393, 217, 393, 218, 393, 219, 393, 220, 393],
	#[65535, 378, 7, 378, 11, 378, 12, 378, 13, 378, 43, 378, 32, 378, 63, 378, 36, 378, 42, 378, 60, 378, 103, 378, 58, 378, 94, 378, 57, 378, 111, 378, 93, 378, 102, 378, 90, 378, 92, 378, 59, 378, 99, 378, 119, 378, 62, 378, 101, 378, 110, 378, 98, 378, 96, 378, 112, 378, 107, 378, 97, 378, 114, 378, 109, 378, 118, 378, 106, 378, 104, 378, 115, 378, 105, 378, 113, 378, 100, 378, 108, 378, 121, 378, 120, 378, 122, 378, 123, 378, 124, 378, 126, 378, 127, 378, 128, 378, 129, 378, 130, 378, 133, 378, 134, 378, 135, 378, 136, 378, 137, 378, 138, 378, 159, 378, 160, 378, 161, 378, 162, 378, 163, 378, 164, 378, 165, 378, 166, 378, 167, 378, 185, 378, 186, 378, 187, 378, 189, 378, 190, 378, 192, 378, 193, 378, 194, 378, 195, 378, 196, 378, 197, 378, 198, 378, 199, 378, 200, 378, 201, 378, 202, 378, 203, 378, 204, 378, 205, 378, 206, 378, 207, 378, 208, 378, 209, 378, 217, 378, 218, 378, 219, 378, 220, 378],
	#[70, -72, 31, -59, 63, -63, 7, -37, 11, -66, 32, -343, 67, -61, 36, -65, 35, -67, 64, -73, 33, -60, 66, -58, 68, -64, 65, -71, 73, -70, 71, -55, 69, -68, 115, -10],
	#[7, 396, 181, -336, 216, -338],
	#[65535, 309, 7, 309, 11, 309, 12, 309, 13, 309, 32, 309, 36, 309, 42, 309, 43, 309, 57, 309, 58, 309, 59, 309, 60, 309, 62, 309, 63, 309, 90, 309, 92, 309, 93, 309, 94, 309, 96, 309, 97, 309, 98, 309, 99, 309, 100, 309, 101, 309, 102, 309, 103, 309, 104, 309, 105, 309, 106, 309, 107, 309, 108, 309, 109, 309, 110, 309, 111, 309, 112, 309, 113, 309, 114, 309, 115, 309, 118, 309, 119, 309, 120, 309, 121, 309, 122, 309, 123, 309, 124, 309, 126, 309, 127, 309, 128, 309, 129, 309, 130, 309, 133, 309, 134, 309, 135, 309, 136, 309, 137, 309, 138, 309, 159, 309, 160, 309, 161, 309, 162, 309, 163, 309, 164, 309, 165, 309, 166, 309, 167, 309, 185, 309, 186, 309, 187, 309, 189, 309, 190, 309, 192, 309, 193, 309, 194, 309, 195, 309, 196, 309, 197, 309, 198, 309, 199, 309, 200, 309, 201, 309, 202, 309, 203, 309, 204, 309, 205, 309, 206, 309, 207, 309, 208, 309, 209, 309, 217, 309, 218, 309, 219, 309, 220, 309],
	#[42, -331],
	#[65535, 382, 7, 382, 11, 382, 12, 382, 13, 382, 32, 382, 36, 382, 42, 382, 43, 382, 57, 382, 58, 382, 59, 382, 60, 382, 62, 382, 63, 382, 98, 382, 90, 382, 92, 382, 93, 382, 94, 382, 96, 382, 97, 382, 99, 382, 100, 382, 101, 382, 102, 382, 103, 382, 104, 382, 105, 382, 106, 382, 107, 382, 108, 382, 109, 382, 110, 382, 111, 382, 112, 382, 113, 382, 114, 382, 115, 382, 118, 382, 119, 382, 120, 382, 121, 382, 122, 382, 123, 382, 124, 382, 126, 382, 127, 382, 128, 382, 129, 382, 130, 382, 133, 382, 134, 382, 135, 382, 136, 382, 137, 382, 138, 382, 159, 382, 160, 382, 161, 382, 162, 382, 163, 382, 164, 382, 165, 382, 166, 382, 167, 382, 185, 382, 186, 382, 187, 382, 189, 382, 190, 382, 192, 382, 193, 382, 194, 382, 195, 382, 196, 382, 197, 382, 198, 382, 199, 382, 200, 382, 201, 382, 202, 382, 203, 382, 204, 382, 205, 382, 206, 382, 207, 382, 208, 382, 209, 382, 217, 382, 218, 382, 219, 382, 220, 382],
	#[116, -9],
	#[13, -333],
	#[7, -37, 115, -10],
	#[43, -335],
	#[65535, 392, 36, 392, 7, 392, 11, 392, 12, 392, 13, 392, 32, 392, 42, 392, 43, 392, 57, 392, 58, 392, 59, 392, 60, 392, 62, 392, 63, 392, 98, 392, 90, 392, 92, 392, 93, 392, 94, 392, 96, 392, 97, 392, 99, 392, 100, 392, 101, 392, 102, 392, 103, 392, 104, 392, 105, 392, 106, 392, 107, 392, 108, 392, 109, 392, 110, 392, 111, 392, 112, 392, 113, 392, 114, 392, 115, 392, 118, 392, 119, 392, 120, 392, 121, 392, 122, 392, 123, 392, 124, 392, 126, 392, 127, 392, 128, 392, 129, 392, 130, 392, 133, 392, 134, 392, 135, 392, 136, 392, 137, 392, 138, 392, 159, 392, 160, 392, 161, 392, 162, 392, 163, 392, 164, 392, 165, 392, 166, 392, 167, 392, 185, 392, 186, 392, 187, 392, 189, 392, 190, 392, 192, 392, 193, 392, 194, 392, 195, 392, 196, 392, 197, 392, 198, 392, 199, 392, 200, 392, 201, 392, 202, 392, 203, 392, 204, 392, 205, 392, 206, 392, 207, 392, 208, 392, 209, 392, 217, 392, 218, 392, 219, 392, 220, 392],
	#[65535, 398, 7, 398],
	#[7, -340],
	#[7, 397, 181, -339],
	#[65535, 399, 7, 399],
	#[13, -341],
	#[7, -342],
	#[65535, 394, 7, 394, 11, 394, 12, 394, 13, 394, 32, 394, 59, 394, 36, 394, 42, 394, 43, 394, 57, 394, 58, 394, 60, 394, 62, 394, 63, 394, 98, 394, 90, 394, 92, 394, 93, 394, 94, 394, 96, 394, 97, 394, 99, 394, 100, 394, 101, 394, 102, 394, 103, 394, 104, 394, 105, 394, 106, 394, 107, 394, 108, 394, 109, 394, 110, 394, 111, 394, 112, 394, 113, 394, 114, 394, 115, 394, 118, 394, 119, 394, 120, 394, 121, 394, 122, 394, 123, 394, 124, 394, 126, 394, 127, 394, 128, 394, 129, 394, 130, 394, 133, 394, 134, 394, 135, 394, 136, 394, 137, 394, 138, 394, 159, 394, 160, 394, 161, 394, 162, 394, 163, 394, 164, 394, 165, 394, 166, 394, 167, 394, 185, 394, 186, 394, 187, 394, 189, 394, 190, 394, 192, 394, 193, 394, 194, 394, 195, 394, 196, 394, 197, 394, 198, 394, 199, 394, 200, 394, 201, 394, 202, 394, 203, 394, 204, 394, 205, 394, 206, 394, 207, 394, 208, 394, 209, 394, 217, 394, 218, 394, 219, 394, 220, 394],
	#[65535, 389, 36, 389, 7, 389, 11, 389, 12, 389, 13, 389, 32, 389, 42, 389, 43, 389, 57, 389, 58, 389, 59, 389, 60, 389, 62, 389, 63, 389, 98, 389, 90, 389, 92, 389, 93, 389, 94, 389, 96, 389, 97, 389, 99, 389, 100, 389, 101, 389, 102, 389, 103, 389, 104, 389, 105, 389, 106, 389, 107, 389, 108, 389, 109, 389, 110, 389, 111, 389, 112, 389, 113, 389, 114, 389, 115, 389, 118, 389, 119, 389, 120, 389, 121, 389, 122, 389, 123, 389, 124, 389, 126, 389, 127, 389, 128, 389, 129, 389, 130, 389, 133, 389, 134, 389, 135, 389, 136, 389, 137, 389, 138, 389, 159, 389, 160, 389, 161, 389, 162, 389, 163, 389, 164, 389, 165, 389, 166, 389, 167, 389, 185, 389, 186, 389, 187, 389, 189, 389, 190, 389, 192, 389, 193, 389, 194, 389, 195, 389, 196, 389, 197, 389, 198, 389, 199, 389, 200, 389, 201, 389, 202, 389, 203, 389, 204, 389, 205, 389, 206, 389, 207, 389, 208, 389, 209, 389, 217, 389, 218, 389, 219, 389, 220, 389],
	#[13, -267, 32, -345],
	#[65535, 388, 13, 388, 7, 388, 123, 388, 59, 388, 159, 388, 62, 388, 11, 388, 98, 388, 133, 388, 36, 388, 58, 388, 63, 388, 92, 388, 102, 388, 127, 388, 12, 388, 90, 388, 57, 388, 43, 388, 42, 388, 99, 388, 32, 388, 108, 388, 115, 388, 94, 388, 60, 388, 103, 388, 93, 388, 126, 388, 104, 388, 101, 388, 100, 388, 96, 388, 97, 388, 105, 388, 106, 388, 107, 388, 130, 388, 109, 388, 110, 388, 111, 388, 112, 388, 113, 388, 114, 388, 129, 388, 138, 388, 118, 388, 119, 388, 120, 388, 121, 388, 122, 388, 128, 388, 124, 388, 134, 388, 135, 388, 136, 388, 137, 388, 167, 388, 162, 388, 160, 388, 163, 388, 161, 388, 164, 388, 165, 388, 166, 388, 185, 388, 186, 388, 187, 388, 189, 388, 190, 388, 192, 388, 193, 388, 194, 388, 195, 388, 196, 388, 197, 388, 198, 388, 199, 388, 200, 388, 201, 388, 202, 388, 203, 388, 204, 388, 205, 388, 206, 388, 207, 388, 208, 388, 209, 388, 217, 388, 218, 388, 219, 388, 220, 388],
	#[65535, 435, 7, 435, 14, 435, 11, 435, 0, 435, 1, 435, 4, 435, 19, 435, 18, 435, 9, 435, 10, 435, 16, 435, 12, 435, 13, 435, 15, 435, 17, 435, 20, 435, 21, 435, 22, 435, 23, 435, 24, 435, 25, 435, 26, 435, 27, 435, 28, 435, 29, 435, 32, 435, 34, 435, 36, 435, 40, 435, 41, 435, 42, 435, 43, 435, 57, 435, 58, 435, 59, 435, 60, 435, 62, 435, 63, 435, 74, 435, 75, 435, 76, 435, 77, 435, 79, 435, 80, 435, 98, 435, 90, 435, 92, 435, 93, 435, 94, 435, 96, 435, 97, 435, 99, 435, 100, 435, 101, 435, 102, 435, 103, 435, 104, 435, 105, 435, 106, 435, 107, 435, 108, 435, 109, 435, 110, 435, 111, 435, 112, 435, 113, 435, 114, 435, 115, 435, 116, 435, 118, 435, 119, 435, 120, 435, 121, 435, 122, 435, 123, 435, 124, 435, 126, 435, 127, 435, 128, 435, 129, 435, 130, 435, 133, 435, 134, 435, 135, 435, 136, 435, 137, 435, 138, 435, 159, 435, 160, 435, 161, 435, 162, 435, 163, 435, 164, 435, 165, 435, 166, 435, 167, 435, 185, 435, 186, 435, 187, 435, 189, 435, 190, 435, 192, 435, 193, 435, 194, 435, 195, 435, 196, 435, 197, 435, 198, 435, 199, 435, 200, 435, 201, 435, 202, 435, 203, 435, 204, 435, 205, 435, 206, 435, 207, 435, 208, 435, 209, 435, 217, 435, 218, 435, 219, 435, 220, 435, 222, 435, 223, 435],
	#[31, -59, 63, -63, 7, -37, 71, -55, 36, -65, 33, -60, 11, -66, 66, -58, 68, -64, 115, -10, 67, -61, 69, -68, 35, -67, 32, -352, 70, -72, 73, -70, 82, -350, 64, -73, 65, -71],
	#[65535, 434, 12, 434, 43, 434, 11, 434, 7, 434, 34, 434, 1, 434, 36, 434, 4, 434, 14, 434, 9, 434, 16, 434, 42, 434, 10, 434, 0, 434, 27, 434, 20, 434, 25, 434, 19, 434, 18, 434, 29, 434, 28, 434, 32, 434, 13, 434, 15, 434, 17, 434, 41, 434, 21, 434, 22, 434, 23, 434, 24, 434, 40, 434, 26, 434, 57, 434, 58, 434, 59, 434, 60, 434, 62, 434, 63, 434, 74, 434, 75, 434, 76, 434, 77, 434, 79, 434, 80, 434, 98, 434, 90, 434, 92, 434, 93, 434, 94, 434, 96, 434, 97, 434, 99, 434, 100, 434, 101, 434, 102, 434, 103, 434, 104, 434, 105, 434, 106, 434, 107, 434, 108, 434, 109, 434, 110, 434, 111, 434, 112, 434, 113, 434, 114, 434, 115, 434, 116, 434, 118, 434, 119, 434, 120, 434, 121, 434, 122, 434, 123, 434, 124, 434, 126, 434, 127, 434, 128, 434, 129, 434, 130, 434, 133, 434, 134, 434, 135, 434, 136, 434, 137, 434, 138, 434, 159, 434, 160, 434, 161, 434, 162, 434, 163, 434, 164, 434, 165, 434, 166, 434, 167, 434, 185, 434, 186, 434, 187, 434, 189, 434, 190, 434, 192, 434, 193, 434, 194, 434, 195, 434, 196, 434, 197, 434, 198, 434, 199, 434, 200, 434, 201, 434, 202, 434, 203, 434, 204, 434, 205, 434, 206, 434, 207, 434, 208, 434, 209, 434, 217, 434, 218, 434, 219, 434, 220, 434, 222, 434, 223, 434],
	#[65535, 427, 13, 427, 32, 427],
	#[65535, 428, 13, 428, 32, 428],
	#[13, -354, 32, -353],
	#[65535, 432, 0, 432, 1, 432, 4, 432, 7, 432, 9, 432, 10, 432, 11, 432, 12, 432, 13, 432, 14, 432, 15, 432, 16, 432, 17, 432, 18, 432, 19, 432, 20, 432, 21, 432, 22, 432, 23, 432, 24, 432, 25, 432, 26, 432, 27, 432, 28, 432, 29, 432, 32, 432, 34, 432, 36, 432, 40, 432, 41, 432, 42, 432, 43, 432, 57, 432, 58, 432, 59, 432, 60, 432, 62, 432, 63, 432, 74, 432, 75, 432, 76, 432, 77, 432, 79, 432, 80, 432, 90, 432, 92, 432, 93, 432, 94, 432, 96, 432, 97, 432, 98, 432, 99, 432, 100, 432, 101, 432, 102, 432, 103, 432, 104, 432, 105, 432, 106, 432, 107, 432, 108, 432, 109, 432, 110, 432, 111, 432, 112, 432, 113, 432, 114, 432, 115, 432, 116, 432, 118, 432, 119, 432, 120, 432, 121, 432, 122, 432, 123, 432, 124, 432, 126, 432, 127, 432, 128, 432, 129, 432, 130, 432, 133, 432, 134, 432, 135, 432, 136, 432, 137, 432, 138, 432, 159, 432, 160, 432, 161, 432, 162, 432, 163, 432, 164, 432, 165, 432, 166, 432, 167, 432, 185, 432, 186, 432, 187, 432, 189, 432, 190, 432, 192, 432, 193, 432, 194, 432, 195, 432, 196, 432, 197, 432, 198, 432, 199, 432, 200, 432, 201, 432, 202, 432, 203, 432, 204, 432, 205, 432, 206, 432, 207, 432, 208, 432, 209, 432, 217, 432, 218, 432, 219, 432, 220, 432, 222, 432, 223, 432],
	#[65535, 433, 7, 433, 36, 433, 10, 433, 34, 433, 1, 433, 0, 433, 4, 433, 32, 433, 9, 433, 29, 433, 11, 433, 12, 433, 13, 433, 14, 433, 15, 433, 16, 433, 17, 433, 18, 433, 19, 433, 20, 433, 21, 433, 22, 433, 23, 433, 24, 433, 25, 433, 26, 433, 27, 433, 28, 433, 40, 433, 41, 433, 42, 433, 43, 433, 167, 433, 57, 433, 58, 433, 59, 433, 60, 433, 62, 433, 63, 433, 74, 433, 75, 433, 76, 433, 77, 433, 79, 433, 80, 433, 90, 433, 92, 433, 93, 433, 94, 433, 96, 433, 97, 433, 98, 433, 99, 433, 100, 433, 101, 433, 102, 433, 103, 433, 104, 433, 105, 433, 106, 433, 107, 433, 108, 433, 109, 433, 110, 433, 111, 433, 112, 433, 113, 433, 114, 433, 115, 433, 116, 433, 118, 433, 119, 433, 120, 433, 121, 433, 122, 433, 123, 433, 124, 433, 126, 433, 127, 433, 128, 433, 129, 433, 130, 433, 133, 433, 134, 433, 135, 433, 136, 433, 137, 433, 138, 433, 159, 433, 160, 433, 161, 433, 162, 433, 163, 433, 164, 433, 165, 433, 166, 433, 190, 433, 189, 433, 187, 433, 186, 433, 192, 433, 185, 433, 193, 433, 194, 433, 195, 433, 196, 433, 197, 433, 198, 433, 199, 433, 200, 433, 201, 433, 202, 433, 203, 433, 204, 433, 205, 433, 206, 433, 207, 433, 208, 433, 209, 433, 217, 433, 218, 433, 219, 433, 220, 433, 222, 433, 223, 433],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 82, -355, 115, -10],
	#[65535, 430, 13, 430, 32, 430],
	#[65535, 429, 13, 429, 32, 429],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -248],
	#[13, -251, 43, -360],
	#[65535, 175, 0, 175, 1, 175, 4, 175, 7, 175, 9, 175, 10, 175, 11, 175, 12, 175, 13, 175, 14, 175, 15, 175, 16, 175, 17, 175, 18, 175, 19, 175, 20, 175, 21, 175, 22, 175, 23, 175, 24, 175, 25, 175, 26, 175, 27, 175, 28, 175, 29, 175, 32, 175, 34, 175, 36, 175, 40, 175, 41, 175, 42, 175, 43, 175, 57, 175, 58, 175, 59, 175, 60, 175, 62, 175, 63, 175, 74, 175, 75, 175, 76, 175, 77, 175, 79, 175, 80, 175, 90, 175, 92, 175, 93, 175, 94, 175, 96, 175, 97, 175, 98, 175, 99, 175, 100, 175, 101, 175, 102, 175, 103, 175, 104, 175, 105, 175, 106, 175, 107, 175, 108, 175, 109, 175, 110, 175, 111, 175, 112, 175, 113, 175, 114, 175, 115, 175, 116, 175, 118, 175, 119, 175, 120, 175, 121, 175, 122, 175, 123, 175, 124, 175, 126, 175, 127, 175, 128, 175, 129, 175, 130, 175, 133, 175, 134, 175, 135, 175, 136, 175, 137, 175, 138, 175, 159, 175, 160, 175, 161, 175, 162, 175, 163, 175, 164, 175, 165, 175, 166, 175, 167, 175, 185, 175, 186, 175, 187, 175, 189, 175, 190, 175, 192, 175, 193, 175, 194, 175, 195, 175, 196, 175, 197, 175, 198, 175, 199, 175, 200, 175, 201, 175, 202, 175, 203, 175, 204, 175, 205, 175, 206, 175, 207, 175, 208, 175, 209, 175, 217, 175, 218, 175, 219, 175, 220, 175, 222, 175, 223, 175],
	#[13, -267, 34, -427],
	#[7, -37, 11, -66, 31, -59, 32, -364, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[87, -221, 7, -37, 134, -184, 101, -214, 84, -204, 135, -213, 10, -212, 33, -202, 92, -189, 102, -178, 37, 105, 133, -197, 31, -220, 119, -231, 38, -200, 63, -219, 86, -224, 110, -233, 82, -186, 85, -205, 127, -187, 42, -93, 98, -192, 94, -210, 109, -180, 88, -208, 11, -195, 138, -230, 116, -9, 100, -193, 126, -198, 111, -222, 32, 161, 122, -190, 99, -218, 89, -235, 136, -188, 115, -10, 104, -185, 13, 161, 107, -229, 128, -216, 118, -181, 96, -182, 81, -176, 120, -226, 105, -173, 103, -228, 108, -199, 83, -236, 123, -174, 113, -194, 106, -227, 72, -50, 121, -201, 129, -203, 114, -183, 124, -232, 130, -191, 137, -175, 97, -217, 112, -206, 223, -209],
	#[126, 94, 111, 94, 63, 94, 37, 94, 127, 94, 11, 94, 10, 94, 110, 94, 100, 94, 84, 94, 87, 94, 34, -368, 31, 94, 82, 94, 7, 94, 99, 94, 85, 94, 102, 94, 81, 94, 33, 94, 38, 94, 86, 94, 42, 94, 98, 94, 88, 94, 92, 94, 94, 94, 114, 94, 101, 94, 103, 94, 72, 94, 104, 94, 119, 94, 89, 94, 83, 94, 96, 94, 97, 94, 105, 94, 106, 94, 107, 94, 108, 94, 109, 94, 123, 94, 122, 94, 112, 94, 113, 94, 133, 94, 115, 94, 116, 94, 118, 94, 130, 94, 120, 94, 121, 94, 129, 94, 128, 94, 124, 94, 134, 94, 135, 94, 136, 94, 137, 94, 138, 94, 223, 94],
	#[13, -267, 32, -366],
	#[34, -367],
	#[65535, 390, 7, 390, 11, 390, 12, 390, 13, 390, 36, 390, 92, 390, 60, 390, 32, 390, 43, 390, 98, 390, 59, 390, 63, 390, 58, 390, 62, 390, 110, 390, 42, 390, 159, 390, 101, 390, 103, 390, 90, 390, 99, 390, 94, 390, 57, 390, 104, 390, 93, 390, 102, 390, 126, 390, 114, 390, 119, 390, 111, 390, 100, 390, 96, 390, 97, 390, 127, 390, 105, 390, 106, 390, 107, 390, 108, 390, 109, 390, 123, 390, 122, 390, 112, 390, 113, 390, 133, 390, 115, 390, 138, 390, 118, 390, 130, 390, 120, 390, 121, 390, 129, 390, 128, 390, 124, 390, 134, 390, 135, 390, 136, 390, 137, 390, 167, 390, 162, 390, 160, 390, 163, 390, 161, 390, 164, 390, 165, 390, 166, 390, 185, 390, 186, 390, 187, 390, 189, 390, 190, 390, 192, 390, 193, 390, 194, 390, 195, 390, 196, 390, 197, 390, 198, 390, 199, 390, 200, 390, 201, 390, 202, 390, 203, 390, 204, 390, 205, 390, 206, 390, 207, 390, 208, 390, 209, 390, 217, 390, 218, 390, 219, 390, 220, 390],
	#[65535, 391, 7, 391, 11, 391, 12, 391, 13, 391, 32, 391, 36, 391, 42, 391, 43, 391, 57, 391, 58, 391, 59, 391, 60, 391, 62, 391, 63, 391, 90, 391, 92, 391, 93, 391, 94, 391, 96, 391, 97, 391, 98, 391, 99, 391, 100, 391, 101, 391, 102, 391, 103, 391, 104, 391, 105, 391, 106, 391, 107, 391, 108, 391, 109, 391, 110, 391, 111, 391, 112, 391, 113, 391, 114, 391, 115, 391, 118, 391, 119, 391, 120, 391, 121, 391, 122, 391, 123, 391, 124, 391, 126, 391, 127, 391, 128, 391, 129, 391, 130, 391, 133, 391, 134, 391, 135, 391, 136, 391, 137, 391, 138, 391, 159, 391, 160, 391, 161, 391, 162, 391, 163, 391, 164, 391, 165, 391, 166, 391, 167, 391, 185, 391, 186, 391, 187, 391, 189, 391, 190, 391, 192, 391, 193, 391, 194, 391, 195, 391, 196, 391, 197, 391, 198, 391, 199, 391, 200, 391, 201, 391, 202, 391, 203, 391, 204, 391, 205, 391, 206, 391, 207, 391, 208, 391, 209, 391, 217, 391, 218, 391, 219, 391, 220, 391],
	#[13, -267, 34, -376],
	#[7, -37, 11, -66, 31, -59, 32, -371, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[87, 94, 101, 94, 7, 94, 85, 94, 10, 94, 11, 94, 38, 94, 86, 94, 37, 94, 34, -375, 92, 94, 94, 94, 33, 94, 31, 94, 84, 94, 63, 94, 82, 94, 89, 94, 110, 94, 42, 94, 98, 94, 83, 94, 103, 94, 81, 94, 99, 94, 118, 94, 115, 94, 72, 94, 88, 94, 104, 94, 107, 94, 102, 94, 126, 94, 114, 94, 119, 94, 111, 94, 100, 94, 96, 94, 97, 94, 127, 94, 105, 94, 106, 94, 108, 94, 109, 94, 123, 94, 122, 94, 112, 94, 113, 94, 133, 94, 129, 94, 116, 94, 128, 94, 130, 94, 120, 94, 121, 94, 124, 94, 134, 94, 135, 94, 136, 94, 137, 94, 138, 94, 223, 94],
	#[13, -267, 32, -373],
	#[34, -374],
	#[65535, 129, 10, 129, 34, 129, 21, 129, 27, 129, 14, 129, 1, 129, 0, 129, 20, 129, 4, 129, 63, 129, 7, 129, 9, 129, 29, 129, 12, 129, 13, 129, 116, 129, 15, 129, 16, 129, 17, 129, 18, 129, 19, 129, 77, 129, 79, 129, 22, 129, 23, 129, 24, 129, 25, 129, 26, 129, 32, 129, 28, 129, 40, 129, 41, 129, 76, 129, 43, 129, 101, 129, 103, 129, 94, 129, 118, 129, 80, 129, 74, 129, 111, 129, 93, 129, 102, 129, 92, 129, 114, 129, 99, 129, 119, 129, 75, 129, 105, 129, 110, 129, 98, 129, 96, 129, 107, 129, 97, 129, 109, 129, 112, 129, 106, 129, 104, 129, 115, 129, 113, 129, 100, 129, 108, 129, 121, 129, 120, 129, 122, 129, 123, 129, 124, 129, 126, 129, 127, 129, 128, 129, 129, 129, 130, 129, 133, 129, 134, 129, 135, 129, 136, 129, 137, 129, 138, 129, 160, 129, 161, 129, 162, 129, 163, 129, 164, 129, 166, 129, 167, 129, 185, 129, 186, 129, 187, 129, 189, 129, 190, 129, 192, 129, 193, 129, 194, 129, 195, 129, 196, 129, 197, 129, 199, 129, 200, 129, 201, 129, 202, 129, 222, 129, 223, 129],
	#[65535, 130, 14, 130, 34, 130, 22, 130, 92, 130, 15, 130, 103, 130, 93, 130, 12, 130, 23, 130, 1, 130, 9, 130, 101, 130, 20, 130, 4, 130, 18, 130, 13, 130, 63, 130, 10, 130, 7, 130, 19, 130, 16, 130, 25, 130, 75, 130, 115, 130, 74, 130, 102, 130, 27, 130, 17, 130, 29, 130, 79, 130, 26, 130, 28, 130, 111, 130, 116, 130, 0, 130, 77, 130, 21, 130, 119, 130, 43, 130, 40, 130, 138, 130, 106, 130, 76, 130, 24, 130, 98, 130, 126, 130, 32, 130, 41, 130, 109, 130, 114, 130, 107, 130, 110, 130, 135, 130, 130, 130, 99, 130, 113, 130, 108, 130, 129, 130, 94, 130, 118, 130, 80, 130, 100, 130, 104, 130, 105, 130, 112, 130, 133, 130, 127, 130, 161, 130, 123, 130, 97, 130, 96, 130, 128, 130, 121, 130, 120, 130, 122, 130, 137, 130, 124, 130, 134, 130, 160, 130, 136, 130, 163, 130, 167, 130, 162, 130, 164, 130, 166, 130, 185, 130, 186, 130, 187, 130, 189, 130, 190, 130, 192, 130, 193, 130, 194, 130, 195, 130, 196, 130, 197, 130, 199, 130, 200, 130, 201, 130, 202, 130, 222, 130, 223, 130],
	#[65535, 126, 0, 126, 1, 126, 4, 126, 7, 126, 9, 126, 10, 126, 12, 126, 13, 126, 14, 126, 15, 126, 16, 126, 17, 126, 18, 126, 19, 126, 20, 126, 21, 126, 22, 126, 23, 126, 24, 126, 25, 126, 26, 126, 27, 126, 28, 126, 29, 126, 32, 126, 34, 126, 40, 126, 41, 126, 43, 126, 63, 126, 74, 126, 75, 126, 76, 126, 77, 126, 79, 126, 80, 126, 92, 126, 93, 126, 94, 126, 96, 126, 97, 126, 98, 126, 99, 126, 100, 126, 101, 126, 102, 126, 103, 126, 104, 126, 105, 126, 106, 126, 107, 126, 108, 126, 109, 126, 110, 126, 111, 126, 112, 126, 113, 126, 114, 126, 115, 126, 116, 126, 118, 126, 119, 126, 120, 126, 121, 126, 122, 126, 123, 126, 124, 126, 126, 126, 127, 126, 128, 126, 129, 126, 130, 126, 133, 126, 134, 126, 135, 126, 136, 126, 137, 126, 138, 126, 160, 126, 161, 126, 162, 126, 163, 126, 164, 126, 166, 126, 167, 126, 185, 126, 186, 126, 187, 126, 189, 126, 190, 126, 192, 126, 193, 126, 194, 126, 195, 126, 196, 126, 197, 126, 199, 126, 200, 126, 201, 126, 202, 126, 222, 126, 223, 126],
	#[65535, 226, 31, 226, 7, 226, 11, 226, 36, 226, 35, 226, 33, 226, 42, 226, 70, 226, 63, 226, 64, 226, 65, 226, 66, 226, 67, 226, 68, 226, 69, 226, 71, 226, 73, 226, 115, 226],
	#[65535, 236, 7, 236, 11, 236, 31, 236, 33, 236, 35, 236, 36, 236, 42, 236, 63, 236, 64, 236, 65, 236, 66, 236, 67, 236, 68, 236, 69, 236, 70, 236, 71, 236, 73, 236, 115, 236],
	#[65535, 124, 1, 124, 10, 124, 13, 124, 15, 124, 12, 124, 32, 124, 21, 124, 98, 124, 19, 124, 14, 124, 4, 124, 29, 124, 18, 124, 27, 124, 7, 124, 94, 124, 116, 124, 77, 124, 25, 124, 41, 124, 16, 124, 103, 124, 17, 124, 76, 124, 79, 124, 26, 124, 23, 124, 0, 124, 20, 124, 43, 124, 63, 124, 101, 124, 92, 124, 9, 124, 28, 124, 34, 124, 105, 124, 40, 124, 24, 124, 22, 124, 75, 124, 97, 124, 118, 124, 80, 124, 74, 124, 111, 124, 93, 124, 102, 124, 114, 124, 99, 124, 119, 124, 106, 124, 115, 124, 110, 124, 96, 124, 107, 124, 104, 124, 109, 124, 112, 124, 133, 124, 113, 124, 100, 124, 108, 124, 121, 124, 120, 124, 122, 124, 123, 124, 124, 124, 126, 124, 127, 124, 128, 124, 129, 124, 130, 124, 134, 124, 135, 124, 136, 124, 137, 124, 138, 124, 160, 124, 161, 124, 162, 124, 163, 124, 164, 124, 166, 124, 167, 124, 185, 124, 186, 124, 187, 124, 189, 124, 190, 124, 192, 124, 193, 124, 194, 124, 195, 124, 196, 124, 197, 124, 199, 124, 200, 124, 201, 124, 202, 124, 222, 124, 223, 124],
	#[12, -381, 13, -267],
	#[65535, 123, 0, 123, 1, 123, 4, 123, 7, 123, 9, 123, 10, 123, 12, 123, 13, 123, 14, 123, 15, 123, 16, 123, 17, 123, 18, 123, 19, 123, 20, 123, 21, 123, 22, 123, 23, 123, 24, 123, 25, 123, 26, 123, 27, 123, 28, 123, 29, 123, 32, 123, 34, 123, 40, 123, 41, 123, 43, 123, 63, 123, 74, 123, 75, 123, 76, 123, 77, 123, 79, 123, 80, 123, 92, 123, 93, 123, 94, 123, 96, 123, 97, 123, 98, 123, 99, 123, 100, 123, 101, 123, 102, 123, 103, 123, 104, 123, 105, 123, 106, 123, 107, 123, 108, 123, 109, 123, 110, 123, 111, 123, 112, 123, 113, 123, 114, 123, 115, 123, 116, 123, 118, 123, 119, 123, 120, 123, 121, 123, 122, 123, 123, 123, 124, 123, 126, 123, 127, 123, 128, 123, 129, 123, 130, 123, 133, 123, 134, 123, 135, 123, 136, 123, 137, 123, 138, 123, 160, 123, 161, 123, 162, 123, 163, 123, 164, 123, 166, 123, 167, 123, 185, 123, 186, 123, 187, 123, 189, 123, 190, 123, 192, 123, 193, 123, 194, 123, 195, 123, 196, 123, 197, 123, 199, 123, 200, 123, 201, 123, 202, 123, 222, 123, 223, 123],
	#[31, -59, 7, -37, 11, -66, 36, -65, 35, -67, 33, -60, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -384],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -386],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[43, -388],
	#[65535, 173, 22, 173, 7, 173, 9, 173, 21, 173, 15, 173, 1, 173, 10, 173, 17, 173, 14, 173, 62, 173, 23, 173, 12, 173, 11, 173, 0, 173, 20, 173, 4, 173, 63, 173, 19, 173, 59, 173, 42, 173, 29, 173, 16, 173, 32, 173, 13, 173, 27, 173, 103, 173, 43, 173, 18, 173, 36, 173, 77, 173, 79, 173, 57, 173, 34, 173, 24, 173, 25, 173, 26, 173, 41, 173, 28, 173, 40, 173, 74, 173, 76, 173, 60, 173, 58, 173, 105, 173, 80, 173, 109, 173, 111, 173, 93, 173, 102, 173, 90, 173, 92, 173, 114, 173, 99, 173, 119, 173, 75, 173, 101, 173, 110, 173, 98, 173, 96, 173, 112, 173, 107, 173, 97, 173, 94, 173, 118, 173, 106, 173, 104, 173, 115, 173, 113, 173, 116, 173, 100, 173, 108, 173, 121, 173, 120, 173, 122, 173, 123, 173, 124, 173, 126, 173, 127, 173, 128, 173, 129, 173, 130, 173, 133, 173, 134, 173, 135, 173, 136, 173, 137, 173, 138, 173, 159, 173, 160, 173, 161, 173, 162, 173, 163, 173, 164, 173, 165, 173, 166, 173, 167, 173, 185, 173, 186, 173, 187, 173, 189, 173, 190, 173, 192, 173, 193, 173, 194, 173, 195, 173, 196, 173, 197, 173, 198, 173, 199, 173, 200, 173, 201, 173, 202, 173, 203, 173, 204, 173, 205, 173, 206, 173, 207, 173, 208, 173, 209, 173, 217, 173, 218, 173, 219, 173, 220, 173, 222, 173, 223, 173],
	#[65535, 260, 7, 260, 11, 260, 31, 260, 33, 260, 35, 260, 36, 260, 42, 260, 63, 260, 64, 260, 65, 260, 66, 260, 67, 260, 68, 260, 69, 260, 70, 260, 71, 260, 73, 260, 115, 260],
	#[65535, 254, 31, 254, 36, 254, 7, 254, 11, 254, 35, 254, 33, 254, 42, 254, 63, 254, 64, 254, 65, 254, 66, 254, 67, 254, 68, 254, 69, 254, 70, 254, 71, 254, 73, 254, 115, 254],
	#[65535, 261, 7, 261, 11, 261, 31, 261, 33, 261, 35, 261, 36, 261, 42, 261, 63, 261, 64, 261, 65, 261, 66, 261, 67, 261, 68, 261, 69, 261, 70, 261, 71, 261, 73, 261, 115, 261],
	#[65535, 263, 7, 263, 11, 263, 31, 263, 33, 263, 35, 263, 36, 263, 42, 263, 63, 263, 64, 263, 65, 263, 66, 263, 67, 263, 68, 263, 69, 263, 70, 263, 71, 263, 73, 263, 115, 263],
	#[65535, 255, 31, 255, 63, 255, 7, 255, 11, 255, 42, 255, 70, 255, 69, 255, 33, 255, 68, 255, 35, 255, 36, 255, 64, 255, 67, 255, 71, 255, 65, 255, 66, 255, 73, 255, 115, 255],
	#[65535, 262, 31, 262, 7, 262, 11, 262, 42, 262, 33, 262, 35, 262, 36, 262, 71, 262, 66, 262, 70, 262, 69, 262, 68, 262, 67, 262, 65, 262, 64, 262, 63, 262, 73, 262, 115, 262],
	#[42, -406],
	#[65535, 269, 31, 269, 36, 269, 69, 269, 11, 269, 7, 269, 67, 269, 65, 269, 35, 269, 33, 269, 70, 269, 63, 269, 73, 269, 66, 269, 64, 269, 42, 269, 68, 269, 71, 269, 115, 269],
	#[65535, 264, 7, 264, 11, 264, 31, 264, 33, 264, 35, 264, 36, 264, 42, 264, 63, 264, 64, 264, 65, 264, 66, 264, 67, 264, 68, 264, 69, 264, 70, 264, 71, 264, 73, 264, 115, 264],
	#[65535, 268, 31, 268, 7, 268, 70, 268, 36, 268, 63, 268, 35, 268, 73, 268, 11, 268, 69, 268, 71, 268, 67, 268, 64, 268, 66, 268, 33, 268, 68, 268, 65, 268, 42, 268, 115, 268],
	#[65535, 257, 31, 257, 7, 257, 11, 257, 71, 257, 68, 257, 70, 257, 69, 257, 33, 257, 63, 257, 35, 257, 36, 257, 67, 257, 42, 257, 65, 257, 73, 257, 66, 257, 64, 257, 115, 257],
	#[65535, 265, 7, 265, 11, 265, 31, 265, 33, 265, 35, 265, 36, 265, 42, 265, 63, 265, 64, 265, 65, 265, 66, 265, 67, 265, 68, 265, 69, 265, 70, 265, 71, 265, 73, 265, 115, 265],
	#[65535, 258, 7, 258, 11, 258, 31, 258, 33, 258, 35, 258, 36, 258, 42, 258, 63, 258, 64, 258, 65, 258, 66, 258, 67, 258, 68, 258, 69, 258, 70, 258, 71, 258, 73, 258, 115, 258],
	#[65535, 266, 7, 266, 11, 266, 31, 266, 33, 266, 35, 266, 36, 266, 42, 266, 63, 266, 64, 266, 65, 266, 66, 266, 67, 266, 68, 266, 69, 266, 70, 266, 71, 266, 73, 266, 115, 266],
	#[65535, 259, 7, 259, 11, 259, 31, 259, 33, 259, 35, 259, 36, 259, 42, 259, 63, 259, 64, 259, 65, 259, 66, 259, 67, 259, 68, 259, 69, 259, 70, 259, 71, 259, 73, 259, 115, 259],
	#[65535, 267, 31, 267, 7, 267, 11, 267, 36, 267, 35, 267, 33, 267, 42, 267, 66, 267, 63, 267, 64, 267, 65, 267, 73, 267, 67, 267, 68, 267, 69, 267, 70, 267, 71, 267, 115, 267],
	#[65535, 256, 7, 256, 11, 256, 31, 256, 33, 256, 35, 256, 36, 256, 42, 256, 63, 256, 64, 256, 65, 256, 66, 256, 67, 256, 68, 256, 69, 256, 70, 256, 71, 256, 73, 256, 115, 256],
	#[68, -64, 31, -59, 63, -63, 7, -37, 11, -66, 69, -68, 67, -61, 36, -65, 70, -72, 33, -60, 66, -58, 64, -73, 35, -67, 65, -71, 71, -55, 73, -70, 115, -10],
	#[13, -408],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[43, -410],
	#[65535, 171, 7, 171, 106, 171, 93, 171, 0, 171, 1, 171, 4, 171, 63, 171, 92, 171, 57, 171, 9, 171, 10, 171, 11, 171, 12, 171, 13, 171, 14, 171, 15, 171, 16, 171, 17, 171, 18, 171, 19, 171, 20, 171, 21, 171, 22, 171, 23, 171, 24, 171, 25, 171, 26, 171, 27, 171, 28, 171, 29, 171, 42, 171, 32, 171, 34, 171, 105, 171, 36, 171, 115, 171, 62, 171, 40, 171, 41, 171, 76, 171, 43, 171, 101, 171, 58, 171, 118, 171, 79, 171, 121, 171, 109, 171, 111, 171, 98, 171, 97, 171, 102, 171, 90, 171, 77, 171, 59, 171, 60, 171, 119, 171, 75, 171, 74, 171, 110, 171, 116, 171, 107, 171, 104, 171, 80, 171, 94, 171, 112, 171, 99, 171, 103, 171, 96, 171, 113, 171, 100, 171, 108, 171, 114, 171, 120, 171, 122, 171, 123, 171, 124, 171, 126, 171, 127, 171, 128, 171, 129, 171, 130, 171, 133, 171, 134, 171, 135, 171, 136, 171, 137, 171, 138, 171, 159, 171, 160, 171, 161, 171, 162, 171, 163, 171, 164, 171, 165, 171, 166, 171, 167, 171, 185, 171, 186, 171, 187, 171, 189, 171, 190, 171, 192, 171, 193, 171, 194, 171, 195, 171, 196, 171, 197, 171, 198, 171, 199, 171, 200, 171, 201, 171, 202, 171, 203, 171, 204, 171, 205, 171, 206, 171, 207, 171, 208, 171, 209, 171, 217, 171, 218, 171, 219, 171, 220, 171, 222, 171, 223, 171],
	#[65535, 221, 31, 221, 11, 221, 68, 221, 70, 221, 36, 221, 63, 221, 7, 221, 73, 221, 66, 221, 69, 221, 71, 221, 64, 221, 67, 221, 35, 221, 65, 221, 33, 221, 42, 221, 115, 221],
	#[65535, 238, 7, 238, 11, 238, 71, 238, 31, 238, 33, 238, 35, 238, 36, 238, 42, 238, 66, 238, 70, 238, 69, 238, 68, 238, 67, 238, 65, 238, 64, 238, 63, 238, 73, 238, 115, 238],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -415],
	#[68, -64, 31, -59, 63, -63, 7, -37, 11, -66, 69, -68, 67, -61, 36, -65, 66, -58, 70, -72, 33, -60, 64, -73, 35, -67, 65, -71, 71, -55, 73, -70, 115, -10],
	#[13, -417],
	#[63, -63, 7, -37, 11, -66, 31, -59, 33, -60, 68, -64, 35, -67, 36, -65, 71, -55, 66, -58, 70, -72, 69, -68, 65, -71, 73, -70, 67, -61, 64, -73, 115, -10],
	#[43, -419],
	#[65535, 167, 43, 167, 13, 167, 14, 167, 62, 167, 21, 167, 159, 167, 103, 167, 167, 167, 23, 167, 111, 167, 92, 167, 93, 167, 12, 167, 119, 167, 11, 167, 90, 167, 58, 167, 7, 167, 107, 167, 1, 167, 22, 167, 42, 167, 59, 167, 94, 167, 15, 167, 60, 167, 57, 167, 0, 167, 135, 167, 32, 167, 106, 167, 102, 167, 126, 167, 16, 167, 9, 167, 17, 167, 20, 167, 4, 167, 26, 167, 29, 167, 76, 167, 27, 167, 63, 167, 10, 167, 97, 167, 34, 167, 19, 167, 100, 167, 77, 167, 25, 167, 18, 167, 162, 167, 101, 167, 108, 167, 96, 167, 79, 167, 105, 167, 28, 167, 99, 167, 116, 167, 24, 167, 36, 167, 118, 167, 104, 167, 40, 167, 123, 167, 163, 167, 74, 167, 75, 167, 127, 167, 112, 167, 41, 167, 109, 167, 80, 167, 110, 167, 98, 167, 113, 167, 122, 167, 120, 167, 114, 167, 133, 167, 130, 167, 121, 167, 115, 167, 129, 167, 128, 167, 124, 167, 134, 167, 160, 167, 136, 167, 137, 167, 138, 167, 161, 167, 164, 167, 165, 167, 166, 167, 185, 167, 186, 167, 187, 167, 189, 167, 190, 167, 192, 167, 193, 167, 194, 167, 195, 167, 196, 167, 197, 167, 198, 167, 199, 167, 200, 167, 201, 167, 202, 167, 203, 167, 204, 167, 205, 167, 206, 167, 207, 167, 208, 167, 209, 167, 217, 167, 218, 167, 219, 167, 220, 167, 222, 167, 223, 167],
	#[65535, 217, 7, 217, 31, 217, 11, 217, 42, 217, 33, 217, 35, 217, 36, 217, 70, 217, 69, 217, 63, 217, 64, 217, 65, 217, 66, 217, 67, 217, 68, 217, 71, 217, 73, 217, 115, 217],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -423],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[43, -425],
	#[65535, 168, 0, 168, 1, 168, 4, 168, 7, 168, 9, 168, 10, 168, 11, 168, 12, 168, 13, 168, 14, 168, 15, 168, 16, 168, 17, 168, 18, 168, 19, 168, 20, 168, 21, 168, 22, 168, 23, 168, 24, 168, 25, 168, 26, 168, 27, 168, 28, 168, 29, 168, 32, 168, 34, 168, 36, 168, 40, 168, 41, 168, 42, 168, 43, 168, 57, 168, 58, 168, 59, 168, 60, 168, 62, 168, 63, 168, 74, 168, 75, 168, 76, 168, 77, 168, 79, 168, 80, 168, 90, 168, 92, 168, 93, 168, 94, 168, 96, 168, 97, 168, 98, 168, 99, 168, 100, 168, 101, 168, 102, 168, 103, 168, 104, 168, 105, 168, 106, 168, 107, 168, 108, 168, 109, 168, 110, 168, 111, 168, 112, 168, 113, 168, 114, 168, 115, 168, 116, 168, 118, 168, 119, 168, 120, 168, 121, 168, 122, 168, 123, 168, 124, 168, 126, 168, 127, 168, 128, 168, 129, 168, 130, 168, 133, 168, 134, 168, 135, 168, 136, 168, 137, 168, 138, 168, 159, 168, 160, 168, 161, 168, 162, 168, 163, 168, 164, 168, 165, 168, 166, 168, 167, 168, 185, 168, 186, 168, 187, 168, 189, 168, 190, 168, 192, 168, 193, 168, 194, 168, 195, 168, 196, 168, 197, 168, 198, 168, 199, 168, 200, 168, 201, 168, 202, 168, 203, 168, 204, 168, 205, 168, 206, 168, 207, 168, 208, 168, 209, 168, 217, 168, 218, 168, 219, 168, 220, 168, 222, 168, 223, 168],
	#[65535, 125, 79, 125, 12, 125, 0, 125, 1, 125, 4, 125, 7, 125, 9, 125, 10, 125, 34, 125, 13, 125, 14, 125, 15, 125, 16, 125, 17, 125, 18, 125, 19, 125, 20, 125, 21, 125, 22, 125, 23, 125, 24, 125, 25, 125, 26, 125, 27, 125, 28, 125, 29, 125, 116, 125, 32, 125, 63, 125, 75, 125, 77, 125, 40, 125, 41, 125, 43, 125, 101, 125, 103, 125, 94, 125, 118, 125, 115, 125, 74, 125, 111, 125, 93, 125, 102, 125, 92, 125, 76, 125, 119, 125, 106, 125, 105, 125, 110, 125, 80, 125, 96, 125, 107, 125, 97, 125, 114, 125, 109, 125, 112, 125, 99, 125, 104, 125, 98, 125, 113, 125, 100, 125, 108, 125, 121, 125, 120, 125, 122, 125, 123, 125, 124, 125, 126, 125, 127, 125, 128, 125, 129, 125, 130, 125, 133, 125, 134, 125, 135, 125, 136, 125, 137, 125, 138, 125, 160, 125, 161, 125, 162, 125, 163, 125, 164, 125, 166, 125, 167, 125, 185, 125, 186, 125, 187, 125, 189, 125, 190, 125, 192, 125, 193, 125, 194, 125, 195, 125, 196, 125, 197, 125, 199, 125, 200, 125, 201, 125, 202, 125, 222, 125, 223, 125],
	#[65535, 384, 7, 384, 11, 384, 12, 384, 13, 384, 43, 384, 42, 384, 32, 384, 59, 384, 36, 384, 57, 384, 58, 384, 60, 384, 62, 384, 63, 384, 90, 384, 92, 384, 93, 384, 94, 384, 96, 384, 97, 384, 98, 384, 99, 384, 100, 384, 101, 384, 102, 384, 103, 384, 104, 384, 105, 384, 106, 384, 107, 384, 108, 384, 109, 384, 110, 384, 111, 384, 112, 384, 113, 384, 114, 384, 115, 384, 118, 384, 119, 384, 120, 384, 121, 384, 122, 384, 123, 384, 124, 384, 126, 384, 127, 384, 128, 384, 129, 384, 130, 384, 133, 384, 134, 384, 135, 384, 136, 384, 137, 384, 138, 384, 159, 384, 160, 384, 161, 384, 162, 384, 163, 384, 164, 384, 165, 384, 166, 384, 167, 384, 189, 384, 185, 384, 186, 384, 187, 384, 190, 384, 192, 384, 193, 384, 194, 384, 195, 384, 196, 384, 197, 384, 198, 384, 199, 384, 200, 384, 201, 384, 202, 384, 203, 384, 204, 384, 205, 384, 206, 384, 207, 384, 208, 384, 209, 384, 217, 384, 218, 384, 219, 384, 220, 384],
	#[65535, 386, 119, 386, 7, 386, 11, 386, 12, 386, 13, 386, 93, 386, 59, 386, 32, 386, 42, 386, 100, 386, 102, 386, 63, 386, 36, 386, 62, 386, 110, 386, 107, 386, 43, 386, 103, 386, 58, 386, 94, 386, 118, 386, 57, 386, 106, 386, 109, 386, 111, 386, 97, 386, 105, 386, 90, 386, 92, 386, 114, 386, 60, 386, 96, 386, 112, 386, 101, 386, 99, 386, 104, 386, 98, 386, 113, 386, 120, 386, 108, 386, 121, 386, 115, 386, 122, 386, 123, 386, 124, 386, 126, 386, 127, 386, 128, 386, 129, 386, 130, 386, 133, 386, 134, 386, 135, 386, 136, 386, 137, 386, 138, 386, 159, 386, 160, 386, 161, 386, 162, 386, 163, 386, 164, 386, 165, 386, 166, 386, 167, 386, 206, 386, 185, 386, 186, 386, 187, 386, 189, 386, 190, 386, 192, 386, 193, 386, 194, 386, 195, 386, 196, 386, 197, 386, 198, 386, 199, 386, 200, 386, 201, 386, 202, 386, 203, 386, 204, 386, 205, 386, 207, 386, 208, 386, 209, 386, 217, 386, 218, 386, 219, 386, 220, 386],
	#[12, -430, 13, -267],
	#[65535, 385, 7, 385, 11, 385, 12, 385, 13, 385, 42, 385, 32, 385, 63, 385, 36, 385, 115, 385, 105, 385, 43, 385, 58, 385, 118, 385, 57, 385, 106, 385, 109, 385, 111, 385, 93, 385, 102, 385, 90, 385, 92, 385, 59, 385, 60, 385, 119, 385, 62, 385, 101, 385, 110, 385, 96, 385, 107, 385, 97, 385, 114, 385, 94, 385, 112, 385, 99, 385, 103, 385, 98, 385, 113, 385, 104, 385, 100, 385, 108, 385, 121, 385, 120, 385, 122, 385, 123, 385, 124, 385, 126, 385, 127, 385, 128, 385, 129, 385, 130, 385, 133, 385, 134, 385, 135, 385, 136, 385, 137, 385, 138, 385, 159, 385, 160, 385, 161, 385, 162, 385, 163, 385, 164, 385, 165, 385, 166, 385, 167, 385, 185, 385, 186, 385, 187, 385, 189, 385, 190, 385, 192, 385, 193, 385, 194, 385, 195, 385, 196, 385, 197, 385, 198, 385, 199, 385, 200, 385, 201, 385, 202, 385, 203, 385, 204, 385, 205, 385, 206, 385, 207, 385, 208, 385, 209, 385, 217, 385, 218, 385, 219, 385, 220, 385],
	#[65535, 387, 7, 387, 11, 387, 12, 387, 13, 387, 32, 387, 36, 387, 42, 387, 43, 387, 57, 387, 58, 387, 59, 387, 60, 387, 62, 387, 63, 387, 90, 387, 92, 387, 93, 387, 94, 387, 96, 387, 97, 387, 98, 387, 99, 387, 100, 387, 101, 387, 102, 387, 103, 387, 104, 387, 105, 387, 106, 387, 107, 387, 108, 387, 109, 387, 110, 387, 111, 387, 112, 387, 113, 387, 114, 387, 115, 387, 118, 387, 119, 387, 120, 387, 121, 387, 122, 387, 123, 387, 124, 387, 126, 387, 127, 387, 128, 387, 129, 387, 130, 387, 133, 387, 134, 387, 135, 387, 136, 387, 137, 387, 138, 387, 159, 387, 160, 387, 161, 387, 162, 387, 163, 387, 164, 387, 165, 387, 166, 387, 167, 387, 185, 387, 186, 387, 187, 387, 189, 387, 190, 387, 192, 387, 193, 387, 194, 387, 195, 387, 196, 387, 197, 387, 198, 387, 199, 387, 200, 387, 201, 387, 202, 387, 203, 387, 204, 387, 205, 387, 206, 387, 207, 387, 208, 387, 209, 387, 217, 387, 218, 387, 219, 387, 220, 387],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -438, 92, -434, 94, -433, 115, -10],
	#[42, 417, 221, -303],
	#[42, -440],
	#[63, -219, 7, -37, 10, -212, 116, -9, 115, -10],
	#[65535, 8, 0, 8, 1, 8, 21, 8, 4, 8, 7, 8, 9, 8, 10, 8, 14, 8, 15, 8, 16, 8, 17, 8, 18, 8, 19, 8, 20, 8, 22, 8, 23, 8, 24, 8, 25, 8, 26, 8, 27, 8, 28, 8, 29, 8, 40, 8, 41, 8, 63, 8, 74, 8, 75, 8, 76, 8, 77, 8, 79, 8, 80, 8, 115, 8, 116, 8, 222, 8, 223, 8],
	#[2, 191, 7, 191, 10, 191, 11, 191, 31, 191, 33, 191, 37, 105, 38, 191, 42, -93, 63, 191, 72, -50, 81, 191, 82, 191, 83, 191, 84, 191, 85, 191, 86, 191, 87, 191, 88, 191, 89, 191, 92, 191, 94, 191, 96, 191, 97, 191, 98, 191, 99, 191, 100, 191, 101, 191, 102, 191, 103, 191, 104, 191, 105, 191, 106, 191, 107, 191, 108, 191, 109, 191, 110, 191, 111, 191, 112, 191, 113, 191, 114, 191, 115, 191, 116, 191, 118, 191, 119, 191, 120, 191, 121, 191, 122, 191, 123, 191, 124, 191, 126, 191, 127, 191, 128, 191, 129, 191, 130, 191, 133, 191, 134, 191, 135, 191, 136, 191, 137, 191, 138, 191, 223, 191],
	#[2, 192, 7, 192, 10, 192, 11, 192, 38, 192, 85, 192, 87, 192, 31, 192, 33, 192, 63, 192, 81, 192, 86, 192, 42, -75, 88, 192, 101, 192, 103, 192, 99, 192, 94, 192, 84, 192, 111, 192, 89, 192, 102, 192, 83, 192, 92, 192, 82, 192, 119, 192, 105, 192, 110, 192, 98, 192, 96, 192, 112, 192, 107, 192, 97, 192, 114, 192, 109, 192, 118, 192, 106, 192, 104, 192, 115, 192, 113, 192, 116, 192, 100, 192, 108, 192, 121, 192, 120, 192, 122, 192, 123, 192, 124, 192, 126, 192, 127, 192, 128, 192, 129, 192, 130, 192, 133, 192, 134, 192, 135, 192, 136, 192, 137, 192, 138, 192, 223, 192],
	#[65535, 148, 0, 148, 1, 148, 4, 148, 7, 148, 9, 148, 10, 148, 14, 148, 15, 148, 16, 148, 17, 148, 18, 148, 19, 148, 20, 148, 21, 148, 22, 148, 23, 148, 24, 148, 25, 148, 26, 148, 27, 148, 28, 148, 29, 148, 63, 148, 40, 148, 41, 148, 79, 148, 77, 148, 75, 148, 74, 148, 80, 148, 93, 148, 76, 148, 116, 148, 115, 148, 222, 148, 223, 148],
	#[31, -59, 36, -65, 11, -66, 7, -37, 68, -64, 35, -67, 33, -60, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 69, -68, 70, -72, 71, -55, 73, -438, 92, -434, 94, -433, 115, -10],
	#[93, -442],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[37, 105, 43, -444, 42, -93, 72, -50],
	#[65535, 149, 14, 149, 18, 149, 10, 149, 0, 149, 1, 149, 4, 149, 19, 149, 7, 149, 9, 149, 17, 149, 16, 149, 15, 149, 20, 149, 21, 149, 22, 149, 23, 149, 24, 149, 25, 149, 26, 149, 27, 149, 28, 149, 29, 149, 41, 149, 40, 149, 63, 149, 74, 149, 75, 149, 76, 149, 77, 149, 79, 149, 80, 149, 93, 149, 115, 149, 116, 149, 222, 149, 223, 149],
	#[42, -446],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 215, 13, 215, 43, 215],
	#[43, -449, 13, -309],
	#[65535, 150, 63, 150, 74, 150, 75, 150, 76, 150, 77, 150, 79, 150, 80, 150, 93, 150, 115, 150, 116, 150, 222, 150, 223, 150, 14, 150, 29, 150, 1, 150, 0, 150, 27, 150, 4, 150, 7, 150, 9, 150, 10, 150, 28, 150, 24, 150, 15, 150, 16, 150, 17, 150, 18, 150, 19, 150, 20, 150, 21, 150, 22, 150, 23, 150, 25, 150, 26, 150, 41, 150, 40, 150],
	#[223, -451],
	#[31, -452],
	#[32, -454, 223, -453],
	#[38, -346],
	#[65535, 421, 21, 421, 18, 421, 15, 421, 10, 421, 14, 421, 23, 421, 17, 421, 22, 421, 0, 421, 1, 421, 20, 421, 4, 421, 7, 421, 9, 421, 40, 421, 28, 421, 27, 421, 16, 421, 25, 421, 19, 421, 41, 421, 79, 421, 75, 421, 24, 421, 26, 421, 29, 421, 74, 421, 63, 421, 77, 421, 76, 421, 116, 421, 80, 421, 115, 421, 222, 421, 223, 421],
	#[13, -458, 32, -457],
	#[65535, 425, 32, 425, 13, 425],
	#[65535, 422, 15, 422, 0, 422, 1, 422, 4, 422, 7, 422, 9, 422, 10, 422, 14, 422, 16, 422, 17, 422, 18, 422, 19, 422, 20, 422, 21, 422, 22, 422, 23, 422, 24, 422, 25, 422, 26, 422, 27, 422, 28, 422, 29, 422, 40, 422, 41, 422, 79, 422, 77, 422, 75, 422, 63, 422, 80, 422, 74, 422, 76, 422, 116, 422, 115, 422, 222, 422, 223, 422],
	#[223, -453],
	#[65535, 426, 13, 426, 32, 426],
	#[7, -37, 11, -465, 31, -462, 33, -468, 35, -466, 36, -463, 71, -55, 70, -72, 69, -68, 68, -64, 67, -61, 66, -58, 65, -71, 64, -73, 63, -63, 73, -70, 115, -10],
	#[0, 42, 1, 42, 4, 42, 7, 42, 9, 42, 10, 42, 14, 42, 15, 42, 16, 42, 17, 42, 18, 42, 19, 42, 20, 42, 21, 42, 22, 42, 23, 42, 24, 42, 25, 42, 26, 42, 27, 42, 28, 42, 29, 42, 37, 87, 40, 42, 41, 42, 42, 87, 63, 42, 72, 87, 74, 42, 75, 42, 76, 42, 77, 42, 79, 42, 80, 42, 115, 42, 116, 42, 222, 42, 223, 42],
	#[7, -37, 11, -66, 33, -60, 31, -59, 32, -486, 35, -67, 36, -65, 70, -72, 69, -68, 67, -61, 65, -71, 63, -63, 64, -73, 66, -58, 68, -64, 71, -55, 73, -70, 115, -10],
	#[0, 41, 1, 41, 4, 41, 7, 41, 9, 41, 10, 41, 14, 41, 15, 41, 16, 41, 17, 41, 18, 41, 19, 41, 20, 41, 21, 41, 22, 41, 23, 41, 24, 41, 25, 41, 26, 41, 27, 41, 28, 41, 29, 41, 37, 86, 40, 41, 41, 41, 42, 86, 63, 41, 72, 86, 74, 41, 75, 41, 76, 41, 77, 41, 79, 41, 80, 41, 115, 41, 116, 41, 222, 41, 223, 41],
	#[14, 46, 0, 46, 1, 46, 21, 46, 4, 46, 7, 46, 9, 46, 10, 46, 16, 46, 15, 46, 17, 46, 18, 46, 19, 46, 20, 46, 22, 46, 23, 46, 24, 46, 25, 46, 26, 46, 27, 46, 28, 46, 29, 46, 37, 84, 40, 46, 41, 46, 42, 84, 116, 46, 79, 46, 80, 46, 72, 84, 77, 46, 75, 46, 63, 46, 74, 46, 76, 46, 115, 46, 222, 46, 223, 46],
	#[38, -481],
	#[0, 40, 1, 40, 4, 40, 7, 40, 9, 40, 10, 40, 14, 40, 15, 40, 16, 40, 17, 40, 18, 40, 19, 40, 20, 40, 21, 40, 22, 40, 23, 40, 24, 40, 25, 40, 26, 40, 27, 40, 28, 40, 29, 40, 37, 85, 40, 40, 41, 40, 42, 85, 76, 40, 63, 40, 72, 85, 74, 40, 75, 40, 77, 40, 79, 40, 80, 40, 115, 40, 116, 40, 222, 40, 223, 40],
	#[37, 105, 42, -93, 72, -50],
	#[31, -470, 38, -469],
	#[39, -476],
	#[7, -37, 11, -66, 31, -59, 32, -472, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -113, 32, -474],
	#[34, -473],
	#[7, 39, 14, 39, 0, 39, 1, 39, 4, 39, 19, 39, 18, 39, 9, 39, 10, 39, 16, 39, 15, 39, 17, 39, 20, 39, 21, 39, 22, 39, 23, 39, 24, 39, 25, 39, 26, 39, 27, 39, 28, 39, 29, 39, 37, 96, 40, 39, 41, 39, 42, 96, 63, 39, 72, 96, 74, 39, 75, 39, 76, 39, 77, 39, 79, 39, 80, 39, 115, 39, 116, 39, 222, 39, 223, 39],
	#[34, -475],
	#[0, 38, 1, 38, 4, 38, 7, 38, 9, 38, 10, 38, 14, 38, 15, 38, 16, 38, 17, 38, 18, 38, 19, 38, 20, 38, 21, 38, 22, 38, 23, 38, 24, 38, 25, 38, 26, 38, 27, 38, 28, 38, 29, 38, 42, 95, 37, 95, 40, 38, 41, 38, 79, 38, 77, 38, 75, 38, 63, 38, 80, 38, 72, 95, 74, 38, 76, 38, 116, 38, 115, 38, 222, 38, 223, 38],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[34, -478, 37, 105, 42, -93, 72, -50],
	#[18, 45, 23, 45, 19, 45, 14, 45, 0, 45, 22, 45, 7, 45, 1, 45, 9, 45, 15, 45, 4, 45, 10, 45, 29, 45, 79, 45, 40, 45, 17, 45, 116, 45, 42, 92, 24, 45, 27, 45, 21, 45, 20, 45, 25, 45, 63, 45, 16, 45, 72, 92, 75, 45, 28, 45, 41, 45, 37, 92, 74, 45, 77, 45, 80, 45, 26, 45, 76, 45, 115, 45, 222, 45, 223, 45],
	#[37, -480],
	#[18, 43, 15, 43, 14, 43, 19, 43, 37, 88, 0, 43, 1, 43, 9, 43, 4, 43, 16, 43, 7, 43, 24, 43, 10, 43, 28, 43, 26, 43, 17, 43, 20, 43, 21, 43, 22, 43, 23, 43, 25, 43, 27, 43, 29, 43, 41, 43, 40, 43, 42, 88, 75, 43, 63, 43, 72, 88, 74, 43, 80, 43, 76, 43, 77, 43, 79, 43, 115, 43, 116, 43, 222, 43, 223, 43],
	#[39, -482],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[12, -484, 42, -93, 37, 105, 72, -50],
	#[7, 44, 14, 44, 0, 44, 1, 44, 4, 44, 19, 44, 18, 44, 9, 44, 10, 44, 16, 44, 15, 44, 17, 44, 20, 44, 21, 44, 22, 44, 23, 44, 24, 44, 25, 44, 26, 44, 27, 44, 28, 44, 29, 44, 37, 91, 40, 44, 41, 44, 42, 91, 63, 44, 72, 91, 74, 44, 75, 44, 76, 44, 77, 44, 79, 44, 80, 44, 115, 44, 116, 44, 222, 44, 223, 44],
	#[13, -113, 32, -487],
	#[23, 37, 72, 94, 27, 37, 63, 37, 28, 37, 37, 94, 77, 37, 21, 37, 15, 37, 4, 37, 10, 37, 14, 37, 79, 37, 18, 37, 116, 37, 42, 94, 0, 37, 1, 37, 9, 37, 26, 37, 40, 37, 16, 37, 7, 37, 74, 37, 24, 37, 75, 37, 17, 37, 25, 37, 19, 37, 20, 37, 22, 37, 29, 37, 41, 37, 80, 37, 76, 37, 115, 37, 222, 37, 223, 37],
	#[15, 36, 0, 36, 1, 36, 4, 36, 7, 36, 9, 36, 10, 36, 14, 36, 26, 36, 16, 36, 17, 36, 18, 36, 19, 36, 20, 36, 21, 36, 22, 36, 23, 36, 24, 36, 25, 36, 27, 36, 28, 36, 29, 36, 42, 93, 41, 36, 40, 36, 37, 93, 63, 36, 72, 93, 74, 36, 75, 36, 76, 36, 77, 36, 79, 36, 80, 36, 115, 36, 116, 36, 222, 36, 223, 36],
	#[65535, 199, 119, 199, 110, 199, 127, 199, 94, 199, 118, 199, 103, 199, 135, 199, 107, 199, 126, 199, 111, 199, 101, 199, 108, 199, 92, 199, 106, 199, 109, 199, 98, 199, 105, 199, 123, 199, 100, 199, 114, 199, 30, 199, 102, 199, 104, 199, 97, 199, 121, 199, 96, 199, 133, 199, 136, 199, 129, 199, 137, 199, 124, 199, 138, 199, 128, 199, 130, 199, 113, 199, 122, 199, 99, 199, 112, 199, 120, 199, 134, 199, 160, 199, 161, 199, 162, 199, 163, 199, 164, 199, 166, 199, 167, 199, 189, 199, 194, 199, 186, 199, 185, 199, 187, 199, 190, 199, 202, 199, 192, 199, 193, 199, 201, 199, 195, 199, 196, 199, 197, 199],
	#[31, -541, 61, -543],
	#[65535, 49, 15, 49, 0, 49, 1, 49, 4, 49, 7, 49, 9, 49, 10, 49, 14, 49, 26, 49, 16, 49, 17, 49, 18, 49, 19, 49, 20, 49, 21, 49, 22, 49, 23, 49, 24, 49, 25, 49, 27, 49, 28, 49, 29, 49, 41, 49, 40, 49, 63, 49, 74, 49, 75, 49, 76, 49, 77, 49, 79, 49, 80, 49, 115, 49, 116, 49, 222, 49, 223, 49],
	#[7, 113, 47, 113, 11, 113, 46, 113, 70, 113, 31, 113, 63, 113, 45, 113, 54, 113, 66, 113, 33, 113, 53, 113, 52, 113, 36, 113, 59, 113, 35, 113, 57, 113, 69, 113, 48, 113, 49, 113, 51, 113, 50, 113, 44, 113, 56, 113, 58, 113, 75, -44, 68, 113, 67, 113, 55, 113, 65, 113, 76, -42, 60, 113, 74, -43, 64, 113, 73, 113, 71, 113, 115, 113],
	#[7, 59, 11, 59, 44, -498, 31, 59, 33, 59, 35, 59, 36, 59, 66, 59, 45, -501, 46, -503, 47, -495, 48, -507, 49, -494, 50, -497, 51, -500, 52, -502, 53, -504, 54, -506, 55, -493, 56, -505, 57, 59, 58, 59, 59, 59, 60, 59, 63, 59, 64, 59, 65, 59, 73, 59, 67, 59, 68, 59, 69, 59, 70, 59, 71, 59, 115, 59],
	#[65535, 71, 7, 71, 11, 71, 31, 71, 33, 71, 35, 71, 36, 71, 57, 71, 58, 71, 59, 71, 60, 71, 63, 71, 64, 71, 65, 71, 66, 71, 67, 71, 68, 71, 69, 71, 70, 71, 71, 71, 73, 71, 115, 71],
	#[65535, 65, 7, 65, 11, 65, 59, 65, 31, 65, 33, 65, 35, 65, 36, 65, 70, 65, 66, 65, 57, 65, 58, 65, 73, 65, 60, 65, 63, 65, 64, 65, 65, 65, 67, 65, 68, 65, 69, 65, 71, 65, 115, 65],
	#[65535, 63, 7, 63, 11, 63, 31, 63, 33, 63, 35, 63, 36, 63, 66, 63, 57, 63, 58, 63, 59, 63, 60, 63, 63, 63, 64, 63, 65, 63, 73, 63, 67, 63, 68, 63, 69, 63, 70, 63, 71, 63, 115, 63],
	#[65535, 52, 31, 52, 61, 52],
	#[65535, 66, 7, 66, 11, 66, 59, 66, 58, 66, 31, 66, 33, 66, 35, 66, 36, 66, 71, 66, 70, 66, 69, 66, 68, 66, 67, 66, 66, 66, 57, 66, 65, 66, 64, 66, 60, 66, 63, 66, 73, 66, 115, 66],
	#[65535, 60, 7, 60, 11, 60, 36, 60, 33, 60, 31, 60, 59, 60, 35, 60, 58, 60, 71, 60, 70, 60, 69, 60, 68, 60, 67, 60, 66, 60, 57, 60, 65, 60, 64, 60, 60, 60, 63, 60, 73, 60, 115, 60],
	#[65535, 73, 31, 73, 7, 73, 11, 73, 36, 73, 33, 73, 69, 73, 59, 73, 35, 73, 58, 73, 73, 73, 57, 73, 70, 73, 66, 73, 65, 73, 60, 73, 63, 73, 64, 73, 67, 73, 68, 73, 71, 73, 115, 73],
	#[65535, 67, 7, 67, 11, 67, 31, 67, 33, 67, 35, 67, 36, 67, 57, 67, 58, 67, 59, 67, 60, 67, 63, 67, 64, 67, 65, 67, 66, 67, 67, 67, 68, 67, 69, 67, 70, 67, 71, 67, 73, 67, 115, 67],
	#[65535, 61, 7, 61, 11, 61, 31, 61, 33, 61, 35, 61, 36, 61, 57, 61, 58, 61, 59, 61, 60, 61, 63, 61, 64, 61, 65, 61, 66, 61, 67, 61, 68, 61, 69, 61, 70, 61, 71, 61, 73, 61, 115, 61],
	#[65535, 68, 7, 68, 11, 68, 31, 68, 33, 68, 35, 68, 36, 68, 70, 68, 69, 68, 65, 68, 67, 68, 57, 68, 58, 68, 59, 68, 60, 68, 63, 68, 64, 68, 66, 68, 68, 68, 71, 68, 73, 68, 115, 68],
	#[65535, 62, 7, 62, 11, 62, 59, 62, 70, 62, 31, 62, 33, 62, 35, 62, 36, 62, 71, 62, 58, 62, 69, 62, 68, 62, 67, 62, 66, 62, 57, 62, 65, 62, 64, 62, 60, 62, 63, 62, 73, 62, 115, 62],
	#[65535, 69, 11, 69, 63, 69, 7, 69, 66, 69, 68, 69, 36, 69, 59, 69, 35, 69, 58, 69, 60, 69, 67, 69, 57, 69, 31, 69, 33, 69, 65, 69, 71, 69, 64, 69, 69, 69, 70, 69, 73, 69, 115, 69],
	#[38, -508],
	#[65535, 70, 7, 70, 11, 70, 31, 70, 33, 70, 35, 70, 36, 70, 57, 70, 58, 70, 59, 70, 60, 70, 63, 70, 64, 70, 65, 70, 66, 70, 67, 70, 68, 70, 69, 70, 70, 70, 71, 70, 73, 70, 115, 70],
	#[65535, 64, 31, 64, 70, 64, 67, 64, 7, 64, 11, 64, 69, 64, 36, 64, 59, 64, 58, 64, 33, 64, 73, 64, 35, 64, 64, 64, 57, 64, 66, 64, 65, 64, 60, 64, 63, 64, 68, 64, 71, 64, 115, 64],
	#[65535, 72, 7, 72, 11, 72, 31, 72, 33, 72, 35, 72, 36, 72, 57, 72, 58, 72, 59, 72, 60, 72, 63, 72, 64, 72, 65, 72, 66, 72, 67, 72, 68, 72, 69, 72, 70, 72, 71, 72, 73, 72, 115, 72],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 57, -510, 58, -514, 59, -515, 60, -513, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -438, 115, -10],
	#[65535, 75, 63, 75, 68, 75, 36, 75, 58, 75, 7, 75, 35, 75, 11, 75, 33, 75, 71, 75, 59, 75, 66, 75, 65, 75, 60, 75, 67, 75, 57, 75, 31, 75, 64, 75, 69, 75, 70, 75, 73, 75, 115, 75],
	#[65535, 74, 7, 74, 11, 74, 31, 74, 33, 74, 35, 74, 36, 74, 57, 74, 58, 74, 59, 74, 60, 74, 63, 74, 64, 74, 65, 74, 66, 74, 67, 74, 68, 74, 69, 74, 70, 74, 71, 74, 73, 74, 115, 74],
	#[10, -518, 116, -9],
	#[65535, 78, 7, 78, 11, 78, 31, 78, 33, 78, 35, 78, 36, 78, 57, 78, 58, 78, 59, 78, 60, 78, 63, 78, 64, 78, 65, 78, 66, 78, 67, 78, 68, 78, 69, 78, 70, 78, 71, 78, 73, 78, 115, 78],
	#[65535, 76, 7, 76, 11, 76, 31, 76, 33, 76, 35, 76, 36, 76, 57, 76, 58, 76, 59, 76, 60, 76, 63, 76, 64, 76, 65, 76, 66, 76, 67, 76, 68, 76, 69, 76, 70, 76, 71, 76, 73, 76, 115, 76],
	#[65535, 77, 11, 77, 36, 77, 63, 77, 7, 77, 66, 77, 68, 77, 33, 77, 35, 77, 58, 77, 60, 77, 67, 77, 57, 77, 31, 77, 59, 77, 65, 77, 71, 77, 64, 77, 69, 77, 70, 77, 73, 77, 115, 77],
	#[65535, 57, 42, 57],
	#[42, -519],
	#[65535, 58, 42, 58],
	#[7, -37, 11, -66, 43, 201, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 95, -522, 115, -10],
	#[43, 202, 13, -537],
	#[43, -526],
	#[65535, 204, 43, 204],
	#[7, -37, 13, 320, 43, 320, 60, -88, 90, -86, 105, -91, 106, -84, 115, -10, 217, -90, 218, -85, 219, -87, 220, -89],
	#[65535, 321, 13, 321, 43, 321],
	#[65535, 205, 13, 205, 43, 205],
	#[0, 119, 1, 119, 58, 119, 4, 119, 7, 119, 9, 119, 10, 119, 14, 119, 15, 119, 16, 119, 17, 119, 18, 119, 19, 119, 20, 119, 21, 119, 22, 119, 23, 119, 24, 119, 25, 119, 26, 119, 27, 119, 28, 119, 29, 119, 31, 119, 41, 119, 40, 119, 59, 119, 57, 119, 77, 119, 76, 119, 61, 119, 63, 119, 74, 119, 75, 119, 78, -49, 79, 119, 80, 119, 90, 119, 91, 119, 171, 119, 117, 119, 116, 119, 115, 119, 172, 119, 168, 119, 169, 119, 170, 119, 173, 119, 174, 119, 175, 119, 176, 119, 177, 119, 178, 119, 179, 119, 180, 119, 181, 119, 182, 119, 183, 119, 184, 119, 222, 119, 223, 119],
	#[65535, 207, 14, 207, 0, 207, 1, 207, 21, 207, 4, 207, 7, 207, 9, 207, 10, 207, 16, 207, 15, 207, 17, 207, 18, 207, 19, 207, 20, 207, 22, 207, 23, 207, 24, 207, 25, 207, 26, 207, 27, 207, 28, 207, 29, 207, 31, 207, 40, 207, 41, 207, 57, 207, 58, 207, 116, 207, 79, 207, 80, 207, 90, 207, 77, 207, 59, 207, 117, 207, 61, 207, 75, 207, 63, 207, 74, 207, 76, 207, 91, 207, 115, 207, 168, 207, 169, 207, 170, 207, 171, 207, 172, 207, 173, 207, 174, 207, 175, 207, 176, 207, 177, 207, 178, 207, 179, 207, 180, 207, 181, 207, 182, 207, 183, 207, 184, 207, 222, 207, 223, 207],
	#[0, 209, 1, 209, 4, 209, 7, 209, 9, 209, 10, 209, 14, 209, 15, 209, 16, 209, 17, 209, 18, 209, 19, 209, 20, 209, 21, 209, 22, 209, 23, 209, 24, 209, 25, 209, 26, 209, 27, 209, 28, 209, 29, 209, 31, 209, 40, 209, 41, 209, 59, -125, 58, -124, 57, -128, 77, 209, 61, 209, 63, 209, 74, 209, 75, 209, 76, 209, 79, 209, 80, 209, 90, 209, 91, -158, 115, 209, 116, 209, 117, 209, 168, -142, 169, -137, 170, -127, 171, -130, 172, -141, 173, -136, 174, -140, 175, -123, 176, -126, 177, -129, 178, -132, 179, -135, 180, -139, 181, -133, 182, -131, 183, -134, 184, -138, 222, 209, 223, 209],
	#[0, 211, 1, 211, 4, 211, 7, 211, 9, 211, 10, 211, 14, 211, 15, 211, 16, 211, 17, 211, 18, 211, 19, 211, 20, 211, 21, 211, 22, 211, 23, 211, 24, 211, 25, 211, 26, 211, 27, 211, 28, 211, 29, 211, 31, 211, 40, 211, 41, 211, 61, 211, 63, 211, 74, 211, 75, 211, 76, 211, 77, 211, 79, 211, 80, 211, 90, -532, 115, 211, 116, 211, 117, 211, 222, 211, 223, 211],
	#[65535, 210, 0, 210, 1, 210, 4, 210, 7, 210, 9, 210, 10, 210, 14, 210, 15, 210, 16, 210, 17, 210, 18, 210, 19, 210, 20, 210, 21, 210, 22, 210, 23, 210, 24, 210, 25, 210, 26, 210, 27, 210, 28, 210, 29, 210, 31, 210, 40, 210, 41, 210, 61, 210, 63, 210, 74, 210, 75, 210, 76, 210, 77, 210, 79, 210, 80, 210, 90, 210, 115, 210, 116, 210, 117, 210, 222, 210, 223, 210],
	#[7, 213, 15, 213, 10, 213, 22, 213, 0, 213, 1, 213, 117, -535, 4, 213, 9, 213, 17, 213, 28, 213, 41, 213, 14, 213, 16, 213, 18, 213, 19, 213, 20, 213, 21, 213, 23, 213, 24, 213, 25, 213, 26, 213, 27, 213, 29, 213, 74, 213, 31, 213, 63, 213, 40, 213, 76, 213, 79, 213, 77, 213, 61, 213, 75, 213, 80, 213, 115, 213, 116, 213, 222, 213, 223, 213],
	#[38, -533],
	#[65535, 212, 0, 212, 1, 212, 4, 212, 7, 212, 9, 212, 10, 212, 14, 212, 15, 212, 16, 212, 17, 212, 18, 212, 19, 212, 20, 212, 21, 212, 22, 212, 23, 212, 24, 212, 25, 212, 26, 212, 27, 212, 28, 212, 29, 212, 31, 212, 40, 212, 41, 212, 61, 212, 63, 212, 74, 212, 75, 212, 76, 212, 77, 212, 79, 212, 80, 212, 115, 212, 116, 212, 117, 212, 222, 212, 223, 212],
	#[65535, 54, 18, 54, 0, 54, 1, 54, 20, 54, 4, 54, 7, 54, 9, 54, 10, 54, 14, 54, 15, 54, 16, 54, 17, 54, 25, 54, 19, 54, 21, 54, 22, 54, 23, 54, 24, 54, 26, 54, 27, 54, 28, 54, 29, 54, 31, 54, 40, 54, 41, 54, 61, 54, 63, 54, 74, 54, 75, 54, 76, 54, 77, 54, 79, 54, 80, 54, 115, 54, 116, 54, 222, 54, 223, 54],
	#[7, -536],
	#[65535, 214, 7, 214, 15, 214, 14, 214, 1, 214, 10, 214, 0, 214, 4, 214, 19, 214, 9, 214, 17, 214, 27, 214, 16, 214, 18, 214, 24, 214, 20, 214, 21, 214, 22, 214, 23, 214, 25, 214, 26, 214, 41, 214, 28, 214, 29, 214, 31, 214, 63, 214, 40, 214, 76, 214, 75, 214, 79, 214, 74, 214, 77, 214, 80, 214, 61, 214, 115, 214, 116, 214, 222, 214, 223, 214],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 66, -58, 63, -63, 64, -73, 65, -71, 95, -539, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, -37, 13, 320, 43, 320, 60, -88, 90, -86, 105, -91, 106, -84, 115, -10, 217, -90, 218, -85, 219, -87, 220, -89],
	#[65535, 203, 43, 203],
	#[65535, 206, 13, 206, 43, 206],
	#[7, 270, 63, 270, 92, 270, 94, 270, 96, 270, 97, 270, 98, 270, 99, 270, 100, 270, 101, 270, 102, 270, 103, 270, 104, 270, 105, 270, 106, 270, 107, 270, 108, 270, 109, 270, 110, 270, 111, 270, 112, 270, 113, 270, 114, 270, 115, 270, 118, 270, 119, 270, 120, 270, 121, 270, 122, 270, 123, 270, 124, 270, 126, 270, 127, 270, 128, 270, 129, 270, 130, 270, 133, 270, 134, 270, 135, 270, 136, 270, 137, 270, 138, 270, 159, -547, 160, 270, 161, 270, 162, 270, 163, 270, 164, 270, 166, 270, 167, 270, 185, 270, 186, 270, 187, 270, 189, 270, 190, 270, 192, 270, 193, 270, 194, 270, 195, 270, 196, 270, 197, 270, 201, 270, 202, 270],
	#[65535, 51, 7, 51, 14, 51, 0, 51, 1, 51, 4, 51, 19, 51, 18, 51, 9, 51, 10, 51, 16, 51, 15, 51, 17, 51, 20, 51, 21, 51, 22, 51, 23, 51, 24, 51, 25, 51, 26, 51, 27, 51, 28, 51, 29, 51, 40, 51, 41, 51, 63, 51, 74, 51, 75, 51, 76, 51, 77, 51, 79, 51, 80, 51, 115, 51, 116, 51, 222, 51, 223, 51],
	#[7, 270, 63, 270, 101, 270, 103, 270, 94, 270, 118, 270, 111, 270, 102, 270, 92, 270, 99, 270, 119, 270, 105, 270, 110, 270, 98, 270, 96, 270, 107, 270, 97, 270, 114, 270, 109, 270, 112, 270, 106, 270, 104, 270, 115, 270, 113, 270, 100, 270, 108, 270, 121, 270, 120, 270, 122, 270, 123, 270, 124, 270, 126, 270, 127, 270, 128, 270, 129, 270, 130, 270, 133, 270, 134, 270, 135, 270, 136, 270, 137, 270, 138, 270, 159, -547, 160, 270, 161, 270, 162, 270, 163, 270, 164, 270, 166, 270, 167, 270, 185, 270, 186, 270, 187, 270, 189, 270, 190, 270, 192, 270, 193, 270, 194, 270, 195, 270, 196, 270, 197, 270, 201, 270, 202, 270],
	#[92, 270, 135, 270, 110, 270, 134, 270, 107, 270, 7, 270, 103, 270, 126, 270, 111, 270, 94, 270, 119, 270, 63, 270, 108, 270, 105, 270, 102, 270, 127, 270, 109, 270, 100, 270, 62, -866, 106, 270, 104, 270, 99, 270, 113, 270, 101, 270, 118, 270, 97, 270, 123, 270, 163, 270, 122, 270, 96, 270, 133, 270, 124, 270, 162, 270, 130, 270, 98, 270, 129, 270, 112, 270, 114, 270, 115, 270, 166, 270, 120, 270, 161, 270, 138, 270, 137, 270, 128, 270, 136, 270, 121, 270, 159, -547, 160, 270, 164, 270, 167, 270, 185, 270, 186, 270, 187, 270, 189, 270, 190, 270, 202, 270, 192, 270, 193, 270, 194, 270, 195, 270, 196, 270, 197, 270, 201, 270],
	#[7, -37, 63, -33, 92, 195, 94, 195, 96, 195, 97, 195, 98, 195, 99, 195, 100, 195, 101, 195, 102, 195, 103, 195, 104, 195, 105, 195, 106, 195, 107, 195, 108, 195, 109, 195, 110, 195, 111, 195, 112, 195, 113, 195, 114, 195, 115, -10, 118, 195, 119, 195, 120, 195, 121, 195, 122, 195, 123, 195, 124, 195, 126, 195, 127, 195, 128, 195, 129, 195, 130, 195, 133, 195, 134, 195, 135, 195, 136, 195, 137, 195, 138, 195, 160, 195, 161, 195, 162, 195, 163, 195, 164, 195, 166, 195, 167, 195, 185, 195, 186, 195, 187, 195, 189, 195, 190, 195, 192, 195, 193, 195, 194, 195, 195, 195, 196, 195, 197, 195, 201, 195, 202, 195],
	#[65535, 81, 7, 81, 32, 81, 62, 81, 63, 81, 92, 81, 94, 81, 96, 81, 97, 81, 98, 81, 99, 81, 100, 81, 101, 81, 102, 81, 103, 81, 104, 81, 105, 81, 106, 81, 107, 81, 108, 81, 109, 81, 110, 81, 111, 81, 112, 81, 113, 81, 114, 81, 115, 81, 118, 81, 119, 81, 120, 81, 121, 81, 122, 81, 123, 81, 124, 81, 126, 81, 127, 81, 128, 81, 129, 81, 130, 81, 133, 81, 134, 81, 135, 81, 136, 81, 137, 81, 138, 81, 159, 81, 160, 81, 161, 81, 162, 81, 163, 81, 164, 81, 166, 81, 167, 81, 185, 81, 186, 81, 187, 81, 189, 81, 190, 81, 192, 81, 193, 81, 194, 81, 195, 81, 196, 81, 197, 81, 201, 81, 202, 81],
	#[65535, 271, 7, 271, 63, 271, 92, 271, 94, 271, 96, 271, 97, 271, 98, 271, 99, 271, 100, 271, 101, 271, 102, 271, 103, 271, 104, 271, 105, 271, 106, 271, 107, 271, 108, 271, 109, 271, 110, 271, 111, 271, 112, 271, 113, 271, 114, 271, 115, 271, 118, 271, 119, 271, 120, 271, 121, 271, 122, 271, 123, 271, 124, 271, 126, 271, 127, 271, 128, 271, 129, 271, 130, 271, 133, 271, 134, 271, 135, 271, 136, 271, 137, 271, 138, 271, 160, 271, 161, 271, 162, 271, 163, 271, 164, 271, 166, 271, 167, 271, 185, 271, 186, 271, 187, 271, 189, 271, 190, 271, 192, 271, 193, 271, 194, 271, 195, 271, 196, 271, 197, 271, 201, 271, 202, 271],
	#[92, -189, 94, -576, 96, -555, 97, -582, 98, -556, 99, -583, 100, -562, 101, -573, 102, -553, 103, -578, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 160, -558, 161, -571, 162, -575, 163, -566, 164, -569, 166, -568, 167, -563, 185, -564, 186, -579, 187, -567, 189, -561, 190, -570, 192, -581, 193, -560, 194, -586, 195, -557, 196, -577, 197, -559, 201, -587, 202, -585],
	#[92, -189, 94, -576, 96, -555, 97, -582, 98, -556, 99, -583, 100, -562, 101, -573, 102, -553, 103, -578, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 160, -558, 161, -571, 162, -575, 163, -566, 164, -569, 166, -568, 167, -563, 185, -564, 186, -579, 187, -567, 189, -561, 190, -570, 192, -581, 193, -560, 194, -586, 195, -557, 196, -577, 197, -559, 201, -587, 202, -585],
	#[65535, 272, 7, 272, 63, 272, 101, 272, 94, 272, 118, 272, 109, 272, 111, 272, 102, 272, 92, 272, 119, 272, 106, 272, 105, 272, 110, 272, 98, 272, 107, 272, 97, 272, 114, 272, 96, 272, 112, 272, 99, 272, 103, 272, 115, 272, 113, 272, 104, 272, 100, 272, 108, 272, 121, 272, 120, 272, 122, 272, 123, 272, 124, 272, 126, 272, 127, 272, 128, 272, 129, 272, 130, 272, 133, 272, 134, 272, 135, 272, 136, 272, 137, 272, 138, 272, 160, 272, 161, 272, 162, 272, 163, 272, 164, 272, 166, 272, 167, 272, 185, 272, 186, 272, 187, 272, 189, 272, 190, 272, 192, 272, 193, 272, 194, 272, 195, 272, 196, 272, 197, 272, 201, 272, 202, 272],
	#[6, -552],
	#[65535, 196, 94, 196, 118, 196, 103, 196, 135, 196, 102, 196, 108, 196, 92, 196, 119, 196, 110, 196, 100, 196, 109, 196, 138, 196, 99, 196, 111, 196, 97, 196, 130, 196, 98, 196, 122, 196, 124, 196, 105, 196, 114, 196, 128, 196, 107, 196, 106, 196, 96, 196, 123, 196, 127, 196, 113, 196, 121, 196, 101, 196, 137, 196, 126, 196, 104, 196, 120, 196, 136, 196, 129, 196, 112, 196, 133, 196, 134, 196, 160, 196, 161, 196, 162, 196, 163, 196, 164, 196, 166, 196, 167, 196, 185, 196, 186, 196, 187, 196, 189, 196, 190, 196, 202, 196, 192, 196, 193, 196, 194, 196, 195, 196, 196, 196, 197, 196, 201, 196],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, -37, 11, -66, 70, -72, 31, -59, 33, -60, 35, -67, 36, -65, 69, -68, 67, -61, 66, -58, 65, -71, 63, -63, 64, -73, 73, -70, 68, -64, 71, -55, 115, -10],
	#[87, -398, 88, -396, 145, -397, 146, -400, 147, -402, 148, -404, 149, -390, 150, -393, 151, -405, 152, -399, 153, -401, 154, -403, 155, -389, 156, -391, 157, -394, 158, -392],
	#[118, 415, 120, 415, 136, 415, 137, 415, 138, 415, 192, -589, 210, 415, 211, 415, 212, 415, 213, 415, 214, 415, 215, 415],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 66, -58, 63, -63, 64, -73, 65, -71, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -819, 115, -10],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[31, 415, 7, 415, 70, 415, 69, 415, 63, 415, 11, 415, 115, 415, 68, 415, 73, 415, 66, 415, 71, 415, 67, 415, 36, 415, 35, 415, 33, 415, 64, 415, 65, 415, 191, -800, 192, -589],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 68, -64, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 194, 7, 194, 32, 194, 62, 194, 63, 194, 92, 194, 94, 194, 96, 194, 97, 194, 98, 194, 99, 194, 100, 194, 101, 194, 102, 194, 103, 194, 104, 194, 105, 194, 106, 194, 107, 194, 108, 194, 109, 194, 110, 194, 111, 194, 112, 194, 113, 194, 114, 194, 115, 194, 118, 194, 119, 194, 120, 194, 121, 194, 122, 194, 123, 194, 124, 194, 126, 194, 127, 194, 128, 194, 129, 194, 130, 194, 133, 194, 134, 194, 135, 194, 136, 194, 137, 194, 138, 194, 159, 194, 160, 194, 161, 194, 162, 194, 163, 194, 164, 194, 166, 194, 167, 194, 185, 194, 186, 194, 187, 194, 189, 194, 190, 194, 192, 194, 193, 194, 194, 194, 195, 194, 196, 194, 197, 194, 201, 194, 202, 194],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 71, -55, 68, -64, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 69, -68, 70, -72, 73, -70, 115, -10],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[119, 436, 63, 436, 127, 436, 124, 436, 92, 436, 103, 436, 135, 436, 100, 436, 126, 436, 111, 436, 101, 436, 62, 436, 114, 436, 115, 436, 134, 436, 106, 436, 129, 436, 109, 436, 107, 436, 7, 436, 122, 436, 123, 436, 163, 436, 99, 436, 159, 436, 136, 436, 113, 436, 121, 436, 108, 436, 96, 436, 112, 436, 128, 436, 110, 436, 98, 436, 120, 436, 94, 436, 118, 436, 137, 436, 130, 436, 102, 436, 164, 436, 32, 436, 13, -606, 105, 436, 133, 436, 167, 436, 196, 436, 97, 436, 104, 436, 161, 436, 160, 436, 138, 436, 185, 436, 162, 436, 197, 436, 166, 436, 187, 436, 195, 436, 194, 436, 186, 436, 201, 436, 189, 436, 190, 436, 202, 436, 192, 436, 193, 436],
	#[7, 59, 11, 59, 46, -503, 31, 59, 33, 59, 35, 59, 36, 59, 44, -498, 45, -501, 47, -495, 48, -507, 49, -494, 50, -497, 51, -500, 52, -502, 53, -504, 54, -506, 55, -493, 56, -505, 57, 59, 58, 59, 59, 59, 60, 59, 63, 59, 64, 59, 65, 59, 66, 59, 67, 59, 68, 59, 69, 59, 70, 59, 71, 59, 73, 59, 115, 59],
	#[7, 415, 11, 415, 31, 415, 33, 415, 35, 415, 36, 415, 63, 415, 64, 415, 65, 415, 66, 415, 67, 415, 68, 415, 69, 415, 70, 415, 71, 415, 73, 415, 115, 415, 191, -733, 192, -589],
	#[69, -722, 36, -721],
	#[7, 59, 11, 59, 33, 59, 31, 59, 66, 59, 35, 59, 36, 59, 44, -498, 45, -501, 46, -503, 47, -495, 48, -507, 49, -494, 50, -497, 51, -500, 52, -502, 53, -504, 54, -506, 55, -493, 56, -505, 57, 59, 58, 59, 59, 59, 60, 59, 63, 59, 64, 59, 65, 59, 73, 59, 67, 59, 68, 59, 69, 59, 70, 59, 71, 59, 115, 59],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[31, -59, 63, -63, 7, -37, 115, -10, 69, -68, 11, -66, 68, -64, 70, -72, 67, -61, 35, -67, 73, -70, 66, -58, 33, -60, 71, -55, 36, -65, 64, -73, 65, -71],
	#[36, -669],
	#[11, 417, 70, 417, 31, 417, 69, 417, 63, 417, 7, 417, 71, 417, 115, 417, 36, 417, 221, -303, 33, 417, 35, 417, 66, 417, 73, 417, 68, 417, 65, 417, 67, 417, 64, 417],
	#[203, -597, 204, 357, 205, 357, 206, 357, 207, 357, 208, 357, 209, 357],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[63, -63, 68, -64, 66, -58, 36, -65, 7, -37, 11, -66, 71, -55, 67, -61, 35, -67, 65, -71, 33, -60, 31, -59, 64, -73, 69, -68, 70, -72, 73, -70, 115, -10],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[190, -624, 193, -623],
	#[148, -283, 147, -281, 141, -282, 143, -274, 140, -279, 142, -284, 139, -277, 146, -278, 145, -276, 144, -275],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 274, 119, 274, 127, 274, 166, 274, 135, 274, 124, 274, 108, 274, 118, 274, 63, 274, 163, 274, 7, 274, 164, 274, 161, 274, 126, 274, 98, 274, 122, 274, 110, 274, 134, 274, 133, 274, 167, 274, 111, 274, 162, 274, 101, 274, 107, 274, 100, 274, 187, 274, 114, 274, 138, 274, 106, 274, 115, 274, 130, 274, 112, 274, 128, 274, 196, 274, 121, 274, 129, 274, 123, 274, 97, 274, 136, 274, 94, 274, 137, 274, 160, 274, 96, 274, 190, 274, 92, 274, 102, 274, 189, 274, 99, 274, 105, 274, 120, 274, 185, 274, 195, 274, 104, 274, 113, 274, 109, 274, 103, 274, 202, 274, 192, 274, 194, 274, 193, 274, 197, 274, 201, 274, 186, 274],
	#[65535, 356, 7, 356, 11, 356, 31, 356, 33, 356, 35, 356, 36, 356, 44, 356, 45, 356, 46, 356, 47, 356, 48, 356, 49, 356, 50, 356, 51, 356, 52, 356, 53, 356, 54, 356, 55, 356, 56, 356, 57, 356, 58, 356, 59, 356, 60, 356, 63, 356, 64, 356, 65, 356, 66, 356, 67, 356, 68, 356, 69, 356, 70, 356, 71, 356, 73, 356, 115, 356],
	#[7, 415, 11, 415, 31, 415, 33, 415, 35, 415, 36, 415, 63, 415, 64, 415, 65, 415, 66, 415, 67, 415, 68, 415, 69, 415, 70, 415, 71, 415, 73, 415, 115, 415, 192, -589],
	#[202, -588],
	#[65535, 355, 7, 355, 11, 355, 46, 355, 31, 355, 33, 355, 35, 355, 36, 355, 44, 355, 45, 355, 47, 355, 48, 355, 49, 355, 50, 355, 51, 355, 52, 355, 53, 355, 54, 355, 55, 355, 56, 355, 57, 355, 58, 355, 59, 355, 60, 355, 63, 355, 64, 355, 65, 355, 66, 355, 67, 355, 68, 355, 69, 355, 70, 355, 71, 355, 73, 355, 115, 355],
	#[65535, 416, 66, 416, 63, 416, 7, 416, 11, 416, 68, 416, 36, 416, 35, 416, 118, 416, 67, 416, 31, 416, 33, 416, 65, 416, 71, 416, 64, 416, 69, 416, 70, 416, 73, 416, 138, 416, 137, 416, 136, 416, 115, 416, 120, 416, 213, 416, 212, 416, 211, 416, 215, 416, 214, 416, 210, 416],
	#[66, -58, 63, -63, 7, -37, 11, -66, 68, -64, 36, -65, 35, -67, 67, -61, 31, -59, 33, -60, 65, -71, 71, -55, 64, -73, 69, -68, 70, -72, 73, -70, 115, -10],
	#[13, -592],
	#[31, -59, 7, -37, 11, -66, 69, -68, 35, -67, 33, -60, 63, -63, 36, -65, 64, -73, 67, -61, 71, -55, 115, -10, 68, -64, 66, -58, 65, -71, 73, -70, 70, -72],
	#[13, -594],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[203, -597, 204, 357, 205, 357, 206, 357, 207, 357, 208, 357, 209, 357],
	#[204, -603, 205, -600, 206, -598, 207, -599, 208, -601, 209, -602],
	#[65535, 358, 204, 358, 205, 358, 206, 358, 207, 358, 208, 358, 209, 358],
	#[65535, 361, 7, 361, 13, 361, 63, 361, 98, 361, 92, 361, 94, 361, 96, 361, 97, 361, 99, 361, 100, 361, 101, 361, 102, 361, 103, 361, 104, 361, 105, 361, 106, 361, 107, 361, 108, 361, 109, 361, 110, 361, 111, 361, 112, 361, 113, 361, 114, 361, 115, 361, 118, 361, 119, 361, 120, 361, 121, 361, 122, 361, 123, 361, 124, 361, 126, 361, 127, 361, 128, 361, 129, 361, 130, 361, 133, 361, 134, 361, 135, 361, 136, 361, 137, 361, 138, 361, 160, 361, 161, 361, 162, 361, 163, 361, 164, 361, 166, 361, 167, 361, 185, 361, 186, 361, 187, 361, 189, 361, 190, 361, 192, 361, 193, 361, 194, 361, 195, 361, 196, 361, 197, 361, 201, 361, 202, 361],
	#[65535, 362, 7, 362, 13, 362, 103, 362, 63, 362, 101, 362, 94, 362, 115, 362, 111, 362, 102, 362, 92, 362, 99, 362, 119, 362, 105, 362, 110, 362, 96, 362, 112, 362, 107, 362, 97, 362, 114, 362, 109, 362, 118, 362, 106, 362, 104, 362, 98, 362, 113, 362, 100, 362, 108, 362, 121, 362, 120, 362, 122, 362, 123, 362, 124, 362, 126, 362, 127, 362, 128, 362, 129, 362, 130, 362, 133, 362, 134, 362, 135, 362, 136, 362, 137, 362, 138, 362, 160, 362, 161, 362, 162, 362, 163, 362, 164, 362, 166, 362, 167, 362, 185, 362, 186, 362, 187, 362, 189, 362, 190, 362, 192, 362, 193, 362, 194, 362, 195, 362, 196, 362, 197, 362, 201, 362, 202, 362],
	#[65535, 360, 127, 360, 115, 360, 135, 360, 167, 360, 161, 360, 163, 360, 110, 360, 92, 360, 106, 360, 13, 360, 63, 360, 7, 360, 122, 360, 118, 360, 133, 360, 129, 360, 138, 360, 194, 360, 130, 360, 126, 360, 108, 360, 107, 360, 196, 360, 123, 360, 164, 360, 111, 360, 134, 360, 100, 360, 98, 360, 160, 360, 136, 360, 94, 360, 120, 360, 101, 360, 162, 360, 102, 360, 137, 360, 128, 360, 104, 360, 114, 360, 103, 360, 97, 360, 121, 360, 105, 360, 113, 360, 109, 360, 124, 360, 119, 360, 112, 360, 96, 360, 99, 360, 197, 360, 166, 360, 186, 360, 195, 360, 185, 360, 193, 360, 187, 360, 189, 360, 190, 360, 202, 360, 192, 360, 201, 360],
	#[65535, 363, 7, 363, 13, 363, 63, 363, 98, 363, 92, 363, 94, 363, 96, 363, 97, 363, 99, 363, 100, 363, 101, 363, 102, 363, 103, 363, 104, 363, 105, 363, 106, 363, 107, 363, 108, 363, 109, 363, 110, 363, 111, 363, 112, 363, 113, 363, 114, 363, 115, 363, 118, 363, 119, 363, 120, 363, 121, 363, 122, 363, 123, 363, 124, 363, 126, 363, 127, 363, 128, 363, 129, 363, 130, 363, 133, 363, 134, 363, 135, 363, 136, 363, 137, 363, 138, 363, 160, 363, 161, 363, 162, 363, 163, 363, 164, 363, 166, 363, 167, 363, 185, 363, 186, 363, 187, 363, 189, 363, 190, 363, 192, 363, 193, 363, 194, 363, 195, 363, 196, 363, 197, 363, 201, 363, 202, 363],
	#[65535, 364, 7, 364, 13, 364, 63, 364, 92, 364, 94, 364, 96, 364, 97, 364, 98, 364, 99, 364, 100, 364, 101, 364, 102, 364, 103, 364, 104, 364, 105, 364, 106, 364, 107, 364, 108, 364, 109, 364, 110, 364, 111, 364, 112, 364, 113, 364, 114, 364, 115, 364, 118, 364, 119, 364, 120, 364, 121, 364, 122, 364, 123, 364, 124, 364, 126, 364, 127, 364, 128, 364, 129, 364, 130, 364, 133, 364, 134, 364, 135, 364, 136, 364, 137, 364, 138, 364, 160, 364, 161, 364, 162, 364, 163, 364, 164, 364, 166, 364, 167, 364, 185, 364, 186, 364, 187, 364, 189, 364, 190, 364, 192, 364, 193, 364, 194, 364, 195, 364, 196, 364, 197, 364, 201, 364, 202, 364],
	#[65535, 359, 63, 359, 7, 359, 13, 359, 103, 359, 110, 359, 109, 359, 111, 359, 98, 359, 97, 359, 108, 359, 92, 359, 94, 359, 106, 359, 96, 359, 99, 359, 100, 359, 101, 359, 102, 359, 114, 359, 104, 359, 105, 359, 113, 359, 107, 359, 112, 359, 115, 359, 118, 359, 119, 359, 120, 359, 121, 359, 122, 359, 123, 359, 124, 359, 126, 359, 127, 359, 128, 359, 129, 359, 130, 359, 133, 359, 134, 359, 135, 359, 136, 359, 137, 359, 138, 359, 160, 359, 161, 359, 162, 359, 163, 359, 164, 359, 166, 359, 167, 359, 185, 359, 186, 359, 187, 359, 189, 359, 190, 359, 192, 359, 193, 359, 194, 359, 195, 359, 196, 359, 197, 359, 201, 359, 202, 359],
	#[7, 436, 13, -606, 63, 436, 98, 436, 92, 436, 94, 436, 96, 436, 97, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 343, 63, 343, 126, 343, 124, 343, 92, 343, 109, 343, 7, 343, 94, 343, 118, 343, 133, 343, 103, 343, 108, 343, 106, 343, 100, 343, 119, 343, 101, 343, 99, 343, 102, 343, 98, 343, 96, 343, 127, 343, 105, 343, 123, 343, 138, 343, 135, 343, 104, 343, 114, 343, 111, 343, 113, 343, 130, 343, 110, 343, 122, 343, 97, 343, 137, 343, 121, 343, 129, 343, 115, 343, 128, 343, 120, 343, 136, 343, 107, 343, 112, 343, 134, 343, 160, 343, 161, 343, 162, 343, 163, 343, 164, 343, 166, 343, 167, 343, 185, 343, 186, 343, 187, 343, 189, 343, 190, 343, 202, 343, 192, 343, 193, 343, 194, 343, 195, 343, 196, 343, 197, 343, 201, 343],
	#[222, -608],
	#[7, 437, 13, -610, 32, 437, 62, 437, 63, 437, 98, 437, 92, 437, 94, 437, 96, 437, 97, 437, 99, 437, 100, 437, 101, 437, 102, 437, 103, 437, 104, 437, 105, 437, 106, 437, 107, 437, 108, 437, 109, 437, 110, 437, 111, 437, 112, 437, 113, 437, 114, 437, 115, 437, 118, 437, 119, 437, 120, 437, 121, 437, 122, 437, 123, 437, 124, 437, 126, 437, 127, 437, 128, 437, 129, 437, 130, 437, 133, 437, 134, 437, 135, 437, 136, 437, 137, 437, 138, 437, 159, 437, 160, 437, 161, 437, 162, 437, 163, 437, 164, 437, 166, 437, 167, 437, 185, 437, 186, 437, 187, 437, 189, 437, 190, 437, 192, 437, 193, 437, 194, 437, 195, 437, 196, 437, 197, 437, 201, 437, 202, 437],
	#[223, -209],
	#[65535, 438, 7, 438, 13, 438, 32, 438, 62, 438, 63, 438, 98, 438, 92, 438, 94, 438, 96, 438, 97, 438, 99, 438, 100, 438, 101, 438, 102, 438, 103, 438, 104, 438, 105, 438, 106, 438, 107, 438, 108, 438, 109, 438, 110, 438, 111, 438, 112, 438, 113, 438, 114, 438, 115, 438, 118, 438, 119, 438, 120, 438, 121, 438, 122, 438, 123, 438, 124, 438, 126, 438, 127, 438, 128, 438, 129, 438, 130, 438, 133, 438, 134, 438, 135, 438, 136, 438, 137, 438, 138, 438, 159, 438, 160, 438, 161, 438, 162, 438, 163, 438, 164, 438, 166, 438, 167, 438, 185, 438, 186, 438, 187, 438, 189, 438, 190, 438, 192, 438, 193, 438, 194, 438, 195, 438, 196, 438, 197, 438, 201, 438, 202, 438],
	#[222, -611],
	#[223, -209],
	#[65535, 439, 7, 439, 13, 439, 32, 439, 62, 439, 63, 439, 98, 439, 92, 439, 94, 439, 96, 439, 97, 439, 99, 439, 100, 439, 101, 439, 102, 439, 103, 439, 104, 439, 105, 439, 106, 439, 107, 439, 108, 439, 109, 439, 110, 439, 111, 439, 112, 439, 113, 439, 114, 439, 115, 439, 118, 439, 119, 439, 120, 439, 121, 439, 122, 439, 123, 439, 124, 439, 126, 439, 127, 439, 128, 439, 129, 439, 130, 439, 133, 439, 134, 439, 135, 439, 136, 439, 137, 439, 138, 439, 159, 439, 160, 439, 161, 439, 162, 439, 163, 439, 164, 439, 166, 439, 167, 439, 185, 439, 186, 439, 187, 439, 189, 439, 190, 439, 192, 439, 193, 439, 194, 439, 195, 439, 196, 439, 197, 439, 201, 439, 202, 439],
	#[13, -614],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, 436, 13, -606, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 138, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 167, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 190, 436, 189, 436, 187, 436, 186, 436, 192, 436, 185, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 329, 7, 329, 167, 329, 63, 329, 103, 329, 92, 329, 94, 329, 106, 329, 96, 329, 97, 329, 98, 329, 99, 329, 100, 329, 101, 329, 102, 329, 114, 329, 104, 329, 105, 329, 113, 329, 107, 329, 108, 329, 109, 329, 110, 329, 111, 329, 112, 329, 115, 329, 118, 329, 119, 329, 120, 329, 121, 329, 122, 329, 123, 329, 124, 329, 126, 329, 127, 329, 128, 329, 129, 329, 130, 329, 133, 329, 134, 329, 135, 329, 136, 329, 137, 329, 138, 329, 160, 329, 161, 329, 162, 329, 163, 329, 164, 329, 166, 329, 190, 329, 189, 329, 187, 329, 192, 329, 185, 329, 186, 329, 193, 329, 194, 329, 195, 329, 196, 329, 197, 329, 201, 329, 202, 329],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 37, 105, 38, -316, 42, -93, 63, -219, 72, -50, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[13, -620],
	#[31, -326, 2, -327, 94, -210, 7, -37, 10, -212, 11, -318, 33, -321, 92, -189, 118, -181, 87, -320, 116, -9, 119, -231, 89, -329, 63, -219, 38, -316, 81, -312, 86, -325, 110, -233, 107, -229, 88, -313, 101, -214, 103, -228, 85, -317, 97, -217, 82, -315, 84, -322, 109, -180, 111, -222, 98, -192, 102, -178, 83, -330, 121, -201, 99, -218, 96, -182, 112, -206, 105, -173, 106, -227, 104, -185, 115, -10, 113, -194, 100, -193, 108, -199, 114, -183, 120, -226, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[7, 436, 13, -606, 63, 436, 98, 436, 97, 436, 101, 436, 100, 436, 94, 436, 99, 436, 103, 436, 102, 436, 104, 436, 105, 436, 92, 436, 107, 436, 96, 436, 106, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 324, 7, 324, 92, 324, 94, 324, 102, 324, 63, 324, 110, 324, 101, 324, 103, 324, 97, 324, 118, 324, 115, 324, 109, 324, 111, 324, 107, 324, 96, 324, 121, 324, 99, 324, 119, 324, 112, 324, 105, 324, 106, 324, 104, 324, 98, 324, 113, 324, 100, 324, 108, 324, 114, 324, 120, 324, 122, 324, 123, 324, 124, 324, 126, 324, 127, 324, 128, 324, 129, 324, 130, 324, 133, 324, 134, 324, 135, 324, 136, 324, 137, 324, 138, 324, 160, 324, 161, 324, 162, 324, 163, 324, 164, 324, 166, 324, 167, 324, 185, 324, 186, 324, 187, 324, 189, 324, 190, 324, 192, 324, 193, 324, 194, 324, 195, 324, 196, 324, 197, 324, 201, 324, 202, 324],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[31, -59, 7, -37, 11, -66, 69, -68, 33, -60, 35, -67, 36, -65, 73, -70, 70, -72, 66, -58, 65, -71, 63, -63, 64, -73, 67, -61, 68, -64, 71, -55, 115, -10],
	#[7, 436, 13, -628, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 339, 7, 339, 63, 339, 92, 339, 94, 339, 96, 339, 97, 339, 98, 339, 99, 339, 100, 339, 101, 339, 102, 339, 103, 339, 104, 339, 105, 339, 106, 339, 107, 339, 108, 339, 109, 339, 110, 339, 111, 339, 112, 339, 113, 339, 114, 339, 115, 339, 118, 339, 119, 339, 120, 339, 121, 339, 122, 339, 123, 339, 124, 339, 126, 339, 127, 339, 128, 339, 129, 339, 130, 339, 133, 339, 134, 339, 135, 339, 136, 339, 137, 339, 138, 339, 160, 339, 161, 339, 162, 339, 163, 339, 164, 339, 166, 339, 167, 339, 185, 339, 186, 339, 187, 339, 189, 339, 190, 339, 192, 339, 193, 339, 194, 339, 195, 339, 196, 339, 197, 339, 201, 339, 202, 339],
	#[65535, 419, 7, 419, 63, 419, 92, 419, 94, 419, 96, 419, 97, 419, 98, 419, 99, 419, 100, 419, 101, 419, 102, 419, 103, 419, 104, 419, 105, 419, 106, 419, 107, 419, 108, 419, 109, 419, 110, 419, 111, 419, 112, 419, 113, 419, 114, 419, 115, 419, 118, 419, 119, 419, 120, 419, 121, 419, 122, 419, 123, 419, 124, 419, 126, 419, 127, 419, 128, 419, 129, 419, 130, 419, 133, 419, 134, 419, 135, 419, 136, 419, 137, 419, 138, 419, 160, 419, 161, 419, 162, 419, 163, 419, 164, 419, 166, 419, 167, 419, 185, 419, 186, 419, 187, 419, 189, 419, 190, 419, 192, 419, 193, 419, 194, 419, 195, 419, 196, 419, 197, 419, 201, 419, 202, 419],
	#[90, -629, 222, -608],
	#[38, -630],
	#[7, 436, 13, -606, 63, 436, 101, 436, 103, 436, 94, 436, 111, 436, 102, 436, 92, 436, 99, 436, 119, 436, 105, 436, 110, 436, 98, 436, 96, 436, 112, 436, 107, 436, 97, 436, 114, 436, 109, 436, 118, 436, 106, 436, 104, 436, 115, 436, 113, 436, 100, 436, 108, 436, 121, 436, 120, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 420, 94, 420, 7, 420, 63, 420, 110, 420, 107, 420, 101, 420, 103, 420, 97, 420, 118, 420, 96, 420, 109, 420, 111, 420, 98, 420, 102, 420, 92, 420, 114, 420, 99, 420, 119, 420, 112, 420, 105, 420, 106, 420, 104, 420, 115, 420, 113, 420, 100, 420, 108, 420, 121, 420, 120, 420, 122, 420, 123, 420, 124, 420, 126, 420, 127, 420, 128, 420, 129, 420, 130, 420, 133, 420, 134, 420, 135, 420, 136, 420, 137, 420, 138, 420, 160, 420, 161, 420, 162, 420, 163, 420, 164, 420, 166, 420, 167, 420, 185, 420, 186, 420, 187, 420, 189, 420, 190, 420, 192, 420, 193, 420, 194, 420, 195, 420, 196, 420, 197, 420, 201, 420, 202, 420],
	#[13, -633],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 71, -55, 70, -72, 69, -68, 68, -64, 67, -61, 66, -58, 65, -71, 64, -73, 63, -63, 73, -70, 115, -10],
	#[7, 436, 13, -628, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 342, 7, 342, 102, 342, 103, 342, 99, 342, 101, 342, 118, 342, 109, 342, 126, 342, 98, 342, 94, 342, 100, 342, 106, 342, 112, 342, 111, 342, 97, 342, 105, 342, 92, 342, 120, 342, 119, 342, 63, 342, 110, 342, 96, 342, 107, 342, 113, 342, 121, 342, 104, 342, 108, 342, 114, 342, 115, 342, 122, 342, 123, 342, 124, 342, 127, 342, 128, 342, 129, 342, 130, 342, 133, 342, 134, 342, 135, 342, 136, 342, 137, 342, 138, 342, 160, 342, 161, 342, 162, 342, 163, 342, 164, 342, 166, 342, 167, 342, 185, 342, 186, 342, 187, 342, 189, 342, 190, 342, 192, 342, 193, 342, 194, 342, 195, 342, 196, 342, 197, 342, 201, 342, 202, 342],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 37, 105, 38, -316, 42, -93, 63, -219, 72, -50, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[13, -638],
	#[2, -327, 7, -37, 10, -212, 11, -318, 86, -325, 118, -181, 87, -320, 31, -326, 33, -321, 63, -219, 38, -316, 81, -312, 89, -329, 88, -313, 85, -317, 94, -210, 82, -315, 115, -10, 109, -180, 111, -222, 98, -192, 102, -178, 83, -330, 92, -189, 99, -218, 119, -231, 101, -214, 110, -233, 116, -9, 100, -193, 84, -322, 107, -229, 105, -173, 96, -182, 112, -206, 106, -227, 103, -228, 97, -217, 113, -194, 104, -185, 121, -201, 108, -199, 114, -183, 120, -226, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[7, 436, 13, -606, 103, 436, 102, 436, 63, 436, 101, 436, 94, 436, 111, 436, 96, 436, 92, 436, 99, 436, 119, 436, 115, 436, 110, 436, 98, 436, 100, 436, 112, 436, 107, 436, 105, 436, 109, 436, 118, 436, 106, 436, 104, 436, 97, 436, 113, 436, 121, 436, 108, 436, 114, 436, 120, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 323, 7, 323, 190, 323, 63, 323, 101, 323, 103, 323, 94, 323, 96, 323, 111, 323, 102, 323, 108, 323, 92, 323, 104, 323, 99, 323, 119, 323, 97, 323, 105, 323, 110, 323, 98, 323, 100, 323, 112, 323, 107, 323, 127, 323, 114, 323, 109, 323, 118, 323, 106, 323, 166, 323, 115, 323, 113, 323, 122, 323, 130, 323, 121, 323, 120, 323, 123, 323, 124, 323, 126, 323, 138, 323, 128, 323, 129, 323, 164, 323, 133, 323, 134, 323, 135, 323, 136, 323, 137, 323, 167, 323, 162, 323, 187, 323, 189, 323, 161, 323, 160, 323, 163, 323, 186, 323, 192, 323, 185, 323, 193, 323, 194, 323, 195, 323, 196, 323, 197, 323, 201, 323, 202, 323],
	#[11, -642, 37, 105, 42, -93, 72, -50],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[63, 436, 7, 436, 13, -645, 110, 436, 118, 436, 102, 436, 126, 436, 115, 436, 121, 436, 94, 436, 100, 436, 113, 436, 106, 436, 112, 436, 111, 436, 98, 436, 105, 436, 92, 436, 104, 436, 99, 436, 119, 436, 97, 436, 101, 436, 107, 436, 109, 436, 103, 436, 96, 436, 108, 436, 114, 436, 120, 436, 122, 436, 123, 436, 124, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 332, 7, 332, 63, 332, 99, 332, 98, 332, 92, 332, 94, 332, 96, 332, 97, 332, 100, 332, 101, 332, 102, 332, 103, 332, 104, 332, 105, 332, 106, 332, 107, 332, 108, 332, 109, 332, 110, 332, 111, 332, 112, 332, 113, 332, 114, 332, 115, 332, 118, 332, 119, 332, 120, 332, 121, 332, 122, 332, 123, 332, 124, 332, 126, 332, 127, 332, 128, 332, 129, 332, 130, 332, 133, 332, 134, 332, 135, 332, 136, 332, 137, 332, 138, 332, 160, 332, 161, 332, 162, 332, 163, 332, 164, 332, 166, 332, 167, 332, 185, 332, 186, 332, 187, 332, 189, 332, 190, 332, 192, 332, 193, 332, 194, 332, 195, 332, 196, 332, 197, 332, 201, 332, 202, 332],
	#[11, -646, 222, -608],
	#[7, -37, 11, -318, 10, -212, 2, -327, 87, -320, 31, -326, 33, -321, 63, -219, 38, -316, 86, -325, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 99, -218, 98, -192, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[13, -648],
	#[2, -327, 63, -219, 7, -37, 10, -212, 11, -318, 38, -316, 135, -213, 92, -189, 85, -317, 87, -320, 31, -326, 33, -321, 86, -325, 103, -228, 130, -191, 84, -322, 127, -187, 83, -330, 82, -315, 138, -230, 133, -197, 88, -313, 81, -312, 111, -222, 97, -217, 123, -174, 113, -194, 136, -188, 110, -233, 134, -184, 89, -329, 109, -180, 108, -199, 114, -183, 94, -210, 137, -175, 96, -182, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 126, -198, 104, -185, 105, -173, 106, -227, 107, -229, 112, -206, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 128, -216, 124, -232, 129, -203, 223, -209],
	#[12, -650],
	#[65535, 414, 7, 414, 13, 414, 63, 414, 92, 414, 94, 414, 96, 414, 97, 414, 98, 414, 99, 414, 100, 414, 101, 414, 102, 414, 103, 414, 104, 414, 105, 414, 106, 414, 107, 414, 108, 414, 109, 414, 110, 414, 111, 414, 112, 414, 113, 414, 114, 414, 115, 414, 118, 414, 119, 414, 120, 414, 121, 414, 122, 414, 123, 414, 124, 414, 126, 414, 127, 414, 128, 414, 129, 414, 130, 414, 133, 414, 134, 414, 135, 414, 136, 414, 137, 414, 138, 414, 160, 414, 161, 414, 162, 414, 163, 414, 164, 414, 166, 414, 167, 414, 185, 414, 186, 414, 187, 414, 189, 414, 190, 414, 192, 414, 193, 414, 194, 414, 195, 414, 196, 414, 197, 414, 201, 414, 202, 414],
	#[13, -652],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[12, -654],
	#[65535, 413, 7, 413, 13, 413, 63, 413, 92, 413, 94, 413, 96, 413, 97, 413, 98, 413, 99, 413, 100, 413, 101, 413, 102, 413, 103, 413, 104, 413, 105, 413, 106, 413, 107, 413, 108, 413, 109, 413, 110, 413, 111, 413, 112, 413, 113, 413, 114, 413, 115, 413, 118, 413, 119, 413, 120, 413, 121, 413, 122, 413, 123, 413, 124, 413, 126, 413, 127, 413, 128, 413, 129, 413, 130, 413, 133, 413, 134, 413, 135, 413, 136, 413, 137, 413, 138, 413, 160, 413, 161, 413, 162, 413, 163, 413, 164, 413, 166, 413, 167, 413, 185, 413, 186, 413, 187, 413, 189, 413, 190, 413, 192, 413, 193, 413, 194, 413, 195, 413, 196, 413, 197, 413, 201, 413, 202, 413],
	#[13, -656],
	#[68, -64, 11, -66, 63, -63, 7, -37, 66, -58, 35, -67, 33, -60, 31, -59, 36, -65, 64, -73, 65, -71, 67, -61, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -248],
	#[7, 436, 13, -660, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 347, 63, 347, 7, 347, 92, 347, 103, 347, 99, 347, 94, 347, 97, 347, 109, 347, 118, 347, 106, 347, 98, 347, 167, 347, 96, 347, 111, 347, 100, 347, 123, 347, 130, 347, 110, 347, 134, 347, 119, 347, 108, 347, 114, 347, 107, 347, 133, 347, 164, 347, 127, 347, 113, 347, 121, 347, 101, 347, 102, 347, 126, 347, 104, 347, 105, 347, 135, 347, 124, 347, 112, 347, 115, 347, 138, 347, 128, 347, 161, 347, 120, 347, 136, 347, 122, 347, 137, 347, 163, 347, 160, 347, 129, 347, 162, 347, 166, 347, 187, 347, 197, 347, 196, 347, 195, 347, 194, 347, 185, 347, 186, 347, 201, 347, 189, 347, 190, 347, 192, 347, 193, 347, 202, 347],
	#[38, -252, 222, -608],
	#[204, -603, 205, -600, 206, -598, 207, -599, 208, -601, 209, -602],
	#[7, 436, 13, -606, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 345, 7, 345, 63, 345, 92, 345, 94, 345, 96, 345, 97, 345, 98, 345, 99, 345, 100, 345, 101, 345, 102, 345, 103, 345, 104, 345, 105, 345, 106, 345, 107, 345, 108, 345, 109, 345, 110, 345, 111, 345, 112, 345, 113, 345, 114, 345, 115, 345, 118, 345, 119, 345, 120, 345, 121, 345, 122, 345, 123, 345, 124, 345, 126, 345, 127, 345, 128, 345, 129, 345, 130, 345, 133, 345, 134, 345, 135, 345, 136, 345, 137, 345, 138, 345, 160, 345, 161, 345, 162, 345, 163, 345, 164, 345, 166, 345, 167, 345, 185, 345, 186, 345, 187, 345, 189, 345, 190, 345, 192, 345, 193, 345, 194, 345, 195, 345, 196, 345, 197, 345, 201, 345, 202, 345],
	#[31, -59, 7, -37, 11, -66, 70, -72, 67, -61, 69, -68, 33, -60, 35, -67, 36, -65, 65, -71, 63, -63, 64, -73, 66, -58, 68, -64, 71, -55, 73, -70, 115, -10],
	#[65535, 215, 7, 215, 13, 215, 63, 215, 92, 215, 94, 215, 96, 215, 97, 215, 98, 215, 99, 215, 100, 215, 101, 215, 102, 215, 103, 215, 104, 215, 105, 215, 106, 215, 107, 215, 108, 215, 109, 215, 110, 215, 111, 215, 112, 215, 113, 215, 114, 215, 115, 215, 118, 215, 119, 215, 120, 215, 121, 215, 122, 215, 123, 215, 124, 215, 126, 215, 127, 215, 128, 215, 129, 215, 130, 215, 133, 215, 134, 215, 135, 215, 136, 215, 137, 215, 138, 215, 160, 215, 161, 215, 162, 215, 163, 215, 164, 215, 166, 215, 167, 215, 185, 215, 186, 215, 187, 215, 189, 215, 190, 215, 192, 215, 193, 215, 194, 215, 195, 215, 196, 215, 197, 215, 201, 215, 202, 215],
	#[7, 436, 13, -668, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 346, 167, 346, 92, 346, 63, 346, 7, 346, 109, 346, 123, 346, 107, 346, 114, 346, 108, 346, 130, 346, 98, 346, 137, 346, 127, 346, 94, 346, 124, 346, 113, 346, 164, 346, 126, 346, 96, 346, 111, 346, 97, 346, 106, 346, 163, 346, 99, 346, 110, 346, 134, 346, 119, 346, 133, 346, 100, 346, 101, 346, 102, 346, 103, 346, 104, 346, 105, 346, 135, 346, 121, 346, 112, 346, 115, 346, 138, 346, 118, 346, 161, 346, 120, 346, 136, 346, 122, 346, 128, 346, 160, 346, 129, 346, 162, 346, 166, 346, 186, 346, 197, 346, 196, 346, 201, 346, 195, 346, 194, 346, 185, 346, 193, 346, 187, 346, 189, 346, 190, 346, 192, 346, 202, 346],
	#[7, -37, 11, -66, 31, -59, 33, -60, 63, -63, 35, -67, 36, -65, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10, 222, -608],
	#[2, -327, 63, -219, 7, -37, 10, -212, 11, -318, 38, -316, 135, -213, 33, -321, 82, -315, 85, -317, 87, -320, 31, -326, 119, -231, 81, -312, 134, -184, 116, -9, 84, -322, 127, -187, 83, -330, 109, -180, 118, -181, 126, -198, 114, -183, 111, -222, 138, -230, 123, -174, 113, -194, 86, -325, 110, -233, 88, -313, 89, -329, 97, -217, 108, -199, 92, -189, 94, -210, 133, -197, 96, -182, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 112, -206, 115, -10, 136, -188, 128, -216, 130, -191, 120, -226, 121, -201, 122, -190, 137, -175, 124, -232, 129, -203, 223, -209],
	#[13, -671],
	#[69, -672],
	#[31, -326, 11, -318, 92, -189, 63, -219, 7, -37, 10, -212, 2, -327, 86, -325, 111, -222, 109, -180, 38, -316, 33, -321, 87, -320, 83, -330, 123, -174, 130, -191, 84, -322, 127, -187, 114, -183, 82, -315, 99, -218, 98, -192, 126, -198, 81, -312, 116, -9, 88, -313, 106, -227, 85, -317, 136, -188, 110, -233, 134, -184, 89, -329, 97, -217, 108, -199, 113, -194, 128, -216, 94, -210, 133, -197, 96, -182, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 135, -213, 107, -229, 112, -206, 115, -10, 138, -230, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 137, -175, 124, -232, 129, -203, 223, -209],
	#[11, -674],
	#[12, -677, 36, -675],
	#[31, -326, 127, -187, 2, -327, 11, -318, 103, -228, 86, -325, 108, -199, 111, -222, 87, -320, 129, -203, 123, -174, 33, -321, 126, -198, 113, -194, 38, -316, 124, -232, 130, -191, 98, -192, 85, -317, 94, -210, 118, -181, 84, -322, 106, -227, 109, -180, 107, -229, 102, -178, 83, -330, 114, -183, 99, -218, 119, -231, 101, -214, 110, -233, 100, -193, 89, -329, 82, -315, 121, -201, 92, -189, 105, -173, 122, -190, 104, -185, 120, -226, 81, -312, 97, -217, 88, -313, 96, -182, 112, -206, 128, -216, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[12, -680, 36, -679],
	#[7, 436, 92, 436, 127, 436, 13, -606, 103, 436, 110, 436, 111, 436, 94, 436, 106, 436, 32, 436, 102, 436, 63, 436, 62, 436, 98, 436, 101, 436, 99, 436, 97, 436, 166, 436, 109, 436, 124, 436, 126, 436, 104, 436, 120, 436, 119, 436, 115, 436, 113, 436, 96, 436, 107, 436, 123, 436, 118, 436, 130, 436, 105, 436, 122, 436, 121, 436, 129, 436, 114, 436, 100, 436, 108, 436, 112, 436, 128, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 159, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 280, 7, 280, 123, 280, 32, 280, 63, 280, 62, 280, 130, 280, 101, 280, 103, 280, 109, 280, 111, 280, 126, 280, 99, 280, 119, 280, 115, 280, 110, 280, 100, 280, 127, 280, 94, 280, 118, 280, 106, 280, 166, 280, 92, 280, 98, 280, 105, 280, 122, 280, 102, 280, 129, 280, 96, 280, 97, 280, 120, 280, 128, 280, 104, 280, 124, 280, 108, 280, 114, 280, 121, 280, 107, 280, 112, 280, 113, 280, 133, 280, 134, 280, 135, 280, 136, 280, 137, 280, 138, 280, 159, 280, 160, 280, 161, 280, 162, 280, 163, 280, 164, 280, 167, 280, 185, 280, 186, 280, 187, 280, 189, 280, 190, 280, 192, 280, 193, 280, 194, 280, 195, 280, 196, 280, 197, 280, 201, 280, 202, 280],
	#[2, -327, 11, -318, 31, -326, 33, -321, 38, -316, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[7, 436, 127, 436, 103, 436, 62, 436, 63, 436, 92, 436, 126, 436, 13, -606, 109, 436, 100, 436, 106, 436, 111, 436, 94, 436, 118, 436, 130, 436, 32, 436, 102, 436, 122, 436, 115, 436, 124, 436, 101, 436, 99, 436, 114, 436, 121, 436, 98, 436, 96, 436, 104, 436, 120, 436, 119, 436, 105, 436, 110, 436, 107, 436, 97, 436, 108, 436, 112, 436, 113, 436, 123, 436, 128, 436, 129, 436, 159, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 166, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 279, 7, 279, 32, 279, 62, 279, 63, 279, 92, 279, 94, 279, 96, 279, 97, 279, 98, 279, 99, 279, 100, 279, 101, 279, 102, 279, 103, 279, 104, 279, 105, 279, 106, 279, 107, 279, 108, 279, 109, 279, 110, 279, 111, 279, 112, 279, 113, 279, 114, 279, 115, 279, 118, 279, 119, 279, 120, 279, 121, 279, 122, 279, 123, 279, 124, 279, 126, 279, 127, 279, 128, 279, 129, 279, 130, 279, 133, 279, 134, 279, 135, 279, 136, 279, 137, 279, 138, 279, 159, 279, 160, 279, 161, 279, 162, 279, 163, 279, 164, 279, 166, 279, 167, 279, 185, 279, 186, 279, 187, 279, 189, 279, 190, 279, 192, 279, 193, 279, 194, 279, 195, 279, 196, 279, 197, 279, 201, 279, 202, 279],
	#[13, -683],
	#[69, -684],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[65535, 312, 12, 312, 36, 312],
	#[13, -687],
	#[69, -688],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[65535, 311, 12, 311, 36, 311],
	#[93, -691],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 70, -72, 69, -68, 67, -61, 65, -71, 63, -63, 64, -73, 66, -58, 68, -64, 71, -55, 73, -70, 115, -10],
	#[7, 436, 13, -606, 37, 105, 42, -93, 63, 436, 72, -50, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 326, 7, 326, 103, 326, 167, 326, 92, 326, 123, 326, 102, 326, 126, 326, 127, 326, 94, 326, 63, 326, 130, 326, 114, 326, 101, 326, 100, 326, 105, 326, 99, 326, 98, 326, 104, 326, 97, 326, 107, 326, 96, 326, 106, 326, 108, 326, 109, 326, 110, 326, 111, 326, 112, 326, 113, 326, 115, 326, 118, 326, 119, 326, 120, 326, 121, 326, 122, 326, 128, 326, 124, 326, 129, 326, 133, 326, 134, 326, 135, 326, 136, 326, 137, 326, 138, 326, 166, 326, 160, 326, 161, 326, 162, 326, 163, 326, 164, 326, 185, 326, 186, 326, 187, 326, 189, 326, 190, 326, 192, 326, 193, 326, 194, 326, 195, 326, 196, 326, 197, 326, 201, 326, 202, 326],
	#[13, -695],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -697],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, 436, 13, -606, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 331, 7, 331, 63, 331, 92, 331, 94, 331, 96, 331, 97, 331, 98, 331, 99, 331, 100, 331, 101, 331, 102, 331, 103, 331, 104, 331, 105, 331, 106, 331, 107, 331, 108, 331, 109, 331, 110, 331, 111, 331, 112, 331, 113, 331, 114, 331, 115, 331, 118, 331, 119, 331, 120, 331, 121, 331, 122, 331, 123, 331, 124, 331, 126, 331, 127, 331, 128, 331, 129, 331, 130, 331, 133, 331, 134, 331, 135, 331, 136, 331, 137, 331, 138, 331, 160, 331, 161, 331, 162, 331, 163, 331, 164, 331, 166, 331, 167, 331, 185, 331, 186, 331, 187, 331, 189, 331, 190, 331, 192, 331, 193, 331, 194, 331, 195, 331, 196, 331, 197, 331, 201, 331, 202, 331],
	#[65535, 73, 7, 73, 11, 73, 31, 73, 33, 73, 35, 73, 36, 73, 57, 73, 58, 73, 59, 73, 60, 73, 63, 73, 64, 73, 65, 73, 66, 73, 67, 73, 68, 73, 69, 73, 70, 73, 71, 73, 73, 73, 115, 73],
	#[7, -37, 11, -66, 69, -68, 70, -72, 67, -61, 31, -59, 33, -60, 35, -67, 36, -65, 59, -515, 58, -514, 57, -510, 65, -71, 60, -513, 63, -63, 64, -73, 66, -58, 68, -64, 71, -55, 73, -438, 115, -10],
	#[2, -327, 31, -326, 7, -37, 10, -212, 11, -318, 86, -325, 118, -181, 87, -320, 116, -9, 119, -231, 33, -321, 63, -219, 38, -316, 124, -232, 127, -187, 92, -189, 101, -214, 103, -228, 81, -312, 85, -317, 94, -210, 82, -315, 84, -322, 106, -227, 109, -180, 111, -222, 98, -192, 102, -178, 83, -330, 99, -218, 89, -329, 105, -173, 110, -233, 122, -190, 104, -185, 126, -198, 96, -182, 100, -193, 120, -226, 88, -313, 115, -10, 97, -217, 108, -199, 114, -183, 121, -201, 107, -229, 112, -206, 113, -194, 123, -174, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[42, -704],
	#[7, -37, 11, -66, 13, 315, 31, -59, 33, -60, 35, -67, 36, -65, 43, 315, 70, -72, 69, -68, 67, -61, 65, -71, 63, -63, 64, -73, 66, -58, 68, -64, 71, -55, 73, -70, 115, -10],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 60, -88, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 90, -86, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -710, 106, -712, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 217, -90, 218, -85, 219, -87, 220, -89, 223, -209],
	#[13, -708, 43, -707],
	#[65535, 207, 7, 207, 13, 207, 57, 207, 58, 207, 59, 207, 63, 207, 92, 207, 94, 207, 96, 207, 97, 207, 98, 207, 99, 207, 100, 207, 101, 207, 102, 207, 103, 207, 104, 207, 105, 207, 106, 207, 107, 207, 108, 207, 109, 207, 110, 207, 111, 207, 112, 207, 113, 207, 114, 207, 115, 207, 118, 207, 119, 207, 120, 207, 121, 207, 122, 207, 123, 207, 124, 207, 126, 207, 127, 207, 128, 207, 129, 207, 130, 207, 133, 207, 134, 207, 135, 207, 136, 207, 137, 207, 138, 207, 160, 207, 161, 207, 162, 207, 163, 207, 164, 207, 166, 207, 167, 207, 168, 207, 169, 207, 170, 207, 171, 207, 172, 207, 173, 207, 174, 207, 175, 207, 176, 207, 177, 207, 178, 207, 179, 207, 180, 207, 181, 207, 182, 207, 183, 207, 184, 207, 185, 207, 186, 207, 187, 207, 189, 207, 190, 207, 192, 207, 193, 207, 194, 207, 195, 207, 196, 207, 197, 207, 201, 207, 202, 207],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[31, -326, 135, -213, 111, -222, 11, -318, 2, -327, 7, -37, 85, -317, 10, -212, 126, -198, 38, -316, 103, -228, 86, -325, 133, -197, 108, -199, 92, -189, 33, -321, 82, -315, 88, -313, 87, -320, 116, -9, 119, -231, 109, -180, 63, -219, 138, -230, 127, -187, 110, -233, 134, -184, 60, -88, 83, -330, 123, -174, 99, -218, 94, -210, 84, -322, 89, -329, 81, -312, 136, -188, 90, -86, 118, -181, 96, -182, 97, -217, 98, -192, 113, -194, 100, -193, 101, -214, 102, -178, 114, -183, 104, -185, 105, -710, 106, -712, 107, -229, 112, -206, 115, -10, 137, -175, 128, -216, 130, -191, 120, -226, 121, -201, 122, -190, 124, -232, 129, -203, 217, -90, 218, -85, 219, -87, 220, -89, 223, -209],
	#[31, 405, 2, 405, 7, 405, 10, 405, 11, 405, 103, 405, 85, 405, 87, 405, 83, 405, 119, 405, 33, 405, 38, 405, 86, 405, 92, 405, 42, 178, 98, 405, 101, 405, 123, 405, 99, 405, 94, 405, 82, 405, 111, 405, 124, 405, 126, 405, 60, 405, 89, 405, 63, 405, 110, 405, 116, 405, 100, 405, 84, 405, 107, 405, 127, 405, 120, 405, 109, 405, 118, 405, 106, 405, 81, 405, 130, 405, 115, 405, 105, 405, 102, 405, 129, 405, 96, 405, 97, 405, 113, 405, 88, 405, 122, 405, 90, 405, 108, 405, 114, 405, 121, 405, 104, 405, 112, 405, 128, 405, 133, 405, 134, 405, 135, 405, 136, 405, 137, 405, 138, 405, 217, 405, 218, 405, 219, 405, 220, 405, 223, 405],
	#[65535, 318, 13, 318, 43, 318, 57, 318, 58, 318, 59, 318, 60, 318, 90, 318, 105, 318, 106, 318, 217, 318, 218, 318, 219, 318, 220, 318],
	#[94, 406, 2, 406, 119, 406, 63, 406, 7, 406, 10, 406, 11, 406, 38, 406, 84, 406, 135, 406, 111, 406, 101, 406, 60, 406, 82, 406, 87, 406, 31, 406, 100, 406, 33, 406, 136, 406, 42, 179, 103, 406, 98, 406, 81, 406, 83, 406, 109, 406, 118, 406, 99, 406, 97, 406, 88, 406, 90, 406, 126, 406, 137, 406, 89, 406, 138, 406, 123, 406, 85, 406, 86, 406, 110, 406, 134, 406, 122, 406, 108, 406, 92, 406, 107, 406, 133, 406, 96, 406, 127, 406, 113, 406, 102, 406, 114, 406, 104, 406, 105, 406, 106, 406, 121, 406, 112, 406, 115, 406, 116, 406, 128, 406, 130, 406, 120, 406, 129, 406, 124, 406, 223, 406, 220, 406, 219, 406, 217, 406, 218, 406],
	#[13, 317, 43, 317, 57, -152, 58, -149, 59, -150, 60, -88, 90, -86, 105, -91, 106, -84, 217, -90, 218, -85, 219, -87, 220, -89],
	#[65535, 400, 13, 400, 43, 400, 57, 400, 58, 400, 59, 400, 60, 400, 90, 400, 105, 400, 106, 400, 217, 400, 218, 400, 219, 400, 220, 400],
	#[65535, 401, 13, 401, 43, 401, 58, 401, 106, 401, 57, 401, 59, 401, 60, 401, 90, 401, 105, 401, 217, 401, 218, 401, 219, 401, 220, 401],
	#[65535, 319, 13, 319, 43, 319, 57, 319, 58, 319, 59, 319, 60, 319, 90, 319, 105, 319, 106, 319, 217, 319, 218, 319, 219, 319, 220, 319],
	#[7, 436, 13, -606, 103, 436, 123, 436, 63, 436, 129, 436, 98, 436, 57, -128, 58, -124, 59, -125, 119, 436, 97, 436, 101, 436, 100, 436, 107, 436, 94, 436, 99, 436, 102, 436, 105, 436, 92, 436, 104, 436, 118, 436, 96, 436, 106, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 128, 436, 130, 436, 120, 436, 121, 436, 122, 436, 124, 436, 126, 436, 127, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 168, -142, 169, -137, 170, -127, 171, -130, 172, -141, 173, -136, 174, -140, 175, -123, 176, -126, 177, -129, 178, -132, 179, -135, 180, -139, 181, -133, 182, -131, 183, -134, 184, -138, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 334, 7, 334, 63, 334, 100, 334, 110, 334, 97, 334, 92, 334, 94, 334, 96, 334, 130, 334, 98, 334, 99, 334, 122, 334, 101, 334, 102, 334, 103, 334, 104, 334, 105, 334, 106, 334, 107, 334, 108, 334, 109, 334, 123, 334, 111, 334, 112, 334, 113, 334, 114, 334, 115, 334, 118, 334, 119, 334, 120, 334, 121, 334, 129, 334, 124, 334, 126, 334, 127, 334, 128, 334, 133, 334, 134, 334, 135, 334, 136, 334, 137, 334, 138, 334, 160, 334, 161, 334, 162, 334, 163, 334, 164, 334, 166, 334, 167, 334, 185, 334, 186, 334, 187, 334, 189, 334, 190, 334, 192, 334, 193, 334, 194, 334, 195, 334, 196, 334, 197, 334, 201, 334, 202, 334],
	#[65535, 318, 13, 318, 43, 318, 57, 318, 58, 318, 59, 318, 60, 318, 90, 318, 105, 318, 106, 318, 217, 318, 218, 318, 219, 318, 220, 318],
	#[13, 316, 43, 316, 57, -152, 58, -149, 59, -150, 60, -88, 90, -86, 105, -91, 106, -84, 217, -90, 218, -85, 219, -87, 220, -89],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[7, 436, 13, -606, 32, 436, 62, 436, 97, 436, 63, 436, 94, 436, 103, 436, 102, 436, 101, 436, 100, 436, 99, 436, 98, 436, 104, 436, 119, 436, 105, 436, 92, 436, 107, 436, 96, 436, 106, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 130, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 159, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 277, 7, 277, 32, 277, 62, 277, 63, 277, 92, 277, 94, 277, 96, 277, 97, 277, 98, 277, 99, 277, 100, 277, 101, 277, 102, 277, 103, 277, 104, 277, 105, 277, 106, 277, 107, 277, 108, 277, 109, 277, 110, 277, 111, 277, 112, 277, 113, 277, 114, 277, 115, 277, 118, 277, 119, 277, 120, 277, 121, 277, 122, 277, 123, 277, 124, 277, 126, 277, 127, 277, 128, 277, 129, 277, 130, 277, 133, 277, 134, 277, 135, 277, 136, 277, 137, 277, 138, 277, 159, 277, 160, 277, 161, 277, 162, 277, 163, 277, 164, 277, 166, 277, 167, 277, 185, 277, 186, 277, 187, 277, 189, 277, 190, 277, 192, 277, 193, 277, 194, 277, 195, 277, 196, 277, 197, 277, 201, 277, 202, 277],
	#[13, -726],
	#[69, -727],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[13, -729],
	#[69, -730],
	#[109, -180, 103, -228, 111, -222, 85, -317, 31, -326, 63, -219, 10, -212, 94, -210, 127, -187, 106, -227, 102, -178, 126, -198, 92, -189, 83, -330, 86, -325, 134, -184, 81, -312, 84, -322, 87, -320, 2, -327, 107, -229, 114, -183, 82, -315, 7, -37, 130, -191, 11, -318, 89, -329, 88, -313, 116, -9, 100, -193, 110, -233, 133, -197, 113, -194, 101, -214, 118, -181, 97, -217, 33, -321, 99, -218, 105, -173, 38, -316, 108, -199, 98, -192, 121, -201, 104, -185, 137, -175, 96, -182, 135, -213, 112, -206, 128, -216, 115, -10, 138, -230, 119, -231, 120, -226, 136, -188, 122, -190, 123, -174, 124, -232, 129, -203, 223, -209],
	#[7, 436, 13, -606, 32, 436, 63, 436, 62, 436, 115, 436, 111, 436, 98, 436, 97, 436, 119, 436, 101, 436, 110, 436, 100, 436, 94, 436, 99, 436, 103, 436, 102, 436, 114, 436, 104, 436, 124, 436, 105, 436, 92, 436, 121, 436, 107, 436, 118, 436, 96, 436, 106, 436, 108, 436, 109, 436, 120, 436, 122, 436, 112, 436, 113, 436, 123, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 166, 436, 159, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 278, 7, 278, 32, 278, 62, 278, 63, 278, 92, 278, 94, 278, 96, 278, 97, 278, 98, 278, 99, 278, 100, 278, 101, 278, 102, 278, 103, 278, 104, 278, 105, 278, 106, 278, 107, 278, 108, 278, 109, 278, 110, 278, 111, 278, 112, 278, 113, 278, 114, 278, 115, 278, 118, 278, 119, 278, 120, 278, 121, 278, 122, 278, 123, 278, 124, 278, 126, 278, 127, 278, 128, 278, 129, 278, 130, 278, 133, 278, 134, 278, 135, 278, 136, 278, 137, 278, 138, 278, 159, 278, 160, 278, 161, 278, 162, 278, 163, 278, 164, 278, 166, 278, 167, 278, 185, 278, 186, 278, 187, 278, 189, 278, 190, 278, 192, 278, 193, 278, 194, 278, 195, 278, 196, 278, 197, 278, 201, 278, 202, 278],
	#[70, 415, 31, 415, 63, 415, 7, 415, 11, 415, 69, 415, 71, 415, 67, 415, 36, 415, 66, 415, 65, 415, 33, 415, 68, 415, 35, 415, 64, 415, 73, 415, 115, 415, 192, -589],
	#[63, -63, 7, -37, 11, -66, 66, -58, 31, -59, 33, -60, 68, -64, 35, -67, 36, -65, 71, -55, 65, -71, 70, -72, 64, -73, 69, -68, 67, -61, 73, -70, 115, -10],
	#[7, 436, 13, -628, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 337, 7, 337, 103, 337, 92, 337, 107, 337, 167, 337, 108, 337, 111, 337, 94, 337, 118, 337, 106, 337, 119, 337, 102, 337, 63, 337, 127, 337, 110, 337, 134, 337, 101, 337, 123, 337, 96, 337, 97, 337, 100, 337, 115, 337, 124, 337, 126, 337, 114, 337, 99, 337, 98, 337, 109, 337, 133, 337, 164, 337, 130, 337, 105, 337, 113, 337, 122, 337, 129, 337, 137, 337, 128, 337, 104, 337, 120, 337, 135, 337, 121, 337, 112, 337, 189, 337, 160, 337, 138, 337, 162, 337, 161, 337, 166, 337, 136, 337, 187, 337, 163, 337, 185, 337, 186, 337, 190, 337, 192, 337, 193, 337, 194, 337, 195, 337, 196, 337, 197, 337, 201, 337, 202, 337],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[203, -597, 204, 357, 205, 357, 206, 357, 207, 357, 208, 357, 209, 357],
	#[204, -603, 205, -600, 206, -598, 207, -599, 208, -601, 209, -602],
	#[7, 436, 13, -628, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 338, 7, 338, 127, 338, 92, 338, 103, 338, 118, 338, 126, 338, 63, 338, 110, 338, 119, 338, 94, 338, 106, 338, 96, 338, 97, 338, 98, 338, 99, 338, 100, 338, 101, 338, 102, 338, 114, 338, 104, 338, 105, 338, 107, 338, 108, 338, 109, 338, 123, 338, 111, 338, 112, 338, 113, 338, 115, 338, 138, 338, 128, 338, 130, 338, 120, 338, 121, 338, 122, 338, 124, 338, 129, 338, 133, 338, 134, 338, 135, 338, 136, 338, 137, 338, 167, 338, 166, 338, 160, 338, 161, 338, 162, 338, 163, 338, 164, 338, 185, 338, 186, 338, 187, 338, 189, 338, 190, 338, 192, 338, 193, 338, 194, 338, 195, 338, 196, 338, 197, 338, 201, 338, 202, 338],
	#[65535, 73, 7, 73, 11, 73, 31, 73, 33, 73, 35, 73, 36, 73, 57, 73, 58, 73, 59, 73, 60, 73, 63, 73, 64, 73, 65, 73, 66, 73, 67, 73, 68, 73, 69, 73, 70, 73, 71, 73, 73, 73, 115, 73],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 57, -510, 58, -514, 59, -515, 60, -513, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -438, 115, -10],
	#[2, -327, 7, -37, 10, -212, 11, -318, 38, -316, 84, -322, 86, -325, 135, -213, 111, -222, 94, -210, 33, -321, 85, -317, 87, -320, 31, -326, 100, -193, 109, -180, 63, -219, 133, -197, 89, -329, 92, -189, 134, -184, 88, -313, 83, -330, 103, -228, 81, -312, 97, -217, 82, -315, 138, -230, 98, -192, 108, -199, 114, -183, 99, -218, 110, -233, 119, -231, 118, -181, 96, -182, 123, -174, 127, -187, 116, -9, 121, -201, 101, -214, 102, -178, 126, -198, 104, -185, 105, -173, 106, -227, 107, -229, 112, -206, 113, -194, 115, -10, 137, -175, 128, -216, 130, -191, 120, -226, 136, -188, 122, -190, 124, -232, 129, -203, 223, -209],
	#[42, -746],
	#[13, 315, 63, -63, 7, -37, 11, -66, 71, -55, 66, -58, 70, -72, 31, -59, 33, -60, 68, -64, 35, -67, 36, -65, 43, 315, 65, -71, 73, -70, 69, -68, 67, -61, 64, -73, 115, -10],
	#[13, -708, 43, -748],
	#[65535, 207, 57, 207, 58, 207, 59, 207, 93, 207, 168, 207, 169, 207, 170, 207, 171, 207, 172, 207, 173, 207, 174, 207, 175, 207, 176, 207, 177, 207, 178, 207, 179, 207, 180, 207, 181, 207, 182, 207, 183, 207, 184, 207],
	#[58, -124, 57, -128, 59, -125, 93, -750, 171, -130, 169, -137, 172, -141, 168, -142, 170, -127, 173, -136, 174, -140, 175, -123, 176, -126, 177, -129, 178, -132, 179, -135, 180, -139, 181, -133, 182, -131, 183, -134, 184, -138],
	#[69, -751],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[165, -753],
	#[69, -754],
	#[31, -326, 86, -325, 2, -327, 7, -37, 10, -212, 11, -318, 38, -316, 33, -321, 82, -315, 63, -219, 134, -184, 107, -229, 94, -210, 81, -312, 92, -189, 111, -222, 83, -330, 84, -322, 85, -317, 96, -182, 87, -320, 88, -313, 89, -329, 97, -217, 133, -197, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 108, -199, 109, -180, 110, -233, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[7, 436, 13, -606, 111, 436, 32, 436, 63, 436, 62, 436, 123, 436, 124, 436, 133, 436, 101, 436, 127, 436, 94, 436, 106, 436, 103, 436, 115, 436, 135, 436, 126, 436, 100, 436, 130, 436, 134, 436, 109, 436, 108, 436, 92, 436, 118, 436, 96, 436, 97, 436, 98, 436, 99, 436, 122, 436, 159, 436, 102, 436, 114, 436, 104, 436, 105, 436, 113, 436, 107, 436, 129, 436, 167, 436, 110, 436, 112, 436, 189, 436, 120, 436, 162, 436, 119, 436, 166, 436, 121, 436, 187, 436, 137, 436, 163, 436, 160, 436, 161, 436, 138, 436, 128, 436, 185, 436, 164, 436, 136, 436, 186, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 283, 7, 283, 32, 283, 62, 283, 63, 283, 92, 283, 94, 283, 96, 283, 97, 283, 98, 283, 99, 283, 100, 283, 101, 283, 102, 283, 103, 283, 104, 283, 105, 283, 106, 283, 107, 283, 108, 283, 109, 283, 110, 283, 111, 283, 112, 283, 113, 283, 114, 283, 115, 283, 118, 283, 119, 283, 120, 283, 121, 283, 122, 283, 123, 283, 124, 283, 126, 283, 127, 283, 128, 283, 129, 283, 130, 283, 133, 283, 134, 283, 135, 283, 136, 283, 137, 283, 138, 283, 159, 283, 160, 283, 161, 283, 162, 283, 163, 283, 164, 283, 166, 283, 167, 283, 185, 283, 186, 283, 187, 283, 189, 283, 190, 283, 192, 283, 193, 283, 194, 283, 195, 283, 196, 283, 197, 283, 201, 283, 202, 283],
	#[65535, 284, 110, 284, 63, 284, 7, 284, 103, 284, 135, 284, 111, 284, 94, 284, 166, 284, 106, 284, 102, 284, 159, 284, 62, 284, 32, 284, 101, 284, 164, 284, 162, 284, 167, 284, 109, 284, 98, 284, 107, 284, 122, 284, 114, 284, 99, 284, 161, 284, 100, 284, 96, 284, 138, 284, 129, 284, 108, 284, 92, 284, 97, 284, 105, 284, 104, 284, 187, 284, 160, 284, 121, 284, 134, 284, 124, 284, 133, 284, 113, 284, 112, 284, 189, 284, 115, 284, 137, 284, 118, 284, 119, 284, 120, 284, 123, 284, 163, 284, 126, 284, 127, 284, 128, 284, 185, 284, 130, 284, 136, 284, 186, 284, 190, 284, 192, 284, 193, 284, 194, 284, 195, 284, 196, 284, 197, 284, 201, 284, 202, 284],
	#[37, 105, 42, -93, 72, -50, 188, -759],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, 350, 13, 350, 110, 350, 111, 350, 119, 350, 63, 350, 126, 350, 101, 350, 94, 350, 118, 350, 106, 350, 103, 350, 97, 350, 102, 350, 167, 350, 107, 350, 100, 350, 99, 350, 98, 350, 134, 350, 109, 350, 108, 350, 92, 350, 133, 350, 96, 350, 123, 350, 127, 350, 113, 350, 122, 350, 129, 350, 115, 350, 114, 350, 104, 350, 105, 350, 135, 350, 124, 350, 112, 350, 189, 350, 120, 350, 162, 350, 130, 350, 166, 350, 121, 350, 187, 350, 137, 350, 163, 350, 160, 350, 161, 350, 138, 350, 128, 350, 185, 350, 164, 350, 136, 350, 186, 350, 190, 350, 192, 350, 193, 350, 194, 350, 195, 350, 196, 350, 197, 350, 198, -762, 199, 350, 200, 350, 201, 350, 202, 350],
	#[65535, 352, 7, 352, 13, 352, 63, 352, 92, 352, 94, 352, 96, 352, 97, 352, 98, 352, 99, 352, 100, 352, 101, 352, 102, 352, 103, 352, 104, 352, 105, 352, 106, 352, 107, 352, 108, 352, 109, 352, 110, 352, 111, 352, 112, 352, 113, 352, 114, 352, 115, 352, 118, 352, 119, 352, 120, 352, 121, 352, 122, 352, 123, 352, 124, 352, 126, 352, 127, 352, 128, 352, 129, 352, 130, 352, 133, 352, 134, 352, 135, 352, 136, 352, 137, 352, 138, 352, 160, 352, 161, 352, 162, 352, 163, 352, 164, 352, 166, 352, 167, 352, 185, 352, 186, 352, 187, 352, 189, 352, 190, 352, 192, 352, 193, 352, 194, 352, 195, 352, 196, 352, 197, 352, 199, 352, 200, 352, 201, 352, 202, 352],
	#[65535, 351, 7, 351, 127, 351, 135, 351, 111, 351, 115, 351, 133, 351, 119, 351, 63, 351, 92, 351, 102, 351, 126, 351, 13, 351, 100, 351, 103, 351, 110, 351, 134, 351, 98, 351, 124, 351, 121, 351, 196, 351, 128, 351, 106, 351, 130, 351, 123, 351, 109, 351, 99, 351, 105, 351, 112, 351, 129, 351, 138, 351, 108, 351, 101, 351, 137, 351, 94, 351, 118, 351, 167, 351, 190, 351, 104, 351, 120, 351, 161, 351, 113, 351, 185, 351, 162, 351, 160, 351, 107, 351, 97, 351, 163, 351, 166, 351, 201, 351, 186, 351, 122, 351, 164, 351, 187, 351, 96, 351, 193, 351, 114, 351, 197, 351, 136, 351, 189, 351, 195, 351, 194, 351, 200, 351, 192, 351, 199, 351, 202, 351],
	#[7, 436, 13, -606, 63, 436, 94, 436, 111, 436, 110, 436, 109, 436, 108, 436, 92, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 166, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 199, -764, 200, -766, 201, 436, 202, 436],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 333, 111, 333, 7, 333, 127, 333, 92, 333, 164, 333, 134, 333, 119, 333, 63, 333, 101, 333, 106, 333, 99, 333, 102, 333, 126, 333, 122, 333, 100, 333, 103, 333, 110, 333, 135, 333, 105, 333, 167, 333, 108, 333, 94, 333, 118, 333, 166, 333, 107, 333, 162, 333, 96, 333, 123, 333, 109, 333, 97, 333, 115, 333, 133, 333, 124, 333, 120, 333, 130, 333, 113, 333, 121, 333, 114, 333, 160, 333, 194, 333, 98, 333, 112, 333, 129, 333, 196, 333, 138, 333, 137, 333, 104, 333, 136, 333, 163, 333, 128, 333, 161, 333, 197, 333, 187, 333, 186, 333, 195, 333, 185, 333, 193, 333, 201, 333, 189, 333, 190, 333, 202, 333, 192, 333],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[65535, 354, 7, 354, 13, 354, 63, 354, 92, 354, 94, 354, 96, 354, 97, 354, 98, 354, 99, 354, 100, 354, 101, 354, 102, 354, 103, 354, 104, 354, 105, 354, 106, 354, 107, 354, 108, 354, 109, 354, 110, 354, 111, 354, 112, 354, 113, 354, 114, 354, 115, 354, 118, 354, 119, 354, 120, 354, 121, 354, 122, 354, 123, 354, 124, 354, 126, 354, 127, 354, 128, 354, 129, 354, 130, 354, 133, 354, 134, 354, 135, 354, 136, 354, 137, 354, 138, 354, 160, 354, 161, 354, 162, 354, 163, 354, 164, 354, 166, 354, 167, 354, 185, 354, 186, 354, 187, 354, 189, 354, 190, 354, 192, 354, 193, 354, 194, 354, 195, 354, 196, 354, 197, 354, 199, 354, 200, 354, 201, 354, 202, 354],
	#[65535, 353, 7, 353, 13, 353, 94, 353, 108, 353, 135, 353, 190, 353, 133, 353, 119, 353, 63, 353, 127, 353, 109, 353, 103, 353, 92, 353, 102, 353, 126, 353, 107, 353, 111, 353, 199, 353, 96, 353, 110, 353, 97, 353, 106, 353, 98, 353, 99, 353, 100, 353, 101, 353, 137, 353, 114, 353, 104, 353, 105, 353, 113, 353, 124, 353, 130, 353, 167, 353, 123, 353, 134, 353, 112, 353, 164, 353, 121, 353, 115, 353, 138, 353, 118, 353, 161, 353, 120, 353, 136, 353, 122, 353, 128, 353, 160, 353, 129, 353, 163, 353, 162, 353, 166, 353, 186, 353, 197, 353, 196, 353, 201, 353, 195, 353, 194, 353, 185, 353, 193, 353, 187, 353, 189, 353, 200, 353, 192, 353, 202, 353],
	#[13, -770],
	#[11, -771],
	#[12, -774, 69, -773],
	#[12, -778, 13, -777],
	#[63, -219, 7, -37, 10, -212, 2, -327, 38, -316, 111, -222, 11, -318, 84, -322, 87, -320, 31, -326, 81, -312, 82, -315, 101, -214, 92, -189, 85, -317, 102, -178, 96, -182, 33, -321, 100, -193, 103, -228, 86, -325, 110, -233, 89, -329, 99, -218, 113, -194, 94, -210, 118, -181, 97, -217, 105, -173, 106, -227, 116, -9, 123, -174, 109, -180, 126, -198, 114, -183, 88, -313, 124, -232, 108, -199, 130, -191, 98, -192, 83, -330, 135, -213, 115, -10, 121, -201, 104, -185, 107, -229, 133, -197, 129, -203, 120, -226, 119, -231, 134, -184, 122, -190, 127, -187, 136, -188, 138, -230, 137, -175, 128, -216, 112, -206, 223, -209],
	#[7, 436, 13, -606, 32, 436, 62, 436, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 159, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 282, 7, 282, 32, 282, 62, 282, 63, 282, 92, 282, 94, 282, 96, 282, 97, 282, 98, 282, 99, 282, 100, 282, 101, 282, 102, 282, 103, 282, 104, 282, 105, 282, 106, 282, 107, 282, 108, 282, 109, 282, 110, 282, 111, 282, 112, 282, 113, 282, 114, 282, 115, 282, 118, 282, 119, 282, 120, 282, 121, 282, 122, 282, 123, 282, 124, 282, 126, 282, 127, 282, 128, 282, 129, 282, 130, 282, 133, 282, 134, 282, 135, 282, 136, 282, 137, 282, 138, 282, 159, 282, 160, 282, 161, 282, 162, 282, 163, 282, 164, 282, 166, 282, 167, 282, 185, 282, 186, 282, 187, 282, 189, 282, 190, 282, 192, 282, 193, 282, 194, 282, 195, 282, 196, 282, 197, 282, 201, 282, 202, 282],
	#[65535, 313, 12, 313, 13, 313],
	#[69, -780],
	#[7, 436, 13, -606, 135, 436, 94, 436, 32, 436, 119, 436, 161, 436, 134, 436, 100, 436, 167, 436, 105, 436, 133, 436, 62, 436, 63, 436, 110, 436, 98, 436, 107, 436, 97, 436, 96, 436, 163, 436, 103, 436, 92, 436, 164, 436, 118, 436, 127, 436, 99, 436, 121, 436, 101, 436, 102, 436, 126, 436, 104, 436, 138, 436, 106, 436, 124, 436, 108, 436, 109, 436, 123, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 136, 436, 128, 436, 130, 436, 120, 436, 122, 436, 137, 436, 129, 436, 160, 436, 162, 436, 159, 436, 166, 436, 187, 436, 186, 436, 197, 436, 196, 436, 195, 436, 194, 436, 185, 436, 193, 436, 201, 436, 189, 436, 190, 436, 192, 436, 202, 436],
	#[65535, 281, 7, 281, 32, 281, 62, 281, 119, 281, 63, 281, 110, 281, 103, 281, 115, 281, 122, 281, 124, 281, 92, 281, 121, 281, 94, 281, 118, 281, 96, 281, 97, 281, 98, 281, 99, 281, 100, 281, 101, 281, 102, 281, 114, 281, 104, 281, 105, 281, 106, 281, 107, 281, 108, 281, 109, 281, 120, 281, 111, 281, 112, 281, 113, 281, 123, 281, 126, 281, 127, 281, 128, 281, 129, 281, 130, 281, 133, 281, 134, 281, 135, 281, 136, 281, 137, 281, 138, 281, 166, 281, 159, 281, 160, 281, 161, 281, 162, 281, 163, 281, 164, 281, 167, 281, 185, 281, 186, 281, 187, 281, 189, 281, 190, 281, 192, 281, 193, 281, 194, 281, 195, 281, 196, 281, 197, 281, 201, 281, 202, 281],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[65535, 314, 12, 314, 13, 314],
	#[13, -783],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[135, 436, 111, 436, 7, 436, 102, 436, 13, -606, 103, 436, 110, 436, 134, 436, 167, 436, 108, 436, 92, 436, 94, 436, 107, 436, 109, 436, 63, 436, 37, 105, 133, 436, 127, 436, 42, -93, 187, 436, 123, 436, 99, 436, 97, 436, 100, 436, 138, 436, 189, 436, 72, -50, 114, 436, 96, 436, 121, 436, 185, 436, 119, 436, 118, 436, 162, 436, 130, 436, 98, 436, 113, 436, 120, 436, 101, 436, 115, 436, 126, 436, 104, 436, 105, 436, 106, 436, 124, 436, 112, 436, 164, 436, 160, 436, 137, 436, 128, 436, 161, 436, 166, 436, 136, 436, 122, 436, 163, 436, 129, 436, 186, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 328, 7, 328, 63, 328, 92, 328, 94, 328, 96, 328, 97, 328, 98, 328, 99, 328, 100, 328, 101, 328, 102, 328, 103, 328, 104, 328, 105, 328, 106, 328, 107, 328, 108, 328, 109, 328, 110, 328, 111, 328, 112, 328, 113, 328, 114, 328, 115, 328, 118, 328, 119, 328, 120, 328, 121, 328, 122, 328, 123, 328, 124, 328, 126, 328, 127, 328, 128, 328, 129, 328, 130, 328, 133, 328, 134, 328, 135, 328, 136, 328, 137, 328, 138, 328, 160, 328, 161, 328, 162, 328, 163, 328, 164, 328, 166, 328, 167, 328, 185, 328, 186, 328, 187, 328, 189, 328, 190, 328, 192, 328, 193, 328, 194, 328, 195, 328, 196, 328, 197, 328, 201, 328, 202, 328],
	#[7, 436, 13, -606, 110, 436, 32, 436, 62, 436, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 123, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 159, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 285, 7, 285, 32, 285, 62, 285, 63, 285, 92, 285, 94, 285, 96, 285, 97, 285, 98, 285, 99, 285, 100, 285, 101, 285, 102, 285, 103, 285, 104, 285, 105, 285, 106, 285, 107, 285, 108, 285, 109, 285, 110, 285, 111, 285, 112, 285, 113, 285, 114, 285, 115, 285, 118, 285, 119, 285, 120, 285, 121, 285, 122, 285, 123, 285, 124, 285, 126, 285, 127, 285, 128, 285, 129, 285, 130, 285, 133, 285, 134, 285, 135, 285, 136, 285, 137, 285, 138, 285, 159, 285, 160, 285, 161, 285, 162, 285, 163, 285, 164, 285, 166, 285, 167, 285, 185, 285, 186, 285, 187, 285, 189, 285, 190, 285, 192, 285, 193, 285, 194, 285, 195, 285, 196, 285, 197, 285, 201, 285, 202, 285],
	#[13, -789],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -791],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, 436, 127, 436, 13, -606, 109, 436, 100, 436, 110, 436, 167, 436, 108, 436, 111, 436, 94, 436, 119, 436, 102, 436, 63, 436, 105, 436, 133, 436, 106, 436, 92, 436, 134, 436, 103, 436, 135, 436, 118, 436, 138, 436, 98, 436, 124, 436, 187, 436, 114, 436, 96, 436, 112, 436, 97, 436, 137, 436, 164, 436, 123, 436, 128, 436, 99, 436, 122, 436, 101, 436, 115, 436, 126, 436, 104, 436, 120, 436, 113, 436, 107, 436, 161, 436, 189, 436, 160, 436, 136, 436, 162, 436, 130, 436, 166, 436, 121, 436, 129, 436, 163, 436, 185, 436, 186, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 330, 7, 330, 63, 330, 92, 330, 94, 330, 96, 330, 97, 330, 98, 330, 99, 330, 100, 330, 101, 330, 102, 330, 103, 330, 104, 330, 105, 330, 106, 330, 107, 330, 108, 330, 109, 330, 110, 330, 111, 330, 112, 330, 113, 330, 114, 330, 115, 330, 118, 330, 119, 330, 120, 330, 121, 330, 122, 330, 123, 330, 124, 330, 126, 330, 127, 330, 128, 330, 129, 330, 130, 330, 133, 330, 134, 330, 135, 330, 136, 330, 137, 330, 138, 330, 160, 330, 161, 330, 162, 330, 163, 330, 164, 330, 166, 330, 167, 330, 185, 330, 186, 330, 187, 330, 189, 330, 190, 330, 192, 330, 193, 330, 194, 330, 195, 330, 196, 330, 197, 330, 201, 330, 202, 330],
	#[103, 436, 110, 436, 124, 436, 94, 436, 7, 436, 102, 436, 106, 436, 13, -796, 109, 436, 135, 436, 133, 436, 111, 436, 42, -93, 101, 436, 100, 436, 119, 436, 99, 436, 138, 436, 126, 436, 162, 436, 108, 436, 92, 436, 37, 105, 127, 436, 107, 436, 129, 436, 134, 436, 123, 436, 163, 436, 105, 436, 197, 436, 112, 436, 98, 436, 104, 436, 122, 436, 97, 436, 72, -50, 128, 436, 187, 436, 96, 436, 164, 436, 186, 436, 120, 436, 115, 436, 63, 436, 137, 436, 114, 436, 161, 436, 113, 436, 130, 436, 193, 436, 136, 436, 118, 436, 166, 436, 202, 436, 201, 436, 167, 436, 160, 436, 121, 436, 185, 436, 192, 436, 195, 436, 196, 436, 189, 436, 194, 436, 190, 436],
	#[65535, 335, 7, 335, 63, 335, 92, 335, 127, 335, 100, 335, 103, 335, 106, 335, 167, 335, 111, 335, 101, 335, 107, 335, 119, 335, 109, 335, 97, 335, 108, 335, 110, 335, 134, 335, 121, 335, 123, 335, 96, 335, 99, 335, 94, 335, 118, 335, 98, 335, 102, 335, 126, 335, 114, 335, 130, 335, 112, 335, 120, 335, 133, 335, 164, 335, 166, 335, 105, 335, 113, 335, 122, 335, 129, 335, 115, 335, 128, 335, 104, 335, 138, 335, 135, 335, 124, 335, 189, 335, 160, 335, 137, 335, 162, 335, 161, 335, 186, 335, 136, 335, 187, 335, 163, 335, 185, 335, 190, 335, 192, 335, 193, 335, 194, 335, 195, 335, 196, 335, 197, 335, 201, 335, 202, 335],
	#[36, -797, 90, -629, 222, -608],
	#[111, -222, 87, -320, 31, -326, 109, -180, 127, -187, 103, -228, 135, -213, 84, -322, 133, -197, 2, -327, 83, -330, 7, -37, 85, -317, 10, -212, 11, -318, 116, -9, 38, -316, 107, -229, 110, -233, 134, -184, 130, -191, 108, -199, 94, -210, 33, -321, 123, -174, 102, -178, 63, -219, 86, -325, 92, -189, 88, -313, 121, -201, 81, -312, 82, -315, 89, -329, 126, -198, 118, -181, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 137, -175, 114, -183, 104, -185, 105, -173, 106, -227, 112, -206, 113, -194, 115, -10, 138, -230, 128, -216, 119, -231, 120, -226, 136, -188, 122, -190, 124, -232, 129, -203, 223, -209],
	#[7, 436, 119, 436, 63, 436, 92, 436, 127, 436, 13, -628, 167, 436, 111, 436, 94, 436, 130, 436, 123, 436, 126, 436, 115, 436, 124, 436, 106, 436, 110, 436, 134, 436, 128, 436, 101, 436, 122, 436, 97, 436, 118, 436, 113, 436, 98, 436, 102, 436, 108, 436, 114, 436, 107, 436, 96, 436, 112, 436, 109, 436, 133, 436, 164, 436, 166, 436, 105, 436, 99, 436, 100, 436, 129, 436, 137, 436, 103, 436, 104, 436, 120, 436, 135, 436, 121, 436, 189, 436, 160, 436, 138, 436, 162, 436, 161, 436, 186, 436, 136, 436, 187, 436, 163, 436, 185, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 336, 7, 336, 63, 336, 92, 336, 94, 336, 96, 336, 97, 336, 98, 336, 99, 336, 100, 336, 101, 336, 102, 336, 103, 336, 104, 336, 105, 336, 106, 336, 107, 336, 108, 336, 109, 336, 110, 336, 111, 336, 112, 336, 113, 336, 114, 336, 115, 336, 118, 336, 119, 336, 120, 336, 121, 336, 122, 336, 123, 336, 124, 336, 126, 336, 127, 336, 128, 336, 129, 336, 130, 336, 133, 336, 134, 336, 135, 336, 136, 336, 137, 336, 138, 336, 160, 336, 161, 336, 162, 336, 163, 336, 164, 336, 166, 336, 167, 336, 185, 336, 186, 336, 187, 336, 189, 336, 190, 336, 192, 336, 193, 336, 194, 336, 195, 336, 196, 336, 197, 336, 201, 336, 202, 336],
	#[31, 415, 63, 415, 7, 415, 70, 415, 35, 415, 67, 415, 66, 415, 36, 415, 11, 415, 65, 415, 73, 415, 68, 415, 71, 415, 115, 415, 33, 415, 69, 415, 64, 415, 192, -589],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[13, -803],
	#[7, -37, 11, -66, 68, -64, 66, -58, 31, -59, 33, -60, 63, -63, 35, -67, 36, -65, 71, -55, 65, -71, 73, -70, 70, -72, 69, -68, 67, -61, 64, -73, 115, -10],
	#[7, 436, 13, -628, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 340, 7, 340, 63, 340, 103, 340, 102, 340, 107, 340, 101, 340, 100, 340, 99, 340, 98, 340, 104, 340, 97, 340, 105, 340, 92, 340, 94, 340, 96, 340, 106, 340, 108, 340, 109, 340, 110, 340, 111, 340, 112, 340, 113, 340, 114, 340, 115, 340, 118, 340, 119, 340, 120, 340, 121, 340, 122, 340, 123, 340, 124, 340, 126, 340, 127, 340, 128, 340, 129, 340, 130, 340, 133, 340, 134, 340, 135, 340, 136, 340, 137, 340, 138, 340, 160, 340, 161, 340, 162, 340, 163, 340, 164, 340, 166, 340, 167, 340, 185, 340, 186, 340, 187, 340, 189, 340, 190, 340, 192, 340, 193, 340, 194, 340, 195, 340, 196, 340, 197, 340, 201, 340, 202, 340],
	#[7, -37, 11, -66, 68, -64, 31, -59, 33, -60, 63, -63, 35, -67, 36, -65, 65, -71, 71, -55, 66, -58, 73, -70, 70, -72, 69, -68, 67, -61, 64, -73, 115, -10],
	#[13, -808],
	#[31, -59, 63, -63, 7, -37, 70, -72, 35, -67, 67, -61, 66, -58, 36, -65, 11, -66, 73, -70, 68, -64, 71, -55, 115, -10, 33, -60, 69, -68, 64, -73, 65, -71],
	#[203, -597, 204, 357, 205, 357, 206, 357, 207, 357, 208, 357, 209, 357],
	#[209, -602, 204, -603, 205, -600, 206, -598, 207, -599, 208, -601],
	#[111, 436, 7, 436, 102, 436, 13, -628, 103, 436, 106, 436, 167, 436, 92, 436, 94, 436, 107, 436, 100, 436, 109, 436, 63, 436, 105, 436, 108, 436, 110, 436, 134, 436, 101, 436, 123, 436, 96, 436, 99, 436, 97, 436, 118, 436, 115, 436, 98, 436, 122, 436, 126, 436, 114, 436, 130, 436, 112, 436, 119, 436, 133, 436, 164, 436, 166, 436, 127, 436, 113, 436, 121, 436, 129, 436, 137, 436, 128, 436, 104, 436, 120, 436, 135, 436, 124, 436, 189, 436, 160, 436, 138, 436, 162, 436, 161, 436, 186, 436, 136, 436, 187, 436, 163, 436, 185, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 341, 7, 341, 63, 341, 92, 341, 94, 341, 96, 341, 97, 341, 98, 341, 99, 341, 100, 341, 101, 341, 102, 341, 103, 341, 104, 341, 105, 341, 106, 341, 107, 341, 108, 341, 109, 341, 110, 341, 111, 341, 112, 341, 113, 341, 114, 341, 115, 341, 118, 341, 119, 341, 120, 341, 121, 341, 122, 341, 123, 341, 124, 341, 126, 341, 127, 341, 128, 341, 129, 341, 130, 341, 133, 341, 134, 341, 135, 341, 136, 341, 137, 341, 138, 341, 160, 341, 161, 341, 162, 341, 163, 341, 164, 341, 166, 341, 167, 341, 185, 341, 186, 341, 187, 341, 189, 341, 190, 341, 192, 341, 193, 341, 194, 341, 195, 341, 196, 341, 197, 341, 201, 341, 202, 341],
	#[13, -814],
	#[38, -815],
	#[7, 436, 13, -606, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 349, 7, 349, 63, 349, 135, 349, 109, 349, 119, 349, 164, 349, 161, 349, 127, 349, 94, 349, 124, 349, 103, 349, 92, 349, 121, 349, 167, 349, 123, 349, 163, 349, 96, 349, 134, 349, 97, 349, 133, 349, 98, 349, 99, 349, 100, 349, 101, 349, 102, 349, 126, 349, 104, 349, 105, 349, 106, 349, 107, 349, 108, 349, 137, 349, 110, 349, 111, 349, 112, 349, 113, 349, 114, 349, 115, 349, 138, 349, 118, 349, 130, 349, 120, 349, 136, 349, 122, 349, 128, 349, 129, 349, 160, 349, 162, 349, 166, 349, 186, 349, 197, 349, 196, 349, 201, 349, 195, 349, 194, 349, 185, 349, 193, 349, 187, 349, 189, 349, 190, 349, 192, 349, 202, 349],
	#[65535, 307, 7, 307, 13, 307, 32, 307, 62, 307, 63, 307, 92, 307, 94, 307, 96, 307, 97, 307, 98, 307, 99, 307, 100, 307, 101, 307, 102, 307, 103, 307, 104, 307, 105, 307, 106, 307, 107, 307, 108, 307, 109, 307, 110, 307, 111, 307, 112, 307, 113, 307, 114, 307, 115, 307, 118, 307, 119, 307, 120, 307, 121, 307, 122, 307, 123, 307, 124, 307, 126, 307, 127, 307, 128, 307, 129, 307, 130, 307, 133, 307, 134, 307, 135, 307, 136, 307, 137, 307, 138, 307, 159, 307, 160, 307, 161, 307, 162, 307, 163, 307, 164, 307, 166, 307, 167, 307, 185, 307, 186, 307, 187, 307, 189, 307, 190, 307, 192, 307, 193, 307, 194, 307, 195, 307, 196, 307, 197, 307, 201, 307, 202, 307],
	#[111, 436, 7, 436, 13, -822, 32, 436, 63, 436, 62, 436, 100, 436, 98, 436, 124, 436, 99, 436, 119, 436, 101, 436, 110, 436, 107, 436, 94, 436, 103, 436, 92, 436, 97, 436, 102, 436, 115, 436, 105, 436, 114, 436, 121, 436, 104, 436, 118, 436, 96, 436, 106, 436, 108, 436, 109, 436, 120, 436, 122, 436, 112, 436, 113, 436, 123, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 166, 436, 159, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[7, 436, 13, -606, 32, 436, 42, -75, 62, 436, 63, 436, 94, 436, 111, 436, 92, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 159, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 276, 7, 276, 32, 276, 62, 276, 63, 276, 92, 276, 94, 276, 96, 276, 97, 276, 98, 276, 99, 276, 100, 276, 101, 276, 102, 276, 103, 276, 104, 276, 105, 276, 106, 276, 107, 276, 108, 276, 109, 276, 110, 276, 111, 276, 112, 276, 113, 276, 114, 276, 115, 276, 118, 276, 119, 276, 120, 276, 121, 276, 122, 276, 123, 276, 124, 276, 126, 276, 127, 276, 128, 276, 129, 276, 130, 276, 133, 276, 134, 276, 135, 276, 136, 276, 137, 276, 138, 276, 159, 276, 160, 276, 161, 276, 162, 276, 163, 276, 164, 276, 166, 276, 167, 276, 185, 276, 186, 276, 187, 276, 189, 276, 190, 276, 192, 276, 193, 276, 194, 276, 195, 276, 196, 276, 197, 276, 201, 276, 202, 276],
	#[65535, 275, 119, 275, 7, 275, 103, 275, 111, 275, 32, 275, 63, 275, 62, 275, 110, 275, 123, 275, 126, 275, 101, 275, 127, 275, 94, 275, 118, 275, 124, 275, 166, 275, 92, 275, 135, 275, 164, 275, 167, 275, 100, 275, 129, 275, 134, 275, 109, 275, 108, 275, 113, 275, 133, 275, 96, 275, 97, 275, 98, 275, 99, 275, 122, 275, 159, 275, 102, 275, 114, 275, 104, 275, 105, 275, 106, 275, 107, 275, 112, 275, 189, 275, 115, 275, 162, 275, 130, 275, 120, 275, 121, 275, 187, 275, 137, 275, 163, 275, 161, 275, 138, 275, 128, 275, 185, 275, 160, 275, 136, 275, 186, 275, 190, 275, 192, 275, 193, 275, 194, 275, 195, 275, 196, 275, 197, 275, 201, 275, 202, 275],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10, 222, -608],
	#[65535, 308, 7, 308, 13, 308, 32, 308, 62, 308, 63, 308, 92, 308, 94, 308, 96, 308, 97, 308, 98, 308, 99, 308, 100, 308, 101, 308, 102, 308, 103, 308, 104, 308, 105, 308, 106, 308, 107, 308, 108, 308, 109, 308, 110, 308, 111, 308, 112, 308, 113, 308, 114, 308, 115, 308, 118, 308, 119, 308, 120, 308, 121, 308, 122, 308, 123, 308, 124, 308, 126, 308, 127, 308, 128, 308, 129, 308, 130, 308, 133, 308, 134, 308, 135, 308, 136, 308, 137, 308, 138, 308, 159, 308, 160, 308, 161, 308, 162, 308, 163, 308, 164, 308, 166, 308, 167, 308, 185, 308, 186, 308, 187, 308, 189, 308, 190, 308, 192, 308, 193, 308, 194, 308, 195, 308, 196, 308, 197, 308, 201, 308, 202, 308],
	#[118, -827, 120, -830, 138, -832, 137, -829, 136, -825, 210, -836, 211, -828, 212, -826, 213, -831, 214, -833, 215, -834],
	#[65535, 368, 7, 368, 11, 368, 31, 368, 33, 368, 35, 368, 36, 368, 63, 368, 64, 368, 65, 368, 66, 368, 67, 368, 68, 368, 69, 368, 70, 368, 71, 368, 73, 368, 115, 368],
	#[65535, 372, 31, 372, 69, 372, 11, 372, 68, 372, 70, 372, 36, 372, 63, 372, 7, 372, 73, 372, 115, 372, 33, 372, 71, 372, 66, 372, 65, 372, 67, 372, 35, 372, 64, 372],
	#[65535, 366, 7, 366, 11, 366, 66, 366, 31, 366, 33, 366, 68, 366, 35, 366, 36, 366, 71, 366, 65, 366, 70, 366, 69, 366, 67, 366, 73, 366, 63, 366, 64, 366, 115, 366],
	#[65535, 369, 7, 369, 11, 369, 31, 369, 33, 369, 35, 369, 36, 369, 63, 369, 64, 369, 65, 369, 66, 369, 67, 369, 68, 369, 69, 369, 70, 369, 71, 369, 73, 369, 115, 369],
	#[65535, 370, 31, 370, 70, 370, 36, 370, 63, 370, 7, 370, 35, 370, 73, 370, 11, 370, 69, 370, 68, 370, 64, 370, 66, 370, 33, 370, 67, 370, 65, 370, 71, 370, 115, 370],
	#[65535, 367, 7, 367, 11, 367, 31, 367, 33, 367, 35, 367, 36, 367, 63, 367, 64, 367, 65, 367, 66, 367, 67, 367, 68, 367, 69, 367, 70, 367, 71, 367, 73, 367, 115, 367],
	#[65535, 373, 7, 373, 11, 373, 31, 373, 33, 373, 35, 373, 36, 373, 63, 373, 64, 373, 65, 373, 66, 373, 67, 373, 68, 373, 69, 373, 70, 373, 71, 373, 73, 373, 115, 373],
	#[65535, 371, 7, 371, 11, 371, 71, 371, 33, 371, 31, 371, 66, 371, 68, 371, 35, 371, 36, 371, 73, 371, 65, 371, 70, 371, 69, 371, 67, 371, 64, 371, 63, 371, 115, 371],
	#[65535, 374, 7, 374, 11, 374, 31, 374, 33, 374, 35, 374, 36, 374, 63, 374, 64, 374, 65, 374, 66, 374, 67, 374, 68, 374, 69, 374, 70, 374, 71, 374, 73, 374, 115, 374],
	#[65535, 375, 7, 375, 11, 375, 71, 375, 66, 375, 31, 375, 33, 375, 68, 375, 35, 375, 36, 375, 73, 375, 65, 375, 70, 375, 69, 375, 67, 375, 64, 375, 63, 375, 115, 375],
	#[7, -37, 11, -66, 33, -60, 31, -59, 66, -58, 68, -64, 35, -67, 36, -65, 71, -55, 65, -71, 70, -72, 69, -68, 67, -61, 73, -70, 63, -63, 64, -73, 115, -10],
	#[65535, 365, 7, 365, 11, 365, 31, 365, 33, 365, 35, 365, 36, 365, 63, 365, 64, 365, 65, 365, 66, 365, 67, 365, 68, 365, 69, 365, 70, 365, 71, 365, 73, 365, 115, 365],
	#[13, -838],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[203, -597, 204, 357, 205, 357, 206, 357, 207, 357, 208, 357, 209, 357],
	#[204, -603, 205, -600, 206, -598, 207, -599, 208, -601, 209, -602],
	#[7, 436, 13, -606, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 344, 111, 344, 110, 344, 63, 344, 164, 344, 7, 344, 127, 344, 126, 344, 135, 344, 167, 344, 163, 344, 129, 344, 189, 344, 119, 344, 166, 344, 123, 344, 92, 344, 102, 344, 115, 344, 128, 344, 109, 344, 160, 344, 103, 344, 138, 344, 137, 344, 97, 344, 124, 344, 108, 344, 94, 344, 118, 344, 106, 344, 162, 344, 100, 344, 122, 344, 161, 344, 114, 344, 133, 344, 120, 344, 134, 344, 98, 344, 121, 344, 136, 344, 99, 344, 105, 344, 112, 344, 187, 344, 101, 344, 130, 344, 107, 344, 113, 344, 185, 344, 104, 344, 96, 344, 186, 344, 190, 344, 192, 344, 193, 344, 194, 344, 195, 344, 196, 344, 197, 344, 201, 344, 202, 344],
	#[7, -37, 11, -66, 33, -60, 31, -59, 66, -58, 68, -64, 35, -67, 36, -65, 71, -55, 65, -71, 70, -72, 64, -73, 69, -68, 67, -61, 73, -70, 63, -63, 115, -10],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 37, 105, 38, -316, 42, -93, 63, -219, 72, -50, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[13, -846],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[7, 436, 13, -606, 63, 436, 92, 436, 94, 436, 96, 436, 97, 436, 98, 436, 99, 436, 100, 436, 101, 436, 102, 436, 103, 436, 104, 436, 105, 436, 106, 436, 107, 436, 108, 436, 109, 436, 110, 436, 111, 436, 112, 436, 113, 436, 114, 436, 115, 436, 118, 436, 119, 436, 120, 436, 121, 436, 122, 436, 123, 436, 124, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 166, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 325, 7, 325, 63, 325, 94, 325, 103, 325, 102, 325, 101, 325, 100, 325, 99, 325, 98, 325, 104, 325, 97, 325, 105, 325, 92, 325, 107, 325, 96, 325, 106, 325, 108, 325, 109, 325, 110, 325, 111, 325, 112, 325, 113, 325, 114, 325, 115, 325, 118, 325, 119, 325, 120, 325, 121, 325, 122, 325, 123, 325, 124, 325, 126, 325, 127, 325, 128, 325, 129, 325, 130, 325, 133, 325, 134, 325, 135, 325, 136, 325, 137, 325, 138, 325, 160, 325, 161, 325, 162, 325, 163, 325, 164, 325, 166, 325, 167, 325, 185, 325, 186, 325, 187, 325, 189, 325, 190, 325, 192, 325, 193, 325, 194, 325, 195, 325, 196, 325, 197, 325, 201, 325, 202, 325],
	#[13, -850],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 70, -72, 69, -68, 67, -61, 65, -71, 63, -63, 64, -73, 66, -58, 68, -64, 71, -55, 73, -70, 115, -10],
	#[13, -852],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[7, 436, 13, -606, 63, 436, 98, 436, 102, 436, 119, 436, 101, 436, 110, 436, 100, 436, 94, 436, 99, 436, 103, 436, 92, 436, 122, 436, 121, 436, 107, 436, 97, 436, 115, 436, 124, 436, 105, 436, 114, 436, 104, 436, 118, 436, 96, 436, 106, 436, 108, 436, 109, 436, 120, 436, 111, 436, 112, 436, 113, 436, 123, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 166, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 327, 7, 327, 63, 327, 92, 327, 94, 327, 96, 327, 97, 327, 98, 327, 99, 327, 100, 327, 101, 327, 102, 327, 103, 327, 104, 327, 105, 327, 106, 327, 107, 327, 108, 327, 109, 327, 110, 327, 111, 327, 112, 327, 113, 327, 114, 327, 115, 327, 118, 327, 119, 327, 120, 327, 121, 327, 122, 327, 123, 327, 124, 327, 126, 327, 127, 327, 128, 327, 129, 327, 130, 327, 133, 327, 134, 327, 135, 327, 136, 327, 137, 327, 138, 327, 160, 327, 161, 327, 162, 327, 163, 327, 164, 327, 166, 327, 167, 327, 185, 327, 186, 327, 187, 327, 189, 327, 190, 327, 192, 327, 193, 327, 194, 327, 195, 327, 196, 327, 197, 327, 201, 327, 202, 327],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 37, 105, 38, -316, 42, -93, 63, -219, 72, -50, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[13, -857],
	#[2, -327, 7, -37, 10, -212, 11, -318, 31, -326, 33, -321, 38, -316, 63, -219, 81, -312, 82, -315, 83, -330, 84, -322, 85, -317, 86, -325, 87, -320, 88, -313, 89, -329, 92, -189, 94, -210, 96, -182, 97, -217, 98, -192, 99, -218, 100, -193, 101, -214, 102, -178, 103, -228, 104, -185, 105, -173, 106, -227, 107, -229, 108, -199, 109, -180, 110, -233, 111, -222, 112, -206, 113, -194, 114, -183, 115, -10, 116, -9, 118, -181, 119, -231, 120, -226, 121, -201, 122, -190, 123, -174, 124, -232, 126, -198, 127, -187, 128, -216, 129, -203, 130, -191, 133, -197, 134, -184, 135, -213, 136, -188, 137, -175, 138, -230, 223, -209],
	#[119, 436, 118, 436, 98, 436, 127, 436, 103, 436, 92, 436, 167, 436, 115, 436, 111, 436, 164, 436, 130, 436, 101, 436, 197, 436, 190, 436, 123, 436, 163, 436, 63, 436, 109, 436, 7, 436, 108, 436, 186, 436, 189, 436, 196, 436, 110, 436, 134, 436, 128, 436, 138, 436, 99, 436, 135, 436, 94, 436, 137, 436, 122, 436, 166, 436, 13, -606, 114, 436, 187, 436, 102, 436, 162, 436, 133, 436, 161, 436, 160, 436, 136, 436, 100, 436, 195, 436, 201, 436, 105, 436, 106, 436, 124, 436, 193, 436, 96, 436, 97, 436, 104, 436, 120, 436, 107, 436, 126, 436, 121, 436, 129, 436, 112, 436, 194, 436, 185, 436, 113, 436, 202, 436, 192, 436],
	#[65535, 322, 7, 322, 111, 322, 119, 322, 63, 322, 110, 322, 94, 322, 103, 322, 102, 322, 124, 322, 92, 322, 107, 322, 118, 322, 96, 322, 97, 322, 98, 322, 99, 322, 100, 322, 101, 322, 115, 322, 114, 322, 104, 322, 105, 322, 106, 322, 121, 322, 108, 322, 109, 322, 120, 322, 122, 322, 112, 322, 113, 322, 123, 322, 126, 322, 127, 322, 128, 322, 129, 322, 130, 322, 133, 322, 134, 322, 135, 322, 136, 322, 137, 322, 138, 322, 166, 322, 160, 322, 161, 322, 162, 322, 163, 322, 164, 322, 167, 322, 185, 322, 186, 322, 187, 322, 189, 322, 190, 322, 192, 322, 193, 322, 194, 322, 195, 322, 196, 322, 197, 322, 201, 322, 202, 322],
	#[13, -248],
	#[7, 436, 13, -660, 63, 436, 124, 436, 119, 436, 101, 436, 110, 436, 94, 436, 121, 436, 99, 436, 103, 436, 92, 436, 98, 436, 122, 436, 102, 436, 107, 436, 100, 436, 115, 436, 97, 436, 105, 436, 114, 436, 104, 436, 118, 436, 96, 436, 106, 436, 108, 436, 109, 436, 120, 436, 111, 436, 112, 436, 113, 436, 123, 436, 126, 436, 127, 436, 128, 436, 129, 436, 130, 436, 133, 436, 134, 436, 135, 436, 136, 436, 137, 436, 138, 436, 166, 436, 160, 436, 161, 436, 162, 436, 163, 436, 164, 436, 167, 436, 185, 436, 186, 436, 187, 436, 189, 436, 190, 436, 192, 436, 193, 436, 194, 436, 195, 436, 196, 436, 197, 436, 201, 436, 202, 436],
	#[65535, 348, 7, 348, 63, 348, 92, 348, 94, 348, 96, 348, 97, 348, 98, 348, 99, 348, 100, 348, 101, 348, 102, 348, 103, 348, 104, 348, 105, 348, 106, 348, 107, 348, 108, 348, 109, 348, 110, 348, 111, 348, 112, 348, 113, 348, 114, 348, 115, 348, 118, 348, 119, 348, 120, 348, 121, 348, 122, 348, 123, 348, 124, 348, 126, 348, 127, 348, 128, 348, 129, 348, 130, 348, 133, 348, 134, 348, 135, 348, 136, 348, 137, 348, 138, 348, 160, 348, 161, 348, 162, 348, 163, 348, 164, 348, 166, 348, 167, 348, 185, 348, 186, 348, 187, 348, 189, 348, 190, 348, 192, 348, 193, 348, 194, 348, 195, 348, 196, 348, 197, 348, 201, 348, 202, 348],
	#[65535, 193, 7, 193, 32, 193, 62, 193, 63, 193, 92, 193, 94, 193, 96, 193, 97, 193, 98, 193, 99, 193, 100, 193, 101, 193, 102, 193, 103, 193, 104, 193, 105, 193, 106, 193, 107, 193, 108, 193, 109, 193, 110, 193, 111, 193, 112, 193, 113, 193, 114, 193, 115, 193, 118, 193, 119, 193, 120, 193, 121, 193, 122, 193, 123, 193, 124, 193, 126, 193, 127, 193, 128, 193, 129, 193, 130, 193, 133, 193, 134, 193, 135, 193, 136, 193, 137, 193, 138, 193, 159, 193, 160, 193, 161, 193, 162, 193, 163, 193, 164, 193, 166, 193, 167, 193, 185, 193, 186, 193, 187, 193, 189, 193, 190, 193, 192, 193, 193, 193, 194, 193, 195, 193, 196, 193, 197, 193, 201, 193, 202, 193],
	#[65535, 273, 7, 273, 63, 273, 127, 273, 92, 273, 94, 273, 96, 273, 97, 273, 98, 273, 99, 273, 100, 273, 101, 273, 102, 273, 103, 273, 104, 273, 105, 273, 106, 273, 107, 273, 108, 273, 109, 273, 110, 273, 111, 273, 112, 273, 113, 273, 114, 273, 115, 273, 118, 273, 119, 273, 120, 273, 121, 273, 122, 273, 123, 273, 124, 273, 126, 273, 138, 273, 128, 273, 129, 273, 130, 273, 133, 273, 134, 273, 135, 273, 136, 273, 137, 273, 160, 273, 161, 273, 162, 273, 163, 273, 164, 273, 166, 273, 167, 273, 197, 273, 196, 273, 194, 273, 185, 273, 186, 273, 187, 273, 189, 273, 190, 273, 202, 273, 192, 273, 193, 273, 201, 273, 195, 273],
	#[65535, 82, 7, 82, 32, 82, 62, 82, 63, 82, 92, 82, 94, 82, 96, 82, 97, 82, 98, 82, 99, 82, 100, 82, 101, 82, 102, 82, 103, 82, 104, 82, 105, 82, 106, 82, 107, 82, 108, 82, 109, 82, 110, 82, 111, 82, 112, 82, 113, 82, 114, 82, 115, 82, 118, 82, 119, 82, 120, 82, 121, 82, 122, 82, 123, 82, 124, 82, 126, 82, 127, 82, 128, 82, 129, 82, 130, 82, 133, 82, 134, 82, 135, 82, 136, 82, 137, 82, 138, 82, 159, 82, 160, 82, 161, 82, 162, 82, 163, 82, 164, 82, 166, 82, 167, 82, 185, 82, 186, 82, 187, 82, 189, 82, 190, 82, 192, 82, 193, 82, 194, 82, 195, 82, 196, 82, 197, 82, 201, 82, 202, 82],
	#[65535, 79, 14, 79, 1, 79, 0, 79, 27, 79, 4, 79, 19, 79, 7, 79, 9, 79, 10, 79, 28, 79, 24, 79, 15, 79, 16, 79, 17, 79, 18, 79, 20, 79, 21, 79, 22, 79, 23, 79, 25, 79, 26, 79, 29, 79, 77, 79, 40, 79, 41, 79, 75, 79, 63, 79, 116, 79, 80, 79, 74, 79, 76, 79, 79, 79, 115, 79, 222, 79, 223, 79],
	#[7, 270, 32, -868, 63, 270, 111, 270, 110, 270, 103, 270, 107, 270, 100, 270, 115, 270, 119, 270, 92, 270, 94, 270, 118, 270, 96, 270, 97, 270, 98, 270, 99, 270, 121, 270, 101, 270, 102, 270, 114, 270, 104, 270, 105, 270, 106, 270, 124, 270, 108, 270, 109, 270, 120, 270, 122, 270, 112, 270, 113, 270, 123, 270, 126, 270, 127, 270, 128, 270, 129, 270, 130, 270, 133, 270, 134, 270, 135, 270, 136, 270, 137, 270, 138, 270, 166, 270, 159, -547, 160, 270, 161, 270, 162, 270, 163, 270, 164, 270, 167, 270, 185, 270, 186, 270, 187, 270, 189, 270, 190, 270, 192, 270, 193, 270, 194, 270, 195, 270, 196, 270, 197, 270, 201, 270, 202, 270],
	#[65535, 80, 0, 80, 1, 80, 4, 80, 7, 80, 9, 80, 10, 80, 14, 80, 15, 80, 16, 80, 17, 80, 18, 80, 19, 80, 20, 80, 21, 80, 22, 80, 23, 80, 24, 80, 25, 80, 26, 80, 27, 80, 28, 80, 29, 80, 40, 80, 41, 80, 63, 80, 74, 80, 75, 80, 76, 80, 77, 80, 79, 80, 80, 80, 115, 80, 116, 80, 222, 80, 223, 80],
	#[11, -871],
	#[65535, 11, 0, 11, 1, 11, 4, 11, 7, 11, 9, 11, 10, 11, 14, 11, 15, 11, 16, 11, 17, 11, 18, 11, 19, 11, 20, 11, 21, 11, 22, 11, 23, 11, 24, 11, 25, 11, 26, 11, 27, 11, 28, 11, 29, 11, 40, 11, 41, 11, 63, 11, 74, 11, 75, 11, 76, 11, 77, 11, 79, 11, 80, 11, 115, 11, 116, 11, 222, 11, 223, 11],
	#[7, -872, 12, 16, 13, 16],
	#[65535, 17, 12, 17, 13, 17],
	#[12, -875, 13, -874],
	#[7, -876],
	#[65535, 15, 0, 15, 1, 15, 4, 15, 7, 15, 9, 15, 10, 15, 14, 15, 15, 15, 16, 15, 17, 15, 18, 15, 19, 15, 20, 15, 21, 15, 22, 15, 23, 15, 24, 15, 25, 15, 26, 15, 27, 15, 28, 15, 29, 15, 40, 15, 41, 15, 63, 15, 74, 15, 75, 15, 76, 15, 77, 15, 79, 15, 80, 15, 116, 15, 115, 15, 222, 15, 223, 15],
	#[65535, 18, 12, 18, 13, 18],
	#[6, -881],
	#[6, -879],
	#[7, -880],
	#[65535, 10, 29, 10, 0, 10, 1, 10, 21, 10, 4, 10, 63, 10, 7, 10, 9, 10, 10, 10, 14, 10, 15, 10, 16, 10, 17, 10, 18, 10, 19, 10, 20, 10, 79, 10, 22, 10, 23, 10, 24, 10, 25, 10, 26, 10, 27, 10, 28, 10, 74, 10, 41, 10, 77, 10, 40, 10, 75, 10, 80, 10, 76, 10, 115, 10, 116, 10, 222, 10, 223, 10],
	#[7, -882],
	#[65535, 9, 0, 9, 1, 9, 4, 9, 19, 9, 7, 9, 9, 9, 10, 9, 14, 9, 15, 9, 16, 9, 17, 9, 18, 9, 24, 9, 20, 9, 21, 9, 22, 9, 23, 9, 25, 9, 26, 9, 27, 9, 28, 9, 29, 9, 40, 9, 41, 9, 76, 9, 75, 9, 63, 9, 74, 9, 80, 9, 77, 9, 79, 9, 116, 9, 115, 9, 222, 9, 223, 9],
	#[65535, 13, 21, 13, 3, 13, 14, 13, 15, 13, 16, 13, 17, 13, 18, 13, 19, 13, 20, 13, 79, 13, 22, 13, 23, 13, 24, 13, 25, 13, 26, 13, 27, 13, 28, 13, 29, 13, 78, 13, 77, 13, 74, 13, 80, 13, 72, 13, 76, 13, 75, 13],
	#[6, -885],
	#[7, -37, 11, -66, 36, -65, 31, -59, 33, -60, 35, -67, 71, -55, 65, -71, 67, -61, 63, -63, 64, -73, 66, -58, 68, -64, 69, -68, 70, -72, 73, -70, 115, -10],
	#[37, 105, 42, -93, 72, -50, 223, -887],
	#[31, -888],
	#[7, -37, 11, -66, 31, -59, 32, -890, 33, -60, 35, -67, 36, -65, 71, -55, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 82, -350, 73, -70, 115, -10],
	#[13, -354, 32, -891],
	#[65535, 423, 0, 423, 1, 423, 4, 423, 19, 423, 7, 423, 9, 423, 10, 423, 14, 423, 15, 423, 16, 423, 17, 423, 18, 423, 24, 423, 20, 423, 21, 423, 22, 423, 23, 423, 25, 423, 26, 423, 27, 423, 28, 423, 29, 423, 41, 423, 40, 423, 75, 423, 63, 423, 74, 423, 80, 423, 76, 423, 77, 423, 79, 423, 116, 423, 115, 423, 222, 423, 223, 423],
	#[65535, 424, 0, 424, 1, 424, 21, 424, 4, 424, 63, 424, 7, 424, 9, 424, 10, 424, 14, 424, 15, 424, 16, 424, 17, 424, 18, 424, 19, 424, 20, 424, 79, 424, 22, 424, 23, 424, 24, 424, 25, 424, 26, 424, 27, 424, 28, 424, 29, 424, 41, 424, 77, 424, 40, 424, 74, 424, 75, 424, 80, 424, 76, 424, 115, 424, 116, 424, 222, 424, 223, 424],
	#[65535, 14, 3, 14, 14, 14, 15, 14, 16, 14, 17, 14, 18, 14, 19, 14, 20, 14, 21, 14, 22, 14, 23, 14, 24, 14, 25, 14, 26, 14, 27, 14, 28, 14, 29, 14, 72, 14, 74, 14, 75, 14, 76, 14, 77, 14, 78, 14, 79, 14, 80, 14],
	#[79, 117, 77, -47, 80, 117],
	#[79, -53, 80, -54],
	#[7, -37, 11, -66, 31, -59, 33, -60, 35, -67, 36, -65, 63, -63, 64, -73, 65, -71, 66, -58, 67, -61, 68, -64, 69, -68, 70, -72, 71, -55, 73, -70, 115, -10],
	#[1, 143, 0, 143, 27, 143, 21, 143, 4, 143, 7, 143, 9, 143, 10, 143, 13, -156, 14, 143, 15, 143, 16, 143, 17, 143, 18, 143, 19, 143, 20, 143, 79, 143, 22, 143, 23, 143, 24, 143, 25, 143, 26, 143, 41, 143, 28, 143, 29, 143, 63, 143, 37, 105, 77, 143, 40, 143, 74, 143, 42, -93, 80, 143, 72, -50, 76, 143, 75, 143, 115, 143, 116, 143, 222, 143, 223, 143],
	#[65535, 7, 74, 7, 76, 7, 79, 7, 80, 7, 115, 7, 116, 7, 222, 7, 223, 7, 7, 7, 41, 7, 0, 7, 1, 7, 4, 7, 40, 7, 9, 7, 10, 7, 14, 7, 15, 7, 16, 7, 17, 7, 18, 7, 19, 7, 20, 7, 21, 7, 22, 7, 23, 7, 24, 7, 25, 7, 26, 7, 27, 7, 28, 7, 29, 7, 77, 7, 75, 7, 63, 7],
	#[7, -900],
	#[65535, 3, 14, 3, 79, 3, 75, 3, 0, 3, 1, 3, 4, 3, 63, 3, 19, 3, 7, 3, 9, 3, 10, 3, 28, 3, 27, 3, 15, 3, 16, 3, 17, 3, 18, 3, 20, 3, 21, 3, 22, 3, 23, 3, 24, 3, 25, 3, 26, 3, 41, 3, 29, 3, 116, 3, 40, 3, 77, 3, 76, 3, 80, 3, 74, 3, 115, 3, 222, 3, 223, 3],
	#[65535, 12, 29, 12, 0, 12, 1, 12, 4, 12, 7, 12, 9, 12, 10, 12, 14, 12, 15, 12, 16, 12, 17, 12, 18, 12, 19, 12, 20, 12, 21, 12, 22, 12, 23, 12, 24, 12, 25, 12, 26, 12, 27, 12, 28, 12, 40, 12, 41, 12, 79, 12, 75, 12, 63, 12, 74, 12, 76, 12, 77, 12, 80, 12, 116, 12, 115, 12, 222, 12, 223, 12],
	#[79, 117, 77, -47, 80, 117],
	#[79, -53, 80, -54],
	#[115, -10, 7, -37, 11, -66, 69, -68, 67, -61, 66, -58, 65, -71, 70, -72, 31, -59, 33, -60, 68, -64, 35, -67, 36, -65, 73, -70, 71, -55, 64, -73, 63, -63],
	#[22, 143, 4, 143, 10, 143, 14, 143, 79, 143, 24, 143, 75, 143, 0, 143, 1, 143, 63, 143, 7, 143, 76, 143, 9, 143, 28, 143, 13, -156, 116, 143, 15, 143, 16, 143, 17, 143, 18, 143, 19, 143, 20, 143, 21, 143, 23, 143, 25, 143, 26, 143, 27, 143, 29, 143, 41, 143, 77, 143, 40, 143, 74, 143, 80, 143, 115, 143, 222, 143, 223, 143],
	#[65535, 5, 15, 5, 115, 5, 14, 5, 20, 5, 41, 5, 29, 5, 63, 5, 10, 5, 7, 5, 116, 5, 9, 5, 77, 5, 21, 5, 18, 5, 4, 5, 27, 5, 17, 5, 40, 5, 76, 5, 79, 5, 23, 5, 74, 5, 22, 5, 0, 5, 1, 5, 19, 5, 24, 5, 75, 5, 28, 5, 16, 5, 25, 5, 26, 5, 80, 5, 222, 5, 223, 5],
	#[65535, 47, 30, 47],
	#[65535, 56, 7, 56, 11, 56, 47, 56, 33, 56, 31, 56, 66, 56, 35, 56, 36, 56, 71, 56, 44, 56, 45, 56, 46, 56, 48, 56, 49, 56, 50, 56, 51, 56, 52, 56, 53, 56, 54, 56, 55, 56, 56, 56, 57, 56, 58, 56, 59, 56, 60, 56, 75, 56, 63, 56, 64, 56, 65, 56, 67, 56, 68, 56, 69, 56, 70, 56, 73, 56, 74, 56, 76, 56, 115, 56],
	#[65535, 50, 29, 50, 63, 50, 7, 50, 27, 50, 14, 50, 79, 50, 28, 50, 115, 50, 10, 50, 0, 50, 1, 50, 21, 50, 20, 50, 4, 50, 19, 50, 18, 50, 9, 50, 17, 50, 16, 50, 41, 50, 24, 50, 15, 50, 25, 50, 77, 50, 22, 50, 23, 50, 76, 50, 26, 50, 74, 50, 40, 50, 80, 50, 75, 50, 116, 50, 222, 50, 223, 50],
	#[65535, 55, 35, 55, 31, 55, 63, 55, 7, 55, 76, 55, 60, 55, 11, 55, 33, 55, 71, 55, 58, 55, 53, 55, 52, 55, 36, 55, 59, 55, 56, 55, 57, 55, 75, 55, 51, 55, 55, 55, 44, 55, 74, 55, 54, 55, 48, 55, 50, 55, 49, 55, 45, 55, 46, 55, 47, 55, 64, 55, 65, 55, 66, 55, 67, 55, 68, 55, 69, 55, 70, 55, 73, 55, 115, 55],
	#[45, 113, 7, 113, 54, 113, 11, 113, 47, 113, 46, 113, 58, 113, 55, 113, 31, 113, 33, 113, 35, 113, 36, 113, 50, 113, 71, 113, 60, 113, 44, 113, 59, 113, 70, 113, 48, 113, 49, 113, 69, 113, 51, 113, 52, 113, 53, 113, 67, 113, 66, 113, 56, 113, 57, 113, 65, 113, 64, 113, 75, -44, 63, 113, 68, 113, 73, 113, 74, -43, 76, -42, 115, 113],
	#[35, 59, 31, 59, 63, 59, 7, 59, 60, 59, 11, 59, 33, 59, 71, 59, 58, 59, 53, -504, 52, -502, 36, 59, 59, 59, 57, 59, 51, -500, 55, -493, 44, -498, 54, -506, 48, -507, 56, -505, 50, -497, 49, -494, 45, -501, 46, -503, 47, -495, 64, 59, 65, 59, 66, 59, 67, 59, 68, 59, 69, 59, 70, 59, 73, 59, 115, 59],
	#[65535, 53, 7, 53, 27, 53, 14, 53, 23, 53, 0, 53, 1, 53, 4, 53, 63, 53, 19, 53, 18, 53, 9, 53, 10, 53, 28, 53, 41, 53, 24, 53, 15, 53, 16, 53, 17, 53, 25, 53, 20, 53, 21, 53, 22, 53, 75, 53, 26, 53, 29, 53, 74, 53, 116, 53, 115, 53, 40, 53, 76, 53, 80, 53, 77, 53, 79, 53, 222, 53, 223, 53]],
  goto-table:
      #[#[225, 1],
	#[272, 31, 273, 25, 233, 33, 231, 14, 232, 17, 226, 20, 222, 34, 228, 38, 271, 11, 223, 21, 116, 8, 115, 9, 41, 7, 40, 29, 63, 32, 29, 30, 26, 37, 24, 27, 27, 22, 28, 6, 17, 16, 9, 28, 19, 24, 25, 13, 21, 35, 16, 2, 0, 12, 18, 4, 14, 3, 1, 15, 7, 36, 10, 18, 22, 19, 4, 26, 20, 10, 23, 5, 15, 23],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[236, 907, 232, 906, 231, 908, 238, 909, 27, 22, 26, 37, 24, 27, 25, 13, 17, 16, 16, 2, 22, 19, 14, 3, 23, 5, 21, 35, 28, 6, 15, 23, 19, 24, 29, 30, 20, 10, 18, 4],
	#[],
	#[],
	#[],
	#[6, 905],
	#[],
	#[],
	#[251, 900, 75, 43, 74, 42, 76, 41],
	#[2, 897],
	#[],
	#[251, 892, 76, 41, 74, 42, 75, 43],
	#[6, 891],
	#[],
	#[],
	#[38, 883],
	#[],
	#[],
	#[],
	#[6, 882],
	#[8, 877, 5, 876],
	#[],
	#[6, 868],
	#[235, 488, 234, 489, 231, 490, 26, 37, 25, 13, 24, 27, 23, 5, 22, 19, 21, 35, 20, 10, 19, 24, 18, 4, 17, 16, 16, 2, 15, 23, 14, 3],
	#[],
	#[],
	#[6, 487],
	#[30, 459],
	#[6, 449],
	#[],
	#[],
	#[],
	#[232, 40, 231, 39, 29, 30, 28, 6, 27, 22, 26, 37, 25, 13, 23, 5, 22, 19, 21, 35, 20, 10, 24, 27, 18, 4, 17, 16, 16, 2, 15, 23, 14, 3, 19, 24],
	#[251, 163, 76, 41, 75, 43, 74, 42],
	#[251, 44, 76, 41, 75, 43, 74, 42],
	#[],
	#[],
	#[],
	#[252, 45, 77, 46],
	#[253, 47, 78, 48],
	#[],
	#[248, 50, 72, 49],
	#[],
	#[42, 95],
	#[254, 51, 80, 53, 79, 52],
	#[249, 55, 246, 61, 245, 68, 247, 56, 271, 73, 115, 9, 68, 63, 67, 60, 33, 59, 64, 72, 73, 69, 71, 54, 69, 67, 65, 70, 66, 57, 70, 71, 11, 65, 63, 62, 36, 64, 35, 66, 31, 58, 7, 36],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[271, 73, 263, 109, 249, 55, 247, 56, 246, 61, 245, 102, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 32, 110, 31, 58, 11, 65, 7, 36],
	#[38, 99, 31, 100],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[38, 105],
	#[],
	#[],
	#[256, 154, 248, 93, 72, 49, 42, 92, 13, 155],
	#[42, 74],
	#[],
	#[],
	#[],
	#[],
	#[271, 73, 262, 76, 261, 78, 250, 81, 249, 77, 247, 56, 246, 75, 245, 80, 115, 9, 95, 79, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[308, 150, 59, 149, 57, 151, 58, 148],
	#[13, 145],
	#[],
	#[43, 152],
	#[],
	#[248, 93, 72, 49, 42, 92],
	#[309, 82, 220, 88, 219, 86, 218, 84, 217, 89, 106, 83, 105, 90, 90, 85, 60, 87],
	#[],
	#[],
	#[],
	#[38, 91],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[247, 56, 246, 75, 261, 98, 262, 76, 249, 77, 250, 81, 115, 9, 271, 73, 95, 79, 245, 80, 71, 54, 70, 71, 67, 60, 73, 69, 65, 70, 63, 62, 64, 72, 68, 63, 69, 67, 66, 57, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[37, 94],
	#[],
	#[38, 96],
	#[43, 97],
	#[],
	#[43, 119],
	#[39, 116],
	#[263, 101, 247, 56, 246, 61, 271, 73, 249, 55, 245, 102, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 32, 103, 31, 58, 11, 65, 7, 36],
	#[32, 114, 13, 112],
	#[248, 93, 72, 49, 42, 92],
	#[34, 104],
	#[],
	#[39, 106],
	#[271, 73, 246, 61, 245, 107, 249, 55, 247, 56, 115, 9, 69, 67, 65, 70, 70, 71, 66, 57, 64, 72, 71, 54, 33, 59, 73, 69, 36, 64, 67, 60, 31, 58, 68, 63, 35, 66, 11, 65, 63, 62, 7, 36],
	#[248, 93, 72, 49, 42, 92, 12, 108],
	#[],
	#[32, 111, 13, 112],
	#[],
	#[],
	#[249, 55, 246, 61, 271, 73, 245, 113, 247, 56, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 65, 70, 66, 57, 64, 72, 67, 60, 36, 64, 35, 66, 63, 62, 33, 59, 31, 58, 11, 65, 7, 36],
	#[248, 93, 72, 49, 42, 92],
	#[34, 115],
	#[],
	#[271, 73, 246, 61, 249, 55, 245, 117, 247, 56, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 7, 36, 31, 58, 11, 65],
	#[248, 93, 72, 49, 42, 92, 34, 118],
	#[],
	#[276, 120],
	#[289, 121, 184, 137, 183, 133, 181, 132, 180, 138, 179, 134, 178, 131, 177, 128, 176, 125, 175, 122, 173, 135, 172, 140, 171, 129, 170, 126, 169, 136, 168, 141, 182, 130, 174, 139, 59, 124, 58, 123, 57, 127],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[42, 142],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[38, 143],
	#[43, 144],
	#[],
	#[271, 73, 250, 146, 249, 77, 247, 56, 246, 75, 245, 80, 115, 9, 95, 147, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[309, 82, 220, 88, 219, 86, 218, 84, 217, 89, 106, 83, 105, 90, 90, 85, 60, 87],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[276, 153],
	#[289, 121, 176, 125, 184, 137, 169, 136, 180, 138, 181, 132, 177, 128, 173, 135, 168, 141, 171, 129, 179, 134, 170, 126, 183, 133, 178, 131, 172, 140, 182, 130, 174, 139, 175, 122, 59, 124, 58, 123, 57, 127],
	#[],
	#[258, 159, 257, 156, 91, 157, 90, 158],
	#[256, 162, 13, 155],
	#[7, 161],
	#[38, 160],
	#[],
	#[],
	#[],
	#[],
	#[252, 165, 77, 46, 3, 164],
	#[231, 431, 25, 13, 23, 5, 21, 35, 20, 10, 24, 27, 16, 2, 26, 37, 14, 3, 17, 16, 19, 24, 22, 19, 18, 4, 15, 23],
	#[248, 166, 72, 49],
	#[253, 167, 78, 48],
	#[254, 168, 80, 53, 79, 52],
	#[271, 73, 255, 170, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[317, 206, 316, 224, 283, 222, 281, 178, 273, 176, 271, 195, 266, 214, 265, 210, 248, 93, 260, 233, 223, 208, 137, 174, 135, 212, 133, 196, 136, 187, 130, 190, 129, 202, 128, 215, 138, 229, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 113, 193, 112, 205, 134, 183, 109, 179, 108, 198, 107, 228, 106, 226, 105, 172, 104, 184, 114, 182, 102, 177, 101, 213, 100, 192, 99, 217, 127, 186, 97, 216, 96, 181, 94, 209, 92, 188, 89, 234, 88, 207, 98, 191, 86, 223, 85, 204, 84, 203, 82, 185, 81, 175, 103, 227, 72, 49, 83, 235, 110, 232, 63, 218, 87, 220, 111, 221, 42, 92, 33, 201, 38, 199, 11, 194, 10, 211, 7, 36, 31, 219],
	#[256, 171, 13, 155],
	#[],
	#[],
	#[],
	#[],
	#[7, 425],
	#[],
	#[42, 356],
	#[42, 420],
	#[],
	#[282, 419, 132, 254, 131, 253],
	#[42, 412],
	#[],
	#[125, 411],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[282, 410, 132, 254, 131, 253],
	#[],
	#[285, 394, 158, 391, 157, 393, 156, 390, 155, 388, 154, 402, 153, 400, 152, 398, 151, 404, 150, 392, 149, 389, 148, 403, 147, 401, 146, 399, 145, 396, 88, 395, 87, 397],
	#[42, 381],
	#[],
	#[271, 73, 264, 379, 255, 262, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 66, 57, 65, 70, 64, 72, 63, 62, 67, 60, 36, 64, 35, 66, 33, 59, 31, 58, 12, 378, 11, 65, 7, 36],
	#[],
	#[282, 377, 132, 254, 131, 253],
	#[125, 376],
	#[],
	#[],
	#[],
	#[271, 73, 264, 368, 249, 55, 246, 61, 245, 169, 247, 56, 255, 262, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 369, 11, 65, 7, 36],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[38, 345, 7, 347, 31, 346],
	#[312, 303, 221, 302],
	#[],
	#[],
	#[125, 301],
	#[42, 294],
	#[42, 289],
	#[],
	#[284, 279, 148, 282, 147, 280, 146, 277, 144, 274, 143, 273, 142, 283, 141, 281, 139, 276, 145, 275, 140, 278],
	#[42, 268],
	#[],
	#[11, 65, 7, 36, 271, 73, 264, 264, 255, 262, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 32, 263, 31, 58],
	#[],
	#[],
	#[42, 257],
	#[],
	#[],
	#[282, 252, 132, 254, 131, 253],
	#[],
	#[42, 242],
	#[],
	#[],
	#[],
	#[125, 241],
	#[],
	#[],
	#[42, 236],
	#[],
	#[273, 237, 116, 8],
	#[13, 238],
	#[271, 239, 115, 9, 7, 36],
	#[43, 240],
	#[],
	#[],
	#[271, 73, 249, 55, 246, 61, 245, 169, 255, 243, 247, 56, 115, 9, 73, 69, 69, 67, 68, 63, 64, 72, 63, 62, 65, 70, 66, 57, 67, 60, 36, 64, 35, 66, 33, 59, 70, 71, 71, 54, 31, 58, 11, 65, 7, 36],
	#[13, 244],
	#[271, 73, 255, 245, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 33, 59, 35, 66, 36, 64, 11, 65, 7, 36, 63, 62, 31, 58],
	#[267, 246, 13, 247],
	#[43, 249, 13, 250],
	#[38, 248],
	#[],
	#[],
	#[38, 251],
	#[],
	#[],
	#[132, 256],
	#[131, 255],
	#[],
	#[],
	#[255, 258, 247, 56, 246, 61, 271, 73, 249, 55, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 259],
	#[247, 56, 246, 61, 249, 55, 255, 260, 271, 73, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[43, 261],
	#[],
	#[],
	#[],
	#[32, 265, 13, 266],
	#[],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 169, 255, 267, 115, 9, 71, 54, 70, 71, 69, 67, 68, 63, 73, 69, 65, 70, 64, 72, 63, 62, 66, 57, 67, 60, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 169, 255, 269, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 270],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 169, 255, 271, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[43, 272],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[42, 284],
	#[],
	#[],
	#[],
	#[],
	#[271, 73, 255, 285, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 286],
	#[271, 73, 255, 287, 247, 56, 246, 61, 245, 169, 249, 55, 115, 9, 64, 72, 67, 60, 33, 59, 73, 69, 66, 57, 68, 63, 65, 70, 71, 54, 69, 67, 11, 65, 35, 66, 7, 36, 70, 71, 63, 62, 31, 58, 36, 64],
	#[43, 288],
	#[],
	#[271, 73, 255, 290, 249, 55, 246, 61, 245, 169, 69, 67, 65, 70, 73, 69, 67, 60, 68, 63, 31, 58, 115, 9, 66, 57, 64, 72, 247, 56, 71, 54, 63, 62, 70, 71, 33, 59, 35, 66, 7, 36, 11, 65, 36, 64],
	#[93, 291],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 292, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[43, 293, 248, 93, 72, 49, 42, 92],
	#[],
	#[271, 73, 255, 295, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 296],
	#[249, 55, 255, 297, 271, 73, 247, 56, 246, 61, 115, 9, 245, 169, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 64, 72, 65, 70, 36, 64, 35, 66, 63, 62, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 298],
	#[271, 73, 255, 299, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[43, 300],
	#[],
	#[],
	#[],
	#[42, 304],
	#[271, 73, 255, 305, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[280, 306],
	#[43, 307, 13, 308],
	#[],
	#[290, 309, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[],
	#[317, 206, 316, 323, 305, 318, 292, 313, 283, 222, 281, 178, 273, 176, 271, 195, 266, 214, 265, 322, 260, 327, 248, 93, 223, 208, 137, 174, 135, 212, 133, 196, 136, 187, 130, 190, 129, 202, 138, 229, 126, 197, 124, 231, 128, 215, 122, 189, 119, 230, 118, 180, 116, 8, 120, 225, 113, 193, 134, 183, 109, 179, 108, 198, 105, 172, 101, 213, 121, 200, 104, 184, 98, 191, 123, 173, 96, 181, 106, 226, 107, 228, 114, 182, 97, 216, 115, 9, 88, 312, 112, 205, 99, 217, 85, 316, 84, 321, 100, 192, 82, 314, 81, 311, 102, 177, 92, 188, 103, 227, 72, 49, 94, 209, 83, 329, 127, 186, 89, 328, 110, 232, 111, 221, 42, 92, 86, 324, 38, 315, 63, 218, 33, 320, 87, 319, 11, 317, 10, 211, 2, 326, 7, 36, 31, 325],
	#[7, 430],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[264, 428, 246, 61, 249, 55, 271, 73, 245, 169, 255, 262, 247, 56, 115, 9, 70, 71, 69, 67, 68, 63, 73, 69, 65, 70, 64, 72, 63, 62, 66, 57, 67, 60, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 71, 54, 12, 427, 7, 36],
	#[],
	#[],
	#[271, 73, 264, 360, 255, 262, 247, 56, 246, 61, 245, 169, 249, 55, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 361, 11, 65, 7, 36],
	#[],
	#[],
	#[],
	#[],
	#[271, 73, 264, 343, 255, 262, 247, 56, 246, 61, 245, 169, 249, 55, 115, 9, 71, 54, 63, 62, 73, 69, 65, 70, 70, 71, 64, 72, 67, 60, 36, 64, 35, 66, 68, 63, 66, 57, 32, 342, 33, 59, 69, 67, 11, 65, 7, 36, 31, 58],
	#[306, 336, 216, 337, 181, 335],
	#[],
	#[42, 330],
	#[],
	#[273, 331, 116, 8],
	#[13, 332],
	#[271, 333, 115, 9, 7, 36],
	#[43, 334],
	#[],
	#[],
	#[7, 339],
	#[181, 338],
	#[],
	#[13, 340],
	#[7, 341],
	#[],
	#[],
	#[32, 344, 13, 266],
	#[],
	#[],
	#[315, 350, 271, 73, 255, 348, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 82, 349, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 32, 351, 31, 58, 11, 65, 7, 36],
	#[],
	#[],
	#[],
	#[32, 352, 13, 353],
	#[],
	#[],
	#[271, 73, 255, 355, 246, 61, 245, 169, 249, 55, 247, 56, 115, 9, 71, 54, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 82, 354, 65, 70, 64, 72, 11, 65, 73, 69, 7, 36, 67, 60, 66, 57, 70, 71, 68, 63, 69, 67],
	#[],
	#[],
	#[271, 73, 255, 357, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[267, 358, 13, 247],
	#[43, 359, 13, 250],
	#[],
	#[34, 426, 13, 266],
	#[271, 73, 264, 364, 255, 262, 249, 55, 247, 56, 246, 61, 245, 362, 263, 109, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 32, 363, 31, 58, 11, 65, 7, 36],
	#[316, 224, 317, 206, 283, 222, 281, 178, 273, 176, 271, 195, 266, 214, 260, 233, 265, 210, 248, 93, 223, 208, 138, 229, 137, 174, 135, 212, 134, 183, 133, 196, 136, 187, 130, 190, 129, 202, 128, 215, 127, 186, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 112, 205, 111, 221, 109, 179, 108, 198, 107, 228, 113, 193, 104, 184, 126, 197, 102, 177, 101, 213, 100, 192, 105, 172, 97, 216, 96, 181, 106, 226, 94, 209, 114, 182, 89, 234, 88, 207, 110, 232, 99, 217, 85, 204, 84, 203, 83, 235, 82, 185, 81, 175, 98, 191, 92, 188, 103, 227, 72, 49, 86, 223, 87, 220, 42, 92, 38, 199, 63, 218, 33, 201, 11, 194, 10, 211, 7, 36, 31, 219],
	#[34, 367],
	#[32, 365, 13, 266],
	#[34, 366],
	#[],
	#[],
	#[34, 375, 13, 266],
	#[271, 73, 264, 371, 263, 109, 255, 262, 249, 55, 247, 56, 246, 61, 245, 362, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 32, 370, 31, 58, 11, 65, 7, 36],
	#[34, 374],
	#[32, 372, 13, 266],
	#[34, 373],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[13, 266, 12, 380],
	#[],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 169, 255, 382, 115, 9, 70, 71, 69, 67, 68, 63, 67, 60, 73, 69, 65, 70, 64, 72, 63, 62, 66, 57, 36, 64, 35, 66, 33, 59, 31, 58, 71, 54, 11, 65, 7, 36],
	#[13, 383],
	#[271, 73, 255, 384, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 385],
	#[271, 73, 255, 386, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[43, 387],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[42, 405],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[271, 73, 255, 406, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 63, 62, 64, 72, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 407],
	#[271, 73, 255, 408, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[43, 409],
	#[],
	#[],
	#[],
	#[271, 73, 255, 413, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 63, 62, 64, 72, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 414],
	#[271, 73, 255, 415, 249, 55, 247, 56, 245, 169, 246, 61, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 416],
	#[271, 73, 255, 417, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 70, 71, 71, 54, 36, 64, 33, 59, 31, 58, 11, 65, 35, 66, 7, 36],
	#[43, 418],
	#[],
	#[],
	#[271, 73, 255, 421, 249, 55, 247, 56, 246, 61, 245, 169, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 422],
	#[271, 73, 255, 423, 246, 61, 245, 169, 249, 55, 247, 56, 115, 9, 69, 67, 67, 60, 73, 69, 66, 57, 64, 72, 68, 63, 70, 71, 65, 70, 71, 54, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 63, 62, 7, 36],
	#[43, 424],
	#[],
	#[],
	#[],
	#[],
	#[13, 266, 12, 429],
	#[],
	#[],
	#[271, 73, 268, 434, 259, 435, 249, 55, 246, 61, 245, 436, 247, 56, 115, 9, 94, 432, 92, 433, 73, 437, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[312, 444, 221, 302],
	#[42, 439],
	#[273, 176, 271, 195, 260, 438, 116, 8, 115, 9, 63, 218, 10, 211, 7, 36],
	#[],
	#[248, 93, 72, 49, 42, 92],
	#[42, 74],
	#[],
	#[271, 73, 268, 434, 259, 440, 249, 55, 247, 56, 246, 61, 245, 436, 115, 9, 94, 432, 92, 433, 70, 71, 69, 67, 68, 63, 67, 60, 73, 437, 65, 70, 64, 72, 66, 57, 71, 54, 36, 64, 35, 66, 63, 62, 33, 59, 31, 58, 11, 65, 7, 36],
	#[93, 441],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 442, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[248, 93, 72, 49, 42, 92, 43, 443],
	#[],
	#[42, 445],
	#[271, 73, 249, 55, 246, 61, 247, 56, 245, 169, 255, 446, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 64, 72, 65, 70, 36, 64, 35, 66, 63, 62, 33, 59, 31, 58, 11, 65, 7, 36],
	#[280, 447],
	#[43, 448, 13, 308],
	#[],
	#[223, 450],
	#[31, 451],
	#[317, 455, 314, 454, 223, 452, 32, 453],
	#[38, 345],
	#[],
	#[32, 456, 13, 457],
	#[],
	#[],
	#[317, 458, 223, 452],
	#[],
	#[271, 73, 249, 463, 247, 460, 245, 466, 246, 61, 115, 9, 71, 54, 67, 60, 64, 72, 73, 69, 69, 67, 70, 71, 65, 70, 36, 462, 35, 465, 66, 57, 31, 461, 33, 467, 68, 63, 11, 464, 63, 62, 7, 36],
	#[],
	#[271, 73, 263, 484, 249, 55, 247, 56, 246, 61, 245, 102, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 32, 485, 31, 58, 11, 65, 7, 36],
	#[],
	#[],
	#[38, 480],
	#[],
	#[248, 478, 72, 49, 42, 92],
	#[38, 468, 31, 469],
	#[39, 475],
	#[271, 73, 263, 470, 249, 55, 247, 56, 246, 61, 245, 102, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 63, 62, 64, 72, 36, 64, 35, 66, 33, 59, 32, 471, 31, 58, 11, 65, 7, 36],
	#[32, 473, 13, 112],
	#[34, 472],
	#[],
	#[34, 474],
	#[],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 476, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[248, 93, 72, 49, 42, 92, 34, 477],
	#[],
	#[37, 479],
	#[],
	#[39, 481],
	#[271, 73, 249, 55, 247, 56, 245, 482, 246, 61, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[248, 93, 72, 49, 42, 92, 12, 483],
	#[],
	#[32, 486, 13, 112],
	#[],
	#[],
	#[],
	#[243, 541, 61, 542, 31, 540],
	#[],
	#[251, 491, 76, 41, 75, 43, 74, 42],
	#[240, 498, 237, 495, 56, 504, 55, 492, 54, 505, 53, 503, 52, 501, 51, 499, 50, 496, 49, 493, 48, 506, 46, 502, 45, 500, 44, 497, 47, 494],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[241, 508],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[38, 507],
	#[],
	#[],
	#[],
	#[271, 73, 268, 511, 249, 55, 242, 510, 247, 56, 245, 436, 246, 61, 115, 9, 73, 437, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 60, 512, 59, 514, 57, 509, 58, 513, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[],
	#[],
	#[10, 517, 273, 515, 239, 516, 116, 8],
	#[],
	#[],
	#[],
	#[],
	#[42, 518],
	#[],
	#[275, 519, 274, 520, 271, 73, 250, 522, 249, 77, 247, 56, 245, 80, 246, 75, 115, 9, 95, 521, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 536],
	#[43, 525],
	#[],
	#[309, 82, 297, 524, 271, 523, 220, 88, 219, 86, 218, 84, 217, 89, 115, 9, 106, 83, 105, 90, 90, 85, 60, 87, 7, 36],
	#[],
	#[],
	#[253, 526, 78, 48],
	#[276, 527],
	#[289, 121, 277, 528, 258, 529, 183, 133, 181, 132, 180, 138, 184, 137, 178, 131, 177, 128, 176, 125, 175, 122, 174, 139, 173, 135, 172, 140, 171, 129, 170, 126, 169, 136, 168, 141, 179, 134, 182, 130, 91, 157, 58, 123, 57, 127, 59, 124],
	#[278, 530, 90, 531],
	#[],
	#[279, 533, 117, 534],
	#[38, 532],
	#[],
	#[],
	#[7, 535],
	#[],
	#[271, 73, 250, 537, 249, 77, 247, 56, 246, 75, 245, 80, 115, 9, 95, 538, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[297, 539, 309, 82, 271, 523, 219, 86, 218, 84, 217, 89, 220, 88, 115, 9, 105, 90, 90, 85, 106, 83, 60, 87, 7, 36],
	#[],
	#[],
	#[286, 544, 244, 866, 269, 545, 159, 546],
	#[],
	#[286, 544, 269, 545, 244, 543, 159, 546],
	#[286, 544, 269, 864, 159, 546, 62, 865],
	#[287, 549, 272, 548, 270, 547, 271, 550, 115, 9, 7, 36, 63, 32],
	#[],
	#[],
	#[298, 863, 288, 862, 283, 579, 301, 571, 281, 553, 266, 573, 197, 558, 196, 576, 195, 556, 194, 585, 193, 559, 202, 584, 185, 563, 163, 565, 162, 574, 201, 586, 192, 580, 186, 578, 187, 566, 190, 569, 166, 567, 160, 557, 167, 562, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 189, 560, 129, 202, 127, 186, 124, 231, 123, 173, 122, 189, 120, 225, 130, 190, 118, 180, 138, 229, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 121, 200, 106, 226, 105, 172, 104, 184, 126, 197, 102, 552, 101, 572, 100, 561, 99, 582, 98, 555, 97, 581, 96, 554, 107, 228, 128, 215, 92, 188, 161, 570, 119, 230, 164, 568, 103, 577, 94, 575],
	#[301, 571, 298, 583, 288, 564, 283, 579, 281, 553, 266, 573, 201, 586, 197, 558, 196, 576, 195, 556, 194, 585, 192, 580, 202, 584, 190, 569, 189, 560, 185, 563, 193, 559, 186, 578, 167, 562, 166, 567, 164, 568, 163, 565, 161, 570, 160, 557, 162, 574, 138, 229, 137, 174, 187, 566, 134, 183, 133, 196, 136, 187, 130, 190, 129, 202, 127, 186, 126, 197, 124, 231, 121, 200, 114, 182, 128, 215, 112, 205, 122, 189, 120, 225, 109, 179, 108, 198, 107, 228, 113, 193, 105, 172, 104, 184, 103, 577, 102, 552, 101, 572, 100, 561, 99, 582, 98, 555, 123, 173, 96, 554, 118, 180, 94, 575, 92, 188, 119, 230, 110, 232, 106, 226, 97, 581, 111, 221, 135, 212],
	#[],
	#[6, 551],
	#[],
	#[290, 859, 271, 73, 249, 55, 247, 56, 245, 310, 246, 61, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 66, 57, 65, 70, 64, 72, 63, 62, 67, 60, 36, 64, 35, 66, 33, 59, 31, 58, 7, 36, 11, 65],
	#[246, 61, 245, 854, 249, 55, 247, 56, 271, 73, 115, 9, 33, 59, 65, 70, 67, 60, 11, 65, 64, 72, 36, 64, 7, 36, 66, 57, 73, 69, 35, 66, 63, 62, 31, 58, 69, 67, 68, 63, 70, 71, 71, 54],
	#[290, 848, 249, 55, 245, 310, 246, 61, 247, 56, 271, 73, 115, 9, 70, 71, 69, 67, 68, 63, 66, 57, 65, 70, 64, 72, 63, 62, 73, 69, 36, 64, 35, 66, 33, 59, 31, 58, 71, 54, 11, 65, 7, 36, 67, 60],
	#[285, 842, 158, 391, 157, 393, 156, 390, 155, 388, 153, 400, 152, 398, 151, 404, 150, 392, 149, 389, 148, 403, 145, 396, 154, 402, 146, 399, 147, 401, 88, 395, 87, 397],
	#[311, 823, 192, 588],
	#[290, 816, 291, 817, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 73, 818, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 71, 54, 36, 64, 33, 59, 31, 58, 70, 71, 35, 66, 7, 36, 11, 65],
	#[290, 812, 246, 61, 245, 310, 249, 55, 36, 64, 33, 59, 115, 9, 65, 70, 35, 66, 70, 71, 66, 57, 64, 72, 73, 69, 11, 65, 247, 56, 67, 60, 69, 67, 63, 62, 31, 58, 71, 54, 68, 63, 271, 73, 7, 36],
	#[311, 800, 192, 588, 191, 799],
	#[271, 73, 115, 9, 246, 61, 245, 793, 35, 66, 33, 59, 71, 54, 69, 67, 249, 55, 64, 72, 73, 69, 70, 71, 11, 65, 66, 57, 67, 60, 247, 56, 36, 64, 65, 70, 63, 62, 31, 58, 68, 63, 7, 36],
	#[290, 787, 271, 73, 249, 55, 245, 310, 247, 56, 246, 61, 115, 9, 73, 69, 71, 54, 70, 71, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 69, 67, 11, 65, 7, 36],
	#[290, 785, 246, 61, 245, 310, 249, 55, 247, 56, 271, 73, 115, 9, 11, 65, 36, 64, 71, 54, 33, 59, 70, 71, 7, 36, 63, 62, 35, 66, 65, 70, 66, 57, 64, 72, 69, 67, 73, 69, 67, 60, 68, 63, 31, 58],
	#[290, 781, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[],
	#[290, 768, 249, 55, 247, 56, 246, 61, 245, 310, 271, 73, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 33, 59, 35, 66, 11, 65, 7, 36, 31, 58],
	#[245, 757, 271, 73, 249, 55, 247, 56, 246, 61, 115, 9, 70, 71, 69, 67, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 73, 69, 36, 64, 35, 66, 33, 59, 31, 58, 68, 63, 11, 65, 7, 36, 71, 54],
	#[318, 756, 13, 605],
	#[240, 741, 54, 505, 53, 503, 48, 506, 56, 504, 45, 500, 50, 496, 49, 493, 44, 497, 52, 501, 46, 502, 55, 492, 47, 494, 51, 499],
	#[311, 733, 191, 732, 192, 588],
	#[69, 721, 36, 720],
	#[240, 699, 56, 504, 55, 492, 54, 505, 53, 503, 52, 501, 51, 499, 50, 496, 49, 493, 48, 506, 47, 494, 46, 502, 45, 500, 44, 497],
	#[290, 693, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 33, 59, 35, 66, 11, 65, 7, 36, 31, 58],
	#[290, 689, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[36, 668],
	#[312, 663, 221, 302],
	#[302, 660, 203, 596],
	#[290, 654, 249, 55, 247, 56, 246, 61, 245, 310, 271, 73, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 640, 71, 54, 69, 67, 67, 60, 66, 57, 65, 70, 64, 72, 115, 9, 73, 69, 70, 71, 36, 64, 35, 66, 63, 62, 33, 59, 31, 58, 68, 63, 11, 65, 7, 36],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 635, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 33, 59, 31, 58, 35, 66, 11, 65, 7, 36],
	#[193, 622, 190, 623],
	#[284, 616, 141, 281, 145, 275, 148, 282, 144, 274, 146, 277, 140, 278, 143, 273, 147, 280, 139, 276, 142, 283],
	#[290, 612, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 64, 72, 65, 70, 36, 64, 35, 66, 63, 62, 33, 59, 31, 58, 11, 65, 7, 36],
	#[],
	#[],
	#[311, 589, 192, 588],
	#[202, 587],
	#[],
	#[],
	#[290, 590, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 591],
	#[290, 592, 271, 73, 249, 55, 245, 310, 246, 61, 247, 56, 115, 9, 70, 71, 69, 67, 68, 63, 67, 60, 73, 69, 65, 70, 64, 72, 63, 62, 66, 57, 71, 54, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 593],
	#[249, 55, 245, 310, 271, 73, 290, 594, 246, 61, 247, 56, 115, 9, 73, 69, 71, 54, 70, 71, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 69, 67, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[302, 595, 203, 596],
	#[303, 603, 204, 602, 209, 601, 208, 600, 206, 597, 205, 599, 207, 598],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[318, 604, 13, 605],
	#[],
	#[319, 606, 222, 607],
	#[13, 609],
	#[317, 206, 316, 608, 223, 208],
	#[],
	#[222, 610],
	#[317, 206, 316, 611, 223, 208],
	#[],
	#[13, 613],
	#[290, 614, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 33, 59, 35, 66, 36, 64, 11, 65, 7, 36, 31, 58],
	#[318, 615, 13, 605],
	#[],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 617, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[317, 206, 316, 323, 305, 318, 292, 618, 283, 222, 281, 178, 273, 176, 266, 214, 265, 322, 260, 327, 248, 93, 271, 195, 223, 208, 138, 229, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 107, 228, 106, 226, 105, 172, 104, 184, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 97, 216, 96, 181, 94, 209, 92, 188, 89, 328, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 103, 227, 72, 49, 63, 218, 42, 92, 38, 315, 33, 320, 31, 325, 11, 317, 10, 211, 7, 36, 2, 326],
	#[13, 619],
	#[317, 206, 316, 323, 305, 318, 292, 620, 283, 222, 273, 176, 271, 195, 281, 178, 266, 214, 265, 322, 260, 327, 223, 208, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 130, 190, 129, 202, 128, 215, 138, 229, 126, 197, 124, 231, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 114, 182, 113, 193, 112, 205, 122, 189, 123, 173, 109, 179, 108, 198, 107, 228, 106, 226, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 100, 192, 99, 217, 127, 186, 97, 216, 96, 181, 94, 209, 92, 188, 89, 328, 88, 312, 98, 191, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 110, 232, 63, 218, 87, 319, 111, 221, 38, 315, 33, 320, 31, 325, 11, 317, 10, 211, 7, 36, 2, 326],
	#[318, 621, 13, 605],
	#[],
	#[290, 631, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 73, 69, 71, 54, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[290, 624, 249, 55, 245, 310, 271, 73, 247, 56, 246, 61, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 66, 57, 65, 70, 64, 72, 67, 60, 36, 64, 63, 62, 33, 59, 31, 58, 35, 66, 11, 65, 7, 36],
	#[318, 626, 313, 625, 13, 627],
	#[],
	#[],
	#[319, 606, 222, 607, 90, 628],
	#[38, 629],
	#[318, 630, 13, 605],
	#[],
	#[13, 632],
	#[290, 633, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 11, 65, 7, 36, 31, 58],
	#[318, 626, 313, 634, 13, 627],
	#[],
	#[316, 323, 317, 206, 292, 636, 305, 318, 283, 222, 281, 178, 265, 322, 260, 327, 273, 176, 266, 214, 248, 93, 223, 208, 271, 195, 138, 229, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 129, 202, 124, 231, 128, 215, 122, 189, 121, 200, 120, 225, 130, 190, 118, 180, 116, 8, 115, 9, 114, 182, 112, 205, 102, 177, 101, 213, 100, 192, 127, 186, 96, 181, 99, 217, 113, 193, 123, 173, 97, 216, 82, 314, 81, 311, 104, 184, 88, 312, 105, 172, 72, 49, 98, 191, 111, 221, 84, 321, 103, 227, 119, 230, 106, 226, 108, 198, 42, 92, 109, 179, 89, 328, 83, 329, 33, 320, 85, 316, 10, 211, 107, 228, 2, 326, 126, 197, 92, 188, 94, 209, 110, 232, 86, 324, 11, 317, 63, 218, 31, 325, 87, 319, 38, 315, 7, 36],
	#[13, 637],
	#[317, 206, 316, 323, 305, 318, 292, 638, 281, 178, 273, 176, 260, 327, 265, 322, 271, 195, 266, 214, 283, 222, 223, 208, 138, 229, 136, 187, 135, 212, 134, 183, 133, 196, 137, 174, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 130, 190, 118, 180, 116, 8, 115, 9, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 107, 228, 106, 226, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 97, 216, 96, 181, 94, 209, 92, 188, 119, 230, 89, 328, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 63, 218, 38, 315, 33, 320, 31, 325, 11, 317, 10, 211, 7, 36, 2, 326],
	#[318, 639, 13, 605],
	#[],
	#[310, 642, 248, 93, 72, 49, 42, 92, 11, 641],
	#[317, 206, 316, 323, 305, 318, 292, 650, 283, 222, 281, 178, 273, 176, 271, 195, 266, 214, 265, 322, 260, 327, 223, 208, 138, 229, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 107, 228, 106, 226, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 97, 216, 96, 181, 94, 209, 92, 188, 89, 328, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 63, 218, 38, 315, 33, 320, 31, 325, 11, 317, 10, 211, 7, 36, 2, 326],
	#[318, 643, 13, 644],
	#[],
	#[319, 606, 222, 607, 11, 645],
	#[316, 323, 305, 318, 292, 646, 283, 222, 281, 178, 273, 176, 271, 195, 266, 214, 265, 322, 317, 206, 260, 327, 223, 208, 137, 174, 133, 196, 130, 190, 129, 202, 128, 215, 127, 186, 136, 187, 124, 231, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 138, 229, 115, 9, 114, 182, 113, 193, 112, 205, 134, 183, 123, 173, 109, 179, 108, 198, 107, 228, 135, 212, 88, 312, 110, 232, 99, 217, 85, 316, 105, 172, 97, 216, 82, 314, 83, 329, 104, 184, 96, 181, 106, 226, 89, 328, 81, 311, 92, 188, 102, 177, 98, 191, 111, 221, 84, 321, 94, 209, 103, 227, 86, 324, 38, 315, 126, 197, 87, 319, 116, 8, 33, 320, 101, 213, 31, 325, 10, 211, 100, 192, 63, 218, 11, 317, 2, 326, 7, 36],
	#[13, 647],
	#[317, 206, 316, 323, 305, 318, 292, 648, 283, 222, 281, 178, 273, 176, 271, 195, 266, 214, 260, 327, 265, 322, 223, 208, 138, 229, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 107, 228, 106, 226, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 97, 216, 96, 181, 94, 209, 92, 188, 89, 328, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 63, 218, 38, 315, 33, 320, 31, 325, 11, 317, 10, 211, 7, 36, 2, 326],
	#[12, 649],
	#[],
	#[13, 651],
	#[317, 206, 316, 323, 305, 318, 292, 652, 283, 222, 281, 178, 273, 176, 271, 195, 266, 214, 265, 322, 260, 327, 223, 208, 138, 229, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 107, 228, 106, 226, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 97, 216, 96, 181, 94, 209, 92, 188, 89, 328, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 63, 218, 38, 315, 33, 320, 31, 325, 11, 317, 10, 211, 7, 36, 2, 326],
	#[12, 653],
	#[],
	#[13, 655],
	#[271, 73, 290, 656, 249, 55, 247, 56, 245, 310, 246, 61, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[267, 657, 13, 247],
	#[13, 659, 318, 658],
	#[],
	#[319, 606, 222, 607, 38, 251],
	#[303, 661, 209, 601, 207, 598, 206, 597, 205, 599, 208, 600, 204, 602],
	#[318, 662, 13, 605],
	#[],
	#[290, 664, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 70, 71, 69, 67, 68, 63, 67, 60, 65, 70, 64, 72, 73, 69, 66, 57, 71, 54, 36, 64, 35, 66, 63, 62, 33, 59, 31, 58, 11, 65, 7, 36],
	#[280, 665],
	#[318, 666, 13, 667],
	#[],
	#[319, 606, 290, 309, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 222, 607, 115, 9, 73, 69, 71, 54, 70, 71, 68, 63, 67, 60, 65, 70, 64, 72, 66, 57, 69, 67, 36, 64, 35, 66, 63, 62, 33, 59, 31, 58, 11, 65, 7, 36],
	#[317, 206, 316, 323, 305, 318, 292, 669, 283, 222, 281, 178, 273, 176, 271, 195, 265, 322, 260, 327, 266, 214, 223, 208, 138, 229, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 120, 225, 119, 230, 118, 180, 115, 9, 114, 182, 113, 193, 112, 205, 122, 189, 110, 232, 109, 179, 108, 198, 107, 228, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 97, 216, 96, 181, 106, 226, 94, 209, 121, 200, 92, 188, 89, 328, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 116, 8, 63, 218, 111, 221, 38, 315, 33, 320, 11, 317, 10, 211, 7, 36, 2, 326, 31, 325],
	#[13, 670],
	#[69, 671],
	#[317, 206, 316, 323, 305, 318, 292, 672, 283, 222, 281, 178, 273, 176, 271, 195, 266, 214, 265, 322, 260, 327, 223, 208, 138, 229, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 107, 228, 106, 226, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 97, 216, 96, 181, 94, 209, 92, 188, 89, 328, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 63, 218, 38, 315, 33, 320, 31, 325, 11, 317, 10, 211, 7, 36, 2, 326],
	#[11, 673],
	#[293, 675, 36, 674, 12, 676],
	#[317, 206, 316, 323, 305, 685, 283, 222, 281, 178, 266, 214, 265, 322, 223, 208, 137, 174, 133, 196, 136, 187, 130, 190, 129, 202, 128, 215, 138, 229, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 114, 182, 113, 193, 112, 205, 134, 183, 110, 232, 109, 179, 108, 198, 107, 228, 135, 212, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 97, 216, 96, 181, 94, 209, 92, 188, 89, 328, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 106, 226, 127, 186, 111, 221, 38, 315, 33, 320, 31, 325, 11, 317, 2, 326],
	#[36, 678, 12, 679],
	#[318, 677, 13, 605],
	#[],
	#[317, 206, 316, 323, 305, 681, 283, 222, 281, 178, 266, 214, 265, 322, 223, 208, 138, 229, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 129, 202, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 106, 226, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 130, 190, 96, 181, 94, 209, 128, 215, 92, 188, 97, 216, 89, 328, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 107, 228, 38, 315, 33, 320, 31, 325, 11, 317, 2, 326],
	#[318, 680, 13, 605],
	#[],
	#[13, 682],
	#[69, 683],
	#[316, 323, 305, 318, 292, 684, 317, 206, 281, 178, 273, 176, 283, 222, 266, 214, 265, 322, 260, 327, 271, 195, 223, 208, 138, 229, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 107, 228, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 121, 200, 99, 217, 98, 191, 97, 216, 96, 181, 106, 226, 94, 209, 92, 188, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 89, 328, 100, 192, 63, 218, 38, 315, 33, 320, 31, 325, 11, 317, 10, 211, 7, 36, 2, 326],
	#[],
	#[13, 686],
	#[69, 687],
	#[317, 206, 316, 323, 305, 318, 292, 688, 283, 222, 281, 178, 273, 176, 266, 214, 265, 322, 260, 327, 271, 195, 223, 208, 138, 229, 137, 174, 136, 187, 135, 212, 134, 183, 133, 196, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 107, 228, 106, 226, 105, 172, 104, 184, 103, 227, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 97, 216, 96, 181, 94, 209, 92, 188, 89, 328, 88, 312, 87, 319, 86, 324, 85, 316, 84, 321, 83, 329, 82, 314, 81, 311, 38, 315, 63, 218, 33, 320, 31, 325, 11, 317, 10, 211, 7, 36, 2, 326],
	#[],
	#[93, 690],
	#[271, 73, 245, 691, 249, 55, 246, 61, 247, 56, 70, 71, 69, 67, 68, 63, 67, 60, 65, 70, 64, 72, 115, 9, 73, 69, 66, 57, 36, 64, 35, 66, 63, 62, 33, 59, 31, 58, 71, 54, 11, 65, 7, 36],
	#[318, 692, 248, 93, 72, 49, 42, 92, 13, 605],
	#[],
	#[13, 694],
	#[290, 695, 249, 55, 246, 61, 247, 56, 115, 9, 71, 54, 67, 60, 66, 57, 65, 70, 64, 72, 33, 59, 35, 66, 7, 36, 73, 69, 68, 63, 36, 64, 31, 58, 245, 310, 11, 65, 69, 67, 70, 71, 271, 73, 63, 62],
	#[13, 696],
	#[290, 697, 246, 61, 247, 56, 249, 55, 245, 310, 271, 73, 115, 9, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 73, 69, 36, 64, 33, 59, 31, 58, 35, 66, 7, 36, 11, 65, 71, 54],
	#[318, 698, 13, 605],
	#[],
	#[241, 700],
	#[268, 701, 246, 61, 242, 510, 249, 55, 247, 56, 271, 73, 245, 436, 115, 9, 73, 437, 70, 71, 69, 67, 68, 63, 67, 60, 65, 70, 64, 72, 63, 62, 58, 513, 57, 509, 59, 514, 66, 57, 60, 512, 71, 54, 36, 64, 33, 59, 11, 65, 35, 66, 7, 36, 31, 58],
	#[317, 206, 316, 323, 305, 318, 292, 702, 283, 222, 281, 178, 273, 176, 265, 322, 260, 327, 266, 214, 223, 208, 271, 195, 137, 174, 135, 212, 134, 183, 133, 196, 129, 202, 128, 215, 127, 186, 136, 187, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 130, 190, 118, 180, 138, 229, 115, 9, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 105, 172, 104, 184, 126, 197, 102, 177, 101, 213, 100, 192, 99, 217, 97, 216, 106, 226, 94, 209, 92, 188, 96, 181, 119, 230, 89, 328, 98, 191, 86, 324, 85, 316, 84, 321, 88, 312, 82, 314, 81, 311, 103, 227, 83, 329, 107, 228, 116, 8, 63, 218, 87, 319, 38, 315, 31, 325, 33, 320, 10, 211, 2, 326, 11, 317, 7, 36],
	#[42, 703],
	#[295, 705, 271, 73, 250, 704, 249, 77, 247, 56, 246, 75, 245, 80, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 31, 58, 33, 59, 35, 66, 36, 64, 11, 65, 7, 36],
	#[317, 206, 309, 82, 305, 318, 316, 323, 292, 718, 283, 222, 281, 178, 218, 84, 265, 322, 219, 86, 266, 214, 273, 176, 217, 89, 137, 174, 135, 212, 129, 202, 127, 186, 136, 187, 120, 225, 109, 179, 130, 190, 113, 193, 104, 184, 98, 191, 121, 200, 85, 316, 82, 314, 126, 197, 88, 312, 105, 709, 92, 188, 81, 311, 118, 180, 114, 182, 97, 216, 100, 192, 110, 232, 112, 205, 96, 181, 123, 173, 106, 711, 115, 9, 99, 217, 90, 85, 122, 189, 134, 183, 38, 315, 119, 230, 31, 325, 128, 215, 33, 320, 111, 221, 108, 198, 133, 196, 83, 329, 86, 324, 84, 321, 116, 8, 89, 328, 107, 228, 10, 211, 87, 319, 94, 209, 2, 326, 220, 88, 11, 317, 101, 213, 60, 87, 138, 229, 103, 227, 124, 231, 223, 208, 271, 195, 260, 327, 7, 36, 63, 218, 102, 177],
	#[43, 706, 13, 707],
	#[276, 716],
	#[271, 73, 250, 708, 249, 77, 247, 56, 245, 80, 246, 75, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 33, 59, 35, 66, 11, 65, 7, 36, 31, 58],
	#[316, 323, 305, 318, 283, 222, 292, 710, 309, 82, 273, 176, 265, 322, 281, 178, 266, 214, 260, 327, 271, 195, 223, 208, 317, 206, 220, 88, 217, 89, 218, 84, 219, 86, 133, 196, 130, 190, 88, 312, 121, 200, 114, 182, 113, 193, 105, 709, 128, 215, 126, 197, 112, 205, 102, 177, 118, 180, 134, 183, 137, 174, 104, 184, 81, 311, 11, 317, 129, 202, 33, 320, 82, 314, 2, 326, 97, 216, 127, 186, 109, 179, 7, 36, 122, 189, 85, 316, 31, 325, 84, 321, 100, 192, 136, 187, 98, 191, 101, 213, 115, 9, 63, 218, 96, 181, 120, 225, 92, 188, 87, 319, 124, 231, 86, 324, 111, 221, 38, 315, 135, 212, 10, 211, 103, 227, 108, 198, 94, 209, 99, 217, 123, 173, 83, 329, 106, 711, 90, 85, 60, 87, 89, 328, 138, 229, 107, 228, 116, 8, 110, 232, 119, 230],
	#[],
	#[296, 712],
	#[],
	#[308, 713, 307, 715, 309, 714, 220, 88, 219, 86, 218, 84, 217, 89, 106, 83, 90, 85, 105, 90, 60, 87, 59, 149, 58, 148, 57, 151],
	#[],
	#[],
	#[],
	#[184, 137, 183, 133, 182, 130, 181, 132, 180, 138, 179, 134, 178, 131, 177, 128, 176, 125, 175, 122, 174, 139, 173, 135, 172, 140, 171, 129, 170, 126, 168, 141, 169, 136, 59, 124, 58, 123, 57, 127, 13, 605, 318, 717, 289, 121],
	#[],
	#[296, 719],
	#[308, 713, 307, 715, 309, 714, 219, 86, 218, 84, 217, 89, 220, 88, 106, 83, 105, 90, 90, 85, 60, 87, 59, 149, 57, 151, 58, 148],
	#[316, 323, 305, 318, 292, 724, 283, 222, 271, 195, 281, 178, 265, 322, 317, 206, 273, 176, 266, 214, 260, 327, 223, 208, 133, 196, 114, 182, 136, 187, 104, 184, 105, 172, 137, 174, 98, 191, 123, 173, 88, 312, 82, 314, 126, 197, 121, 200, 122, 189, 112, 205, 96, 181, 92, 188, 118, 180, 120, 225, 127, 186, 128, 215, 81, 311, 85, 316, 134, 183, 130, 190, 115, 9, 97, 216, 138, 229, 99, 217, 129, 202, 87, 319, 113, 193, 102, 177, 111, 221, 106, 226, 84, 321, 100, 192, 94, 209, 89, 328, 107, 228, 108, 198, 38, 315, 63, 218, 109, 179, 119, 230, 31, 325, 33, 320, 101, 213, 124, 231, 86, 324, 103, 227, 116, 8, 7, 36, 2, 326, 10, 211, 110, 232, 83, 329, 135, 212, 11, 317],
	#[317, 206, 316, 323, 305, 318, 292, 722, 283, 222, 281, 178, 273, 176, 271, 195, 265, 322, 260, 327, 266, 214, 223, 208, 138, 229, 137, 174, 135, 212, 134, 183, 133, 196, 136, 187, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 101, 213, 100, 192, 104, 184, 105, 172, 112, 205, 96, 181, 106, 226, 94, 209, 92, 188, 109, 179, 89, 328, 98, 191, 86, 324, 85, 316, 84, 321, 88, 312, 82, 314, 81, 311, 102, 177, 113, 193, 108, 198, 114, 182, 97, 216, 107, 228, 110, 232, 63, 218, 99, 217, 87, 319, 83, 329, 103, 227, 38, 315, 33, 320, 31, 325, 111, 221, 11, 317, 10, 211, 7, 36, 2, 326],
	#[318, 723, 13, 605],
	#[],
	#[13, 725],
	#[69, 726],
	#[317, 206, 316, 323, 305, 318, 283, 222, 281, 178, 292, 727, 273, 176, 271, 195, 266, 214, 265, 322, 260, 327, 223, 208, 138, 229, 137, 174, 135, 212, 134, 183, 133, 196, 136, 187, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 116, 8, 115, 9, 112, 205, 101, 213, 100, 192, 104, 184, 105, 172, 97, 216, 96, 181, 94, 209, 92, 188, 108, 198, 109, 179, 89, 328, 88, 312, 98, 191, 86, 324, 81, 311, 102, 177, 113, 193, 103, 227, 114, 182, 107, 228, 84, 321, 85, 316, 110, 232, 63, 218, 106, 226, 82, 314, 87, 319, 83, 329, 111, 221, 118, 180, 38, 315, 99, 217, 33, 320, 31, 325, 11, 317, 10, 211, 7, 36, 2, 326],
	#[13, 728],
	#[69, 729],
	#[317, 206, 316, 323, 305, 318, 292, 730, 283, 222, 281, 178, 273, 176, 271, 195, 266, 214, 265, 322, 260, 327, 223, 208, 138, 229, 136, 187, 126, 197, 124, 231, 123, 173, 122, 189, 120, 225, 110, 232, 137, 174, 130, 190, 121, 200, 113, 193, 105, 172, 104, 184, 114, 182, 115, 9, 129, 202, 127, 186, 96, 181, 94, 209, 128, 215, 92, 188, 108, 198, 109, 179, 89, 328, 134, 183, 98, 191, 86, 324, 85, 316, 82, 314, 88, 312, 112, 205, 133, 196, 99, 217, 83, 329, 97, 216, 107, 228, 81, 311, 100, 192, 116, 8, 101, 213, 106, 226, 119, 230, 87, 319, 102, 177, 84, 321, 118, 180, 135, 212, 103, 227, 38, 315, 63, 218, 33, 320, 31, 325, 111, 221, 10, 211, 11, 317, 2, 326, 7, 36],
	#[13, 605, 318, 731],
	#[],
	#[311, 736, 192, 588],
	#[290, 734, 246, 61, 249, 55, 247, 56, 245, 310, 271, 73, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 66, 57, 64, 72, 65, 70, 67, 60, 36, 64, 63, 62, 33, 59, 35, 66, 68, 63, 11, 65, 7, 36, 31, 58],
	#[313, 735, 318, 626, 13, 627],
	#[],
	#[271, 73, 290, 737, 249, 55, 247, 56, 245, 310, 246, 61, 115, 9, 70, 71, 69, 67, 68, 63, 66, 57, 65, 70, 64, 72, 73, 69, 63, 62, 31, 58, 33, 59, 67, 60, 35, 66, 36, 64, 11, 65, 71, 54, 7, 36],
	#[302, 738, 203, 596],
	#[303, 739, 208, 600, 207, 598, 205, 599, 204, 602, 209, 601, 206, 597],
	#[318, 626, 313, 740, 13, 627],
	#[],
	#[241, 742],
	#[268, 743, 246, 61, 249, 55, 242, 510, 271, 73, 245, 436, 247, 56, 115, 9, 73, 437, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 60, 512, 58, 513, 57, 509, 59, 514, 36, 64, 35, 66, 63, 62, 31, 58, 33, 59, 11, 65, 7, 36],
	#[317, 206, 316, 323, 305, 318, 292, 744, 283, 222, 281, 178, 273, 176, 265, 322, 260, 327, 266, 214, 223, 208, 271, 195, 137, 174, 135, 212, 134, 183, 133, 196, 129, 202, 128, 215, 127, 186, 136, 187, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 130, 190, 118, 180, 138, 229, 115, 9, 114, 182, 113, 193, 112, 205, 111, 221, 110, 232, 109, 179, 108, 198, 107, 228, 106, 226, 105, 172, 104, 184, 126, 197, 102, 177, 101, 213, 100, 192, 99, 217, 98, 191, 97, 216, 96, 181, 94, 209, 92, 188, 119, 230, 89, 328, 87, 319, 86, 324, 85, 316, 84, 321, 88, 312, 82, 314, 81, 311, 103, 227, 83, 329, 63, 218, 38, 315, 33, 320, 31, 325, 116, 8, 11, 317, 10, 211, 7, 36, 2, 326],
	#[42, 745],
	#[295, 746, 271, 73, 250, 704, 247, 56, 246, 75, 245, 80, 249, 77, 115, 9, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 73, 69, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[13, 707, 43, 747],
	#[276, 748],
	#[289, 121, 173, 135, 177, 128, 181, 132, 176, 125, 169, 136, 183, 133, 180, 138, 184, 137, 170, 126, 178, 131, 179, 134, 57, 127, 172, 140, 168, 141, 171, 129, 58, 123, 59, 124, 93, 749, 174, 139, 182, 130, 175, 122],
	#[69, 750],
	#[316, 323, 305, 318, 317, 206, 292, 751, 283, 222, 281, 178, 273, 176, 271, 195, 266, 214, 265, 322, 260, 327, 223, 208, 138, 229, 137, 174, 135, 212, 134, 183, 136, 187, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 121, 200, 116, 8, 115, 9, 102, 177, 101, 213, 122, 189, 105, 172, 112, 205, 96, 181, 118, 180, 104, 184, 92, 188, 89, 328, 98, 191, 86, 324, 85, 316, 84, 321, 88, 312, 82, 314, 81, 311, 120, 225, 113, 193, 108, 198, 114, 182, 97, 216, 107, 228, 100, 192, 110, 232, 63, 218, 119, 230, 99, 217, 87, 319, 133, 196, 103, 227, 83, 329, 106, 226, 38, 315, 31, 325, 33, 320, 109, 179, 11, 317, 10, 211, 7, 36, 111, 221, 2, 326, 94, 209],
	#[165, 752],
	#[69, 753],
	#[281, 178, 104, 184, 121, 200, 105, 172, 137, 174, 81, 311, 273, 176, 88, 312, 122, 189, 96, 181, 92, 188, 123, 173, 82, 314, 114, 182, 112, 205, 136, 187, 133, 196, 98, 191, 130, 190, 113, 193, 85, 316, 126, 197, 102, 177, 292, 754, 11, 317, 33, 320, 305, 318, 127, 186, 100, 192, 118, 180, 94, 209, 7, 36, 10, 211, 109, 179, 129, 202, 106, 226, 134, 183, 108, 198, 87, 319, 120, 225, 38, 315, 101, 213, 111, 221, 84, 321, 265, 322, 128, 215, 135, 212, 115, 9, 103, 227, 97, 216, 317, 206, 83, 329, 99, 217, 316, 323, 107, 228, 124, 231, 266, 214, 89, 328, 223, 208, 138, 229, 110, 232, 86, 324, 271, 195, 260, 327, 283, 222, 63, 218, 31, 325, 2, 326, 116, 8, 119, 230],
	#[318, 755, 13, 605],
	#[],
	#[],
	#[248, 93, 188, 758, 72, 49, 42, 92],
	#[290, 759, 271, 73, 249, 55, 247, 56, 245, 310, 246, 61, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 35, 66, 33, 59, 11, 65, 7, 36, 31, 58],
	#[299, 760, 198, 761],
	#[300, 762],
	#[],
	#[318, 764, 200, 765, 199, 763, 13, 605],
	#[290, 767, 249, 55, 245, 310, 69, 67, 64, 72, 246, 61, 68, 63, 115, 9, 66, 57, 71, 54, 11, 65, 35, 66, 33, 59, 247, 56, 36, 64, 73, 69, 65, 70, 70, 71, 271, 73, 67, 60, 7, 36, 63, 62, 31, 58],
	#[],
	#[271, 73, 249, 55, 247, 56, 246, 61, 245, 169, 255, 766, 115, 9, 73, 69, 71, 54, 70, 71, 68, 63, 67, 60, 65, 70, 64, 72, 63, 62, 66, 57, 35, 66, 31, 58, 33, 59, 11, 65, 36, 64, 7, 36, 69, 67],
	#[],
	#[],
	#[13, 769],
	#[11, 770],
	#[294, 771, 69, 772, 12, 773],
	#[12, 777, 13, 776],
	#[317, 206, 305, 318, 292, 775, 283, 222, 316, 323, 281, 178, 273, 176, 271, 195, 266, 214, 265, 322, 260, 327, 223, 208, 138, 229, 137, 174, 135, 212, 134, 183, 133, 196, 136, 187, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 116, 8, 115, 9, 102, 177, 101, 213, 100, 192, 104, 184, 105, 172, 112, 205, 96, 181, 106, 226, 94, 209, 92, 188, 109, 179, 89, 328, 98, 191, 86, 324, 85, 316, 84, 321, 88, 312, 82, 314, 81, 311, 113, 193, 108, 198, 114, 182, 97, 216, 107, 228, 110, 232, 63, 218, 99, 217, 87, 319, 83, 329, 111, 221, 103, 227, 38, 315, 33, 320, 31, 325, 10, 211, 2, 326, 11, 317, 7, 36],
	#[318, 774, 13, 605],
	#[],
	#[],
	#[69, 779],
	#[318, 778, 13, 605],
	#[],
	#[317, 206, 316, 323, 305, 318, 292, 780, 283, 222, 266, 214, 265, 322, 273, 176, 137, 174, 281, 178, 136, 187, 130, 190, 118, 180, 138, 229, 129, 202, 115, 9, 101, 213, 104, 184, 105, 172, 123, 173, 96, 181, 133, 196, 94, 209, 121, 200, 109, 179, 86, 324, 85, 316, 84, 321, 88, 312, 82, 314, 81, 311, 126, 197, 102, 177, 122, 189, 112, 205, 108, 198, 114, 182, 127, 186, 271, 195, 100, 192, 113, 193, 119, 230, 120, 225, 92, 188, 83, 329, 124, 231, 128, 215, 223, 208, 103, 227, 134, 183, 97, 216, 106, 226, 260, 327, 38, 315, 63, 218, 98, 191, 31, 325, 33, 320, 111, 221, 89, 328, 11, 317, 110, 232, 2, 326, 99, 217, 10, 211, 7, 36, 116, 8, 87, 319, 107, 228, 135, 212],
	#[],
	#[13, 782],
	#[249, 55, 247, 56, 245, 783, 246, 61, 271, 73, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 33, 59, 35, 66, 36, 64, 11, 65, 7, 36, 31, 58],
	#[318, 784, 248, 93, 72, 49, 42, 92, 13, 605],
	#[],
	#[318, 786, 13, 605],
	#[],
	#[13, 788],
	#[290, 789, 246, 61, 249, 55, 271, 73, 245, 310, 247, 56, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 63, 62, 64, 72, 36, 64, 35, 66, 33, 59, 31, 58, 7, 36, 11, 65],
	#[13, 790],
	#[271, 73, 290, 791, 249, 55, 247, 56, 245, 310, 246, 61, 115, 9, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 73, 69, 68, 63, 69, 67, 70, 71, 71, 54, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[318, 792, 13, 605],
	#[],
	#[318, 626, 313, 794, 248, 93, 72, 49, 42, 92, 13, 795],
	#[],
	#[319, 606, 222, 607, 90, 628, 36, 796],
	#[305, 318, 317, 206, 283, 222, 281, 178, 292, 797, 260, 327, 266, 214, 273, 176, 265, 322, 271, 195, 316, 323, 223, 208, 137, 174, 133, 196, 136, 187, 130, 190, 129, 202, 128, 215, 127, 186, 126, 197, 124, 231, 123, 173, 122, 189, 121, 200, 120, 225, 119, 230, 118, 180, 138, 229, 115, 9, 101, 213, 100, 192, 104, 184, 105, 172, 96, 181, 86, 324, 85, 316, 84, 321, 88, 312, 82, 314, 81, 311, 102, 177, 112, 205, 108, 198, 107, 228, 113, 193, 63, 218, 97, 216, 92, 188, 98, 191, 111, 221, 109, 179, 114, 182, 103, 227, 106, 226, 38, 315, 99, 217, 33, 320, 94, 209, 134, 183, 110, 232, 116, 8, 11, 317, 7, 36, 89, 328, 31, 325, 87, 319, 10, 211, 135, 212, 83, 329, 2, 326],
	#[318, 626, 313, 798, 13, 627],
	#[],
	#[311, 805, 192, 588],
	#[290, 801, 249, 55, 245, 310, 246, 61, 247, 56, 271, 73, 73, 69, 70, 71, 64, 72, 68, 63, 66, 57, 71, 54, 115, 9, 36, 64, 33, 59, 31, 58, 65, 70, 35, 66, 7, 36, 63, 62, 67, 60, 11, 65, 69, 67],
	#[13, 802],
	#[290, 803, 246, 61, 249, 55, 247, 56, 245, 310, 271, 73, 65, 70, 73, 69, 70, 71, 66, 57, 115, 9, 35, 66, 33, 59, 31, 58, 67, 60, 64, 72, 68, 63, 36, 64, 11, 65, 71, 54, 69, 67, 7, 36, 63, 62],
	#[13, 627, 318, 626, 313, 804],
	#[],
	#[290, 806, 271, 73, 249, 55, 246, 61, 247, 56, 245, 310, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 66, 57, 65, 70, 64, 72, 36, 64, 63, 62, 33, 59, 35, 66, 11, 65, 7, 36, 67, 60, 31, 58],
	#[13, 807],
	#[249, 55, 245, 310, 290, 808, 246, 61, 271, 73, 247, 56, 115, 9, 73, 69, 71, 54, 69, 67, 68, 63, 66, 57, 65, 70, 67, 60, 64, 72, 36, 64, 63, 62, 33, 59, 70, 71, 11, 65, 31, 58, 35, 66, 7, 36],
	#[302, 809, 203, 596],
	#[208, 600, 209, 601, 205, 599, 207, 598, 206, 597, 303, 810, 204, 602],
	#[313, 811, 318, 626, 13, 627],
	#[],
	#[13, 813],
	#[38, 814],
	#[318, 815, 13, 605],
	#[],
	#[],
	#[318, 820, 13, 821],
	#[318, 819, 42, 74, 13, 605],
	#[],
	#[],
	#[271, 73, 290, 822, 319, 606, 247, 56, 246, 61, 249, 55, 245, 310, 73, 69, 69, 67, 64, 72, 115, 9, 70, 71, 65, 70, 35, 66, 31, 58, 33, 59, 71, 54, 7, 36, 36, 64, 68, 63, 67, 60, 11, 65, 66, 57, 63, 62, 222, 607],
	#[],
	#[304, 834, 215, 833, 214, 832, 212, 825, 210, 835, 211, 827, 213, 830, 136, 824, 120, 829, 138, 831, 118, 826, 137, 828],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[290, 836, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 73, 69, 71, 54, 36, 64, 35, 66, 33, 59, 31, 58, 11, 65, 7, 36],
	#[],
	#[13, 837],
	#[290, 838, 249, 55, 247, 56, 246, 61, 245, 310, 271, 73, 115, 9, 73, 69, 71, 54, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 33, 59, 35, 66, 36, 64, 11, 65, 31, 58, 7, 36],
	#[302, 839, 203, 596],
	#[303, 840, 209, 601, 207, 598, 205, 599, 204, 602, 208, 600, 206, 597],
	#[318, 841, 13, 605],
	#[],
	#[249, 55, 247, 56, 245, 843, 246, 61, 271, 73, 115, 9, 73, 69, 70, 71, 68, 63, 67, 60, 66, 57, 64, 72, 69, 67, 71, 54, 65, 70, 63, 62, 33, 59, 35, 66, 7, 36, 31, 58, 11, 65, 36, 64],
	#[317, 206, 316, 323, 305, 318, 292, 844, 283, 222, 281, 178, 273, 176, 265, 322, 248, 93, 266, 214, 260, 327, 271, 195, 223, 208, 138, 229, 136, 187, 126, 197, 124, 231, 120, 225, 123, 173, 137, 174, 130, 190, 113, 193, 105, 172, 115, 9, 129, 202, 98, 191, 96, 181, 104, 184, 92, 188, 108, 198, 109, 179, 122, 189, 134, 183, 87, 319, 85, 316, 82, 314, 88, 312, 133, 196, 81, 311, 128, 215, 72, 49, 114, 182, 127, 186, 107, 228, 84, 321, 121, 200, 110, 232, 112, 205, 119, 230, 99, 217, 102, 177, 89, 328, 111, 221, 106, 226, 118, 180, 94, 209, 103, 227, 101, 213, 42, 92, 86, 324, 38, 315, 97, 216, 33, 320, 100, 192, 11, 317, 83, 329, 7, 36, 116, 8, 2, 326, 10, 211, 135, 212, 63, 218, 31, 325],
	#[13, 845],
	#[316, 323, 305, 318, 317, 206, 292, 846, 283, 222, 281, 178, 273, 176, 265, 322, 260, 327, 266, 214, 271, 195, 223, 208, 133, 196, 136, 187, 137, 174, 98, 191, 85, 316, 126, 197, 92, 188, 81, 311, 88, 312, 105, 172, 127, 186, 122, 189, 130, 190, 82, 314, 123, 173, 113, 193, 96, 181, 104, 184, 109, 179, 118, 180, 112, 205, 129, 202, 121, 200, 128, 215, 134, 183, 108, 198, 114, 182, 120, 225, 97, 216, 33, 320, 100, 192, 31, 325, 138, 229, 89, 328, 94, 209, 115, 9, 124, 231, 99, 217, 84, 321, 116, 8, 7, 36, 63, 218, 107, 228, 2, 326, 87, 319, 10, 211, 11, 317, 106, 226, 111, 221, 83, 329, 119, 230, 86, 324, 101, 213, 102, 177, 135, 212, 110, 232, 103, 227, 38, 315],
	#[318, 847, 13, 605],
	#[],
	#[13, 849],
	#[290, 850, 271, 73, 249, 55, 247, 56, 246, 61, 245, 310, 115, 9, 71, 54, 70, 71, 66, 57, 64, 72, 65, 70, 69, 67, 73, 69, 67, 60, 36, 64, 31, 58, 33, 59, 35, 66, 68, 63, 11, 65, 7, 36, 63, 62],
	#[13, 851],
	#[290, 852, 249, 55, 247, 56, 246, 61, 271, 73, 245, 310, 115, 9, 73, 69, 71, 54, 70, 71, 67, 60, 66, 57, 65, 70, 64, 72, 68, 63, 69, 67, 35, 66, 33, 59, 7, 36, 63, 62, 31, 58, 36, 64, 11, 65],
	#[318, 853, 13, 605],
	#[],
	#[292, 855, 273, 176, 281, 178, 266, 214, 265, 322, 305, 318, 316, 323, 283, 222, 271, 195, 248, 93, 260, 327, 223, 208, 317, 206, 88, 312, 72, 49, 123, 173, 137, 174, 122, 189, 113, 193, 105, 172, 109, 179, 129, 202, 104, 184, 33, 320, 87, 319, 96, 181, 121, 200, 108, 198, 98, 191, 130, 190, 136, 187, 85, 316, 82, 314, 100, 192, 2, 326, 134, 183, 110, 232, 10, 211, 42, 92, 38, 315, 97, 216, 114, 182, 112, 205, 120, 225, 118, 180, 94, 209, 107, 228, 89, 328, 81, 311, 128, 215, 103, 227, 86, 324, 99, 217, 106, 226, 138, 229, 127, 186, 126, 197, 102, 177, 7, 36, 11, 317, 119, 230, 31, 325, 92, 188, 133, 196, 111, 221, 115, 9, 124, 231, 63, 218, 84, 321, 116, 8, 135, 212, 101, 213, 83, 329],
	#[13, 856],
	#[317, 206, 316, 323, 305, 318, 273, 176, 292, 857, 271, 195, 265, 322, 283, 222, 266, 214, 281, 178, 223, 208, 260, 327, 130, 190, 114, 182, 136, 187, 105, 172, 123, 173, 82, 314, 104, 184, 112, 205, 96, 181, 92, 188, 137, 174, 88, 312, 109, 179, 120, 225, 128, 215, 81, 311, 113, 193, 115, 9, 138, 229, 129, 202, 122, 189, 98, 191, 111, 221, 121, 200, 94, 209, 85, 316, 103, 227, 101, 213, 38, 315, 108, 198, 124, 231, 134, 183, 97, 216, 33, 320, 118, 180, 100, 192, 116, 8, 99, 217, 83, 329, 2, 326, 135, 212, 126, 197, 102, 177, 11, 317, 89, 328, 127, 186, 10, 211, 106, 226, 31, 325, 110, 232, 87, 319, 84, 321, 107, 228, 63, 218, 133, 196, 86, 324, 7, 36, 119, 230],
	#[318, 858, 13, 605],
	#[],
	#[267, 860, 13, 247],
	#[318, 861, 13, 659],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[286, 544, 269, 864, 159, 546, 32, 867],
	#[],
	#[229, 869, 11, 870],
	#[],
	#[230, 872, 7, 871],
	#[],
	#[13, 873, 12, 874],
	#[7, 875],
	#[],
	#[],
	#[6, 880],
	#[6, 878],
	#[7, 879],
	#[],
	#[7, 881],
	#[],
	#[],
	#[6, 884],
	#[271, 73, 249, 55, 247, 56, 245, 885, 246, 61, 115, 9, 73, 69, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 63, 62, 36, 64, 33, 59, 31, 58, 7, 36, 35, 66, 11, 65, 70, 71, 71, 54],
	#[248, 93, 72, 49, 42, 92, 223, 886],
	#[31, 887],
	#[315, 888, 255, 348, 249, 55, 247, 56, 245, 169, 115, 9, 73, 69, 82, 349, 70, 71, 69, 67, 68, 63, 67, 60, 66, 57, 63, 62, 32, 889, 33, 59, 64, 72, 11, 65, 65, 70, 35, 66, 71, 54, 7, 36, 36, 64, 31, 58, 271, 73, 246, 61],
	#[32, 890, 13, 353],
	#[],
	#[],
	#[],
	#[252, 893, 77, 46],
	#[254, 894, 79, 52, 80, 53],
	#[271, 73, 249, 55, 246, 61, 245, 895, 247, 56, 68, 63, 65, 70, 67, 60, 70, 71, 31, 58, 33, 59, 115, 9, 66, 57, 35, 66, 73, 69, 7, 36, 63, 62, 36, 64, 64, 72, 11, 65, 69, 67, 71, 54],
	#[256, 896, 248, 93, 72, 49, 42, 92, 13, 155],
	#[],
	#[227, 898, 7, 899],
	#[],
	#[],
	#[252, 901, 77, 46],
	#[254, 902, 80, 53, 79, 52],
	#[246, 61, 249, 55, 247, 56, 255, 903, 271, 73, 245, 169, 73, 69, 71, 54, 69, 67, 68, 63, 67, 60, 66, 57, 65, 70, 64, 72, 115, 9, 36, 64, 70, 71, 33, 59, 63, 62, 11, 65, 7, 36, 31, 58, 35, 66],
	#[256, 904, 13, 155],
	#[],
	#[],
	#[],
	#[],
	#[],
	#[74, 42, 75, 43, 76, 41, 251, 910],
	#[240, 498, 237, 911, 56, 504, 55, 492, 54, 505, 53, 503, 52, 501, 51, 499, 50, 496, 48, 506, 47, 494, 46, 502, 49, 493, 45, 500, 44, 497],
	#[]],
  action-function-table:
	 vector(llvm-parser-action0,
		llvm-parser-action1,
		llvm-parser-action0,
		llvm-parser-action3,
		llvm-parser-action4,
		llvm-parser-action5,
		llvm-parser-action6,
		llvm-parser-action7,
		llvm-parser-action8,
		llvm-parser-action9,
		llvm-parser-action10,
		llvm-parser-action11,
		llvm-parser-action12,
		llvm-parser-action13,
		llvm-parser-action13,
		llvm-parser-action15,
		llvm-parser-action16,
		llvm-parser-action17,
		llvm-parser-action18,
		llvm-parser-action19,
		llvm-parser-action20,
		llvm-parser-action21,
		llvm-parser-action22,
		llvm-parser-action23,
		llvm-parser-action24,
		llvm-parser-action25,
		llvm-parser-action26,
		llvm-parser-action27,
		llvm-parser-action28,
		llvm-parser-action29,
		llvm-parser-action30,
		llvm-parser-action31,
		llvm-parser-action32,
		llvm-parser-action33,
		llvm-parser-action34,
		llvm-parser-action35,
		llvm-parser-action36,
		llvm-parser-action37,
		llvm-parser-action38,
		llvm-parser-action39,
		llvm-parser-action40,
		llvm-parser-action41,
		llvm-parser-action42,
		llvm-parser-action43,
		llvm-parser-action44,
		llvm-parser-action45,
		llvm-parser-action46,
		llvm-parser-action13,
		llvm-parser-action12,
		llvm-parser-action0,
		llvm-parser-action0,
		llvm-parser-action51,
		llvm-parser-action52,
		llvm-parser-action53,
		llvm-parser-action54,
		llvm-parser-action12,
		llvm-parser-action12,
		llvm-parser-action12,
		llvm-parser-action12,
		llvm-parser-action59,
		llvm-parser-action60,
		llvm-parser-action61,
		llvm-parser-action62,
		llvm-parser-action63,
		llvm-parser-action64,
		llvm-parser-action65,
		llvm-parser-action66,
		llvm-parser-action67,
		llvm-parser-action68,
		llvm-parser-action69,
		llvm-parser-action70,
		llvm-parser-action71,
		llvm-parser-action72,
		llvm-parser-action73,
		llvm-parser-action74,
		llvm-parser-action75,
		llvm-parser-action76,
		llvm-parser-action77,
		llvm-parser-action78,
		llvm-parser-action79,
		llvm-parser-action79,
		llvm-parser-action81,
		llvm-parser-action82,
		llvm-parser-action12,
		llvm-parser-action12,
		llvm-parser-action85,
		llvm-parser-action86,
		llvm-parser-action87,
		llvm-parser-action88,
		llvm-parser-action89,
		llvm-parser-action90,
		llvm-parser-action91,
		llvm-parser-action92,
		llvm-parser-action93,
		llvm-parser-action94,
		llvm-parser-action95,
		llvm-parser-action96,
		llvm-parser-action97,
		llvm-parser-action98,
		llvm-parser-action99,
		llvm-parser-action100,
		llvm-parser-action101,
		llvm-parser-action102,
		llvm-parser-action103,
		llvm-parser-action104,
		llvm-parser-action105,
		llvm-parser-action106,
		llvm-parser-action107,
		llvm-parser-action108,
		llvm-parser-action109,
		llvm-parser-action109,
		llvm-parser-action111,
		llvm-parser-action112,
		llvm-parser-action113,
		llvm-parser-action114,
		llvm-parser-action115,
		llvm-parser-action116,
		llvm-parser-action1,
		llvm-parser-action118,
		llvm-parser-action1,
		llvm-parser-action118,
		llvm-parser-action121,
		llvm-parser-action122,
		llvm-parser-action123,
		llvm-parser-action124,
		llvm-parser-action125,
		llvm-parser-action123,
		llvm-parser-action123,
		llvm-parser-action124,
		llvm-parser-action129,
		llvm-parser-action130,
		llvm-parser-action131,
		llvm-parser-action132,
		llvm-parser-action133,
		llvm-parser-action133,
		llvm-parser-action131,
		llvm-parser-action136,
		llvm-parser-action136,
		llvm-parser-action138,
		llvm-parser-action139,
		llvm-parser-action140,
		llvm-parser-action141,
		llvm-parser-action133,
		llvm-parser-action143,
		llvm-parser-action144,
		llvm-parser-action145,
		llvm-parser-action146,
		llvm-parser-action72,
		llvm-parser-action133,
		llvm-parser-action149,
		llvm-parser-action150,
		llvm-parser-action151,
		llvm-parser-action152,
		llvm-parser-action153,
		llvm-parser-action154,
		llvm-parser-action155,
		llvm-parser-action156,
		llvm-parser-action157,
		llvm-parser-action158,
		llvm-parser-action17,
		llvm-parser-action18,
		llvm-parser-action17,
		llvm-parser-action18,
		llvm-parser-action17,
		llvm-parser-action18,
		llvm-parser-action165,
		llvm-parser-action150,
		llvm-parser-action167,
		llvm-parser-action168,
		llvm-parser-action169,
		llvm-parser-action170,
		llvm-parser-action171,
		llvm-parser-action172,
		llvm-parser-action173,
		llvm-parser-action174,
		llvm-parser-action175,
		llvm-parser-action176,
		llvm-parser-action177,
		llvm-parser-action178,
		llvm-parser-action179,
		llvm-parser-action180,
		llvm-parser-action181,
		llvm-parser-action182,
		llvm-parser-action183,
		llvm-parser-action184,
		llvm-parser-action185,
		llvm-parser-action186,
		llvm-parser-action187,
		llvm-parser-action188,
		llvm-parser-action189,
		llvm-parser-action18,
		llvm-parser-action12,
		llvm-parser-action192,
		llvm-parser-action193,
		llvm-parser-action193,
		llvm-parser-action1,
		llvm-parser-action13,
		llvm-parser-action12,
		llvm-parser-action12,
		llvm-parser-action13,
		llvm-parser-action12,
		llvm-parser-action155,
		llvm-parser-action156,
		llvm-parser-action157,
		llvm-parser-action158,
		llvm-parser-action205,
		llvm-parser-action206,
		llvm-parser-action73,
		llvm-parser-action74,
		llvm-parser-action1,
		llvm-parser-action12,
		llvm-parser-action1,
		llvm-parser-action72,
		llvm-parser-action1,
		llvm-parser-action72,
		llvm-parser-action16,
		llvm-parser-action18,
		llvm-parser-action217,
		llvm-parser-action218,
		llvm-parser-action219,
		llvm-parser-action220,
		llvm-parser-action221,
		llvm-parser-action222,
		llvm-parser-action223,
		llvm-parser-action224,
		llvm-parser-action225,
		llvm-parser-action226,
		llvm-parser-action227,
		llvm-parser-action228,
		llvm-parser-action229,
		llvm-parser-action230,
		llvm-parser-action16,
		llvm-parser-action232,
		llvm-parser-action233,
		llvm-parser-action234,
		llvm-parser-action234,
		llvm-parser-action236,
		llvm-parser-action237,
		llvm-parser-action238,
		llvm-parser-action239,
		llvm-parser-action240,
		llvm-parser-action241,
		llvm-parser-action242,
		llvm-parser-action243,
		llvm-parser-action244,
		llvm-parser-action245,
		llvm-parser-action246,
		llvm-parser-action247,
		llvm-parser-action248,
		llvm-parser-action249,
		llvm-parser-action250,
		llvm-parser-action251,
		llvm-parser-action252,
		llvm-parser-action253,
		llvm-parser-action254,
		llvm-parser-action255,
		llvm-parser-action256,
		llvm-parser-action257,
		llvm-parser-action258,
		llvm-parser-action259,
		llvm-parser-action260,
		llvm-parser-action261,
		llvm-parser-action262,
		llvm-parser-action263,
		llvm-parser-action250,
		llvm-parser-action251,
		llvm-parser-action252,
		llvm-parser-action253,
		llvm-parser-action268,
		llvm-parser-action269,
		llvm-parser-action270,
		llvm-parser-action271,
		llvm-parser-action272,
		llvm-parser-action273,
		llvm-parser-action273,
		llvm-parser-action275,
		llvm-parser-action276,
		llvm-parser-action277,
		llvm-parser-action278,
		llvm-parser-action279,
		llvm-parser-action280,
		llvm-parser-action281,
		llvm-parser-action282,
		llvm-parser-action283,
		llvm-parser-action284,
		llvm-parser-action285,
		llvm-parser-action286,
		llvm-parser-action287,
		llvm-parser-action75,
		llvm-parser-action76,
		llvm-parser-action77,
		llvm-parser-action291,
		llvm-parser-action292,
		llvm-parser-action293,
		llvm-parser-action294,
		llvm-parser-action295,
		llvm-parser-action296,
		llvm-parser-action297,
		llvm-parser-action298,
		llvm-parser-action299,
		llvm-parser-action300,
		llvm-parser-action301,
		llvm-parser-action302,
		llvm-parser-action303,
		llvm-parser-action304,
		llvm-parser-action305,
		llvm-parser-action306,
		llvm-parser-action17,
		llvm-parser-action18,
		llvm-parser-action12,
		llvm-parser-action12,
		llvm-parser-action311,
		llvm-parser-action312,
		llvm-parser-action0,
		llvm-parser-action314,
		llvm-parser-action16,
		llvm-parser-action316,
		llvm-parser-action317,
		llvm-parser-action73,
		llvm-parser-action74,
		llvm-parser-action1,
		llvm-parser-action12,
		llvm-parser-action322,
		llvm-parser-action323,
		llvm-parser-action324,
		llvm-parser-action325,
		llvm-parser-action326,
		llvm-parser-action327,
		llvm-parser-action328,
		llvm-parser-action329,
		llvm-parser-action330,
		llvm-parser-action331,
		llvm-parser-action332,
		llvm-parser-action333,
		llvm-parser-action334,
		llvm-parser-action335,
		llvm-parser-action336,
		llvm-parser-action337,
		llvm-parser-action338,
		llvm-parser-action339,
		llvm-parser-action340,
		llvm-parser-action341,
		llvm-parser-action342,
		llvm-parser-action343,
		llvm-parser-action344,
		llvm-parser-action345,
		llvm-parser-action346,
		llvm-parser-action347,
		llvm-parser-action348,
		llvm-parser-action349,
		llvm-parser-action1,
		llvm-parser-action118,
		llvm-parser-action16,
		llvm-parser-action353,
		llvm-parser-action353,
		llvm-parser-action355,
		llvm-parser-action356,
		llvm-parser-action357,
		llvm-parser-action358,
		llvm-parser-action359,
		llvm-parser-action360,
		llvm-parser-action361,
		llvm-parser-action362,
		llvm-parser-action363,
		llvm-parser-action364,
		llvm-parser-action365,
		llvm-parser-action366,
		llvm-parser-action367,
		llvm-parser-action368,
		llvm-parser-action369,
		llvm-parser-action370,
		llvm-parser-action371,
		llvm-parser-action372,
		llvm-parser-action373,
		llvm-parser-action374,
		llvm-parser-action375,
		llvm-parser-action376,
		llvm-parser-action376,
		llvm-parser-action378,
		llvm-parser-action379,
		llvm-parser-action380,
		llvm-parser-action381,
		llvm-parser-action382,
		llvm-parser-action381,
		llvm-parser-action384,
		llvm-parser-action384,
		llvm-parser-action386,
		llvm-parser-action387,
		llvm-parser-action384,
		llvm-parser-action386,
		llvm-parser-action390,
		llvm-parser-action391,
		llvm-parser-action392,
		llvm-parser-action12,
		llvm-parser-action394,
		llvm-parser-action12,
		llvm-parser-action143,
		llvm-parser-action397,
		llvm-parser-action398,
		llvm-parser-action399,
		llvm-parser-action12,
		llvm-parser-action12,
		llvm-parser-action76,
		llvm-parser-action77,
		llvm-parser-action75,
		llvm-parser-action76,
		llvm-parser-action77,
		llvm-parser-action407,
		llvm-parser-action78,
		llvm-parser-action409,
		llvm-parser-action410,
		llvm-parser-action411,
		llvm-parser-action412,
		llvm-parser-action413,
		llvm-parser-action414,
		llvm-parser-action1,
		llvm-parser-action118,
		llvm-parser-action1,
		llvm-parser-action118,
		llvm-parser-action419,
		llvm-parser-action420,
		llvm-parser-action421,
		llvm-parser-action422,
		llvm-parser-action423,
		llvm-parser-action424,
		llvm-parser-action17,
		llvm-parser-action18,
		llvm-parser-action17,
		llvm-parser-action428,
		llvm-parser-action18,
		llvm-parser-action430,
		llvm-parser-action12,
		llvm-parser-action432,
		llvm-parser-action433,
		llvm-parser-action434,
		llvm-parser-action435,
		llvm-parser-action16,
		llvm-parser-action72,
		llvm-parser-action438,
		llvm-parser-action439),
  action-nargs-table: #[2, 0, 2, 3, 9, 6, 9, 6, 6, 4, 4, 3, 1, 2, 2, 3, 0, 1, 3, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 5, 4, 7, 6, 3, 3, 3, 5, 7, 7, 3, 2, 1, 2, 2, 2, 3, 3, 12, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 0, 2, 1, 1, 1, 1, 3, 3, 1, 2, 1, 1, 1, 1, 1, 3, 1, 1, 5, 5, 3, 2, 5, 4, 1, 1, 1, 1, 1, 1, 1, 1, 0, 4, 5, 5, 1, 1, 2, 2, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 4, 3, 3, 4, 4, 3, 6, 5, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 7, 2, 0, 3, 1, 2, 2, 2, 6, 6, 1, 1, 1, 1, 0, 1, 3, 1, 1, 3, 1, 3, 1, 3, 6, 6, 8, 6, 6, 7, 7, 6, 8, 8, 5, 7, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 3, 1, 1, 3, 3, 0, 2, 1, 1, 2, 1, 0, 1, 3, 1, 2, 4, 0, 2, 0, 1, 0, 2, 0, 2, 0, 3, 2, 1, 2, 1, 2, 1, 1, 2, 1, 2, 1, 1, 1, 1, 0, 1, 1, 2, 2, 2, 1, 2, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 2, 2, 2, 3, 3, 4, 10, 10, 9, 7, 6, 16, 2, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 4, 1, 1, 1, 2, 1, 3, 1, 1, 5, 6, 2, 4, 0, 3, 5, 0, 2, 0, 1, 6, 6, 7, 7, 5, 7, 5, 5, 7, 7, 4, 7, 10, 3, 6, 4, 7, 4, 6, 9, 6, 10, 9, 4, 5, 6, 4, 5, 0, 1, 0, 3, 3, 2, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 2, 2, 3, 2, 5, 4, 6, 1, 5, 1, 0, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 5, 7, 0, 1, 0, 1, 1, 4, 5, 6, 7, 8, 1, 3, 1, 1, 3, 3, 1, 3, 4, 2, 2, 0, 2, 2, 4],
  action-nt-table: #[224, 225, 225, 226, 226, 226, 226, 226, 226, 226, 226, 226, 227, 228, 228, 229, 230, 230, 230, 231, 231, 231, 231, 231, 231, 231, 231, 231, 231, 231, 231, 231, 231, 232, 232, 232, 226, 226, 226, 226, 226, 226, 226, 226, 226, 226, 226, 233, 233, 226, 226, 234, 235, 236, 237, 238, 238, 239, 239, 240, 240, 240, 240, 240, 240, 240, 240, 240, 240, 240, 240, 240, 240, 241, 241, 242, 242, 242, 242, 243, 243, 244, 244, 245, 245, 246, 246, 246, 246, 246, 246, 246, 246, 246, 246, 246, 246, 247, 247, 247, 247, 247, 247, 247, 247, 248, 248, 249, 249, 250, 250, 250, 250, 251, 251, 251, 251, 252, 252, 253, 253, 254, 254, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 256, 256, 257, 257, 258, 259, 259, 259, 260, 260, 260, 260, 261, 261, 261, 261, 262, 262, 263, 263, 264, 264, 265, 265, 265, 265, 265, 265, 265, 265, 265, 265, 265, 265, 266, 266, 266, 266, 266, 266, 266, 266, 266, 266, 266, 266, 267, 267, 268, 268, 269, 269, 270, 270, 271, 271, 272, 273, 274, 274, 274, 274, 275, 275, 276, 276, 277, 277, 278, 278, 279, 279, 280, 280, 281, 281, 281, 281, 281, 281, 281, 281, 281, 281, 281, 281, 281, 281, 282, 282, 282, 282, 282, 283, 283, 283, 283, 283, 283, 283, 283, 284, 284, 284, 284, 284, 284, 284, 284, 284, 284, 285, 285, 285, 285, 285, 285, 285, 285, 285, 285, 285, 285, 285, 285, 285, 285, 286, 286, 286, 287, 287, 288, 288, 288, 288, 288, 288, 288, 288, 288, 288, 288, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 289, 290, 291, 291, 292, 292, 293, 293, 294, 294, 295, 295, 295, 296, 296, 297, 297, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 298, 299, 299, 300, 300, 300, 301, 301, 302, 302, 303, 303, 303, 303, 303, 303, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 305, 306, 306, 306, 306, 307, 307, 308, 308, 308, 309, 309, 309, 309, 309, 309, 309, 309, 310, 310, 311, 311, 312, 312, 313, 313, 226, 226, 226, 226, 314, 314, 315, 315, 315, 315, 316, 316, 316, 316, 317, 318, 318, 319, 319]);
